/*
   BrightnessProfileUnit.cpp

   definin soubor okna pro zobrazen jasovho profilu dku, i sloupce

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/	
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "BrightnessProfileUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "PERFGRAP"
#pragma resource "*.dfm"

TBrightnessProfileForm *BrightnessProfileForm;
//---------------------------------------------------------------------------
// initialization
__fastcall TBrightnessProfileForm::TBrightnessProfileForm(TComponent* Owner)
        : TForm(Owner)
{
   original_image = new GreyImage();
   profile_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   profile_handler->Set(*original_image);

   Rows->Checked = true;
   Columns->Checked = false;

   max_points = original_image->Cols();
   last_x = 0;
   last_y = 0;

   StatusBar1->SimpleText =  "Click to 'Options' and select a type of showing.";
   ShowAxes();
}
//---------------------------------------------------------------------------
// destructor
__fastcall TBrightnessProfileForm::~TBrightnessProfileForm()
{
   delete original_image;
   delete profile_handler;
}
//---------------------------------------------------------------------------
// close BrightnessProfileForm
void __fastcall TBrightnessProfileForm::Close1Click(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
// change to Show-Rows-Mode
void __fastcall TBrightnessProfileForm::RowsClick(TObject *Sender)
{
   Rows->Checked = true;
   Columns->Checked = false;
   max_points = original_image->Cols();
   StatusBar1->SimpleText =  "You drag mouse cursor to main window for show rows.";
   ShowAxes();
}
//---------------------------------------------------------------------------
// change to Show-Columns-Mode
void __fastcall TBrightnessProfileForm::ColumnsClick(TObject *Sender)
{
   Rows->Checked = false;
   Columns->Checked = true;
   max_points = original_image->Rows();
      StatusBar1->SimpleText =  "You drag mouse cursor to main window for show columns.";
   ShowAxes();
}
//---------------------------------------------------------------------------
// drawing the axis
void __fastcall TBrightnessProfileForm::ShowAxes()
{
  Width = max_points + 57;
  Top = PictureForm->Top+30;
  Left = PictureForm->Left + PictureForm->Width - 30 - Width;

  // draw fill-rectangle
  Image1->Canvas->Brush->Color = clWhite;
  Image1->Canvas->FillRect(Rect(0, 0, max_points+54, 325));

  // draw rectangle
  Image1->Canvas->Pen->Color = clSilver; // color of axes
  Image1->Canvas->Pen->Width = 1; // width of line
  Image1->Canvas->Rectangle(25, 17, max_points + 27, 275);

  // X-axis
  for (int i=0; i<=max_points; i+=50){
     Image1->Canvas->TextOutA(i + 23, 277, IntToStr(i));
  }

  // Y-axis
  Image1->Canvas->TextOutA(15, 268, "0");
  for (int i=32; i<=96; i+=32){
     Image1->Canvas->TextOutA(9, 268 - i, IntToStr(i));
  }
  for (int i=128; i<=256; i+=32){
     if (i == 256) {i = 255;}
     Image1->Canvas->TextOutA(3, 268 - i, IntToStr(i));
  }
}
//---------------------------------------------------------------------------
// drawing the graph
void __fastcall TBrightnessProfileForm::ShowGraph(int x, int y)
{
   if ((last_x!=x)&&(Rows->Checked == false)){
      // clear last graph (if is active Show-Columns-Mode)
      Image1->Canvas->Pen->Color = clWhite;
      for (int i=0; i<max_points; i++){
         //points[i].x = max_points + 25 - i;
         points[i].x =  i + 26;
         points[i].y = 273 - original_image->Data(last_x, i);
      }
      Image1->Canvas->Polyline(points, max_points-1);
      last_x = x;
      // draw new graph (if is active Show-Columns-Mode)
      Image1->Canvas->Pen->Color = clBlack; // color of graph
      for (int i=0; i<max_points; i++){
         //points[i].x = max_points + 25 - i;
         points[i].x = i + 26;
         points[i].y = 273 - original_image->Data(x, i);
      }
      Image1->Canvas->Polyline(points, max_points-1);
      StatusBar1->SimpleText =  "Column: "+AnsiString(x);
   }else{
      if ((last_y!=y)&&(Rows->Checked == true)){
         // clear last graph (if is active Show-Rows-Mode)
         Image1->Canvas->Pen->Color = clWhite;
         for (int i=0; i<max_points; i++){
            points[i].x = i + 26;
            points[i].y = 273 - original_image->Data(i, last_y);
         }
         Image1->Canvas->Polyline(points, max_points-1);
         last_y = y;
         // draw new graph (if is active Show-Rows-Mode)
         Image1->Canvas->Pen->Color = clBlack; // color of graph
         for (int i=0; i<max_points; i++){
            points[i].x = i + 26;
            points[i].y = 273 - original_image->Data(i, y);
         }
         Image1->Canvas->Polyline(points, max_points-1);
         StatusBar1->SimpleText =  "Row: "+AnsiString(y);
      }
   }
   Save->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TBrightnessProfileForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   PictureForm->ShowProfileOff();
   PictureForm->Open1->Visible = true;
   PictureForm->MSave->Visible = true;
   PictureForm->MSaveAs->Visible = true;
   PictureForm->MView->Visible = true;
   PictureForm->MEdit->Visible = true;
   PictureForm->MImage->Visible = true;
   PictureForm->MColor->Visible = true;
   Release();
}
//---------------------------------------------------------------------------

void __fastcall TBrightnessProfileForm::SaveClick(TObject *Sender)
{
   try{
      if(SaveDialog->Execute()){
         if(!IfFileExistsThenOverWrite(SaveDialog->FileName))
                   return;
         file_path = SaveDialog->FileName;
         // Save data
         ofstream _file(file_path.c_str());
         for (int i = 0; i < max_points; i++){
            _file << 273 - points[i].y << endl;
         }
         _file.close();
      }
   }catch(EConvertError &E){
        Application->MessageBox(E.Message.c_str(),
            "Line Profile Unit",MB_ICONQUESTION|MB_OK);
   }catch(Exception &E){
         Application->MessageBox(E.Message.c_str(),
            "Line Profile Unit",MB_ICONQUESTION|MB_OK);
   }
}
//---------------------------------------------------------------------------
bool
__fastcall TBrightnessProfileForm::IfFileExistsThenOverWrite(AnsiString _path)
{
     if(FileExists(_path))
        if(Application->MessageBox(
            ("File "+file_path+" exsits\nDo you want to overwrite it?").c_str(),
                "SaveDialog Message", MB_YESNO|MB_ICONQUESTION)==mrYes)
           return  true;
        else
           return  false;
     return true;
}
//---------------------------------------------------------------------------


