/*
   CircleProfileUnit.cpp

   definin soubor okna pro zobrazen profilu po krunici

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "CircleProfileUnit.h"
#include "PictureUnit.h"
#include "FftFiltrationUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TCircleProfileForm *CircleProfileForm;
//---------------------------------------------------------------------------
// constructor
__fastcall TCircleProfileForm::TCircleProfileForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   original_image = new GreyImage();
   sized_image = new GreyImage();
   circle_image = new GreyImage();
   graph_image = new GreyImage();
   
   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*original_image);
   Scale scale(*original_image, *sized_image, Image1->Width);

   if (sized_image->Cols() > sized_image->Rows()){
     Image1->Top=Image2->Top + (sized_image->Cols()-sized_image->Rows())/2;
   }else if (sized_image->Rows() > sized_image->Cols()){
     Image1->Left=Image2->Left + (sized_image->Rows()-sized_image->Cols())/2;
   }

   MaxRow = original_image->Rows();
   MaxCol = original_image->Cols();

   CircleCentreR = (int)(MaxRow/2);
   CircleCentreC = (int)(MaxCol/2);
   CircleRadius = (int)((MaxRow > MaxCol) ? MaxRow/5 : MaxCol/5);
   RS = (float)(original_image->Rows())/(float)(sized_image->Rows()); // reduction

   RecountLimits();

   UpDown1->Position = -(CircleCentreR);
   UpDown2->Position = CircleCentreC;
   UpDown3->Position = CircleRadius;

   last_c = CircleCentreC;
   last_r = CircleCentreR;
   last_rad = CircleRadius - 1;

   ShowCircle();
   ShowAxes();
   ShowGraph(CircleCentreR, CircleCentreC, CircleRadius);
}
//---------------------------------------------------------------------------
// destructor
__fastcall TCircleProfileForm::~TCircleProfileForm()
{
   delete (handler);
   delete (original_image);
   delete (sized_image);
   delete (circle_image);
   delete (graph_image);
}
//---------------------------------------------------------------------------
// release memory before exit
void __fastcall TCircleProfileForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();           
}
//---------------------------------------------------------------------------
// OK Button
void __fastcall TCircleProfileForm::OKBitBtnClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
// linear method of approximation (using four points)
void __fastcall TCircleProfileForm::ComputeGraph(GreyImage* image, int r, int c, int rad)
{
   float x, y;
   int p1, p2, p3, p4; // for interpolation
   for (int i = 0; i<360; i++){

     x = (float)c + cos((float)(2*3.141592654)*((float)(i))/(float)(360))*(float)rad;
     y = (float)r - sin((float)(2*3.141592654)*((float)(i))/(float)(360))*(float)rad;

     p1 = image->Data(  floor(x) ,  ceil(y)  );
     p2 = image->Data(  ceil(x)  ,  ceil(y)  );
     p3 = image->Data(  ceil(x)  ,  floor(y) );
     p4 = image->Data(  floor(x) ,  floor(y) );

     points[i].x = i + 26;
     points[i].y = 273 - (int)((p1+p2+p3+p4)/4);
   }
}
//---------------------------------------------------------------------------
// Move circle to up/down
void __fastcall TCircleProfileForm::UpDown1Click(TObject *Sender,
      TUDBtnType Button)
{
   CircleCentreR = -(UpDown1->Position);
   RecountLimits();
   ShowCircle();
   ShowGraph(CircleCentreR, CircleCentreC, CircleRadius);
}
//---------------------------------------------------------------------------
// Move circle to left/right
void __fastcall TCircleProfileForm::UpDown2Click(TObject *Sender,
      TUDBtnType Button)
{
   CircleCentreC = UpDown2->Position;
   RecountLimits();
   ShowCircle();
   ShowGraph(CircleCentreR, CircleCentreC, CircleRadius);
}
//---------------------------------------------------------------------------
// Change radius of circle
void __fastcall TCircleProfileForm::UpDown3Click(TObject *Sender,
      TUDBtnType Button)
{
   CircleRadius = UpDown3->Position;
   RecountLimits();
   ShowCircle();
   ShowGraph(CircleCentreR, CircleCentreC, CircleRadius);
}
//---------------------------------------------------------------------------
// recount max limits for moving of circle
void __fastcall TCircleProfileForm::RecountLimits()
{
   max_c = (CircleCentreC < (MaxCol - CircleCentreC)) ? CircleCentreC : (MaxCol - CircleCentreC);
   max_r = (CircleCentreR < (MaxRow - CircleCentreR)) ? CircleCentreR : (MaxRow - CircleCentreR);
   max_rad = (max_c < max_r) ? max_c : max_r;

   UpDown1->Min = - CircleRadius;
   UpDown1->Max = -(MaxRow - CircleRadius);

   UpDown2->Min = CircleRadius;
   UpDown2->Max = MaxCol - CircleRadius;

   UpDown3->Min = 10;
   UpDown3->Max = max_rad;

   StatusBar1->Panels->Items[0]->Text = "Center Row: " + AnsiString(CircleCentreR);
   StatusBar1->Panels->Items[1]->Text = "Center Column: " + AnsiString(CircleCentreC);
   StatusBar1->Panels->Items[2]->Text = "Circle Radius: " + AnsiString(CircleRadius);
}
//---------------------------------------------------------------------------
// drawing the circle
void __fastcall TCircleProfileForm::ShowCircle()
{
   handler->Assign(*sized_image);
   handler->Set(*circle_image);

   circle_image->DrawCircle(CircleCentreC/RS, CircleCentreR/RS, CircleRadius/RS - 1, 255);
   circle_image->DrawCircle(CircleCentreC/RS, CircleCentreR/RS, CircleRadius/RS, 0);

   handler->Assign(*circle_image);
   handler->Show(Image1);
}
//---------------------------------------------------------------------------
// drawing the axis
void __fastcall TCircleProfileForm::ShowAxes()
{
  // draw fill-rectangle
  Image2->Canvas->Brush->Color = clWhite;
  Image2->Canvas->FillRect(Rect(0, 0, Image2->Height, Image2->Width));

  // draw rectangle
  Image2->Canvas->Pen->Color = clSilver; // color of axes
  Image2->Canvas->Pen->Width = 1; // width of line
  Image2->Canvas->Rectangle(25, 17, 387, 275);

  // X-axis
  Image2->Canvas->Font->Color = clBlack;
  for (int i=0; i<=360; i+=45){
     Image2->Canvas->TextOutA(i + 21, 277, IntToStr(i));
  }

  // Y-axis
  Image2->Canvas->TextOutA(15, 267, "0");
  for (int i=32; i<=96; i+=32){
     Image2->Canvas->TextOutA(9, 267 - i, IntToStr(i));
  }
  for (int i=128; i<=256; i+=32){
     if (i == 256) {i = 255;}
     Image2->Canvas->TextOutA(3, 267 - i, IntToStr(i));
  }
}
//---------------------------------------------------------------------------
// drawing the graph
void __fastcall  TCircleProfileForm::ShowGraph(int new_r, int new_c, int new_rad)
{
   if ((last_r!=CircleCentreR)|((last_c!=CircleCentreC)|(last_rad!=CircleRadius))){

      // clear last graph
      Image2->Canvas->Pen->Color = clWhite;
      ComputeGraph(original_image, last_r, last_c, last_rad);
      Image2->Canvas->Polyline(points, 359);

      // draw new graph
      Image2->Canvas->Pen->Color = clBlack; // color of graph
      ComputeGraph(original_image, new_r, new_c, new_rad);
      Image2->Canvas->Polyline(points, 359);

      last_r = new_r;
      last_c = new_c;
      last_rad = new_rad;
   }
}
//-------------------------------------------------------------------------
void __fastcall TCircleProfileForm::BitBtn1Click(TObject *Sender)
{
   try{
      if(SaveDialog->Execute()){
         if(!IfFileExistsThenOverWrite(SaveDialog->FileName))
                   return;
         file_path = SaveDialog->FileName;
         // Save data
         file_txt->SaveProfile(points, file_path);
   }
   }catch(EConvertError &E){
        Application->MessageBox(E.Message.c_str(),
            "Circle Profile Unit",MB_ICONQUESTION|MB_OK);
   }catch(Exception &E){
         Application->MessageBox(E.Message.c_str(),
            "Circle Profile Unit",MB_ICONQUESTION|MB_OK);
   }
}
//---------------------------------------------------------------------------
bool
__fastcall TCircleProfileForm::IfFileExistsThenOverWrite(AnsiString _path)
{
     if(FileExists(_path))
        if(Application->MessageBox(
            ("File "+file_path+" exsits\nDo you want to overwrite it?").c_str(),
                "SaveDialog Message", MB_YESNO|MB_ICONQUESTION)==mrYes)
           return  true;
        else
           return  false;
     return true;
}
//---------------------------------------------------------------------------


