/*
   LineProfileUnit.cpp

   definin soubor okna pro zobrazen profilu po sece

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "LineProfileUnit.h"
#include "PictureUnit.h"
#include "FftFiltrationUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TLineProfileForm *LineProfileForm;
//---------------------------------------------------------------------------
// constructor
__fastcall TLineProfileForm::TLineProfileForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   original_image = new GreyImage();
   sized_image = new GreyImage();
   line_image = new GreyImage();
   graph_image = new GreyImage();
   
   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*original_image);
   Scale scale(*original_image, *sized_image, Image1->Width);

   if (sized_image->Cols() > sized_image->Rows()){
     Image1->Top=Image2->Top + (sized_image->Cols()-sized_image->Rows())/2;
   }else if (sized_image->Rows() > sized_image->Cols()){
     Image1->Left=Image2->Left + (sized_image->Rows()-sized_image->Cols())/2;
   }

   MaxRow = original_image->Rows();
   MaxCol = original_image->Cols();

   P1r = (int)(MaxRow/10);
   P1c = (int)(MaxCol/10);
   P2r = (int)(MaxRow - MaxRow/10);
   P2c = (int)(MaxCol - MaxCol/10);
   RS = (float)(original_image->Rows())/(float)(sized_image->Rows()); // reduction
   UpDown1->Max = MaxRow;
   UpDown2->Max = MaxCol;
   UpDown3->Max = MaxRow;
   UpDown4->Max = MaxCol;

   UpDown1->Position = P1r;
   UpDown2->Position = P1c;
   UpDown3->Position = P2r;
   UpDown4->Position = P2c;

   last_p1r = P1r;
   last_p1c = P1c;
   last_p2r = P2r - 1;
   last_p2c = P2c - 1;

   ShowLine();
   ShowAxes();
   ShowGraph(P1r, P1c, P2r, P2c);
}
//---------------------------------------------------------------------------
__fastcall TLineProfileForm::~TLineProfileForm()
{
  delete (handler);
  delete (original_image);
  delete (sized_image);
  delete (line_image);
  delete (graph_image);
}
//---------------------------------------------------------------------------
// linear method of approximation (using four points)
void __fastcall TLineProfileForm::ComputeGraph(GreyImage* image, int r1, int c1, int r2, int c2)
{
   float x, y;
   int p1, p2, p3, p4; // for interpolation
   float step_r, step_c;

   step_r = ((float)((r2 - r1)))/(float)(360);
   step_c = ((float)((c2 - c1)))/(float)(360);

   x = c1;
   y = r1;

   for (int i = 0; i<360; i++){

     x += step_c;
     y += step_r;

     p1 = image->Data(  floor(x) ,  ceil(y)  );
     p2 = image->Data(  ceil(x)  ,  ceil(y)  );
     p3 = image->Data(  ceil(x)  ,  floor(y) );
     p4 = image->Data(  floor(x) ,  floor(y) );

     points[i].x = i + 26;
     points[i].y = 273 - (int)((p1+p2+p3+p4)/4);
   }
}
//---------------------------------------------------------------------------
// drawing the line
void __fastcall TLineProfileForm::ShowLine()
{
   handler->Assign(*sized_image);
   handler->Set(*line_image);

   line_image->DrawCircle(P1c/RS, P1r/RS, 5, 255);
   line_image->DrawCircle(P1c/RS, P1r/RS, 3, 0);

   line_image->Drawline(P1r/RS - 1, P1c/RS - 1, P2r/RS - 1, P2c/RS - 1, 255);
   line_image->Drawline(P1r/RS, P1c/RS, P2r/RS, P2c/RS, 0);

   handler->Assign(*line_image);
   handler->Show(Image1);
}
//---------------------------------------------------------------------------
// drawing the axis
void __fastcall TLineProfileForm::ShowAxes()
{
  // draw fill-rectangle
  Image2->Canvas->Brush->Color = clWhite;
  Image2->Canvas->FillRect(Rect(0, 0, Image2->Height, Image2->Width));

  // draw rectangle
  Image2->Canvas->Pen->Color = clSilver; // color of axes
  Image2->Canvas->Pen->Width = 1; // width of line
  Image2->Canvas->Rectangle(25, 17, 387, 275);

  // X-axis
  Image2->Canvas->Font->Color = clBlack;
  for (int i=0; i<=360; i+=90){
     Image2->Canvas->TextOutA(i + 21, 277, IntToStr((int)((float)sqrt((P1r-P2r)*(P1r-P2r)+(P1c-P2c)*(P1c-P2c))*((float)(i)/360))));
  }

  // Y-axis
  Image2->Canvas->TextOutA(15, 267, "0");
  for (int i=32; i<=96; i+=32){
     Image2->Canvas->TextOutA(9, 267 - i, IntToStr(i));
  }
  for (int i=128; i<=256; i+=32){
     if (i == 256) {i = 255;}
     Image2->Canvas->TextOutA(3, 267 - i, IntToStr(i));
  }
}
//---------------------------------------------------------------------------
// drawing the graph
void __fastcall  TLineProfileForm::ShowGraph(int new_p1r, int new_p1c, int new_p2r, int new_p2c)
{
   if (((last_p1r!=new_p1r)|(last_p1c!=new_p1c))|((last_p2r!=new_p2r)|(last_p2c!=new_p2c))){

      // Clear last graph
      Image2->Canvas->Pen->Color = clWhite;
      ComputeGraph(original_image, last_p1r, last_p1c, last_p2r, last_p2c);
      Image2->Canvas->Polyline(points, 359);

      // Draw new graph
      Image2->Canvas->Pen->Color = clBlack; // color of graph
      ComputeGraph(original_image, new_p1r, new_p1c, new_p2r, new_p2c);
      Image2->Canvas->Polyline(points, 359);

      // Clear X-axis legends
      Image2->Canvas->Font->Color = clWhite;
      for (int i=0; i<=360; i+=45){
         Image2->Canvas->TextOutA(i + 21, 277, IntToStr((int)((float)sqrt((last_p1r-last_p2r)*(last_p1r-last_p2r)+(last_p1c-last_p2c)*(last_p1c-last_p2c))*((float)(i)/360))));
      }

      // Refresh X-axis legends
      Image2->Canvas->Font->Color = clBlack;
      for (int i=0; i<=360; i+=45){
         Image2->Canvas->TextOutA(i + 21, 277, IntToStr((int)((float)sqrt((P1r-P2r)*(P1r-P2r)+(P1c-P2c)*(P1c-P2c))*((float)(i)/360))));
      }

      last_p1r = new_p1r;
      last_p1c = new_p1c;
      last_p2r = new_p2r;
      last_p2c = new_p2c;

      StatusBar1->Panels->Items[0]->Text = "P1 (row): " + AnsiString(P1r);
      StatusBar1->Panels->Items[1]->Text = "P1 (column): " + AnsiString(P1c);
      StatusBar1->Panels->Items[2]->Text = "P2 (row): " + AnsiString(P2r);
      StatusBar1->Panels->Items[3]->Text = "P2 (column): " + AnsiString(P2c);
   }
}
//---------------------------------------------------------------------------
void __fastcall TLineProfileForm::UpDown1Click(TObject *Sender,
      TUDBtnType Button)
{
   P1r = UpDown1->Position;
   ShowLine();
   ShowGraph(P1r, P1c, P2r, P2c);
}
//---------------------------------------------------------------------------
void __fastcall TLineProfileForm::UpDown2Click(TObject *Sender,
      TUDBtnType Button)
{
   P1c = UpDown2->Position;
   ShowLine();
   ShowGraph(P1r, P1c, P2r, P2c);
}
//---------------------------------------------------------------------------
void __fastcall TLineProfileForm::UpDown3Click(TObject *Sender,
      TUDBtnType Button)
{
   P2r = UpDown3->Position;
   ShowLine();
   ShowGraph(P1r, P1c, P2r, P2c);
}
//---------------------------------------------------------------------------
void __fastcall TLineProfileForm::UpDown4Click(TObject *Sender,
      TUDBtnType Button)
{
   P2c = UpDown4->Position;
   ShowLine();
   ShowGraph(P1r, P1c, P2r, P2c);
}
//---------------------------------------------------------------------------
void __fastcall TLineProfileForm::BitBtn1Click(TObject *Sender)
{
   try{
      if(SaveDialog->Execute()){
         if(!IfFileExistsThenOverWrite(SaveDialog->FileName))
                   return;
         file_path = SaveDialog->FileName;
         // Save data
         file_txt->SaveProfile(points, file_path);
      }
   }catch(EConvertError &E){
        Application->MessageBox(E.Message.c_str(),
            "Line Profile Unit",MB_ICONQUESTION|MB_OK);
   }catch(Exception &E){
         Application->MessageBox(E.Message.c_str(),
            "Line Profile Unit",MB_ICONQUESTION|MB_OK);
   }
}
//---------------------------------------------------------------------------
bool
__fastcall TLineProfileForm::IfFileExistsThenOverWrite(AnsiString _path)
{
     if(FileExists(_path))
        if(Application->MessageBox(
            ("File "+file_path+" exsits\nDo you want to overwrite it?").c_str(),
                "SaveDialog Message", MB_YESNO|MB_ICONQUESTION)==mrYes)
           return  true;
        else
           return  false;
     return true;
}
//---------------------------------------------------------------------------



