/*
   CentroidUnit.cpp

   definin soubor okna pro vpoet tit obrazu - centroidu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "CentroidUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TCentroidForm *CentroidForm;
//---------------------------------------------------------------------------
__fastcall TCentroidForm::TCentroidForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   centroid_image = new GreyImage();
   sized_image = new GreyImage();
   centroid_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   centroid_handler->Set(*centroid_image);
   Scale scale(*centroid_image, *sized_image, Image1->Width);

   if (sized_image->Cols() > sized_image->Rows()){
     Image1->Top=Image1->Top + (sized_image->Cols()-sized_image->Rows())/2;
   }else if (sized_image->Rows() > sized_image->Cols()){
     Image1->Left=Image1->Left + (sized_image->Rows()-sized_image->Cols())/2;
   }

   centroid_handler->Assign(*sized_image);
   centroid_handler->Show(Image1);
   x_rate =  (float)centroid_handler->Width();
   y_rate =  (float)centroid_handler->Height();
   centroid_handler->Assign(*centroid_image);
   x_rate /= centroid_handler->Width();
   y_rate /= centroid_handler->Height();

   object_value = 255;
   ComputeCentre();
}
//---------------------------------------------------------------------------
__fastcall TCentroidForm::~TCentroidForm()
{
   delete (centroid_image);
   delete (sized_image);
   delete (centroid_handler);
}
//---------------------------------------------------------------------------
// cancel button
void __fastcall TCentroidForm::BitBtn1Click(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
// compute geographical centre of object
void __fastcall TCentroidForm::ComputeCentre()
{
  object_area = 0;
  x_coordinate = 0;
  y_coordinate = 0;
  for (int i=0; i < centroid_handler->Width(); i++){
     for (int j=0; j < centroid_handler->Height(); j++){
        if (centroid_image->Data(i,j) == object_value){
            x_coordinate += i;
            y_coordinate += j;
            object_area++;
        }
     }
  }

  centroid_handler->Assign(*sized_image);
  centroid_handler->Show(Image1);
  centroid_handler->Assign(*centroid_image);

  if (object_area!=0){
     x_position = (int)(x_coordinate/object_area);
     y_position = (int)(y_coordinate/object_area);
     Edit1->Text = IntToStr(x_position);
     Edit2->Text = IntToStr(y_position);
     // light cross
     Image1->Canvas->Pen->Color = clSilver;
     Image1->Canvas->Pen->Width = 3;
     Image1->Canvas->MoveTo((int)(x_rate*x_position), 0);
     Image1->Canvas->LineTo((int)(x_rate*x_position), Image1->Height);
     Image1->Canvas->MoveTo(0, (int)(y_rate*y_position));
     Image1->Canvas->LineTo(Image1->Width, (int)(y_rate*y_position));
      // dark cross
     Image1->Canvas->Pen->Color = clGray;
     Image1->Canvas->Pen->Width = 1;
     Image1->Canvas->MoveTo((int)(x_rate*x_position), 0);
     Image1->Canvas->LineTo((int)(x_rate*x_position), Image1->Height);
     Image1->Canvas->MoveTo(0, (int)(y_rate*y_position));
     Image1->Canvas->LineTo(Image1->Width, (int)(y_rate*y_position));
   }else{
     Edit1->Text = "Object not found!";
     Edit2->Text = "Object not found!";
  }
}
//---------------------------------------------------------------------------
//change information about object - object_value
void __fastcall TCentroidForm::Edit3Change(TObject *Sender)
{
   try{
      if(Edit3->Text=="") throw VEMsg("");
      object_value = Edit3->Text.ToInt();
      if(object_value < 0){
         Edit3->Text = 0;
         object_value = 0;
         throw VEMsg("");
      }
      if(object_value > 255){
         Edit3->Text = 255;
         object_value = 255;
         throw VEMsg("");
      }
      PercentUpDown->Position = Edit3->Text.ToInt();
      // compute geographical centre of object
      ComputeCentre();
   }catch(EConvertError& e){
         MessageBox(Handle, e.Message.c_str(),
            "Centroid:", MB_ICONWARNING | MB_OK);
   }catch(VEMsg& e){
       ; // empty statemnet-just exit the function which is in a turn called
         // with a new value of Edit1->Text
   }
}
//---------------------------------------------------------------------------
// global close
void __fastcall TCentroidForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------

void __fastcall TCentroidForm::PercentUpDownClick(TObject *Sender,
      TUDBtnType Button)
{
   Edit3->Text = PercentUpDown->Position;        
}
//---------------------------------------------------------------------------



