/*
   SnrPsnrUnit.cpp

   definin soubor okna pro vpoet parametr SNR/PSNR

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "SnrPsnrUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TSnrPsnrForm *SnrPsnrForm;
//---------------------------------------------------------------------------
// constructor
__fastcall TSnrPsnrForm::TSnrPsnrForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   OpenDialog->Filter="Bitmap picture (*.bmp)|*.bmp";
   OpenDialog->DefaultExt = "bmp";

   reconstructed_image = new GreyImage();
   original_image = new GreyImage();
   error_image = new GreyImage();

   sized_reconstructed_image = new GreyImage();
   sized_original_image = new GreyImage();
   sized_error_image = new GreyImage();

   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*reconstructed_image);
   handler->Set(*original_image);
   handler->Set(*error_image);
   Scale scale(*reconstructed_image, *sized_reconstructed_image, Image1->Width);

   handler->Assign(*sized_reconstructed_image);
   handler->Set(*sized_original_image);
   handler->Set(*sized_error_image);
   handler->Show(Image1);
   StatusBar1->SimpleText = "Click to load button and select original image.";
}
//---------------------------------------------------------------------------
// destructor
__fastcall TSnrPsnrForm::~TSnrPsnrForm()
{
  delete (handler);

  delete (reconstructed_image);
  delete (original_image);
  delete (error_image);

  delete (sized_reconstructed_image);
  delete (sized_original_image);
  delete (sized_error_image);
}
//---------------------------------------------------------------------------
// Exit
void __fastcall TSnrPsnrForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------
// open original image
void __fastcall TSnrPsnrForm::Button1Click(TObject *Sender)
{

if(OpenDialog->Execute()){
    delete handler;

    handler = new BMPHandler(OpenDialog->FileName);

    handler->Set(*original_image);
    original_file_name = ExtractFileName(OpenDialog->FileName);

    if(reconstructed_image->Cols()!= original_image->Cols()||reconstructed_image->Rows()!= original_image->Rows()){
       AnsiString message = "Picture " + original_file_name + " isn't of the same size"+".\nRequired size is "+
       AnsiString(reconstructed_image->Cols())+ " x "+ AnsiString(reconstructed_image->Rows());
       MessageBox(Handle, message.c_str(), "SNR/PSNR", MB_ICONQUESTION | MB_OK);
    }else{
       Scale scale(*original_image, *sized_original_image, Image2->Width);
       handler->Assign(*sized_original_image);
       handler->Show(Image2);
       ///
          ComputeAndShowErrorImage();
          ComputeSNR();
          ComputePSNR();

          if (SNR != 0){
             // three decimal places
             Edit1->Text = FormatFloat("0.000", SNR);
             Edit1->Text = Edit1->Text + " dB";
          }else
             Edit1->Text = "infinite";

          if (PSNR != 0){
             // three decimal places
             Edit2->Text = FormatFloat("0.000", PSNR);
             Edit2->Text = Edit2->Text + " dB";
          }else
             Edit2->Text = "infinite";
       ///
       Image2->ShowHint = true;
    }
  }
}
//---------------------------------------------------------------------------
// compute and show Error image
void __fastcall TSnrPsnrForm::ComputeAndShowErrorImage()
{
  for (int r = 0; r < reconstructed_image->Rows(); r++){
     for (int c = 0; c < reconstructed_image->Cols(); c++){
       point1 = (float)reconstructed_image->Data(c, r);
       point2 = (float)original_image->Data(c, r);
       point_diff = 128+2*(point2-point1);
       point_diff = (point_diff < 0) ? 0 : point_diff;
       point_diff = (point_diff > 255) ? 255 : point_diff;
       error_image->Data(c, r) = (unsigned char)(point_diff);
     }
   }

   Scale scale(*error_image, *sized_error_image, Image3->Width);
   handler->Assign(*sized_error_image);
   handler->Show(Image3);
   StatusBar1->SimpleText = "Click to OK for use Error image.";
   BitBtn1->Enabled = true;
   Image3->ShowHint = true;
}
//---------------------------------------------------------------------------
// compute SNR
void __fastcall TSnrPsnrForm::ComputeSNR()
{
   float sum_mse = 0;
   float sum_original = 0;

   for (int r = 0; r < reconstructed_image->Rows(); r++){
     for (int c = 0; c < reconstructed_image->Cols(); c++){
       point1 = (float)original_image->Data(c, r);
       point2 = (float)reconstructed_image->Data(c, r);
       sum_mse += (float)pow(point1 - point2, 2);
       sum_original += (float)pow(point1, 2);
     }
   }
   if ((sum_original != 0)&&(sum_mse != 0))
        SNR = (float)(10*log(sum_original / sum_mse));
   else SNR = 0;
}
//---------------------------------------------------------------------------
// compute PSNR
void __fastcall TSnrPsnrForm::ComputePSNR()
{
  float sum_mse = 0;
  float MSE, RMSE;

  for (int r = 0; r < reconstructed_image->Rows(); r++){
     for (int c = 0; c < reconstructed_image->Cols(); c++){
       point1 = (float)original_image->Data(c, r);
       point2 = (float)reconstructed_image->Data(c, r);
       sum_mse += (float)pow(point1 - point2, 2);
     }
   }
   if (sum_mse != 0){
      MSE = (sum_mse / ((float)reconstructed_image->Rows()*(float)reconstructed_image->Cols()));
      RMSE = sqrt(MSE);
      PSNR = (float)(10*log((float)(255) / RMSE));
   }else PSNR = 0;
}
//---------------------------------------------------------------------------
// OK button
void __fastcall TSnrPsnrForm::BitBtn1Click(TObject *Sender)
{
   PictureForm->AssignList(error_image);
   Close();
}
//---------------------------------------------------------------------------
// Cancel button
void __fastcall TSnrPsnrForm::BitBtn2Click(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------

