/*
   StatisticsUnit.cpp

   definin soubor okna pro vpoet statistickch veliin

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "StatisticsUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TStatisticsForm *StatisticsForm;
//---------------------------------------------------------------------------
__fastcall TStatisticsForm::TStatisticsForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   statistics_image = new GreyImage();
   sized_image = new GreyImage();
   statistics_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   statistics_handler->Set(*statistics_image);
   Scale scale(*statistics_image, *sized_image, Image1->Width);
   statistics_handler->Assign(*sized_image);
   statistics_handler->Show(Image1);

   ComputeMean(statistics_image);
   ComputeDispersion(statistics_image);
   ComputeEntropy(statistics_image);
}
//---------------------------------------------------------------------------
__fastcall TStatisticsForm::~TStatisticsForm()
{
   delete (statistics_image);
   delete (sized_image);
   delete (statistics_handler);
}
//---------------------------------------------------------------------------
void __fastcall TStatisticsForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------
// compute Arithmetic mean
void __fastcall TStatisticsForm::ComputeMean(GreyImage* image)
{
  float sum = 0;

  for (int i=0; i < image->Cols(); i++){
     for (int j=0; j < image->Rows(); j++){
        sum += image->Data(i,j);
     }
  }
  mean = sum / (float)(image->Rows()*image->Cols());
  Edit1->Text = FormatFloat("0.000", mean);
}
//---------------------------------------------------------------------------
// compute Dispersion and Standart deviation
void __fastcall TStatisticsForm::ComputeDispersion(GreyImage* image)
{
  float sum = 0;
  float points = (float)(image->Rows()*image->Cols());

  for (int i=0; i < image->Cols(); i++){
     for (int j=0; j < image->Rows(); j++){
        sum += (((float)(image->Data(i,j)-mean)*(float)(image->Data(i,j)-mean)));
     }
  }
  
  dispersion = (sum) / points;
  // / (float)(image->Rows()*image->Cols());
  deviation = sqrt(dispersion);
  
  Edit2->Text = FormatFloat("0.000", dispersion);
  Edit3->Text = FormatFloat("0.000", deviation);
}
//---------------------------------------------------------------------------
// compute Entropia, max and min
void __fastcall TStatisticsForm::ComputeEntropy(GreyImage* image)
{
  int maximum = 0;
  int minimum = 255;
  int hist[256];
  float entropy_log;
  float points = (float)(image->Rows()*image->Cols());

  // find max and min
  for (int i=0; i < image->Cols(); i++){
     for (int j=0; j < image->Rows(); j++){
        if (image->Data(i,j)>maximum){ maximum = image->Data(i,j);}
        if (image->Data(i,j)<minimum){ minimum = image->Data(i,j);}
     }
  }

  // prepare histogram
  for (int i=0; i <= maximum; i++){
    hist[i] = 0;
  }

  // compute histogram
  for (int i=0; i < image->Cols(); i++){
    for (int j=0; j < image->Rows(); j++){
      hist[image->Data(i,j)]++;
    }
  }
  
  // compute entropia
  entropy = 0;
  for (int i=0; i <= maximum; i++){
    if (hist[i] == 0){
      hist[i]=1;
    }
    entropy_log = (float)(hist[i]/points)*log10((float)(hist[i]/points));
    entropy += entropy_log;
  }
  entropy = (-1) * entropy;
  Edit4->Text = FormatFloat("0.000", entropy);
  Edit5->Text = FormatFloat("0.000", maximum);
  Edit6->Text = FormatFloat("0.000", minimum);
}
//---------------------------------------------------------------------------

