/*
   CMYToolUnit.cpp

   definin soubor okna pro pravu jednotlivch kanl v CMY modelu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "CMYToolUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TCMYToolForm *CMYToolForm;
//---------------------------------------------------------------------------
__fastcall TCMYToolForm::TCMYToolForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   color_handler = new BMPHandlerColor(*(PictureForm->GetBMPHandlerColor()));
   color_image = new ColorImage();
   color_handler->Set(*color_image);

   scaled_color_image = new ColorImage();
   Scale_color scale(*color_image, *scaled_color_image, Image1->Width);

   if (scaled_color_image->Cols() > scaled_color_image->Rows()){
     Image1->Top=Image1->Top + (scaled_color_image->Cols()-scaled_color_image->Rows())/2;
   }else if (scaled_color_image->Rows() > scaled_color_image->Cols()){
     Image1->Left=Image1->Left + (scaled_color_image->Rows()-scaled_color_image->Cols())/2;
   }

   c_image = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());
   m_image = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());
   y_image = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());

   c_image_temp = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());
   m_image_temp = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());
   y_image_temp = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());

   scaled_color_image->RGBtoCMY(c_image, m_image, y_image);
   scaled_color_image->RGBtoCMY(c_image_temp, m_image_temp, y_image_temp);

   color_handler->CreateBitmapAndAssign(scaled_color_image);
   color_handler->Show(Image1);

   // create scale image for showing
   CMY_scale = new ColorImage(255,1);

   c_scale = new FloatImage(255,1);
   m_scale = new FloatImage(255,1);
   y_scale = new FloatImage(255,1);

   c_scale_temp = new FloatImage(255,1);
   m_scale_temp = new FloatImage(255,1);
   y_scale_temp = new FloatImage(255,1);

   for (int c = 0; c < c_scale->Cols(); c++){
     c_scale->Data(c, 0) = c_scale_temp->Data(c, 0) = (float)(1) - ((float)(c) / (float)(255));
     m_scale->Data(c, 0) = m_scale_temp->Data(c, 0) = (float)(1) - ((float)(c) / (float)(255));
     y_scale->Data(c, 0) = y_scale_temp->Data(c, 0) = (float)(1) - ((float)(c) / (float)(255));
   }

   CMY_scale->CMYtoRGB(c_scale, m_scale, y_scale);
   color_handler->CreateBitmapAndAssign(CMY_scale);
   color_handler->Show(Image2);

   StatusBar1->SimpleText = ("You can change Cyan, Magenta and Yellow levels.");
}
//---------------------------------------------------------------------------
// destructor
__fastcall TCMYToolForm::~TCMYToolForm()
{
   delete color_image;
   delete scaled_color_image;
   delete color_handler;

   delete c_image;
   delete m_image;
   delete y_image;

   delete c_image_temp;
   delete m_image_temp;
   delete y_image_temp;

   delete c_scale;
   delete m_scale;
   delete y_scale;

   delete c_scale_temp;
   delete m_scale_temp;
   delete y_scale_temp;

   delete CMY_scale;
}
//---------------------------------------------------------------------------
void __fastcall TCMYToolForm::BitBtn2Click(TObject *Sender)
{
   Close();          
}
//---------------------------------------------------------------------------
void __fastcall TCMYToolForm::TrackBar1Change(TObject *Sender)
{
  change_c = true;
  Edit1->Text = TrackBar1->Position;
  Show_Scale();
  Timer1->Enabled = false;
  Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TCMYToolForm::TrackBar2Change(TObject *Sender)
{
  change_m = true;
  Edit2->Text = TrackBar2->Position;
  Show_Scale();
  Timer1->Enabled = false;
  Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TCMYToolForm::TrackBar3Change(TObject *Sender)
{
  change_y = true;
  Edit3->Text = TrackBar3->Position;
  Show_Scale();
  Timer1->Enabled = false;
  Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TCMYToolForm::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   
   if (change_c == true) {
      Change(c_image, c_image_temp, (float)(0.01)*(float)(TrackBar1->Position));
      change_c = false;
   }
   if (change_m == true) {
      Change(m_image, m_image_temp, (float)(0.01)*(float)(TrackBar2->Position));
      change_m = false;
   }
   if (change_y == true) {
      Change(y_image, y_image_temp, (float)(0.01)*(float)(TrackBar3->Position));
      change_y = false;
   }

   scaled_color_image->CMYtoRGB(c_image_temp, m_image_temp, y_image_temp);
   color_handler->CreateBitmapAndAssign(scaled_color_image);
   color_handler->Show(Image1);
}
//---------------------------------------------------------------------------
void __fastcall TCMYToolForm::Change(FloatImage* old_values, FloatImage* new_values, float _f)
{
   for (int r = 0; r < old_values->Rows(); r++){
      for (int c = 0; c < old_values->Cols(); c++){
         new_values->Data(c, r) = old_values->Data(c, r) + _f;
         if (new_values->Data(c,r) < 0) { new_values->Data(c, r) = 0; }
         if (new_values->Data(c,r) > 1) { new_values->Data(c, r) = 1; }
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TCMYToolForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------

void __fastcall TCMYToolForm::Button1Click(TObject *Sender)
{
   Timer1->Enabled = false;

   TrackBar1->Position = 0;
   TrackBar2->Position = 0;
   TrackBar3->Position = 0;

   Change(c_image, c_image_temp, 0);
   Change(m_image, m_image_temp, 0);
   Change(y_image, y_image_temp, 0);

   for (int c = 0; c < c_scale->Cols(); c++){
     c_scale_temp->Data(c, 0) = (float)(1) - ((float)(c) / (float)(255));
     m_scale_temp->Data(c, 0) = (float)(1) - ((float)(c) / (float)(255));
     y_scale_temp->Data(c, 0) = (float)(1) - ((float)(c) / (float)(255));
   }

   scaled_color_image->CMYtoRGB(c_image_temp, m_image_temp, y_image_temp);
   color_handler->CreateBitmapAndAssign(scaled_color_image);
   color_handler->Show(Image1);

   CMY_scale->CMYtoRGB(c_scale_temp, m_scale_temp, y_scale_temp);
   color_handler->CreateBitmapAndAssign(CMY_scale);
   color_handler->Show(Image2);
}
//---------------------------------------------------------------------------

void __fastcall TCMYToolForm::BitBtn1Click(TObject *Sender)
{
   // Prepare variable
   delete c_image;
   delete m_image;
   delete y_image;

   c_image = new FloatImage(color_image->Cols(), color_image->Rows());
   m_image = new FloatImage(color_image->Cols(), color_image->Rows());
   y_image = new FloatImage(color_image->Cols(), color_image->Rows());

   // RGB -> CMY
   color_image->RGBtoCMY(c_image, m_image, y_image);

   // Change HSV coecifients
   Change(c_image, c_image, ((float)(0.01)*(float)(TrackBar1->Position)));
   Change(m_image, m_image, ((float)(0.01)*(float)(TrackBar2->Position)));
   Change(y_image, y_image, ((float)(0.01)*(float)(TrackBar3->Position)));

   // CMY -> RGB
   color_image->CMYtoRGB(c_image, m_image, y_image);

   // Copy to main Window
   PictureForm->CreateColorImage(color_image);
   Close();
}
//---------------------------------------------------------------------------
// Show RBG and CMY scale image
void __fastcall TCMYToolForm::Show_Scale()
{
   if (change_c == true){
      Change(c_scale, c_scale_temp, ((float)(0.01)*(float)(TrackBar1->Position)));
    }
   if (change_m == true){
      Change(m_scale, m_scale_temp, ((float)(0.01)*(float)(TrackBar2->Position)));
   }
   if (change_y == true){
      Change(y_scale, y_scale_temp, ((float)(0.01)*(float)(TrackBar3->Position)));
   }

   CMY_scale->CMYtoRGB(c_scale_temp, m_scale_temp, y_scale_temp);
   color_handler->CreateBitmapAndAssign(CMY_scale);
   color_handler->Show(Image2);
}
//---------------------------------------------------------------------------

