/*
   HSLToolUnit.cpp

   definin soubor okna pro pravu jednotlivch kanl v HSL modelu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "HSLToolUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
THSLToolForm *HSLToolForm;
//---------------------------------------------------------------------------
// constructor
__fastcall THSLToolForm::THSLToolForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   color_handler = new BMPHandlerColor(*(PictureForm->GetBMPHandlerColor()));
   color_image = new ColorImage();
   color_handler->Set(*color_image);

   scaled_color_image = new ColorImage();
   Scale_color scale(*color_image, *scaled_color_image, Image1->Width);

   if (scaled_color_image->Cols() > scaled_color_image->Rows()){
     Image1->Top=Image1->Top + (scaled_color_image->Cols()-scaled_color_image->Rows())/2;
   }else if (scaled_color_image->Rows() > scaled_color_image->Cols()){
     Image1->Left=Image1->Left + (scaled_color_image->Rows()-scaled_color_image->Cols())/2;
   }

   h_image = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());
   s_image = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());
   l_image = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());

   h_image_temp = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());
   s_image_temp = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());
   l_image_temp = new FloatImage(scaled_color_image->Cols(), scaled_color_image->Rows());

   scaled_color_image->RGBtoHSL(h_image, s_image, l_image);
   scaled_color_image->RGBtoHSL(h_image_temp, s_image_temp, l_image_temp);
   scaled_color_image->HSLtoRGB(h_image_temp, s_image_temp, l_image_temp);

   color_handler->CreateBitmapAndAssign(scaled_color_image);
   color_handler->Show(Image1);

   // create scale image for showing
   HSL_scale = new ColorImage(255,1);

   h_scale = new FloatImage(255,1);
   s_scale = new FloatImage(255,1);
   l_scale = new FloatImage(255,1);

   h_scale_temp = new FloatImage(255,1);
   s_scale_temp = new FloatImage(255,1);
   l_scale_temp = new FloatImage(255,1);

   for (int c = 0; c < h_scale->Cols(); c++){
     h_scale->Data(c, 0) = h_scale_temp->Data(c, 0) = (float)(c)/(float)(255);
     s_scale->Data(c, 0) = (float)(1);
     l_scale->Data(c, 0) = (float)(1);
   }

   HSL_scale->HSVtoRGB(h_scale, s_scale, l_scale);
   color_handler->CreateBitmapAndAssign(HSL_scale);
   color_handler->Show(Image2);

   for (int c = 0; c < h_scale->Cols(); c++){
     s_scale->Data(c, 0) = s_scale_temp->Data(c, 0) = (float)(0.5);
     l_scale->Data(c, 0) = l_scale_temp->Data(c, 0) = (float)(0.5);
   }

   HSL_scale->HSLtoRGB(h_scale_temp, s_scale_temp, l_scale_temp);
   color_handler->CreateBitmapAndAssign(HSL_scale);
   color_handler->Show(Image3);

   StatusBar1->SimpleText = ("You can change Hue, Saturation and Lightness levels.");
}
//---------------------------------------------------------------------------
// destructor
__fastcall THSLToolForm::~THSLToolForm()
{
   delete color_image;
   delete scaled_color_image;
   delete color_handler;

   delete h_image;
   delete s_image;
   delete l_image;

   delete h_image_temp;
   delete s_image_temp;
   delete l_image_temp;

   delete h_scale;
   delete s_scale;
   delete l_scale;

   delete h_scale_temp;
   delete s_scale_temp;
   delete l_scale_temp;

   delete HSL_scale;
}
//---------------------------------------------------------------------------
// Change one part of HSL model
void __fastcall THSLToolForm::Change(FloatImage* old_values, FloatImage* new_values,
                float _f, bool overflow)
{
   for (int r = 0; r < old_values->Rows(); r++){
      for (int c = 0; c < old_values->Cols(); c++){
         new_values->Data(c, r) = old_values->Data(c, r) + _f;
         if (new_values->Data(c,r) < 0){
            if (overflow == true) { new_values->Data(c, r) += 1; }
            else  { new_values->Data(c, r) = 0; }
         } else
         if (new_values->Data(c,r) > 1){
            if (overflow == true) { new_values->Data(c, r) -= 1; }
            else  { new_values->Data(c, r) = 1; }
         }
      }
   }
}
//---------------------------------------------------------------------------
// Show RBG and HSL scale image
void __fastcall THSLToolForm::Show_Scale()
{
   if (change_h == true){
      Change(h_scale, h_scale_temp, ((float)(0.01)*(float)(TrackBar1->Position)), true);
    }
   if (change_s == true){
      Change(s_scale, s_scale_temp, ((float)(0.01)*(float)(TrackBar2->Position)), false);
   }
   if (change_l == true){
      Change(l_scale, l_scale_temp, ((float)(0.01)*(float)(TrackBar3->Position)), false);
   }

   HSL_scale->HSLtoRGB(h_scale_temp, s_scale_temp, l_scale_temp);
   color_handler->CreateBitmapAndAssign(HSL_scale);
   color_handler->Show(Image3);
}
//---------------------------------------------------------------------------
// Change Hue
void __fastcall THSLToolForm::TrackBar1Change(TObject *Sender)
{
  change_h = true;

  Edit1->Text =(int)(180*TrackBar1->Position/50);
  Show_Scale();
  Timer1->Enabled = false;
  Timer1->Enabled = true;        
}
//---------------------------------------------------------------------------
// Change Saturation
void __fastcall THSLToolForm::TrackBar2Change(TObject *Sender)
{
  change_s = true;
  Edit2->Text = TrackBar2->Position;
  Show_Scale();
  Timer1->Enabled = false;
  Timer1->Enabled = true;        
}
//---------------------------------------------------------------------------
// Change Lightness
void __fastcall THSLToolForm::TrackBar3Change(TObject *Sender)
{
  change_l = true;
  Edit3->Text = TrackBar3->Position;
  Show_Scale();
  Timer1->Enabled = false;
  Timer1->Enabled = true;        
}
//---------------------------------------------------------------------------
// Delay of  compute HSL model
void __fastcall THSLToolForm::Timer1Timer(TObject *Sender)
{
   if (change_h == true) {
      Change(h_image, h_image_temp, ((float)(0.01)*(float)(TrackBar1->Position)), true);
      change_h = false;
   }
   if (change_s == true) {
      Change(s_image, s_image_temp, ((float)(0.01)*(float)(TrackBar2->Position)), false);
      change_s = false;
   }
   if (change_l == true) {
      Change(l_image, l_image_temp, ((float)(0.01)*(float)(TrackBar3->Position)), false);
      change_l = false;
   }

   scaled_color_image->HSLtoRGB(h_image_temp, s_image_temp, l_image_temp);
   color_handler->CreateBitmapAndAssign(scaled_color_image);
   color_handler->Show(Image1);
   Timer1->Enabled = false;
}
//---------------------------------------------------------------------------
// Reset button
void __fastcall THSLToolForm::Button1Click(TObject *Sender)
{
   Timer1->Enabled = false;
   TrackBar1->Position = 0;
   TrackBar2->Position = 0;
   TrackBar3->Position = 0;

   Change(h_image, h_image_temp, 0, true);
   Change(s_image, s_image_temp, 0, false);
   Change(l_image, l_image_temp, 0, false);

   for (int c = 0; c < h_scale->Cols(); c++){
     h_scale_temp->Data(c, 0) = (float)(c)/(float)(255);
     s_scale_temp->Data(c, 0) = (float)(0.5);
     l_scale_temp->Data(c, 0) = (float)(0.5);
   }

   scaled_color_image->HSLtoRGB(h_image_temp, s_image_temp, l_image_temp);
   color_handler->CreateBitmapAndAssign(scaled_color_image);
   color_handler->Show(Image1);

   HSL_scale->HSLtoRGB(h_scale_temp, s_scale_temp, l_scale_temp);
   color_handler->CreateBitmapAndAssign(HSL_scale);
   color_handler->Show(Image3);
}
//---------------------------------------------------------------------------
// Cancel button
void __fastcall THSLToolForm::BitBtn2Click(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
void __fastcall THSLToolForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------
// OK button
void __fastcall THSLToolForm::BitBtn1Click(TObject *Sender)
{
   // Prepare variable
   delete h_image;
   delete s_image;
   delete l_image;

   h_image = new FloatImage(color_image->Cols(), color_image->Rows());
   s_image = new FloatImage(color_image->Cols(), color_image->Rows());
   l_image = new FloatImage(color_image->Cols(), color_image->Rows());

   // RGB -> HSV
   color_image->RGBtoHSL(h_image, s_image, l_image);

   // Change HSV coecifients
   Change(h_image, h_image, ((float)(0.01)*(float)(TrackBar1->Position)), true);
   Change(s_image, s_image, ((float)(0.01)*(float)(TrackBar2->Position)), false);
   Change(l_image, l_image, ((float)(0.01)*(float)(TrackBar3->Position)), false);

   // HSV -> RGB
   color_image->HSLtoRGB(h_image, s_image, l_image);

   // Copy to main Window
   PictureForm->CreateColorImage(color_image);
   Close();
}
//---------------------------------------------------------------------------
