/*
   RGBComponentUnit.cpp

   definin soubor okna pro editaci jednotlivch kanl (erven, modr, zelen)

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "RGBComponentUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TRGBComponentsForm *RGBComponentsForm;
//---------------------------------------------------------------------------
__fastcall TRGBComponentsForm::TRGBComponentsForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   color_handler = new BMPHandlerColor(*(PictureForm->GetBMPHandlerColor()));
   color_image = new ColorImage();
   original_color_image = new ColorImage();
   color_handler->Set(*color_image);

   original_text = PictureForm->StatusBar1->Panels->Items[4]->Text;
   color_handler->Set(*original_color_image);

   scaled_red_image = new GreyImage();
   scaled_green_image = new GreyImage();
   scaled_blue_image = new GreyImage();

   red_image = new GreyImage(color_image->Cols(), color_image->Rows());
   green_image = new GreyImage(color_image->Cols(), color_image->Rows());
   blue_image = new GreyImage(color_image->Cols(), color_image->Rows());

   color_image->SplitToRGB(red_image, green_image, blue_image);

   Scale scaleR(*red_image, *scaled_red_image, Image1->Width);
   Scale scaleG(*green_image, *scaled_green_image, Image2->Width);
   Scale scaleB(*blue_image, *scaled_blue_image, Image3->Width);

   scaled_color_image = new ColorImage();
   Scale_color scale(*color_image, *scaled_color_image, Image4->Width);

   if (scaled_color_image->Cols() > scaled_color_image->Rows()){
     Image4->Top=Image4->Top + (scaled_color_image->Cols()-scaled_color_image->Rows())/2;
   }else if (scaled_color_image->Rows() > scaled_color_image->Cols()){
     Image4->Left=Image4->Left + (scaled_color_image->Rows()-scaled_color_image->Cols())/2;
   }

   red_handler = new BMPHandler();
   green_handler = new BMPHandler();
   blue_handler = new BMPHandler();

   red_handler->CreateBitmapAndAssign(scaled_red_image);
   green_handler->CreateBitmapAndAssign(scaled_green_image);
   blue_handler->CreateBitmapAndAssign(scaled_blue_image);
   color_handler->CreateBitmapAndAssign(scaled_color_image);

   red_handler->Show(Image1);
   green_handler->Show(Image2);
   blue_handler->Show(Image3);
   color_handler->Show(Image4);

   StatusBar1->SimpleText = ("You can edit color components.");
   only_exit = true; // for Alt+F4
};
//---------------------------------------------------------------------------
__fastcall TRGBComponentsForm::~TRGBComponentsForm()
{
   delete red_image;
   delete green_image;
   delete blue_image;
   delete color_image;
   delete original_color_image;

   delete scaled_red_image;
   delete scaled_green_image;
   delete scaled_blue_image;
   delete scaled_color_image;

   delete red_handler;
   delete green_handler;
   delete blue_handler;
   delete color_handler;

   PictureForm->ChangeTypeOfRGB(0);
}
//---------------------------------------------------------------------------
// OK button
void __fastcall TRGBComponentsForm::BitBtn4Click(TObject *Sender)
{
   PictureForm->CreateColorImage(color_image);
   PrepareMenuColor();
   only_exit = false;
   Close();
}
//---------------------------------------------------------------------------
// Red edit button
void __fastcall TRGBComponentsForm::BitBtn1Click(TObject *Sender)
{
   PictureForm->ChangeTypeOfRGB(1);
   PictureForm->CreateGreyImage(red_image);
   PictureForm->StatusBar1->Panels->Items[4]->Text = "Red component from RGB";
   PrepareMenuEdit();
}
//---------------------------------------------------------------------------
// Green edit button
void __fastcall TRGBComponentsForm::BitBtn2Click(TObject *Sender)
{
   PictureForm->ChangeTypeOfRGB(2);
   PictureForm->CreateGreyImage(green_image);
   PictureForm->StatusBar1->Panels->Items[4]->Text = "Green component from RGB";
   PrepareMenuEdit();
}
//---------------------------------------------------------------------------
// Blue edit button
void __fastcall TRGBComponentsForm::BitBtn3Click(TObject *Sender)
{
   PictureForm->ChangeTypeOfRGB(3);
   PictureForm->CreateGreyImage(blue_image);
   PictureForm->StatusBar1->Panels->Items[4]->Text = "Blue component from RGB";
   PrepareMenuEdit();
}
//---------------------------------------------------------------------------
// global close
void __fastcall TRGBComponentsForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   if (only_exit==true) {
      PictureForm->CreateColorImage(original_color_image);
      PictureForm->StatusBar1->Panels->Items[4]->Text = original_text;
      PrepareMenuColor();
      PictureForm->MUndo1->Enabled = false;
   }
   Release();
}
//---------------------------------------------------------------------------
// assign changed image (R or G or B component)
void __fastcall TRGBComponentsForm::AssignChangedImage(GreyImage _image, int _type)
{
   switch (_type){
      case(1):
         // assign red component
         *red_image = _image;
         break;
      case(2):
         // assign green component
         *green_image = _image;
         break;
      case(3):
      // assign blue component
         *blue_image = _image;
         break;
   }

   // create color image from RGB
   if (_type == 1) { Scale scaleR(*red_image, *scaled_red_image, Image1->Width); }else
   if (_type == 2) { Scale scaleG(*green_image, *scaled_green_image, Image2->Width); }else
   if (_type == 3) { Scale scaleB(*blue_image, *scaled_blue_image, Image3->Width); }

   red_handler->Assign(*scaled_red_image);
   green_handler->Assign(*scaled_green_image);
   blue_handler->Assign(*scaled_blue_image);

   color_image->CombineFromRGB(red_image, green_image, blue_image);
   delete scaled_color_image;
   scaled_color_image = new ColorImage();
   Scale_color scale(*color_image, *scaled_color_image, Image4->Width);
   color_handler->CreateBitmapAndAssign(scaled_color_image);

   red_handler->Show(Image1);
   green_handler->Show(Image2);
   blue_handler->Show(Image3);
   color_handler->Show(Image4);

   StatusBar1->SimpleText = ("Some of colors is changed. You can change any color again.");
}
//---------------------------------------------------------------------------

void __fastcall TRGBComponentsForm::PrepareMenuEdit()
{
   PictureForm->MSave->Visible = false;
   PictureForm->MSaveAs->Visible = true;
   PictureForm->MView->Visible = true;
   PictureForm->MActualView1->Visible = false;
   PictureForm->MMagnifyingGlass->Visible = true;
   PictureForm->MZoomIn1->Visible = false;
   PictureForm->MZoomOut1->Visible = false;
   PictureForm->MShowMoreImage->Visible = true;
   PictureForm->MEdit->Visible = true;
   PictureForm->MUndo1->Visible = true;
   PictureForm->MCut1->Visible = false;
   PictureForm->MResize1->Visible = false;
   PictureForm->MTranslate->Visible = false;
   PictureForm->MFFTSize->Visible = false;
   PictureForm->MRotate1->Visible = false;
   PictureForm->MImage->Visible = true;
   PictureForm->MNoise1->Visible = true;
   PictureForm->MContrast->Visible = true;
   PictureForm->MBrightness->Visible = true;
   PictureForm->MProfile->Visible = true;
   PictureForm->MHistogram1->Visible = true;
   PictureForm->MConvolution1->Visible = true;
   PictureForm->LUT1->Visible = true;
   PictureForm->DFT1->Visible = true;
   PictureForm->MLogicArith1->Visible = true;
   PictureForm->MObject1->Visible = true;
   PictureForm->MMorpholog1->Visible = true;
   PictureForm->MNonlinearFiltres1->Visible = true;
   PictureForm->MBitFields1->Visible = true;
   PictureForm->MSobelFilters1->Visible = true;
   PictureForm->MCharacteristics->Visible = true;
   PictureForm->MCorrection->Visible = true;
   PictureForm->MAveraging->Visible = false;
   PictureForm->MColor->Visible = true;
   PictureForm->MPseudoColor->Visible = true;
   PictureForm->MSplitToRGB->Visible = false;
   PictureForm->MGreyScale->Visible = false;
   PictureForm->MAdjust->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TRGBComponentsForm::PrepareMenuColor()
{
   PictureForm->Open1->Visible = true;
   PictureForm->MSave->Visible = true;
   PictureForm->MSaveAs->Visible = true;
   PictureForm->MView->Visible = true;
   PictureForm->MMagnifyingGlass->Visible = false;
   PictureForm->MEdit->Visible = true;
   PictureForm->MImage->Visible = true;
   PictureForm->MNoise1->Visible = false;
   PictureForm->MContrast->Visible = false;
   PictureForm->MBrightness->Visible = false;
   PictureForm->MProfile->Visible = false;
   PictureForm->MHistogram1->Visible = false;
   PictureForm->MConvolution1->Visible = false;
   PictureForm->LUT1->Visible = false;
   PictureForm->DFT1->Visible = false;
   PictureForm->MLogicArith1->Visible = false;
   PictureForm->MObject1->Visible = false;
   PictureForm->MMorpholog1->Visible = false;
   PictureForm->MNonlinearFiltres1->Visible = false;
   PictureForm->MBitFields1->Visible = false;
   PictureForm->MSobelFilters1->Visible = false;
   PictureForm->MCharacteristics->Visible = false;
   PictureForm->MCorrection->Visible = false;
   PictureForm->MAveraging->Visible = true;
   PictureForm->MColor->Visible = true;
   PictureForm->MPseudoColor->Visible = false;
   PictureForm->MSplitToRGB->Visible = true;
   PictureForm->MGreyScale->Visible = true;
   PictureForm->MAdjust->Visible = true;
}
//---------------------------------------------------------------------------


void __fastcall TRGBComponentsForm::BitBtn5Click(TObject *Sender)
{
   PictureForm->CreateColorImage(original_color_image);
   PictureForm->StatusBar1->Panels->Items[4]->Text = original_text;
   PrepareMenuColor();
   only_exit = false;
   PictureForm->MUndo1->Enabled = false;
   Close();
}
//---------------------------------------------------------------------------

