/*
   ContrastUnit.cpp

   definin soubor okna pro korekci kontrastu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "ContrastUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TContrastForm *ContrastForm;
//---------------------------------------------------------------------------
__fastcall TContrastForm::TContrastForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;
   original_image = new GreyImage();
   sized_image = new GeometricImage();
   contrast_image = new GreyImage();
   contrast_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   contrast_handler->Set(*original_image);
   Scale scale(*original_image, *sized_image, Image1->Width);
   contrast_handler->Assign(*sized_image);

   if (sized_image->Cols() > sized_image->Rows()){
     Image2->Top=Image2->Top + (sized_image->Cols()-sized_image->Rows())/2;
   }else if (sized_image->Rows() > sized_image->Cols()){
     Image2->Left=Image2->Left + (sized_image->Rows()-sized_image->Cols())/2;
   }


   contrast = new Contrast(*sized_image, *contrast_image, 0, 0,
                           sized_image->Cols(),sized_image->Rows());
   contrast_handler->Show(Image1);
   contrast_handler->Show(Image2);

   position = (float)1;
}
//---------------------------------------------------------------------------

__fastcall TContrastForm::~TContrastForm()
{
   delete (contrast_handler);
   delete (sized_image);
   delete (contrast_image);
   delete (contrast);
   delete (original_image);
}
//---------------------------------------------------------------------------
void __fastcall TContrastForm::TrackBarChange(TObject *Sender)
{
   if(TrackBar->Position>0)
      position = (float)(1 + (float)TrackBar->Position/25);
   else
      position = (float)(1 + (float)TrackBar->Position/200);
   contrast -> ContrastCorrection(position);
   contrast_handler->Assign(*contrast_image);
   contrast_handler->Show(Image2);
   PositionEdit->Text = TrackBar->Position;
}
//---------------------------------------------------------------------------
void __fastcall TContrastForm::CancelBitBtnClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
void __fastcall TContrastForm::ResetButtonClick(TObject *Sender)
{
   TrackBar -> Position = 0;
   contrast -> ContrastCorrection((float)1);
   contrast_handler->Assign(*contrast_image);
   contrast_handler->Show(Image2);
   PositionEdit->Text = TrackBar -> Position;
}
//---------------------------------------------------------------------------
void __fastcall TContrastForm::OKBitBtnClick(TObject *Sender)
{
   contrast = new Contrast(*original_image, *contrast_image,0,0,
            original_image->Cols(), original_image->Rows());

   contrast -> ContrastCorrection(position);
   PictureForm->AssignList(contrast_image);
   Close();
}
//---------------------------------------------------------------------------
void __fastcall TContrastForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------




