/*
   ConvolutionUnit.cpp

   definin soubor hlavnho konvolunho okna

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "ConvolutionUnit.h"
#include "PictureUnit.h"
#include "NewKernelUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TConvolutionForm *ConvolutionForm;
//---------------------------------------------------------------------------
__fastcall TConvolutionForm::TConvolutionForm(TComponent* Owner)
        : TForm(Owner), c_scale(0), c_absolute(0), kernels_number(0)
{
   Top = PictureForm->Top+50;
   Left = PictureForm->Left +50;

   OpenDialog->Filter="Kernel file (*.ker)|*.ker";
   OpenDialog->DefaultExt = "ker";
   OpenDialog->FilterIndex = 1;
   original_image = new GreyImage();
   sized_image = new GreyImage();
   convol_image = new GreyImage();
   convol_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   sized_handler = new BMPHandler(*convol_handler);
   convol_handler->Set(*original_image);
   Scale scale(*original_image, *sized_image, Image1->Width);

   convol_handler->Assign(*sized_image);
   convol_handler->Show(Image2);
   convolution  = NULL;
   kernel = NULL;
   sized_handler->Assign(*sized_image);
   sized_handler->Show(Image1);
   StatusBar->Panels->Items[1]->Text = "Click to 'Kernel' and select a convolution kernel.";
}
//---------------------------------------------------------------------------
__fastcall TConvolutionForm::~TConvolutionForm()
{
   delete (convol_handler);
   delete (sized_handler);
   delete (sized_image);
   delete (convol_image);
   delete (convolution);
   delete (original_image);
   delete (kernel);
}
//---------------------------------------------------------------------------
void __fastcall TConvolutionForm::PerformButtonClick(TObject *Sender)
{
   if(kernel != NULL){
      convolution = new Convolution(*kernel, *sized_image, *convol_image);
       if(CylindricCheckBox->Checked == true)
      convolution -> CylindricalConvolution(0, 0, sized_image->Cols(),
           sized_image->Rows(), kernel->Scale(), kernel->Absolute());
      else
         convolution -> PerformRealConvolution(0, 0, sized_image->Cols(),
              sized_image->Rows(), kernel->Scale(), kernel->Absolute());

      convol_handler->Assign(*convol_image);
      convol_handler->Show(Image2);
      if(preview){
         convolution = new Convolution(*kernel, *original_image, *convol_image);
         convolution -> PerformRealConvolution(0, 0, original_image->Cols(),
              original_image->Rows(), kernel->Scale(), kernel->Absolute());
         PictureForm->GetBMPHandler()->Assign(*convol_image);
         PictureForm->GetBMPHandler()->Show(PictureForm->Image);
      }
   }
   else
      MessageBox(Handle, "No Kernel was selected yet.", "Convolution",
                                        MB_ICONQUESTION | MB_OK);

}
//---------------------------------------------------------------------------



void __fastcall TConvolutionForm::OKBitBtnClick(TObject *Sender)
{
   if(kernel != NULL){
      convolution = new Convolution(*kernel, *original_image, *convol_image);
       if(CylindricCheckBox->Checked == true)
      convolution -> CylindricalConvolution(0, 0, original_image->Cols(),
           original_image->Rows(),kernel->Scale(), kernel->Absolute());
      else
         convolution -> PerformRealConvolution(0, 0, original_image->Cols(),
              original_image->Rows(),kernel->Scale(), kernel->Absolute());
        PictureForm->AssignList(convol_image);
//      PictureForm->GetBMPHandler()->Assign(*convol_image);
//      PictureForm->GetBMPHandler()->Show(PictureForm->Image);
      //PreviewCheckBox->Checked = false;
      Close();
   }
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::CancelBitBtnClick(TObject *Sender)
{
   if(preview){
      PictureForm->GetBMPHandler()->Assign(*original_image);
      PictureForm->GetBMPHandler()->Show(PictureForm->Image);
   }
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLP1Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->LP1();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLP2Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->LP2();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLP3Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->LP3();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------
void __fastcall TConvolutionForm::MHP3Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->HP3();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------
void __fastcall TConvolutionForm::MHP2Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->HP2();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MHP1Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->HP1();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MVerticalClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->SDVerticalEdges();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MHorizontalClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->SDHorizontalEdges();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MHorizVerticalClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->SDHAndVEdges();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MVertClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->MVerticalEdges();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MHorClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->MHorizontalEdges();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MNorthClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->North();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MNorthEastClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->NorthEast();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MEastClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->East();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MSouthEastClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->SouthEast();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MSouthClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->South();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------


void __fastcall TConvolutionForm::MSouthWestClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->SouthWest();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MWestClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->West();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MNOrthWestClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->NorthWest();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::B3x3Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->Blur3x3();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::B5X5Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->Blur5x5();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MOpenClick(TObject *Sender)
{
   if(OpenDialog->Execute()){
      delete kernel;
      kernel = new Kernel(OpenDialog->FileName);
      convolution = new Convolution(*kernel, *sized_image, *convol_image);
      SetStatusBar(kernel);
   }
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MNewKernelClick(TObject *Sender)
{
   Application->CreateForm(__classid(TNewKernelForm),
                          &NewKernelForm);
   NewKernelForm->SetModeKernel(new_kernel_mode);                      ;;;
   NewKernelForm->Top = Top+20;
   NewKernelForm->Left = Left+20;
   NewKernelForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::SetKernel(Kernel& _kernel)
{
   delete kernel;
   kernel = new Kernel(_kernel);
}
//---------------------------------------------------------------------------
void __fastcall TConvolutionForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------
void __fastcall TConvolutionForm::SetStatusBar(Kernel* kernel)
{
   StatusBar->Panels->Items[0]->Text=AnsiString(kernel->KerCols())+"x"+
                                     AnsiString(kernel->KerRows());
   StatusBar->Panels->Items[1]->Text=kernel->Name();
}
//---------------------------------------------------------------------------
void __fastcall TConvolutionForm::PreviewCheckBoxClick(TObject *Sender)
{
   if(preview)
      preview = false;
   else
     preview = true;
   if(!preview){
      PictureForm->GetBMPHandler()->Assign(*original_image);
      PictureForm->GetBMPHandler()->Show(PictureForm->Image);
   }
}
//---------------------------------------------------------------------------
void __fastcall TConvolutionForm::ViewBitBtnClick(TObject *Sender)
{
   if(kernel!=NULL){
      Application->CreateForm(__classid(TNewKernelForm),
                          &NewKernelForm);
      NewKernelForm->SetModeKernel(view_kernel_mode);
      NewKernelForm->Top = Top-20;
      NewKernelForm->Left = Left+20;
      NewKernelForm->ShowModal();
   }else
      MessageBox(Handle, "No Kernel was selected yet.", "Convolution",
                                        MB_ICONQUESTION | MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLAP1Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->LAP1();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLAP2Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->LAP2();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLAP3Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->LAP3();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLAP4Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->LAP4();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MSaveClick(TObject *Sender)
{
   try{
      if(kernel!=NULL){
        if(SaveDialog->Execute())
           kernel->SaveKernelAs(SaveDialog->FileName);
      }else
        MessageBox(Handle, "No Kernel was selected yet.", "Convolution",
                                        MB_ICONQUESTION | MB_OK);
   }catch(VEMsg &e){
      ShowMessage(e.msg);
   }
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLineVerticalClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->LineVertical();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLineHorizontalClick(TObject *Sender)
{
   kernel = new Kernel();
   kernel->LineHorizontal();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);

}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLine_45Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->Line_45();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);

}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::MLine_135Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->Line_135();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------


void __fastcall TConvolutionForm::DerivativeNorth11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_01();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeNorth21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_02();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeNorthEast11Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_03();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------


void __fastcall TConvolutionForm::DerivativeEast11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_05();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeEast21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_06();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivatedSouthEast11Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_07();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeSouthEast21Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_08();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeSouth11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_09();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeSouth21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_10();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeSouthWest11Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_11();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeSouthWest21Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_12();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeWest11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_13();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeWest21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_14();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeNorthWest11Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_15();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeNorthWest21Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_16();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Laplacian11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_17();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Laplacian21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_18();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Laplacian31Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_19();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Laplacian41Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_20();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Enhancementsmall1Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_21();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Enhancementnormal1Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_22();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Enhancementstrong1Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_23();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Enhancementmaximum1Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_24();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Enhancementtype11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_25();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::EnhancementType21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_26();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::EnhancementType31Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_27();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::EnhancementType41Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_28();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Smoothing11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_29();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Smoothing21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_30();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Smoothing31Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_31();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Smoothing41Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_32();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeHorizontal11Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_33();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeHorizontal21Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_34();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeVertical11Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_35();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::DerivativeVertical21Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_36();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::VerticalLineDetect1Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_37();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::HorizontalLineDetect1Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_38();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N1stDiagonalDetection1Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_39();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N2ndDiagonalDetection2Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_40();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_41A();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_41B();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask31Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_41C();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask41Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_41D();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask15x51Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_42A();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask25x51Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_42B();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask35x51Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_42C();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask45x51Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_42D();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask17x71Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_43A();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask27x71Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_43B();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask37x71Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_43C();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask47x71Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_43D();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask19x91Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_44A();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask29x91Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_44B();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask39x91Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_44C();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N3levelMaxabsmask49x91Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_44D();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N5levelMaxabsmask13x31Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_45A();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N5levelMaxabsmask23x31Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_45B();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N5levelMaxabsmask33x31Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_45C();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::N5levelMaxabsmask43x31Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_45D();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Robertsmeansquaremask11Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_46A();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::Robertsmeansquaremask21Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_46B();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::SobelMeanSquaremask11Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_47A();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::SobelMeanSquaremask21Click(
      TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_47B();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::CompassMaxmask11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_48A();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::CompassMaxmask21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_48B();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::CompassMaxmask31Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_48C();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::CompassMaxmask41Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_48D();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::CompassMaxmask51Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_48E();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::CompassMaxmask61Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_48F();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::CompassMaxmask71Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_48G();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::CompassMaxmask81Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_48H();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::KirschMaxmask11Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_49A();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::KirschMaxmask21Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_49B();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::KirschMaxmask31Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_49C();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::KirschMaxmask41Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_49D();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::KirschMaxmask51Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_49E();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::KirschMaxmask61Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_49F();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::KirschMaxmask71Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_49G();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------

void __fastcall TConvolutionForm::KirschMaxmask81Click(TObject *Sender)
{
   kernel = new Kernel();
   kernel->FILTER_49H();
   convolution = new Convolution(*kernel, *sized_image, *convol_image);
   SetStatusBar(kernel);
}
//---------------------------------------------------------------------------


