/*
   NewKernelUnit.cpp

   definin soubor okna pro definici novch konvolunch masek

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "NewKernelUnit.h"
#include "ConvolutionUnit.h"
#include "NewKernelSizeUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNewKernelForm *NewKernelForm;
//---------------------------------------------------------------------------
__fastcall TNewKernelForm::TNewKernelForm(TComponent* Owner)
        : TForm(Owner), rows_num(3), cols_num(3)

{
   SaveDialog->Filter="Kernel (*.ker)|*.ker";
   SaveDialog->DefaultExt = "ker";
   SaveDialog->FilterIndex = 1;
   form_width = Width;
   form_height = Height;
   kernel = NULL;
}
//---------------------------------------------------------------------------
__fastcall TNewKernelForm::~TNewKernelForm()
{

    DeleteEdit();
    delete kernel;
}
//---------------------------------------------------------------------------
void __fastcall TNewKernelForm::ClearBitBtnClick(TObject *Sender)
{
  for(int i(0); i<cols_num*rows_num; i++)
     edit[i]->Text = 0;
  AbsoluteCheckBox->Checked = true;
  SumEdit->Text = 1;
}
//---------------------------------------------------------------------------
void __fastcall TNewKernelForm::CancelBitBtnClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

void TNewKernelForm::SetSize(int _cols, int _rows)
{
   Assert((_cols>=1)&&(_cols<=25)&&(_rows>=1)&&(_rows<=25),
      VEMsg(AnsiString
     ("Number of rows and columns must be between 1 and 25")));
   cols_num = _cols;
   rows_num = _rows;

   if(cols_num<4)
      Width = form_width;
   else
      Width = form_width + (cols_num-4)*diff_left;
   if(rows_num<4)
      Height = form_height;
   else
      Height = form_height + (rows_num-4)*diff_top;

   // when move from ViewKernel to modify the original values can be modified

   edit = new TEdit*[cols_num*rows_num];
   for(int r(0); r<rows_num; r++){
      for(int c(0); c<cols_num; c++){
         edit[c+r*cols_num] = new TEdit(Panel);
         edit[c+r*cols_num]->Visible = false;
         edit[c+r*cols_num]->Parent = Panel;
         edit[c+r*cols_num]->Width = edit_width;
         edit[c+r*cols_num]->Height = edit_height;
         edit[c+r*cols_num]->Top = edit_top + r*diff_top;
         edit[c+r*cols_num]->Left = edit_left + c*diff_left;
         switch(mode_kernel){
            case(0):   // view kernel
                edit[c+r*cols_num]->Text = FormatFloat("0.00", kernel->KerData(c,r));
                edit[c+r*cols_num]->ReadOnly = true;
                break;
            case(1):   // new kernel
                edit[c+r*cols_num]->Text = 0;
                edit[c+r*cols_num]->ReadOnly = false;
                break;
            case(2):   // modify kernel
                edit[c+r*cols_num]->Text = FormatFloat("0.00", kernel->KerData(c,r));
                edit[c+r*cols_num]->ReadOnly = false;
                break;
         }
         edit[c+r*cols_num]->Show();
         edit[c+r*cols_num]->Visible = true;
      }
   }
}
//---------------------------------------------------------------------------
void
TNewKernelForm::DeleteEdit()
{
   for(int i(0); i<cols_num*rows_num; i++)
      delete edit[i];
   delete [] edit;
}
//---------------------------------------------------------------------------
void __fastcall TNewKernelForm::M2X2Click(TObject *Sender)
{
   DeleteEdit();
   cols_num = 2;
   rows_num = 2;
   SetModeKernel(new_kernel_mode);
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::M3x3Click(TObject *Sender)
{
   DeleteEdit();
   cols_num = 3;
   rows_num = 3;
   SetModeKernel(new_kernel_mode);
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::M4X4Click(TObject *Sender)
{
   DeleteEdit();
   cols_num = 4;
   rows_num = 4;
   SetModeKernel(new_kernel_mode);
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::M5x5Click(TObject *Sender)
{
   DeleteEdit();
   cols_num = 5;
   rows_num = 5;
   SetModeKernel(new_kernel_mode);
}
//---------------------------------------------------------------------------
// returning value true OK
//                 false Exception was thrown and checked
bool TNewKernelForm::GetKernel()
{
   try{
      delete kernel;
      kernel = new Kernel(cols_num, rows_num);
      for(int r(0); r<rows_num; r++)
         for(int c(0); c<cols_num; c++){
            kernel->KerData(c,r) = edit[c+r*cols_num]->Text.ToDouble();
            kernel->Sum() += kernel->KerData(c,r);
         }
      kernel->Scale() = SumEdit->Text.ToDouble();
      kernel->Absolute() = AbsoluteCheckBox->Checked;
      return (true);
   }catch(EConvertError &E){
      Application->MessageBox(E.Message.c_str(),
         "User's Kernel",MB_ICONQUESTION|MB_OK);
   }
   return (false);
}
//---------------------------------------------------------------------------
void __fastcall TNewKernelForm::OKButtonClick(TObject *Sender)
{
  if(GetKernel()==true){
     ConvolutionForm->SetKernel(*kernel);
     kernel->Name() = AnsiString("User's Kernel");
     ConvolutionForm->SetStatusBar(kernel);
     Close();
  }
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::MExitClick(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::MClearClick(TObject *Sender)
{
   for(int i(0); i<cols_num*rows_num; i++)
      edit[i]->Text = 0;
   AbsoluteCheckBox->Checked = true;
   SumEdit->Text = 1;
}
//---------------------------------------------------------------------------


void __fastcall TNewKernelForm::MSaveAsClick(TObject *Sender)
{
   try{
      if(SaveDialog->Execute()){
        if(GetKernel()){
           kernel->SaveKernelAs(SaveDialog->FileName);
        }
      }
   }catch(VEMsg &e){
      ShowMessage(e.msg);
   }
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::FormResize(TObject *Sender)
{
   OKButton->Left = ClientWidth - buttons_differ;
   CancelBitBtn->Left = ClientWidth - buttons_differ;
   ClearBitBtn->Left = ClientWidth - buttons_differ;
   ModifyBitBtn->Left = ClientWidth - buttons_differ;
   Set1BitBtn->Left = ClientWidth - buttons_differ;
   AbsoluteCheckBox->Top = ClientHeight - absolute_differ;
   SumLabel->Top = ClientHeight - absolute_differ;
   SumEdit->Top = ClientHeight - absolute_differ - 2;

}
//---------------------------------------------------------------------------
void __fastcall TNewKernelForm::SetNewSizeKernel(int _c, int _r)
{
   DeleteEdit();
   cols_num = _c;
   rows_num = _r;
   SetModeKernel(new_kernel_mode);
}

void __fastcall TNewKernelForm::AnothClick(TObject *Sender)
{
   Application->CreateForm(__classid(TNewKernelSizeForm),
                          &NewKernelSizeForm);
   NewKernelSizeForm->Top = Top+20;
   NewKernelSizeForm->Left = Left+20;
   NewKernelSizeForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::Set1BitBtnClick(TObject *Sender)
{
   for(int i(0); i<cols_num*rows_num; i++)
      edit[i]->Text = 1;
   AbsoluteCheckBox->Checked = true;
   SumEdit->Text = cols_num*rows_num;
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::Setto11Click(TObject *Sender)
{
   for(int i(0); i<cols_num*rows_num; i++)
      edit[i]->Text = 1;
   AbsoluteCheckBox->Checked = true;
   SumEdit->Text = cols_num*rows_num;
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::ExitClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------
void __fastcall TNewKernelForm::SetModeKernel(ModeKernel mode)
{
   mode_kernel = mode;
   switch (mode_kernel){
      case(0):         // view kernel
         kernel = new Kernel(*ConvolutionForm->GetKernel());
         cols_num = kernel->KerCols();
         rows_num = kernel->KerRows();
         AbsoluteCheckBox->Enabled = false;
         SumEdit->ReadOnly = true;
         SetSize(cols_num, rows_num);
         MSize->Visible = false;
         MClear->Visible = false;
         Setto11->Visible = false;
         Set1BitBtn->Visible = false;
         ClearBitBtn->Visible = false;
         ModifyBitBtn->Visible = true;
         SumEdit->Text = kernel->Sum();
         AbsoluteCheckBox->Checked=kernel->Absolute();
         break;
      case(2):        // modify kernel
         //kernel = new Kernel(*ConvolutionForm->GetKernel());
         cols_num = kernel->KerCols();
         rows_num = kernel->KerRows();
         SumEdit->Text = kernel->Sum();
         AbsoluteCheckBox->Checked=kernel->Absolute();
         // without break!
      case(1):           // new kernel
         SetSize(cols_num, rows_num);
         MSize->Visible = true;
         MClear->Visible = true;
         Setto11->Visible = true;
         SumEdit->ReadOnly = false;
         Set1BitBtn->Visible = true;
         ClearBitBtn->Visible = true;
         ModifyBitBtn->Visible = false;
         AbsoluteCheckBox->Enabled = true;
         break;
   }
}
void __fastcall TNewKernelForm::ModifyBitBtnClick(TObject *Sender)
{
    DeleteEdit();
    SetModeKernel(modify_kernel_mode);
}
//---------------------------------------------------------------------------

void __fastcall TNewKernelForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------

