/*
   FlatFieldUnit.cpp

   definin soubor okna Flat-Field korekce

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "FlatFieldUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFlatFieldForm *FlatFieldForm;
//---------------------------------------------------------------------------
// constructor
__fastcall TFlatFieldForm::TFlatFieldForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;
   BL = 128; // brightness level

   OpenDialog->Filter="Bitmap picture (*.bmp)|*.bmp";
   OpenDialog->DefaultExt = "bmp";

   original_image = new GreyImage();
   sized_original_image = new GreyImage();

   background_image = NULL;
   dark_image = NULL;
   corrected_image = NULL;

   frame_original_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   frame_original_handler->Set(*original_image);

   Scale scale(*original_image, *sized_original_image, Image1->Width);

   if (sized_original_image->Cols() > sized_original_image->Rows()){
     Image4->Top=Image4->Top + (sized_original_image->Cols()-sized_original_image->Rows())/2;
   }else if (sized_original_image->Rows() > sized_original_image->Cols()){
     Image4->Left=Image4->Left + (sized_original_image->Rows()-sized_original_image->Cols())/2;
   }

   frame_original_handler->Assign(*sized_original_image);
   frame_original_handler->Show(Image1);
   StatusBar->SimpleText = "Click to load button and select background and dark images.";
}
//---------------------------------------------------------------------------
// destructor
__fastcall TFlatFieldForm::~TFlatFieldForm()
{
  delete (frame_original_handler);
  delete (frame_background_handler);
  delete (frame_dark_handler);
  delete (frame_corrected_handler);
  delete (original_image);
  delete (background_image);
  delete (dark_image);
  delete (corrected_image);
  delete (sized_original_image);
  delete (sized_background_image);
  delete (sized_dark_image);
  delete (sized_corrected_image);
}
//---------------------------------------------------------------------------
// Load Background Image
void __fastcall TFlatFieldForm::Button1Click(TObject *Sender)
{
  if(OpenDialog->Execute()){
    delete frame_background_handler;
    delete background_image;
    delete sized_background_image;

    frame_background_handler = new BMPHandler(OpenDialog->FileName);
    background_image = new GreyImage();
    sized_background_image = new GreyImage();

    frame_background_handler->Set(*background_image);
    background_file_name = ExtractFileName(OpenDialog->FileName);

    if(original_image->Cols()!= background_image->Cols()||original_image->Rows()!= background_image->Rows()){
       AnsiString message = "Picture " + background_file_name + " isn't of the same size"+".\nRequired size is "+
       AnsiString(original_image->Cols())+ " x "+ AnsiString(original_image->Rows());
       MessageBox(Handle, message.c_str(), "Flat-Field Correction", MB_ICONQUESTION | MB_OK);
       delete background_image;
       background_image = NULL;
    }else{
       Scale scale(*background_image, *sized_background_image, Image2->Width);
       frame_background_handler->Assign(*sized_background_image);
       frame_background_handler->Show(Image2);
       if (background_image!=NULL&&dark_image!=NULL){
          FlatFieldCompute();
          TrackBar->Enabled = true;
       }
       Image2->ShowHint = true;
    }
  }
}
//---------------------------------------------------------------------------
// Load Dark Image
void __fastcall TFlatFieldForm::Button3Click(TObject *Sender)
{
  if(OpenDialog->Execute()){
    delete frame_dark_handler;
    delete dark_image;
    delete sized_dark_image;

    frame_dark_handler = new BMPHandler(OpenDialog->FileName);
    dark_image = new GreyImage();
    sized_dark_image = new GreyImage();
    
    frame_dark_handler->Set(*dark_image);
    dark_file_name = ExtractFileName(OpenDialog->FileName);

    if(original_image->Cols()!= dark_image->Cols()||original_image->Rows()!= dark_image->Rows()){
       AnsiString message = "Picture " + dark_file_name + " isn't of the same size"+".\nRequired size is "+
       AnsiString(original_image->Cols())+ " x "+ AnsiString(original_image->Rows());
       MessageBox(Handle, message.c_str(), "Flat-Field Correction", MB_ICONQUESTION | MB_OK);
       delete dark_image;
       dark_image = NULL;
    }else{
       Scale scale(*dark_image, *sized_dark_image, Image3->Width);
       frame_dark_handler->Assign(*sized_dark_image);
       frame_dark_handler->Show(Image3);
       if (background_image!=NULL&&dark_image!=NULL){
          FlatFieldCompute();
       }
       Image3->ShowHint = true;
       TrackBar->Enabled = true;
    }
  }
}
//---------------------------------------------------------------------------
// Create Dark Image (black image)
void __fastcall TFlatFieldForm::Button4Click(TObject *Sender)
{
  delete frame_dark_handler;
  delete dark_image;
  delete sized_dark_image;

  dark_image = new GreyImage();
  sized_dark_image = new GreyImage();
  frame_dark_handler = new BMPHandler();

  frame_dark_handler->CreateBitmapAndAssign(original_image);
  frame_dark_handler->Set(*dark_image);

  dark_image->ClearImageArea(BLACK, dark_image->Cols(), dark_image->Rows());
  Scale scale(*dark_image, *sized_dark_image, Image3->Width);

  frame_dark_handler->Assign(*sized_dark_image);
  frame_dark_handler->Show(Image3);
  if (background_image!=NULL&&dark_image!=NULL){
     FlatFieldCompute();
  }
  Image3->ShowHint = true;
  TrackBar->Enabled = true;
}
//---------------------------------------------------------------------------
//Compute flat-field image from three images
void __fastcall TFlatFieldForm::FlatFieldSubtract(GreyImage& _image1, GreyImage& _image2,
                                                 GreyImage& _image3, GreyImage& _image4){

   image1 = &_image1; //original_image
   image2 = &_image2; //background_image
   image3 = &_image3; //dark_image
   image4 = &_image4; //corrected_image
   
   // IO-original, IB-background, ID-dark, IC-corrected, BL-brightness level
   for (int r = 0; r < image1->Rows(); r++){
     for (int c = 0; c < image1->Cols(); c++){
       IO = (float)image1->Data(c, r);
       IB = (float)image2->Data(c, r);
       ID = (float)image3->Data(c, r);
       if ((IB - ID) != 0){
         IC = (((IO - ID) * BL) / (IB - ID));
         IC = (IC < 0) ? 0 : IC;
         IC = (IC > 255) ? 255 : IC;
       }else{
         IC = 0;
       }
       image4->Data(c, r) = (unsigned char) IC;
     }
   }
}
//---------------------------------------------------------------------------
//compute flat-field image
void __fastcall TFlatFieldForm::FlatFieldCompute()
{
   delete sized_corrected_image;
   delete frame_corrected_handler;

   sized_corrected_image = new GreyImage();
   frame_corrected_handler = new BMPHandler();

   frame_corrected_handler->CreateBitmapAndAssign(sized_original_image);
   frame_corrected_handler->Set(*sized_corrected_image);

   FlatFieldSubtract(*sized_original_image, *sized_background_image, *sized_dark_image,
                     *sized_corrected_image);

   frame_corrected_handler->Assign(*sized_corrected_image);
   frame_corrected_handler->Show(Image4);
   StatusBar->SimpleText = "Click to OK for use corrected image. You can select background image or dark image again.";
   BitBtn1->Enabled = true;
   Image4->ShowHint = true;
}
//---------------------------------------------------------------------------
// perform flat-field correction and transfer this image to main window
void __fastcall TFlatFieldForm::BitBtn1Click(TObject *Sender)
{
   try{
     delete corrected_image;
     delete sized_corrected_image;
     delete frame_corrected_handler;

     corrected_image = new GreyImage();
     sized_corrected_image = new GreyImage();
     frame_corrected_handler = new BMPHandler();

     frame_corrected_handler->CreateBitmapAndAssign(original_image);
     frame_corrected_handler->Set(*corrected_image);

     FlatFieldSubtract(*original_image, *background_image, *dark_image, *corrected_image);
     PictureForm->AssignList(corrected_image);
     Close();
   }catch(VEMsg &e){
      ShowMessage(e.msg);
   }
}
//---------------------------------------------------------------------------
// close form
void __fastcall TFlatFieldForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------
// change brightness level and compute flat-field correction
void __fastcall TFlatFieldForm::TrackBarChange(TObject *Sender)
{
   BL = (float)TrackBar->Position; // brightness level
   PositionEdit->Text = TrackBar->Position;
   FlatFieldCompute();
}
//---------------------------------------------------------------------------






