/*
   NonUniformIlluminationUnit.cpp

   definin soubor okna pro korekci nehomogennho pozad

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "NonUniformIlluminationUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNonUniformIlluminationForm *NonUniformIlluminationForm;
//---------------------------------------------------------------------------
// Constructor
__fastcall TNonUniformIlluminationForm::TNonUniformIlluminationForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;
   size_mask = 10;

   original_image = new GreyImage();
   sized_image = new GreyImage();
   background_image = new GreyImage();
   corrected_image = new GreyImage();

   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*original_image);
   Scale scale(*original_image, *sized_image, Image1->Width);
   handler->Assign(*sized_image);
   handler->Set(*background_image);
   handler->Set(*corrected_image);
   handler->Show(Image1);

   StatusBar1->SimpleText = ("Click the Preview Button.");
}
//---------------------------------------------------------------------------
// Destructor
__fastcall TNonUniformIlluminationForm::~TNonUniformIlluminationForm()
{
   delete (handler);
   delete (original_image);
   delete (sized_image);
   delete (background_image);
   delete (corrected_image);
}
//---------------------------------------------------------------------------
// OK Button (Unreduced original image is used for calculation)
void __fastcall TNonUniformIlluminationForm::BitBtn1Click(TObject *Sender)
{
   StatusBar1->SimpleText = ("Unreduced original image is used for calculation now.");
   handler->Assign(*original_image);
   handler->Set(*background_image);
   handler->Set(*corrected_image);
   ComputeBackground(*original_image, *background_image, *corrected_image, size_mask);
   PictureForm->AssignList(corrected_image);
   Close();
}
//---------------------------------------------------------------------------
// Cancel Button
void __fastcall TNonUniformIlluminationForm::BitBtn2Click(TObject *Sender)
{
   Close();        
}
//---------------------------------------------------------------------------
// Release memory before exit
void __fastcall TNonUniformIlluminationForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------
// Estimate Background Intensity
void __fastcall TNonUniformIlluminationForm::ComputeBackground
                (GreyImage& _image1, GreyImage& _image2, GreyImage& _image3, int size)
{
   image1 = &_image1; //original_image
   image2 = &_image2; //background_image
   image3 = &_image3; //corrected_image

   int number_point = 0;
   long number_sum = 0;
   int number_result;
   int min = 1000;
   int max = -1000;

   ProgressBar1->Max = image1->Rows();

   for (int r = 0; r < image1->Rows(); r++){
     for (int c = 0; c < image1->Cols(); c++){
       for (int i = -size; i <= size; i++){
         for (int j = -size; j <= size; j++){
           // Average 1/2
           if (((c+j >= 0)&(c+j <= image1->Cols()))&((r+i >= 0)&(r+i <= image1->Rows()))){
             number_point++;
             number_sum += (long)(image1->Data(c+j, r+i));
           }
         }
       }

       // Average 2/2
       number_result = (int)(number_sum/number_point);
       if (number_result < 0) number_result = 0;
       if (number_result > 255) number_result = 255;
       image2->Data(c, r) = (unsigned char)(number_result);

       // Subtract Background from Original Image
       number_result = ((int)(image1->Data(c, r)) - (int)(image2->Data(c, r)));
       if (number_result < 0) number_result = 0;
       if (number_result > 255) number_result = 255;
       image3->Data(c, r) = (unsigned char)(number_result);

       number_point = 0;
       number_sum = 0;
     }
     ProgressBar1->Position += 1;
   }

   // Adjust Image Intensities
   for (int r = 0; r < image3->Rows(); r++){
     for (int c = 0; c < image3->Cols(); c++){
       min = (image3->Data(c, r) < min) ? image3->Data(c, r) : min;
       max = (image3->Data(c, r) > max) ? image3->Data(c, r) : max;
     }
   }
   for (int r = 0; r < image3->Rows(); r++){
     for (int c = 0; c < image3->Cols(); c++){
       number_result = (int)(255*(((int)(image3->Data(c, r))) - min)/(max - min));
       if (number_result < 0) number_result = 0;
       if (number_result > 255) number_result = 255;
       image3->Data(c, r) = (unsigned char)(number_result);
     }
   }
   ProgressBar1->Position = 0;
}
//---------------------------------------------------------------------------
// Compute Preview and show it
void __fastcall TNonUniformIlluminationForm::Button1Click(TObject *Sender)
{
   StatusBar1->SimpleText = ("You can change block-size for estimate.");
   ComputeBackground(*sized_image, *background_image, *corrected_image,
                    (int)(size_mask/(original_image->Rows()/sized_image->Rows())));

   handler->Assign(*background_image);
   handler->Show(Image2);

   handler->Assign(*corrected_image);
   handler->Show(Image3);
}
//---------------------------------------------------------------------------
// Change block-size for Estimate
void __fastcall TNonUniformIlluminationForm::TrackBarChange(
      TObject *Sender)
{
   PositionEdit->Text = AnsiString(TrackBar->Position)+" x "+AnsiString(TrackBar->Position);
   size_mask = TrackBar->Position;
   StatusBar1->SimpleText = ("Click the Preview Button.");
}
//---------------------------------------------------------------------------

