/*
   DrawUnit.cpp

   definin soubor okna pro dodn syntetickch objekt do obrazu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DrawUnit.h"
#include "PictureUnit.h"
#include "DrawNewSizeUnit.h"
#include "ColoBoxUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDrawForm *DrawForm;
//---------------------------------------------------------------------------
__fastcall TDrawForm::TDrawForm(TComponent* Owner)
        : TForm(Owner),image(NULL),list(NULL),orig_image(NULL),
          pixels_per_line(10), pen_color((TColor)0x0), brush_color((TColor)0x0),
          drawing(false)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;
   cancel_press = false;
}
//---------------------------------------------------------------------------
__fastcall TDrawForm::~TDrawForm()
{
}
//---------------------------------------------------------------------------
void
__fastcall TDrawForm::DrawShape(TPoint TopLeft, TPoint BottomRight, TPenMode AMode)
{

   Image->Canvas->Pen->Mode = AMode;
   Image->Canvas->Brush->Color = brush_color;
   Image->Canvas->Brush->Style = brush_style;

   switch (drawing_tool){
      case dtLine :
         Image->Canvas->MoveTo(TopLeft.x, TopLeft.y);
         Image->Canvas->LineTo(BottomRight.x, BottomRight.y);
         break;
      case dtRectangle :
         Image->Canvas->Rectangle(TopLeft.x, TopLeft.y, BottomRight.x,
                                                         BottomRight.y);
         break;
      case dtEllipse :
         Image->Canvas->Ellipse(TopLeft.x, TopLeft.y, BottomRight.x,
                                                       BottomRight.y);
         break;
      case dtRoundRect :
         Image->Canvas->RoundRect(TopLeft.x, TopLeft.y, BottomRight.x,
                   BottomRight.y, (TopLeft.x - BottomRight.x)/2,
                   (TopLeft.y - BottomRight.y)/2);
         break;
      case dtWeb :
         TPoint topleft, bottomright;
         topleft.x = (TopLeft.x>BottomRight.x)?BottomRight.x:TopLeft.x;
         topleft.y = (TopLeft.y>BottomRight.y)?BottomRight.y:TopLeft.y;
         bottomright.x = (TopLeft.x<BottomRight.x)?BottomRight.x:TopLeft.x;
         bottomright.y = (TopLeft.y<BottomRight.y)?BottomRight.y:TopLeft.y;
         int num_width = (bottomright.x-topleft.x)/pixels_per_line;
         int num_height = (bottomright.y-topleft.y)/pixels_per_line;
         for(int w(0); w<num_width;w++){
            Image->Canvas->MoveTo(topleft.x+(w+1)*pixels_per_line, topleft.y);
            Image->Canvas->LineTo(topleft.x+(w+1)*pixels_per_line, bottomright.y);
         }
         for(int h(0); h<num_height;h++){
            Image->Canvas->MoveTo(topleft.x, topleft.y+(h+1)*pixels_per_line);
            Image->Canvas->LineTo(bottomright.x, topleft.y+(h+1)*pixels_per_line);
         }
         break;
   }
}
//---------------------------------------------------------------------------
void
__fastcall TDrawForm::AssignImage()
{
   Image = PictureForm->Image;
   image = new GreyImage();
   orig_image = new GreyImage();
   list = new ListI<GreyImage>();
   PictureForm->GetBMPHandler()->Set(*image);
   PictureForm->GetBMPHandler()->Set(*orig_image);
   list->Append(*image);
}
//---------------------------------------------------------------------------
void __fastcall TDrawForm::OnMouseDown(TMouseButton Button, int X, int Y)
{
  if (drawing_allowed){
     /*if(PictureForm->ClientWidth!=PictureForm->GetBMPHandler()->Cols())
        PictureForm->ClientWidth=PictureForm->GetBMPHandler()->Cols();
     if(PictureForm->ClientHeight!=PictureForm->GetBMPHandler()->Rows())
        PictureForm->ClientHeight=PictureForm->GetBMPHandler()->Rows();*/
     drawing = True;
     Image->Canvas->MoveTo(X, Y);
     origin = TPoint(X, Y);
     move_pt = origin;
  }
}
//---------------------------------------------------------------------------
void __fastcall TDrawForm::OnMouseMove(int X, int Y)
{
 if (drawing_allowed)
 if (drawing){
    //Image->Canvas->Pen->Color = clWhite;
    DrawShape(origin, move_pt, pmNotXor);
    move_pt = Point(X, Y);
    DrawShape(origin, move_pt, pmNotXor);
  }
}
//---------------------------------------------------------------------------
void __fastcall TDrawForm::OnMouseUp(TMouseButton Button, int X, int Y)
{
  if (drawing_allowed)
  if (drawing){
    //Image->Canvas->Pen->Color = pen_color;
    //Image->Canvas->Brush->Color = brush_color;
    DrawShape(origin, TPoint(X, Y), pmCopy);
    drawing = False;
    PictureForm->GetBMPHandler()->Assign(PictureForm->Image->Picture->Bitmap);
    PictureForm->GetBMPHandler()->Set(*image);
    list->Append(*image);
  }
}
//---------------------------------------------------------------------------
void __fastcall TDrawForm::NewClick(TObject *Sender)
{
   Application->CreateForm(__classid(TDrawNewSizeForm),
                          &DrawNewSizeForm);
   DrawNewSizeForm->ActiveControl = DrawNewSizeForm->WidthEdit;
   DrawNewSizeForm->WidthEdit->Text = IntToStr(Image->Picture->Graphic->Width);
   DrawNewSizeForm->HeightEdit->Text = IntToStr(Image->Picture->Graphic->Height);
   if (DrawNewSizeForm->ShowModal() != IDCANCEL){
      int width = StrToInt(DrawNewSizeForm->WidthEdit->Text);
      int height = StrToInt(DrawNewSizeForm->HeightEdit->Text);
      GreyImage temp_image(width,height);
      temp_image.ClearImageArea(WHITE, temp_image.Cols(),
                                  temp_image.Rows() ,0, 0);
      PictureForm->AssignImage(&temp_image);
      delete (list);
      list = new ListI<GreyImage>();
      list->Append(temp_image);
   }
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::LineSpeedButtonClick(TObject *Sender)
{
   drawing_tool = dtLine;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::EllipseSpeedButtonClick(TObject *Sender)
{
   drawing_tool = dtEllipse;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::RoundRectSpeedButtonClick(TObject *Sender)
{
   drawing_tool = dtRoundRect;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::RectSpeedButtonClick(TObject *Sender)
{
   drawing_tool = dtRectangle;
}
//---------------------------------------------------------------------------
void __fastcall TDrawForm::SetBrushStyle(TObject *Sender)
{
 if (Sender == SolidSpeedButton){
       Image->Canvas->Brush->Style = bsSolid;
    }
    else if (Sender == ClearSpeedButton){
         Image->Canvas->Brush->Style = bsClear;
    }
    else if (Sender == HorizontalSpeedButton){
         Image->Canvas->Brush->Style = bsHorizontal;
    }
    else if (Sender == VerticalSpeedButton){
         Image->Canvas->Brush->Style = bsVertical;
    }
    else if (Sender == FDiagonalSpeedButton){
         Image->Canvas->Brush->Style = bsFDiagonal;
    }
    else if (Sender == BDiagonalSpeedButton){
         Image->Canvas->Brush->Style = bsBDiagonal;
    }
    else if (Sender == CrossSpeedButton){
         Image->Canvas->Brush->Style = bsCross;
    }
    else if (Sender == DiagCrossSpeedButton){
         Image->Canvas->Brush->Style = bsDiagCross;
    }
    brush_style = Image->Canvas->Brush->Style;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::ColorBrushSpeedButtonClick(TObject *Sender)
{
  Application->CreateForm(__classid(TColorBoxForm),
                        &ColorBoxForm);
  ColorBoxForm->KindColor(brush);
  ColorBoxForm->ResultPanel->Color = brush_color;
  ColorBoxForm->ShowModal();
  brush_color = Image->Canvas->Brush->Color;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::Size1Click(TObject *Sender)
{
   Image->Canvas->Pen->Width = 1;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::Size2Click(TObject *Sender)
{
   Image->Canvas->Pen->Width = 2;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::Size3Click(TObject *Sender)
{
   Image->Canvas->Pen->Width = 3;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::Size4Click(TObject *Sender)
{
   Image->Canvas->Pen->Width = 4;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::Size5Click(TObject *Sender)
{
   Image->Canvas->Pen->Width = 5;
}
//---------------------------------------------------------------------------



void __fastcall TDrawForm::ColorPenSpeedButtonClick(TObject *Sender)
{
   Application->CreateForm(__classid(TColorBoxForm),
                          &ColorBoxForm);

   ColorBoxForm->KindColor(pen);
   ColorBoxForm->ResultPanel->Color = pen_color;
   ColorBoxForm->ShowModal();
   pen_color = Image->Canvas->Pen->Color;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::MUndoClick(TObject *Sender)
{
   list->Delete();
   *image = (list->Member());
   PictureForm->AssignImage(image);
}
//---------------------------------------------------------------------------



void __fastcall TDrawForm::MCancelClick(TObject *Sender)
{
   cancel_press = true;
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::MResetClick(TObject *Sender)
{
   PictureForm->AssignImage(orig_image);
   list->Reset();
   AssignImage();
   SolidSpeedButton->Down = true;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::FormClose(TObject *Sender, TCloseAction &Action)
{
   if (cancel_press == true)
      PictureForm->AssignImage(orig_image);
   else
      PictureForm->AssignImage(image);
   delete (image);
   delete (orig_image);
   delete (list);
   drawing = false;
   drawing_allowed = false;
   PictureForm->MImage->Visible = true;
   PictureForm->MEdit->Visible = true;
   PictureForm->Open1->Visible = true;
   PictureForm->MSave->Visible = true;
   PictureForm->MSaveAs->Visible = true;
   PictureForm->MView->Visible = true;
   PictureForm->MColor->Visible = true;
}

//---------------------------------------------------------------------------


void __fastcall TDrawForm::WebSpeedButtonClick(TObject *Sender)
{
   drawing_tool = dtWeb;
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::PixelsPerLineClick(TObject *Sender)
{
   if (Sender == web10){
      pixels_per_line = 10;
   }
   else if (Sender == web20){
      pixels_per_line = 20;
   }
   else if (Sender == web30){
      pixels_per_line = 30;
   }
   else if (Sender == web40){
      pixels_per_line = 40;
   }
}
//---------------------------------------------------------------------------

void __fastcall TDrawForm::FormShow(TObject *Sender)
{
   drawing_allowed = true;
}
//---------------------------------------------------------------------------


void __fastcall TDrawForm::OK1Click(TObject *Sender)
{
   Close();        
}
//---------------------------------------------------------------------------

