/*
   AverageUnit.cpp

   definin soubor okna pro prmrovn z vce obraz

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "AverageUnit.h"
#include "PictureUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TAverageForm *AverageForm;
//---------------------------------------------------------------------------
__fastcall TAverageForm::TAverageForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   number_images = 0;
   x_size = 0; // must while next - detect first image for primary size
   y_size = 0;
   a_image = new GreyImage();
   b_image = new GreyImage();
   d_image = new GreyImage();
   sized_image = new GreyImage();
   average_list = new ListI<GreyImage>();
   a_list = NULL;
   average_path = new TStringList;
   file = NULL;
   a_handler = NULL;
   frame = NULL;

   Button2->Enabled = false;
   Label2->Enabled = false;
   ComboBox1->Enabled = false;
   BitBtn1->Enabled = false;

   StatusBar1->SimpleText = ("Click to 'Add image(s)' and select image(s).");
}
//---------------------------------------------------------------------------
__fastcall TAverageForm::~TAverageForm()
{
   //for (int i=1; i<number_images; i++){
   //   average_list->Delete();
   //}
   average_list->Reset();
   if (a_list!=NULL) {a_list->Reset();}
   delete a_image;
   delete b_image;
   delete d_image;
   delete sized_image;
   delete average_path;
   if (a_handler!=NULL) {delete a_handler;}
   if (file!=NULL) {delete file;}
   if (frame!=NULL) {delete frame;}
}
//---------------------------------------------------------------------------
// averaging operations
void __fastcall TAverageForm::Button1Click(TObject *Sender)
{
   bool a_ok = false; // every message only once
   bool b_ok = false; // every message only once
   bool c_ok = false; // every message only once
   try{
      if (OpenDialog1->Execute()){
         delete average_path;
         average_path = new TStringList;
         average_path->AddStrings(OpenDialog1->Files);
         if  (average_list == NULL) {average_list = new ListI<GreyImage>();}
         // size of first image = size of others images
         if (x_size == 0){
            delete file;
            file = new VBitmap(average_path->Strings[0]);
            delete a_handler;
            a_handler = new BMPHandler();
            file->ConvertToBMP(a_handler);
            // max size of images must MAX_COLS x MAX_ROWS
            if ((a_handler->Cols()<=MAX_COLS)&&(a_handler->Rows()<=MAX_ROWS)){
               x_size = a_handler->Cols();
               y_size = a_handler->Rows();
               Edit2->Text = (IntToStr(x_size)+" x "+IntToStr(y_size));
            }
         }
         for (int i = 0; i < OpenDialog1->Files->Count; i++){
           if (Check_Validity(average_path->Strings[i])){
            delete file;
            file = new VBitmap(average_path->Strings[i]);
            delete a_handler;
            a_handler = new BMPHandler();
            file->ConvertToBMP(a_handler);
            // all conditions
            if ((number_images < 10)&&(a_handler->Cols() == x_size)
            &&(a_handler->Rows() == y_size)&&(a_handler->Cols()<=MAX_COLS)
            &&(a_handler->Rows()<=MAX_ROWS)){
                AddToAverageList(average_path->Strings[i]);
                number_images++;
            }else{
                if (((a_handler->Cols() != x_size)||(a_handler->Rows() != y_size))
                &&(a_ok == false)){
                   Application->MessageBox("All images must have a same size.",
                   "Average Tool Window",MB_ICONINFORMATION|MB_OK);
                   a_ok = true;
                }
                if ((number_images==10)&&(b_ok == false)){
                   Application->MessageBox("Too much images.",
                   "Average Tool Window",MB_ICONINFORMATION|MB_OK);
                   b_ok = true;
                }
                if (((a_handler->Cols()>MAX_COLS)||(a_handler->Rows()>MAX_ROWS))
                &&(c_ok == false)){
                   Application->MessageBox("Some image(s) is too large.",
                   "Average Tool Window",MB_ICONINFORMATION|MB_OK);
                   c_ok = true;
                }
            }
           }
         }
         average_path->Clear();
         Edit1->Text = AnsiString(number_images);
         Button2->Enabled = true;
         Label2->Enabled = true;
         ComboBox1->Enabled = true;
         StatusBar1->SimpleText = ("Select averaging operation.");
      }
   }catch(VEMsg &E){
      Application->MessageBox(E.msg.c_str(),
      "Exception Catched",MB_ICONHAND|MB_OK);
   }catch(VECorruptedFile &E){
      Application->MessageBox(E.msg.c_str(),
      "Exception Catched",MB_ICONHAND|MB_OK);

   }
}
//---------------------------------------------------------------------------
void __fastcall TAverageForm::AddToAverageList(AnsiString file_path)
{
   try{
      //delete a_image;
      delete file;
      file = new VBitmap(file_path);
      delete a_handler;
      a_handler = new BMPHandler();
      file->ConvertToBMP(a_handler);
      //a_image = new GreyImage();
      a_handler->Set(*a_image);
      average_list->Append(*a_image);
   }catch(VEMsg &E){
      Application->MessageBox(E.msg.c_str(),
      "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TAverageForm::Button2Click(TObject *Sender)
{
   average_list->Reset();
   average_path->Clear();
   
   delete average_list;
   delete a_image;
   delete b_image;
   delete d_image;
   delete sized_image;

   average_list = new ListI<GreyImage>();
   a_image = new GreyImage();
   b_image = new GreyImage();
   d_image = new GreyImage();
   sized_image = new GreyImage();

   number_images = 0;
   x_size = 0;
   y_size = 0;
   Edit1->Text = "0";
   Edit2->Text = "";

   Button2->Enabled = false;
   Label2->Enabled = false;
   ComboBox1->Enabled = false;

   StatusBar1->SimpleText = ("Click to 'Add image(s)' and select image(s).");
}
//---------------------------------------------------------------------------
void __fastcall TAverageForm::BitBtn2Click(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
void __fastcall TAverageForm::ComboBox1Change(TObject *Sender)
{
   a_list = average_list;
   a_image->ClearImageArea(BLACK, a_image->Cols(), a_image->Rows());
   switch(ComboBox1->ItemIndex){
      case(0):
         ArithmeticAverage();
         StatusBar1->SimpleText = ("Click to OK button for use or add another image.");
         break;
   }
   Scale scale(*a_image, *sized_image, Image1->Width);
   a_handler->Assign(*sized_image);
   a_handler->Show(Image1);
   BitBtn1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TAverageForm::ArithmeticAverage()
{
   for (int i = 1; i<=number_images; i++){
      try{
         delete frame;
         frame = new FrameImage(a_list->Member(), a_list->Member(), *d_image);
         frame ->CombineImages(0, 0, a_image->Cols(),
                         a_image->Rows(), 0, 0, fDIV, 0,
                         number_images, 1, 0, 255);
         delete frame;
         frame = new FrameImage(*d_image, *a_image, *b_image);
         frame ->CombineImages(0, 0, a_image->Cols(),
                         a_image->Rows(), 0, 0, ADD, 0,
                         0, 1, 0, 255);
         *a_image = *b_image;
         a_list->Next();
      }catch(VEMsg &e){
         ShowMessage(e.msg);
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TAverageForm::BitBtn1Click(TObject *Sender)
{
   try{
     PictureForm->CreateGreyImage(a_image);
     PictureForm->StatusBar1->Panels->Items[4]->Text = "Average image";
     PictureForm->MSave->Visible = false;
     PictureForm->MSaveAs->Visible = true;
     PictureForm->MZoomIn1->Visible = true;
     PictureForm->MZoomOut1->Visible = true;
     PictureForm->MEdit->Visible = true;
     PictureForm->MNoise1->Visible = true;
     PictureForm->MContrast->Visible = true;
     PictureForm->MBrightness->Visible = true;
     PictureForm->MProfile->Visible = true;
     PictureForm->MHistogram1->Visible = true;
     PictureForm->MConvolution1->Visible = true;
     PictureForm->LUT1->Visible = true;
     PictureForm->DFT1->Visible = true;
     PictureForm->MLogicArith1->Visible = true;
     PictureForm->MObject1->Visible = true;
     PictureForm->MMorpholog1->Visible = true;
     PictureForm->MNonlinearFiltres1->Visible = true;
     PictureForm->MBitFields1->Visible = true;
     PictureForm->MSobelFilters1->Visible = true;
     PictureForm->MCharacteristics->Visible = true;
     PictureForm->MCorrection->Visible = true;
     PictureForm->MColor->Visible = true;
     PictureForm->MPseudoColor->Visible = true;
     PictureForm->MSplitToRGB->Visible = false;
   }catch(VEMsg &e){
      ShowMessage(e.msg);
   }
   Close();
}
//---------------------------------------------------------------------------
// divne - nutno predelat
bool
__fastcall TAverageForm::Check_Validity(AnsiString file_path)
{
   char buff[54];
   bool valid = true;
   TFileStream* stream = new TFileStream(file_path,fmOpenRead);
   stream->Read(buff, 54);
   delete(stream);
   for (int index(0); index<2; index++){
      switch(index){
         case(0): valid = (buff[0]=='B') ? true : false; break;
         case(1): valid = (buff[1]=='M') ? true : false; break;
      }
   }
   Assert(valid==true,
         VECorruptedFile(AnsiString("Bitmap is not valid"), AnsiString("bmp")));
   valid = (buff[26]==0x01) ? true : false;
   valid = (buff[28]==0x08) ? true : false;
   Assert(valid==true,
         VECorruptedFile(AnsiString("Color images aren't supported"), AnsiString("bmp")));
   return valid;
}
//---------------------------------------------------------------------------
void __fastcall TAverageForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------





