/*
   MagnifyingGlassUnit.cpp

   definin soubor okna pro pouit lupy v obraze

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "MagnifyingGlassUnit.h"
#include "DrawUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMagnifyingGlassForm *MagnifyingGlassForm;
//---------------------------------------------------------------------------
__fastcall TMagnifyingGlassForm::TMagnifyingGlassForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;
   original_image = new GreyImage();
   sized_image_light = new GreyImage();
   sized_image_dark = new GreyImage(); 
   sized_image_lightdark = new GreyImage();
   sized_detail_image = new GreyImage();
   magnifying_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   magnifying_handler->Set(*original_image);
   Scale scale(*original_image, *sized_image_light, Image1->Width);
   magnifying_handler->Assign(*sized_image_light);
   magnifying_handler->Set(*sized_image_dark);
   magnifying_handler->Set(*sized_image_lightdark);

   if (sized_image_lightdark->Cols() > sized_image_lightdark->Rows()){
     Image1->Top=Image1->Top + (sized_image_lightdark->Cols()-sized_image_lightdark->Rows())/2;
   }else if (sized_image_lightdark->Rows() > sized_image_lightdark->Cols()){
     Image1->Left=Image1->Left + (sized_image_lightdark->Rows()-sized_image_lightdark->Cols())/2;
   }

   x_ordinate = sized_image_lightdark->Cols()/2;
   y_ordinate = sized_image_lightdark->Rows()/2;
   interp = false;
   lightness = 80; // 0 = normal; 255=black
   can_move = false;
   Edit2->Text = lightness;
   TrackBar2->Position = lightness;
   min_size = 1;
   max_size = (sized_image_lightdark->Rows() > sized_image_lightdark->Cols())
             ? sized_image_lightdark->Cols() : sized_image_lightdark->Rows();
   max_size = (max_size-min_size)/2;
   TrackBar1->Position = 32;
   size_glass = TrackBar1->Position*(max_size-min_size)/99 + min_size - (max_size-min_size)/99;
   resize_koecifient = (float)(original_image->Rows())/(float)(sized_image_lightdark->Rows());
   Edit1->Text = TrackBar1->Position;
   ReCount(); // pre-compute axes
   MakeDark(sized_image_light, sized_image_dark);
   CombineDetailAndDark(sized_image_light, sized_image_dark, sized_image_lightdark,
                       x_ordinate, y_ordinate, size_glass);
   magnifying_handler->Assign(*sized_image_lightdark);
   magnifying_handler->Show(Image1);

   detail_image = new GreyImage(size_glass_original, size_glass_original);
   ViewDetail(x_ordinate_original, y_ordinate_original);
   StatusBar1->SimpleText = ("You can change magnification, shade level or position of magnifying glass.");
}
//---------------------------------------------------------------------------
__fastcall TMagnifyingGlassForm::~TMagnifyingGlassForm()
{
   delete (magnifying_handler);
   delete (original_image);
   delete (sized_image_light);
   delete (sized_image_dark);
   delete (sized_image_lightdark);
   delete (detail_image);
   delete (sized_detail_image);
}
//---------------------------------------------------------------------------
void __fastcall TMagnifyingGlassForm::BitBtn1Click(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
// make dark image from light image
void __fastcall TMagnifyingGlassForm::MakeDark(GreyImage* im_in, GreyImage* im_out)
{
  int p;
  for (int r = 0; r < im_in->Rows(); r++){
     for (int c = 0; c < im_in->Cols(); c++){
        p = im_in->Data(c,r) - lightness;
        if (p < 0) p = 0;
        im_out->Data(c,r) = p;
     }
  }
}
//---------------------------------------------------------------------------
// Create left image (dark and light in center)
void __fastcall TMagnifyingGlassForm::CombineDetailAndDark(GreyImage* im_light, GreyImage* im_dark,
                GreyImage* im_lightdark, int x, int y, int size)
{
  for (int r = 0; r < im_lightdark->Rows(); r++){
     for (int c = 0; c < im_lightdark->Cols(); c++){
        if ((r >= y-size/2)&&(r <= y+size/2)&&(c >= x-size/2)&&(c <= x+size/2)){
           if ((r == y-size/2)||(r == y+size/2)||(c == x-size/2)||(c == x+size/2)){
              im_lightdark->Data(c,r) = 255;
           }else{
           if ((r == y-size/2+1)||(r == y+size/2-1)||(c == x-size/2+1)||(c == x+size/2-1)){
              im_lightdark->Data(c,r) = 0;
           }else
              im_lightdark->Data(c,r) = im_light->Data(c,r);
           }
        }else{
           im_lightdark->Data(c,r) = im_dark->Data(c,r);
        }
     }
  }
}
//---------------------------------------------------------------------------
// multiplication koecifients
void __fastcall TMagnifyingGlassForm::ReCount()
{
   x_ordinate_original = (int)(x_ordinate * resize_koecifient);
   y_ordinate_original = (int)(y_ordinate * resize_koecifient);
   size_glass_original = (int)(size_glass * resize_koecifient);
}
//---------------------------------------------------------------------------
void __fastcall TMagnifyingGlassForm::ViewDetail(int x, int y)
{
  original_image->ReadImageAreaToImage(*detail_image, x-(size_glass_original/2),
                  y-(size_glass_original/2), size_glass_original, size_glass_original);
  if (interp == true){
     Scale scale2(*detail_image, *sized_detail_image, Image2->Width);
     magnifying_handler->Assign(*sized_detail_image);
  } else magnifying_handler->Assign(*detail_image);
  magnifying_handler->Show(Image2);
}
//---------------------------------------------------------------------------

void __fastcall TMagnifyingGlassForm::Image1MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   if(Button==mbLeft){
      can_move = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TMagnifyingGlassForm::Image1MouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   can_move = false;
}
//---------------------------------------------------------------------------


void __fastcall TMagnifyingGlassForm::Timer1Timer(TObject *Sender)
{
  Timer1->Enabled = false;
  ReCount();
  ViewDetail(x_ordinate_original, y_ordinate_original);
}
//---------------------------------------------------------------------------
 
void __fastcall TMagnifyingGlassForm::Image1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  if (can_move == true){
     Timer1->Enabled = false;
     X = (X<size_glass/2+2) ? size_glass/2+2 : X;
     X = (X>Image1->Width-size_glass/2-2) ? Image1->Width-size_glass/2-2 : X;
     Y = (Y<size_glass/2+2) ? size_glass/2+2 : Y;
     Y = (Y>Image1->Height-size_glass/2-2) ? Image1->Height-size_glass/2-2 : Y;
     x_ordinate = X;
     y_ordinate = Y;
     CombineDetailAndDark(sized_image_light, sized_image_dark, sized_image_lightdark,
                         x_ordinate, y_ordinate, size_glass);
     magnifying_handler->Assign(*sized_image_lightdark);
     magnifying_handler->Show(Image1);
     Timer1->Enabled = true;
  }
}
//---------------------------------------------------------------------------


void __fastcall TMagnifyingGlassForm::TrackBar2Change(TObject *Sender)
{
   Edit2->Text = TrackBar2->Position;
   lightness = TrackBar2->Position;
   MakeDark(sized_image_light, sized_image_dark);
   CombineDetailAndDark(sized_image_light, sized_image_dark, sized_image_lightdark,
                       x_ordinate, y_ordinate, size_glass);
   magnifying_handler->Assign(*sized_image_lightdark);
   magnifying_handler->Show(Image1);
}
//---------------------------------------------------------------------------

void __fastcall TMagnifyingGlassForm::TrackBar1Change(TObject *Sender)
{
   Timer1->Enabled = false;
   Edit1->Text = TrackBar1->Position;
   x_ordinate = sized_image_lightdark->Cols()/2;
   y_ordinate = sized_image_lightdark->Rows()/2;
   size_glass = TrackBar1->Position*(max_size-min_size)/99 + min_size - (max_size-min_size)/99;
   ReCount(); // pre-compute axes
   CombineDetailAndDark(sized_image_light, sized_image_dark, sized_image_lightdark,
                       x_ordinate, y_ordinate, size_glass);
   magnifying_handler->Assign(*sized_image_lightdark);
   magnifying_handler->Show(Image1);
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TMagnifyingGlassForm::CheckBox1Click(TObject *Sender)
{
  bool interp_old = interp;
  interp = (CheckBox1->Checked == true) ? true : false;
  if (interp != interp_old){
     ViewDetail(x_ordinate_original, y_ordinate_original);
  }
}
//---------------------------------------------------------------------------

void __fastcall TMagnifyingGlassForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------

