/*
   dft.cpp

   definin soubor tdy implementujc fft algoritmus

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "dft.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

//---------------------------------------------------------------------------
// constructor
DFT::DFT(ComplexImage* _image,  TProgressBar* _prog_bar)
:image(_image), data(NULL), prog_bar(_prog_bar)
{
}
//---------------------------------------------------------------------------
DFT::~DFT()
{
   delete [] data;
}
//---------------------------------------------------------------------------
void
DFT::fft(double* data, int num, double dir)
{
   int array_size, bits, ind, j, j1;
   int i, i1, u, step, inc;
   double *sine = new double[num+1], *cose = new double[num+1];
   double wcos, wsin, tr, ti, temp;
   bits = log(num)/log(2) + 0.5;
   for(i=0; i<num+1; i++)
   {
      sine[i] = dir*sin(3.141592654*i/num);
      cose[i] = cos(3.141592654*i/num);
   }
   array_size=num<<1;
   for(i=0; i<num; i++)
   {
      ind=0;
      for(j=0; j<bits; j++)
      {
         u=1<<j;ind=(ind<<1)+((u&i)>>j);
      }
      ind = ind<<1; j=i<<1;
      if(j<ind)
      {
         temp=data[j];
         data[j]=data[ind];
         data[ind]=temp;
         temp=data[j+1];
         data[j+1]=data[ind+1];
         data[ind+1]=temp;
      }
   }
   for(inc=2; inc<array_size; inc=step)
   {
      step=inc<<1;
      for(u=0; u<inc; u+=2)
      {
         ind=((long)u<<bits)/inc;
         wcos=cose[ind];
         wsin=sine[ind];
         for(i=u; i<array_size; i+=step)
         {
            j=i+inc;
            j1=j+1;
            i1=i+1;
            tr=wcos*data[j]-wsin*data[j1];
            ti=wcos*data[j1]+wsin*data[j];
            data[j]=data[i]-tr;
            data[i]=data[i]+tr;
            data[j1]=data[i1]-ti;
            data[i1]=data[i1]+ti;
         }
      }
   }
   delete [] sine;
   delete [] cose;
}
//---------------------------------------------------------------------------
void
DFT::PerformDFT(double dir)
{
   double scale;
   int index=0, num = image->Rows();
   prog_bar->Min = 0;
   prog_bar->Max = 2*num;
   prog_bar->Step = 1;
   prog_bar->Position = 0;
   data = new double[2*num];
   if (dir == 1.0)
      scale = num*num;
   else
      scale=1.0;
   for(int row(0); row<num; row++){
      for(int col(0); col< num; col++){
         data[index++] = image->Data(col,row).real();
         data[index++] = image->Data(col,row).imag();
      }
      fft(data, num, dir);
      prog_bar->StepIt();
      index = 0;
      for(int col(0); col< num; col++){
         image->Data(col,row) = vcomplex(data[2*col],data[2*col+1]);
      }
   }
   for(int col(0); col<num; col++){
      for(int row(0); row<num; row++){
         data[2*row]   = image->Data(col,row).real();
         data[2*row+1] = image->Data(col,row).imag();
      }
      fft(data, num, dir);
      prog_bar->StepIt();
      for(int row(0); row<num; row++){
         image->Data(col,row) = vcomplex(data[2*row], data[2*row+1])/scale;
      }
   }
}
//---------------------------------------------------------------------------
bool
DFT::CheckSize()
{
   return true;
}



