/*
   DFTVCircSymFilterUnit.cpp

   definin soubor okna filtru Cilcurary Symmetric Filter

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DFTCircSymFilterUnit.h"
#include "DFTUnit.h"
#include "DFTProgressUnit.h"
#include "PictureUnit.h"
#include "dft.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDFTCircSymFilterForm *DFTCircSymFilterForm;
//---------------------------------------------------------------------------
__fastcall TDFTCircSymFilterForm::TDFTCircSymFilterForm(TComponent* Owner)
        : TForm(Owner), circular_kind(LowPass)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   orig_grey_image = new GreyImage();
   PictureForm->GetBMPHandler()->Set(*orig_grey_image);
   orig_image = new ComplexImage(*DFTForm->GetDFTComplexImage());
   filtered_image = new ComplexImage(orig_image->Cols(), orig_image->Rows());
}
//---------------------------------------------------------------------------
__fastcall TDFTCircSymFilterForm::~TDFTCircSymFilterForm()
{
   delete (orig_image);
   delete (filtered_image);
   delete (orig_grey_image);   
}
//---------------------------------------------------------------------------
void __fastcall TDFTCircSymFilterForm::PerformBitBtnClick(TObject *Sender)
{
   try{
      int cut_off = CutOffEdit->Text.ToInt();
      if (cut_off > orig_image->Cols()/2)
         throw Exception("Examined Image has only " +
                AnsiString(orig_image->Cols()/2) + "  coefficients in both direction");
      if (cut_off<= 0)
         throw Exception(" CutOff frequency must be a non-zero positive integer number");
      int order = OrderEdit->Text.ToInt();
      if (order <= 0)
         throw Exception("The Order of the Filter must be a positive integer number");
      *filtered_image = *orig_image;
      filtered_image->CyrcularSymFilter(cut_off, order, circular_kind);

      DFTForm->AssignComplexImageAndView(filtered_image);


      Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
      DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
      DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;

      DFTProgressForm->Caption = " ";
      DFTProgressForm->Show();
      DFTProgressForm->Caption = "Inverse FFT in Process";

      DFT dft(filtered_image, DFTProgressForm->ProgressBar);
      dft.PerformDFT(-1);
      DFTProgressForm->Release();
      GreyImage temp(filtered_image->Cols(), filtered_image->Rows());
      temp = filtered_image->ReImage();
      PictureForm->AssignImage(&temp);
      // Change for AssignList ! 

   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "Save Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "Save Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TDFTCircSymFilterForm::CheckBoxClick(TObject *Sender)
{
   if(CheckBox->Checked)
      circular_kind = LowPass;
   else
      circular_kind = HighPass;
}
//---------------------------------------------------------------------------
void __fastcall TDFTCircSymFilterForm::BitBtn2Click(TObject *Sender)
{
   DFTForm->AssignComplexImageAndView(orig_image);
   PictureForm->AssignImage(orig_grey_image);
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TDFTCircSymFilterForm::BitBtn1Click(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TDFTCircSymFilterForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------



