/*
   DFTFreqConvolutionUnit.cpp

   definin soubor okna, umoujcho provdt konvoluci a dekonvoluci
   natenho obrazu a s natenou maskou.

   tato funkce nen implementovna v tto verzi programu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DFTFreqConvolutionUnit.h"
#include "DFTUnit.h"
#include "PictureUnit.h"
#include "dft.h"
#include "DFTProgressUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDFTFreqConvolutionForm *DFTFreqConvolutionForm;
//---------------------------------------------------------------------------
__fastcall TDFTFreqConvolutionForm::TDFTFreqConvolutionForm(TComponent* Owner)
        : TForm(Owner), kernel(NULL), kernel_image(NULL), conv_image(NULL)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   orig_grey_image = new GreyImage();
   PictureForm->GetBMPHandler()->Set(*orig_grey_image);

   orig_image = new ComplexImage(*DFTForm->GetDFTComplexImage());
   conv_image = new ComplexImage(orig_image->Cols(), orig_image->Rows());
   image = new ComplexImage(*orig_image);
}
//---------------------------------------------------------------------------
__fastcall TDFTFreqConvolutionForm::~TDFTFreqConvolutionForm()
{
   delete (kernel);
   delete (orig_image);
   delete (conv_image);
   delete (kernel_image);
   delete (image);
   delete (orig_grey_image);
}
//---------------------------------------------------------------------------
void __fastcall TDFTFreqConvolutionForm::MLoadKernelClick(TObject *Sender)
{
   if(OpenDialog->Execute()){
      delete kernel;
      kernel = new Kernel(OpenDialog->FileName);
      kernel_image = new ComplexImage(orig_image->Cols(), orig_image->Rows());
      GreyImage kernel_grey_image(orig_image->Cols(), orig_image->Rows());
      // to get know if the number of rows and columns is odd or even.
      //bool even_rows = (kernel->KerRows() % 2 == 1) ? false : true;
      //bool even_cols = (kernel->KerCols() % 2 == 1) ? false : true;

      for(register int c(0); c<kernel_grey_image.Cols(); c++)
         for(register int r(0); r<kernel_grey_image.Rows(); r++){
              kernel_grey_image.Data(c,r) = 0;
         }

      int ker_cols = kernel->KerCols();
      int ker_rows = kernel->KerRows();

      for(register int c(0); c<ker_cols; c++)
         for(register int r(0); r<ker_rows; r++){
            if(c < ker_cols/2){
               if(r < ker_rows/2){
                  kernel_grey_image.Data(kernel_grey_image.Cols()-r-1,
                     kernel_grey_image.Rows()-c-1) = kernel->KerData(c,r);
               }else{
                  kernel_grey_image.Data(kernel_grey_image.Cols()-c-1,r-ker_rows/2)
                          = kernel->KerData(c,r);
               }
            }else{
               if(r < ker_rows/2){
                  kernel_grey_image.Data(c-ker_cols/2, kernel_grey_image.Rows()-r-1)
                          = kernel->KerData(c,r);
               }else{
                  kernel_grey_image.Data(c-ker_cols/2,r-ker_rows/2)
                          = kernel->KerData(c,r);
               }
            }
         }

      kernel_image->ReImage(&kernel_grey_image);
      if (kernel->Sum()) // if sum is different from 0
         *kernel_image *= kernel_image->Cols()*kernel_image->Rows()/abs(kernel->Sum());
      else               // if sum is equal to zero
         *kernel_image *= kernel_image->Cols()*kernel_image->Rows();

      Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
      DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
      DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;

      DFTProgressForm->Caption = " ";
      DFTProgressForm->Show();
      DFTProgressForm->Caption = "Forward FFT of Kernel in Process";
      DFT dft(kernel_image, DFTProgressForm->ProgressBar);
      dft.PerformDFT(1);
      DFTProgressForm->Release();

      DFTForm->AssignComplexImageAndView(kernel_image);
   }
}
//---------------------------------------------------------------------------

void __fastcall TDFTFreqConvolutionForm::PerformConvolutionBitBtnClick(
      TObject *Sender)
{
   try{
      if(kernel==NULL)
         throw Exception("No Kernel was selected yet.");
      *conv_image = *image*(*kernel_image);
      *image = *conv_image;


      conv_image->Data(128,128) = vcomplex(orig_image->Data(128,128).real(),0);

      Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
      DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
      DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;

      DFTProgressForm->Caption = " ";
      DFTProgressForm->Show();
      DFTProgressForm->Caption = "Inverse FFT of Convolved Image in Process";

      DFTForm->AssignComplexImageAndView(conv_image);
      DFT dft(conv_image, DFTProgressForm->ProgressBar);
      dft.PerformDFT(-1);
      DFTProgressForm->Release();

      GreyImage temp = conv_image->MagImage(1);
      PictureForm->AssignImage(&temp);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "Frequency Convolution Error" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------

void __fastcall TDFTFreqConvolutionForm::PerformDeconvolutionBitBtnClick(
      TObject *Sender)
{
   try{
      if(kernel==NULL)
         throw Exception("No Kernel was selected yet.");

      double bound = BoundEdit->Text.ToDouble();
      if(bound <= 0)
         throw Exception("The float point number for omiting\n division must be a non-zero positive number.");

      *conv_image = image->Divide(*kernel_image,bound);
      //*image = *conv_image;

      conv_image->Data(128,128) = vcomplex(orig_image->Data(128,128).real(),0);

      Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
      DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
      DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;

      DFTProgressForm->Caption = " ";
      DFTProgressForm->Show();
      DFTProgressForm->Caption = "Inverse FFT of Convolved Image in Process";

      DFTForm->AssignComplexImageAndView(conv_image);
      DFT dft(conv_image, DFTProgressForm->ProgressBar);
      dft.PerformDFT(-1);
      DFTProgressForm->Release();

      GreyImage temp = conv_image->MagImage(1);
      PictureForm->AssignImage(&temp);
   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "Frequency Convolution Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "Frequency Convolution Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------


void __fastcall TDFTFreqConvolutionForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{

   Release();
}
//---------------------------------------------------------------------------

void __fastcall TDFTFreqConvolutionForm::CancelBitBtnClick(TObject *Sender)
{
   DFTForm->AssignComplexImageAndView(orig_image);
   PictureForm->AssignImage(orig_grey_image);
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TDFTFreqConvolutionForm::MResetClick(TObject *Sender)
{
   DFTForm->AssignComplexImageAndView(orig_image);
   PictureForm->AssignImage(orig_grey_image);
   *image = *orig_image;
}
//---------------------------------------------------------------------------


void __fastcall TDFTFreqConvolutionForm::BitBtn1Click(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

