/*
   DFTFrequencyMaskUnit.cpp

   definin soubor okna filtru Frequency Mask, umoujcho vybrat
   jakoukoli obdlnkovou st spektra a tu vyfiltrovat

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DFTFrequencyMaskUnit.h"
#include "DFTUnit.h"
#include "PictureUnit.h"
#include "DFTProgressUnit.h"
#include "dft.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDFTFrequencyMaskForm *DFTFrequencyMaskForm;
//---------------------------------------------------------------------------
__fastcall TDFTFrequencyMaskForm::TDFTFrequencyMaskForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   DFTForm->fmrect->Allow() = true;
   DFTForm->fm_is_opened = true;
   
   orig_grey_image = new GreyImage();
   PictureForm->GetBMPHandler()->Set(*orig_grey_image);
   orig_image = new ComplexImage(*DFTForm->GetDFTComplexImage());
   filtered_image = new ComplexImage(orig_image->Cols(), orig_image->Rows());
   *filtered_image = *orig_image;
}
//---------------------------------------------------------------------------
__fastcall TDFTFrequencyMaskForm::~TDFTFrequencyMaskForm()
{
   delete (orig_image);
   delete (filtered_image);
   delete (orig_grey_image);
}
//---------------------------------------------------------------------------
void __fastcall TDFTFrequencyMaskForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   DFTForm->fmrect->Clear();
   DFTForm->fmrect->Allow() = false;
   DFTForm->fm_is_opened = false;
   Release();
}
//---------------------------------------------------------------------------

void __fastcall TDFTFrequencyMaskForm::PerformBitBtnClick(TObject *Sender)
{
   try{
      TPoint p1 = DFTForm->fmrect->Point1();
      TPoint p2 = DFTForm->fmrect->Point2();

      switch(RectRadioGroup->ItemIndex){
         case 0 :     // set to Zero
              //*filtered_image = *orig_image;
              filtered_image->RectFrequencyMask(p1, p2);
              break;
         case 1 :     // every coeff multiply be mtp
              double mtp = MtpEdit->Text.ToDouble();
              //*filtered_image = *orig_image;
              filtered_image->RectFrequencyMask(p1, p2, mtp);
              break;
      }

      DFTForm->AssignComplexImageAndView(filtered_image);


      Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
      DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
      DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;

      DFTProgressForm->Caption = " ";
      DFTProgressForm->Show();
      DFTProgressForm->Caption = "Inverse FFT in Process";

      ComplexImage filtered_image_temp(*filtered_image);
      DFT dft(&filtered_image_temp, DFTProgressForm->ProgressBar);
      dft.PerformDFT(-1);
      DFTProgressForm->Release();
      GreyImage temp(filtered_image->Cols(), filtered_image->Rows());
      temp = filtered_image_temp.ReImage();
      PictureForm->AssignImage(&temp);
      DFTForm->fmrect->Rect() = false;
      // Change for AssignList !

   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "Frequency Mask Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "Frequency Mask Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------

void __fastcall TDFTFrequencyMaskForm::CancelBitBtnClick(TObject *Sender)
{
   DFTForm->AssignComplexImageAndView(orig_image);
   PictureForm->AssignImage(orig_grey_image);
   Close();
}
//---------------------------------------------------------------------------


void __fastcall TDFTFrequencyMaskForm::RectRadioGroupClick(TObject *Sender)
{
   switch(RectRadioGroup->ItemIndex){
      case 0 :
         MtpEdit->Enabled = false;
         break;
      case 1 :
         MtpEdit->Enabled = true;
         break;
   }
}
//---------------------------------------------------------------------------

void __fastcall TDFTFrequencyMaskForm::MResetClick(TObject *Sender)
{
   DFTForm->AssignComplexImageAndView(orig_image);
   PictureForm->AssignImage(orig_grey_image);
   *filtered_image = *orig_image;
}
//---------------------------------------------------------------------------

void __fastcall TDFTFrequencyMaskForm::OKBitBtnClick(TObject *Sender)
{
   Close();        
}
//---------------------------------------------------------------------------

