/*
   DFTRingFrequencyMaskUnit.cpp

   definin soubor okna filtru Frequency Mask, umoujcho vybrat
   jakoukoli kruhovou st spektra a tu vyfiltrovat

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DFTRingFrequencyMaskUnit.h"
#include "PictureUnit.h"
#include "DFTUnit.h"
#include "DFTProgressUnit.h"
#include "dft.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDFTRingFrequencyMaskForm *DFTRingFrequencyMaskForm;
//---------------------------------------------------------------------------
__fastcall TDFTRingFrequencyMaskForm::TDFTRingFrequencyMaskForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   orig_grey_image = new GreyImage();
   PictureForm->GetBMPHandler()->Set(*orig_grey_image);

   orig_image = new ComplexImage(*DFTForm->GetDFTComplexImage());
   filtered_image = new ComplexImage(orig_image->Cols(), orig_image->Rows());
}
//---------------------------------------------------------------------------
__fastcall TDFTRingFrequencyMaskForm::~TDFTRingFrequencyMaskForm()
{
   delete (orig_image);
   delete (filtered_image);
   delete (orig_grey_image);
}
void __fastcall TDFTRingFrequencyMaskForm::PerformBitBtnClick(
      TObject *Sender)
{

   try{
      int half = orig_image->Cols()/2 - 1;
      int freq_1 = Freq1Edit->Text.ToInt();
      if(freq_1 < 0)
         throw Exception("1. cut-off frequency must be a non-negative integer number");
      if(freq_1 > half)
         throw Exception("1. cut-off frequency must be a number between 0 and "+AnsiString(half-1));
      int freq_2 = Freq2Edit->Text.ToInt();
      if(freq_2 < 0)
         throw Exception("2. cut-off frequency must be a non-negative integer value");
      if(freq_2 > half)
         throw Exception("2. cut-off frequency must be a number between 0 and "+AnsiString(half-1));
      if(freq_1 >= freq_2)
         throw Exception("2. cut-off frequency must be of a bigger value than 1. cut-off frequency");

      switch(RingRadioGroup->ItemIndex){
         case 0 :     // set to Zero
              *filtered_image = *orig_image;
              if(KindCheckBox->Checked == true) // inverse
                 filtered_image->RingFrequencyMask(freq_1, freq_2, 0, ring_inverse);
              else                              // normal
                 filtered_image->RingFrequencyMask(freq_1, freq_2, 0, ring_normal);
              break;
         case 1 :     // every coeff multiply be mtp
              double mtp = MtpEdit->Text.ToDouble();
              *filtered_image = *orig_image;
              if(KindCheckBox->Checked == true) // inverse
                 filtered_image->RingFrequencyMask(freq_1, freq_2, mtp, ring_inverse);
              else                              // normal
                 filtered_image->RingFrequencyMask(freq_1, freq_2, mtp, ring_normal);
              break;
      }
      DFTForm->AssignComplexImageAndView(filtered_image);


      Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
      DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
      DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;

      DFTProgressForm->Caption = " ";
      DFTProgressForm->Show();
      DFTProgressForm->Caption = "Inverse FFT in Process";

      DFT dft(filtered_image, DFTProgressForm->ProgressBar);
      dft.PerformDFT(-1);
      DFTProgressForm->Release();
      GreyImage temp(filtered_image->Cols(), filtered_image->Rows());
      temp = filtered_image->ReImage();
      PictureForm->AssignImage(&temp);
      // Change for AssignList !

   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "Ring Frequency Mask Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "Ring Frequency Mask Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------


void __fastcall TDFTRingFrequencyMaskForm::BitBtn1Click(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TDFTRingFrequencyMaskForm::RingRadioGroupClick(
      TObject *Sender)
{
   switch(RingRadioGroup->ItemIndex){
      case 0 :
         MtpEdit->Enabled = false;
         break;
      case 1 :
         MtpEdit->Enabled = true;
         break;
   }
}
//---------------------------------------------------------------------------

void __fastcall TDFTRingFrequencyMaskForm::CancelClick(
      TObject *Sender)
{
   DFTForm->AssignComplexImageAndView(orig_image);
   PictureForm->AssignImage(orig_grey_image);
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TDFTRingFrequencyMaskForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  Release();
}
//---------------------------------------------------------------------------

