/*
   DFTSaveUnit.cpp

   definin soubor okna, pomoc kterho lze uloit hodnoty koefficient do
   ascii souboru.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DFTSaveUnit.h"
#include "DFTUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDFTSaveForm *DFTSaveForm;
//---------------------------------------------------------------------------
__fastcall TDFTSaveForm::TDFTSaveForm(TComponent* Owner)
        : TForm(Owner)
{
   Top =  PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   c_image = DFTForm->GetDFTComplexImage();
   half = c_image->Cols()/2;
}
__fastcall TDFTSaveForm::~TDFTSaveForm()
{
   
}
//---------------------------------------------------------------------------

bool
__fastcall TDFTSaveForm::ReadNumbersFromEdit(AnsiString all_string,
                                             vector& coeff, int pos_neg)
{
 //  try{
   int index = 0, pos;
   bool finish = false;
   AnsiString tmp, tmp2, all_string_tmp = all_string;
   //tests if the Edit is set to nil "-"
   if(all_string.Trim().AnsiCompare(AnsiString("-"))==0){
       coeff.resize(2);
       coeff[0] = 0;
       coeff[1] = 0;
       return false;
   }
   // this do-while loop serves for scaning the numbers from Edit in to
   // the coeff array
   do{
      pos=all_string.AnsiPos(",");
      if(pos==0){
         pos = all_string.Length();
         finish = true;
      }
      tmp = all_string.SubString(0,pos);
      all_string = all_string.SubString(pos+1,all_string.Length()-pos);
      if((pos = tmp.AnsiPos("-"))==0){
          if(finish)
             tmp = tmp.SubString(0,tmp.Length());
          else
             tmp = tmp.SubString(0,tmp.Length()-1);
          coeff.resize(index+2);

          if(pos_neg == 1){
             if (tmp.Trim().ToInt() > half - 1)
                throw Exception("Positive coefficients must be between 0 and "+AnsiString(half-1));
          }else{
             if (tmp.Trim().ToInt() > half)
                throw Exception("Negative coefficients must be between 0 and "+AnsiString(half));
          }
          coeff[index++] = tmp.Trim().ToInt();
          coeff[index++] = tmp.Trim().ToInt();
      }else{
          tmp2 = tmp.SubString(0,pos-1);
          coeff.resize(index+1);

          if(pos_neg == 1){
             if  (tmp2.Trim().ToInt() > half - 1)
                throw Exception("Positive coefficients must be between 0 and "+AnsiString(half-1));
          }else{
             if  (tmp2.Trim().ToInt() > half)
                throw Exception("Negative coefficients must be between 0 and "+AnsiString(half));
          }

          coeff[index++] = tmp2.Trim().ToInt();
          if(finish)
             tmp = tmp.SubString(pos+1,tmp.Length()-pos);
          else
             tmp = tmp.SubString(pos+1,tmp.Length()-pos-1);
          coeff.resize(index+1);

          if(pos_neg == 1){
             if  (tmp.Trim().ToInt() > half - 1)
                throw Exception("Positive coefficients must be between 0 and "+AnsiString(half-1));
          }else{
             if  (tmp.Trim().ToInt() > half)
                throw Exception("Negative coefficients must be between 0 and "+AnsiString(half));
          }

          coeff[index++] = tmp.Trim().ToInt();
       }
   }while(!finish);
   return true;
   /*}catch(Exception &E){
      Application->MessageBox(E.Message.c_str(),
         "Discrete Fourier Transform",MB_ICONQUESTION|MB_OK);
   } */
}
//---------------------------------------------------------------------------
void
__fastcall TDFTSaveForm::SaveCoefficients(DFTSaveCoefficients coeff)
{
   double data;
   double pi = 3.1415927;
   AnsiString test = coeff.file_path;
   ofstream o_file(coeff.file_path.c_str());
   //int tmp, tmpx, tmpy;
   if(o_file==0)
      throw Exception("Can't open file\n" + coeff.file_path);
   o_file << "Discrete Fourier Transformation Coefficients\n";
   o_file << "--------------------------------------------\n\n\n";
   o_file << setw(30) <<"Real part" << setw(15) <<"Imag part";
   o_file << setw(15) <<"Magnitude" << setw(15) <<"Phase\n";
   // positive coeff in X
   for(register int x(0); x<coeff.coeff_x.length()/2; x++){
      for(register int xx(coeff.coeff_x[2*x]); xx<=coeff.coeff_x[2*x+1]; xx++){
         // positive coeff in Y
         for(register int y(0); y<coeff.coeff_y.length()/2; y++){
            for(register int yy(coeff.coeff_y[2*y]);
                   yy<=coeff.coeff_y[2*y+1]; yy++){
               o_file <<"[fx = "<<xx<<",fy= "<<yy<<"]";
               data = c_image->Data(half+xx,half+yy).real();
               o_file << setw(15) << data;
               data = c_image->Data(half+xx,half+yy).imag();
               o_file << setw(15) << data;
               data = abs(c_image->Data(half+xx,half+yy));
               o_file << setw(15) << data;
               //data = arg(c_image->Data(half+xx,half+yy));
               data = atan2(c_image->Data(half+xx,half+yy).imag(),
                            c_image->Data(half+xx,half+yy).real())*180/(pi);
               o_file << setw(15) << data << endl;
            }
         }
         // negative coeff in Y
         for(register int y(0); y<coeff.n_coeff_y.length()/2; y++){
            for(register int yy(coeff.n_coeff_y[2*y]);
                   yy<=coeff.coeff_y[2*y+1]; yy++){
               o_file <<"[fx = "<<xx<<",fy= -"<<yy<<"]";
               data = c_image->Data(half+xx,half-yy).real();
               o_file << setw(15) << data;
               data = c_image->Data(half+xx,half-yy).imag();
               o_file << setw(15) << data;
               data = abs(c_image->Data(half+xx,half-yy));
               o_file << setw(15) << data;
               data = atan2(c_image->Data(half+xx,half+yy).imag(),
                            c_image->Data(half+xx,half+yy).real())*180/(pi);
//               data = arg(c_image->Data(half+xx,half-yy));
               o_file << setw(15) << data << endl;
            }
         }
      }
   }
   // negative coeff in X
   for(register int x(0); x<coeff.n_coeff_x.length()/2;x++){
      for(register int xx(coeff.n_coeff_x[2*x]);
                               xx<=coeff.n_coeff_x[2*x+1]; xx++){
         // positive coeff in Y
         for(register int y(0); y<coeff.coeff_y.length()/2; y++){
            for(register int yy(coeff.coeff_y[2*y]);
                   yy<=coeff.coeff_y[2*y+1]; yy++){
               o_file <<"[fx = -"<<xx<<",fy= "<<yy<<"]";
               data = c_image->Data(half-xx,half+yy).real();
               o_file << setw(15) << data;
               data = c_image->Data(half-xx,half+yy).imag();
               o_file << setw(15) << data;
               data = abs(c_image->Data(half-xx,half+yy));
               o_file << setw(15) << data;
               data = atan2(c_image->Data(half+xx,half+yy).imag(),
                            c_image->Data(half+xx,half+yy).real())*180/(pi);
//               data = arg(c_image->Data(half-xx,half+yy));
               o_file << setw(15) << data << endl;
            }
         }
         // negative coeff in Y
         for(register int y(0); y<coeff.n_coeff_y.length()/2; y++){
            for(register int yy(coeff.n_coeff_y[2*y]);
                   yy<=coeff.coeff_y[2*y+1]; yy++){
               o_file <<"[fx = -"<<xx<<",fy= -"<<yy<<"]";
               data = c_image->Data(half-xx,half-yy).real();
               o_file << setw(15) << data;
               data = c_image->Data(half-xx,half-yy).imag();
               o_file << setw(15) << data;
               data = abs(c_image->Data(half-xx,half-yy));
               o_file << setw(15) << data;
               data = atan2(c_image->Data(half+xx,half+yy).imag(),
                            c_image->Data(half+xx,half+yy).real())*180/(pi);
//               data = arg(c_image->Data(half-xx,half-yy));
               o_file << setw(15) << data << endl;
            }
         }
      }
   }
   o_file.close();

}
//---------------------------------------------------------------------------
bool
__fastcall TDFTSaveForm::IfFileExistsOverWrite(AnsiString file_path)
{
     if(FileExists(file_path))
        if(Application->MessageBox(
            ("File "+file_path+" exsits\nDo you want to overwrite it?").c_str(),
                "SaveDialog Message", MB_YESNO|MB_ICONQUESTION)==mrYes)
           return  true;
        else
           return  false;
     return true;
}

void __fastcall TDFTSaveForm::BitBtn1Click(TObject *Sender)
{
   try{
      ReadNumbersFromEdit(XPosEdit->Text, save_coeff.coeff_x, 1);
      ReadNumbersFromEdit(XNegEdit->Text, save_coeff.n_coeff_x, 0);
      ReadNumbersFromEdit(YPosEdit->Text, save_coeff.coeff_y, 1);
      ReadNumbersFromEdit(YNegEdit->Text, save_coeff.n_coeff_y, 0);

      save_coeff.save = true;
      if(SaveDialog->Execute()){
         if(!IfFileExistsOverWrite(SaveDialog->FileName))
                   return;
         save_coeff.file_path = SaveDialog->FileName;
      }
      SaveCoefficients(save_coeff);
      Close();
   }catch(EConvertError &E){
      Application->MessageBox(E.Message.c_str(),
         "Discrete Fourier Transform",MB_ICONQUESTION|MB_OK);
   }catch(Exception &E){
      Application->MessageBox(E.Message.c_str(),
         "Discrete Fourier Transform",MB_ICONQUESTION|MB_OK);
   }
}
//---------------------------------------------------------------------------

