/*
   DFTScissorsFrequencyMaskUnit.cpp

   definin soubor okna filtru Scissors Frequency Mask, umoujc
   filtrovat oblast ve tvaru nek a to bu k horizontln, nebo
   vertikln ose.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DFTScissorsFrequencyMaskUnit.h"
#include "PictureUnit.h"
#include "DFTUnit.h"
#include "DFTProgressUnit.h"
#include "dft.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDFTScissorsFrequencyMaskForm *DFTScissorsFrequencyMaskForm;
//---------------------------------------------------------------------------
__fastcall TDFTScissorsFrequencyMaskForm::TDFTScissorsFrequencyMaskForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   orig_grey_image = new GreyImage();
   PictureForm->GetBMPHandler()->Set(*orig_grey_image);

   orig_image = new ComplexImage(*DFTForm->GetDFTComplexImage());
   filtered_image = new ComplexImage(orig_image->Cols(), orig_image->Rows());
}
//---------------------------------------------------------------------------
__fastcall TDFTScissorsFrequencyMaskForm::~TDFTScissorsFrequencyMaskForm()
{
   delete (orig_image);
   delete (filtered_image);
   delete (orig_grey_image);
}
//---------------------------------------------------------------------------
void __fastcall TDFTScissorsFrequencyMaskForm::PerformBitBtnClick(
      TObject *Sender)
{
   const double pi = 3.1415927;
   try{
      int angle = AngleEdit->Text.ToDouble();
      if(angle < 0)
         throw Exception("Angle must be a non-negative number");
      switch(ScissRadioGroup->ItemIndex){
         case 0 :     // set to Zero
              *filtered_image = *orig_image;
              if(KindCheckBox->Checked == true) // Horizontal
                 filtered_image->ScissorsFrequencyMask(
                                angle*pi/180, 0, scissor_horizontal);
              else                              // Vertical
                 filtered_image->ScissorsFrequencyMask(
                                angle*pi/180, 0, scissor_vertical);
              break;
         case 1 :     // every coeff multiply be mtp
              double mtp = MtpEdit->Text.ToDouble();
              *filtered_image = *orig_image;
              if(KindCheckBox->Checked == true) // Horizontal
                 filtered_image->ScissorsFrequencyMask(
                                angle*pi/180, mtp, scissor_horizontal);
              else                              // Vertical
                 filtered_image->ScissorsFrequencyMask(
                                angle*pi/180, mtp, scissor_vertical);
              break;
      }
      DFTForm->AssignComplexImageAndView(filtered_image);


      Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
      DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
      DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;

      DFTProgressForm->Caption = " ";
      DFTProgressForm->Show();
      DFTProgressForm->Caption = "Inverse FFT in Process";

      DFT dft(filtered_image, DFTProgressForm->ProgressBar);
      dft.PerformDFT(-1);
      DFTProgressForm->Release();
      GreyImage temp(filtered_image->Cols(), filtered_image->Rows());
      temp = filtered_image->ReImage();
      PictureForm->AssignImage(&temp);
      // Change for AssignList !

   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "Scissors Frequency Mask Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "Scissors Frequency Mask Tool Window Error" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TDFTScissorsFrequencyMaskForm::CancelBitBtnClick(
      TObject *Sender)
{
   DFTForm->AssignComplexImageAndView(orig_image);
   PictureForm->AssignImage(orig_grey_image);
   Close();        
}
//---------------------------------------------------------------------------
void __fastcall TDFTScissorsFrequencyMaskForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------
void __fastcall TDFTScissorsFrequencyMaskForm::BitBtn1Click(
      TObject *Sender)
{
   Close();        
}
//---------------------------------------------------------------------------
void __fastcall TDFTScissorsFrequencyMaskForm::ScissRadioGroupClick(
      TObject *Sender)
{
   switch(ScissRadioGroup->ItemIndex){
      case 0 :
         MtpEdit->Enabled = false;
         break;
      case 1 :
         MtpEdit->Enabled = true;
         break;
   }
}
//---------------------------------------------------------------------------

