/*
   DFTSizeUnit.cpp

   definin soubor okna, zarovnvajcho obraz do velikosti, na kterou
   lze aplikovat FFT algoritmus.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DFTSizeUnit.h"
#include "PictureUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDFTSizeForm *DFTSizeForm;
//---------------------------------------------------------------------------
__fastcall TDFTSizeForm::TDFTSizeForm(TComponent* Owner)
        : TForm(Owner), terminate(true)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;
   original_image = new GreyImage();
   sized_image = new GreyImage();
   sized_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   sized_handler->Set(*original_image);

   CheckSize(original_image->Cols(), original_image->Rows());
   cancel = false;
   sqr_sized_image = new GreyImage(Image1->Width, Image1->Height);
   Scale scale(*original_image, *sized_image,
       Image1->Width*max_size/pow(2,which_pow_of2));
   BlackBackground(sqr_sized_image, sized_image, Image1->Width);
   sized_handler->Assign(*sqr_sized_image);
   sized_handler->Show(Image1);
}
//---------------------------------------------------------------------------
__fastcall TDFTSizeForm::~TDFTSizeForm()
{
   delete (sized_handler);
   delete (original_image);
   delete (sized_image);
   delete (sqr_sized_image);
   delete (sqr_original_image);
}
//---------------------------------------------------------------------------
bool
__fastcall TDFTSizeForm::CheckSize(int cols, int rows)
{
   max_size = (cols>rows)? cols : rows;
   int pow_of2=1;
   which_pow_of2=0;
   while (max_size>pow_of2){
       pow_of2 = pow(2, ++which_pow_of2);
       if((max_size==pow_of2)&&(max_size==cols)&&(max_size==rows))
          return true;
   }
   return false;
}
//---------------------------------------------------------------------------
void
__fastcall TDFTSizeForm::BlackBackground(GreyImage* back_image,
                     GreyImage* forw_image, int size_of_window)
{
   // makeup with black background
   try{
      back_image->ClearImageArea(0, back_image->Cols(),
                                 back_image->Rows());
      back_image->WriteImageAreaFromImage(*forw_image, 0, 0,
                          forw_image->Cols(),forw_image->Rows());
   }catch(VEMsg &E){
      ShowMessage(E.msg.c_str());
   }
}
//---------------------------------------------------------------------------
void
__fastcall TDFTSizeForm::MakeUpWithImage(GreyImage* back_image,
                     GreyImage* forw_image, int size_of_window)
{
   // make up with the image
   int s_cols, s_rows;
   try{
      back_image->WriteImageAreaFromImage(*forw_image,
                 0, 0, forw_image->Cols(), forw_image->Rows());
      if(2*forw_image->Cols()>size_of_window){
         s_cols = size_of_window - forw_image->Cols()-1;
         s_rows = forw_image->Rows();
      }else{
         s_cols = forw_image->Cols();
         s_rows = forw_image->Rows();
      }
      back_image->WriteImageAreaFromImage(*forw_image,
                 forw_image->Cols(), 0, s_cols, s_rows);

      if(2*forw_image->Rows()>size_of_window){
         s_rows = size_of_window - forw_image->Rows()-1;
         s_cols = forw_image->Cols();
      }else{
         s_cols = forw_image->Cols();
         s_rows = forw_image->Rows();
      }
      back_image->WriteImageAreaFromImage(*forw_image,
                 0, forw_image->Rows(), s_cols, s_rows);

      if(2*forw_image->Rows()>size_of_window){
         s_rows = size_of_window - forw_image->Rows()-1;
      }else
         s_rows = forw_image->Rows();
      if(2*forw_image->Cols()>size_of_window){
         s_cols = size_of_window - forw_image->Cols()-1;
      }else
         s_cols = forw_image->Cols();

      back_image->WriteImageAreaFromImage(*forw_image,
                 forw_image->Cols(), forw_image->Rows(),
                 s_cols, s_rows);

   }catch(VEMsg &E){
      ShowMessage(E.msg.c_str());
   }
}
//---------------------------------------------------------------------------
void
__fastcall TDFTSizeForm::MakeUpWithMirror(GreyImage* back_image,
                      GreyImage* forw_image, int size_of_window)
{
   // make up with the mirror image
   try{
      GreyImage HImage(forw_image->Cols(), forw_image->Rows());
      GreyImage VImage(forw_image->Cols(), forw_image->Rows());
      GreyImage HVImage(forw_image->Cols(), forw_image->Rows());
      Translate transH(*forw_image, HImage);
      transH.MirrorImage(0, 0, forw_image->Cols(), forw_image->Rows(),
                         0, 0, HorizMirror);
      Translate transV(*forw_image, VImage);
      transV.MirrorImage(0, 0, forw_image->Cols(), forw_image->Rows(),
                      0, 0, VertMirror);
      Translate transHV(HImage, HVImage);
      transHV.MirrorImage(0, 0, forw_image->Cols(), forw_image->Rows(),
                      0, 0, VertMirror);

      int s_cols, s_rows;
      back_image->WriteImageAreaFromImage(*forw_image,
                 0, 0, forw_image->Cols(), forw_image->Rows());
      if(2*forw_image->Cols()>size_of_window){
         s_cols = size_of_window - forw_image->Cols()-1;
         s_rows = forw_image->Rows();
      }else{
         s_cols = forw_image->Cols();
         s_rows = forw_image->Rows();
      }
      back_image->WriteImageAreaFromImage(HImage,
                 forw_image->Cols(), 0, s_cols, s_rows);

      if(2*forw_image->Rows()>size_of_window){
         s_rows = size_of_window - forw_image->Rows()-1;
         s_cols = forw_image->Cols();
      }else{
         s_cols = forw_image->Cols();
         s_rows = forw_image->Rows();
      }
      back_image->WriteImageAreaFromImage(VImage,
                 0, forw_image->Rows(), s_cols, s_rows);

      if(2*forw_image->Rows()>size_of_window){
         s_rows = size_of_window - forw_image->Rows()-1;
      }else
         s_rows = forw_image->Rows();
      if(2*forw_image->Cols()>size_of_window){
         s_cols = size_of_window - forw_image->Cols()-1;
      }else
         s_cols = forw_image->Cols();

      back_image->WriteImageAreaFromImage(HVImage,
                 forw_image->Cols(), forw_image->Rows(),
                 s_cols, s_rows);

   }catch(VEMsg &E){
      ShowMessage(E.msg.c_str());
   }
}
//----------------------------------------------------------------------------
void __fastcall TDFTSizeForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
      Release();
}
//---------------------------------------------------------------------------

void __fastcall TDFTSizeForm::OKBitBtnClick(TObject *Sender)
{
   if(cancel==false){
      int sqr_size = pow(2, which_pow_of2);
      GreyImage sqr_original_image(pow(2, which_pow_of2),pow(2, which_pow_of2));
    /*  DFTForm->GetDFTGreyImage()->Resize(sqr_original_image.Cols(),
                                      sqr_original_image.Rows());*/
      switch (RadioGroup->ItemIndex){
         case 0:  // make up with black background
            BlackBackground(&sqr_original_image, original_image, sqr_size);
            //*(DFTForm->GetDFTGreyImage()) = sqr_original_image;
            break;
         case 1:  // make up with the image
            MakeUpWithImage(&sqr_original_image, original_image, sqr_size);
          //  *(DFTForm->GetDFTGreyImage()) = sqr_original_image;
            break;
         case 2:  // make up with the mirroring image
            MakeUpWithMirror(&sqr_original_image, original_image, sqr_size);
            //*(DFTForm->GetDFTGreyImage()) = sqr_original_image;
            break;
      }
      PictureForm->AssignList(&sqr_original_image);
   }
}
//---------------------------------------------------------------------------
void __fastcall TDFTSizeForm::RadioGroupClick(TObject *Sender)
{
   switch (RadioGroup->ItemIndex){
      case 0:  // make up with black background
         BlackBackground(sqr_sized_image, sized_image, Image1->Width);
         sized_handler->Assign(*sqr_sized_image);
         sized_handler->Show(Image1);
         break;
      case 1:  // make up with the image
         MakeUpWithImage(sqr_sized_image, sized_image, Image1->Width);
         sized_handler->Assign(*sqr_sized_image);
         sized_handler->Show(Image1);
         break;
      case 2:  // make up with the mirroring image
         MakeUpWithMirror(sqr_sized_image, sized_image, Image1->Width);
         sized_handler->Assign(*sqr_sized_image);
         sized_handler->Show(Image1);
         break;
   }
}
//---------------------------------------------------------------------------

