/*
   DFTUnit.cpp

   definin soubor hlavnho DFT okna.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "DFTUnit.h"
#include "DFTSizeUnit.h"
#include "dft.h"
#include "DFTProgressUnit.h"
#include "DFTOptionsUnit.h"
#include "DFTSaveUnit.h"
#include "PictureUnit.h"
#include "DFTViewCoeffUnit.h"
#include "DFTCircSymFilterUnit.h"
#include "DFTFrequencyMaskUnit.h"
#include "DFTScissorsFrequencyMaskUnit.h"
#include "DFTFreqConvolutionUnit.h"
#include "DFTRingFrequencyMaskUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDFTForm *DFTForm;
//---------------------------------------------------------------------------
__fastcall TDFTForm::TDFTForm(TComponent* Owner)
        : TForm(Owner), dft_terminate(false)
{

   Top = PictureForm->Top+50;
   Left = PictureForm->Left +50;

   fmrect = new FreqMask(Image1);

   try{
      original_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
      dft_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
      grey_image = new GreyImage;
      original_handler->Set(*grey_image);
      dft_handler->Assign(*grey_image);
      FFTForward(options);
   }catch(EDFTTooLargeImage &E){
      Application->MessageBox(E.Message.c_str(),
         "Discrete Fourier Transform",MB_ICONQUESTION|MB_OK);
      Release(); // !!!!!!!!!!!!
   }catch(EDFTTerminate &E){
      Release();
   }
}
//---------------------------------------------------------------------------
__fastcall TDFTForm::~TDFTForm()
{
   delete (dft_handler);
   delete (original_handler);
   delete (complex_image);
   delete (grey_image);
}
//---------------------------------------------------------------------------
bool
__fastcall TDFTForm::CheckSize(int cols, int rows)
{
   int max_size = (cols>rows)? cols : rows;
   if(max_size>512)
      throw EDFTTooLargeImage(AnsiString("Image is too large!\n\
      The longer size can't be bigger than 512"));
   int pow_of2=1;
   int which_pow_of2=0;
   while (max_size>pow_of2){
       pow_of2 = pow(2, ++which_pow_of2);
       if((max_size==pow_of2)&&(max_size==cols)&&(max_size==rows))
          return true;
   }
   return false;
}
//---------------------------------------------------------------------------
void
__fastcall TDFTForm::FormClose(TObject *Sender, TCloseAction &Action)
{
   if(fm_is_opened){
      DFTFrequencyMaskForm->Release();
   }
   PictureForm->MEdit->Visible = true;
   PictureForm->Open1->Visible = true;
   PictureForm->MSave->Visible = true;
   PictureForm->MSaveAs->Visible = true;
   PictureForm->MImage->Visible = true;
   PictureForm->MView->Visible = true;
   PictureForm->MColor->Visible = true;
   Release();
}
//---------------------------------------------------------------------------
void
__fastcall TDFTForm::PerformDFTWithOptions(DFTOptions &options)
{
   try{
      if(dft_terminate)
         throw EDFTTerminate("Terminated from DFTOptionsForm");
      Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
      DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
      DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;

      DFTProgressForm->Caption = " ";
      FFTForward(options);


      dft_handler->Assign(*grey_image);
      dft_handler->Show(Image1);
      DFTProgressForm->Close();
   }catch(EDFTTerminate &E){
   }
}
//---------------------------------------------------------------------------
void
__fastcall TDFTForm::FFTForward(DFTOptions &options)
{
   complex_image = new ComplexImage(grey_image->Rows(),
                                   grey_image->Rows());
   complex_image->ReImage(grey_image);

   Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
   DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
   DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;

   DFTProgressForm->Caption = " ";
   DFTProgressForm->Show();
   DFTProgressForm->Caption = "Forward FFT in Process";

   DFT dft(complex_image, DFTProgressForm->ProgressBar);
   dft.PerformDFT(1);
   DFTProgressForm->Close();
   DFTProgressForm->Release();
   SetDFTOptions(options);
}
//---------------------------------------------------------------------------
void
__fastcall TDFTForm::FFTInverse(DFTOptions &options)
{

}
//---------------------------------------------------------------------------
void
__fastcall TDFTForm::FFTForwardInverse(DFTOptions &options)
{

}
//---------------------------------------------------------------------------
void __fastcall TDFTForm::MOptionsClick(TObject *Sender)
{
   Application->CreateForm(__classid(TDFTOptionsForm),
                                     &DFTOptionsForm);
   DFTOptionsForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TDFTForm::SetDFTOptions(DFTOptions &opt)
{
   switch(opt.ViewAs){
      case(0):
         *grey_image = complex_image->MagImage(opt.Scale);
         break;
      case(1):
         *grey_image = complex_image->ReImage(opt.Scale);
         break;
      case(2):
         *grey_image = complex_image->ImImage(opt.Scale);
         break;
      case(3):
         *grey_image = complex_image->PhaseImage();
         break;

   }
   if(dft_handler->Cols()<MAX_CLIENT_SIZE){
      ClientWidth = MAX_CLIENT_SIZE;
      ClientHeight = MAX_CLIENT_SIZE;
      Panel->Width = dft_handler->Cols();
      Panel->Height = dft_handler->Rows();
      Panel->Top = ClientHeight/2 - Panel->Height/2;
      Panel->Left = ClientWidth/2 - Panel->Width/2;
      if(opt.ViewValues){
         StatusBar->Visible = true;
         ClientHeight += StatusBar->Height;
      }
      else{
         StatusBar->Visible = false;
      }
   }else{
      Panel->Width = dft_handler->Cols();
      Panel->Height = dft_handler->Rows();
      ClientWidth = Panel->Width;
      ClientHeight = Panel->Height;
      Panel->Top = 0;
      Panel->Left = 0;
      if(opt.ViewValues){
         StatusBar->Visible = true;
         ClientHeight += StatusBar->Height;
      }
      else{
         StatusBar->Visible = false;
      }
   }

   dft_handler->Assign(*grey_image);
   dft_handler->Show(Image1);
   Save->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TDFTForm::Image1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   if(options.Fft_trans == forward)
      if(options.ViewValues){
         int rows = complex_image->Rows();
         int cols = complex_image->Cols();
         int x_off = (Image1->Width - complex_image->Cols())/2;
         int y_off = (Image1->Height - complex_image->Rows())/2;
         if(((X-x_off)<0)||((X-x_off)>=complex_image->Cols())||
            ((Y-y_off)<0)||((Y-y_off)>=complex_image->Rows())){
            StatusBar->Panels->Items[1]->Text =
                        AnsiString("Fx: -");
            StatusBar->Panels->Items[2]->Text =
                        AnsiString("Fy: -");
            StatusBar->Panels->Items[3]->Text =
                        AnsiString(" - ");
         }
         else{
            StatusBar->Panels->Items[1]->Text =
                        AnsiString("Fx: ")+AnsiString(- cols/2 + X-x_off);
            StatusBar->Panels->Items[2]->Text =
                        AnsiString("Fy: ")+AnsiString(- rows/2 + Y-y_off);

            switch(options.ViewAs){
               case(0):
                  StatusBar->Panels->Items[0]->Text = "Magnitude";
                  StatusBar->Panels->Items[3]->Text =
                     FormatFloat("0.00000",abs(complex_image->Data(
                                                  X-x_off, Y-y_off)));
                  break;
               case(1):
                  StatusBar->Panels->Items[0]->Text = "Real part";
                  StatusBar->Panels->Items[3]->Text =
                     FormatFloat("0.00000",complex_image->Data(
                                       X-x_off, Y-y_off).real());
                  break;
               case(2):
                  StatusBar->Panels->Items[0]->Text = "Imag. part";
                  StatusBar->Panels->Items[3]->Text =
                     FormatFloat("0.00000",complex_image->Data(
                                        X-x_off, Y-y_off).imag());
                  break;
               case(3): // Phase
                  double pi_2 = 3.14159276/2;
                  StatusBar->Panels->Items[0]->Text = "Phase";
                  StatusBar->Panels->Items[3]->Text =
                     FormatFloat("0",arg(complex_image->Data(
                                        X-x_off, Y-y_off))*180/pi_2/2 );
                  break;

            }
         }
      }

   fmrect->MouseMove(X,Y);

}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::SaveClick(TObject *Sender)
{
   Application->CreateForm(__classid(TDFTSaveForm),
                                     &DFTSaveForm);
   DFTSaveForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::MViewCoeffClick(TObject *Sender)
{
   Application->CreateForm(__classid(TDFTViewCoeffForm),
                                     &DFTViewCoeffForm);
   DFTViewCoeffForm->ShowModal();
}
//---------------------------------------------------------------------------
void
__fastcall TDFTForm::AssignComplexImageAndView(ComplexImage* image)
{
   *complex_image = *image;
   SetDFTOptions(options);
}


void __fastcall TDFTForm::MCircSymFilterClick(TObject *Sender)
{
   Application->CreateForm(__classid(TDFTCircSymFilterForm),
                                     &DFTCircSymFilterForm);
   DFTCircSymFilterForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::MFrequencyMasksClick(TObject *Sender)
{
   Application->CreateForm(__classid(TDFTFrequencyMaskForm),
                                     &DFTFrequencyMaskForm);
   DFTFrequencyMaskForm->Show();
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::Image1MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   fmrect->MouseDown(Button,X,Y);
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::Image1MouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   fmrect->MouseUp(Button,X,Y);
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::MScissorsMaskClick(TObject *Sender)
{
   Application->CreateForm(__classid(TDFTScissorsFrequencyMaskForm),
                                     &DFTScissorsFrequencyMaskForm);
   DFTScissorsFrequencyMaskForm->ShowModal();
}
//---------------------------------------------------------------------------


void __fastcall TDFTForm::MConDeconClick(TObject *Sender)
{
  Application->CreateForm(__classid(TDFTFreqConvolutionForm),
                          &DFTFreqConvolutionForm);
  DFTFreqConvolutionForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::MInverseFFTClick(TObject *Sender)
{
   ComplexImage inverse_image(complex_image->Cols(), complex_image->Rows());
   Application->CreateForm(__classid(TDFTProgressForm),
                                     &DFTProgressForm);
   DFTProgressForm->Top = PictureForm->Top + PictureForm->ClientHeight/2 - DFTProgressForm->Height/2;
   DFTProgressForm->Left = PictureForm->Left + PictureForm->ClientWidth/2 - DFTProgressForm->Width/2;
   DFTProgressForm->Caption = " ";
   DFTProgressForm->Show();
   DFTProgressForm->Caption = "Inverse FFT in Process";

   switch(options.Inverse_kind){
      case(0):
          inverse_image = *complex_image;
          break;
      case(1):
          inverse_image = complex_image->ConstPhase();
          break;
      case(2):
          inverse_image = complex_image->ConstMagnitude();      
          break;
   }

   DFT dft(&inverse_image, DFTProgressForm->ProgressBar);
   dft.PerformDFT(-1);
   DFTProgressForm->Close();
   DFTProgressForm->Release();

   *grey_image = inverse_image.MagImage();
   dft_handler->Assign(*grey_image);
   dft_handler->Show(Image1);

   //SetDFTOptions(options);
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::MOKClick(TObject *Sender)
{
   GreyImage temp;
   dft_handler->Set(temp);
   PictureForm->AssignImage(&temp);
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::MRingMaskClick(TObject *Sender)
{
   Application->CreateForm(__classid(TDFTRingFrequencyMaskForm),
                                     &DFTRingFrequencyMaskForm);
   DFTRingFrequencyMaskForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::MExitClick(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------

void __fastcall TDFTForm::FormMouseMove(TObject *Sender, TShiftState Shift,
      int X, int Y)
{
   StatusBar->Panels->Items[1]->Text =
                        AnsiString("Fx: -");
   StatusBar->Panels->Items[2]->Text =
                        AnsiString("Fy: -");
   StatusBar->Panels->Items[3]->Text =
                        AnsiString(" - ");
}
//---------------------------------------------------------------------------

