/*
   DFTUnit.h

   hlavikov soubor hlavnho DFT okna.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#ifndef DFTUnitH
#define DFTUnitH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "..\Headers\bmp_handler.h"
#include <ComCtrls.hpp>
#include <ExtCtrls.hpp>
#include "..\Headers\image_complex.h"
#include "..\Headers\image_grey.h"
#include "..\Headers\dft_options.h"
#include "..\Headers\dft_save_coefficients.h"
#include "..\Headers\freq_mask.h"
#include <Menus.hpp>
#include <ToolWin.hpp>
//---------------------------------------------------------------------------
const int MAX_CLIENT_SIZE = 256;
//---------------------------------------------------------------------------
class TDFTForm : public TForm
{
__published:	// IDE-managed Components
        TStatusBar *StatusBar;
        TMainMenu *MainMenu1;
        TMenuItem *MCoefficients;
        TMenuItem *Save;
        TMenuItem *MSpatialFiltres;
        TMenuItem *MOptions;
        TMenuItem *MViewCoeff;
        TMenuItem *MCircSymFilter;
        TMenuItem *MFrequencyMasks;
        TMenuItem *MScissorsMask;
        TMenuItem *MConDecon;
        TMenuItem *MInverseFFT;
        TMenuItem *MFile;
        TMenuItem *MOK;
        TMenuItem *N2;
        TMenuItem *MExit;
        TMenuItem *FFT;
        TMenuItem *MRingMask;
        TPanel *Panel;
        TImage *Image1;
        void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
        void __fastcall MOptionsClick(TObject *Sender);
        void __fastcall Image1MouseMove(TObject *Sender, TShiftState Shift,
          int X, int Y);
        void __fastcall SaveClick(TObject *Sender);
        void __fastcall MViewCoeffClick(TObject *Sender);
        void __fastcall MCircSymFilterClick(TObject *Sender);
        void __fastcall MFrequencyMasksClick(TObject *Sender);
        void __fastcall Image1MouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
        void __fastcall Image1MouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
        void __fastcall MScissorsMaskClick(TObject *Sender);
        void __fastcall MConDeconClick(TObject *Sender);
        void __fastcall MInverseFFTClick(TObject *Sender);
        void __fastcall MOKClick(TObject *Sender);
        void __fastcall MRingMaskClick(TObject *Sender);
        void __fastcall MExitClick(TObject *Sender);
        void __fastcall FormMouseMove(TObject *Sender, TShiftState Shift,
          int X, int Y);
private:	// User declarations
        BMPHandler* dft_handler, *original_handler;
        ComplexImage* complex_image;
        GreyImage* grey_image;
        DFTOptions options;
        DFTSaveCoefficients save_coeff;
        bool dft_terminate;
        bool __fastcall CheckSize(int cols, int rows);
        void __fastcall FFTForward(DFTOptions &options);
        void __fastcall FFTInverse(DFTOptions &options);
        void __fastcall FFTForwardInverse(DFTOptions &options);
public:		// User declarations
        __fastcall TDFTForm(TComponent* Owner);
        __fastcall ~TDFTForm();
        FreqMask* fmrect;
        bool fm_is_opened;
        void         __fastcall PerformDFTWithOptions(DFTOptions &options);
        BMPHandler*  __fastcall GetBMPHandler() { return dft_handler; }
        GreyImage*   __fastcall GetDFTGreyImage() { return grey_image; }
        DFTOptions&  __fastcall GetDFTOptions() { return options; }
        void         __fastcall SetDFTOptions(DFTOptions &opt);
        void         __fastcall DFTTerminate() { dft_terminate = true; }
        void         __fastcall SetDFTSaveCoefficients(DFTSaveCoefficients init)
                                { save_coeff = init; }
        ComplexImage*__fastcall GetDFTComplexImage() { return complex_image; }
        void         __fastcall AssignComplexImageAndView(ComplexImage* image);

};
//---------------------------------------------------------------------------
extern PACKAGE TDFTForm *DFTForm;
//---------------------------------------------------------------------------
#endif
