/*
   FftFiltrationUnit.cpp

   definin soubor okna umonujc filtraci pomoc FFT

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "FftFiltrationUnit.h"
#include "PictureUnit.h"
#include "DFTProgressUnit.h"
#include "ViewUnit.h"
#include "dft.h"
#include "LineProfileUnit.h"
#include "CircleProfileUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFftFiltrationForm *FftFiltrationForm;
//---------------------------------------------------------------------------
__fastcall TFftFiltrationForm::TFftFiltrationForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   original_image = new GreyImage();
   sized_image = new GreyImage();
   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*original_image);
   cut_complete_image = new GreyImage(original_image->Cols(), original_image->Rows());

   // show original size of image
   Edit1->Text = AnsiString(handler->Cols())+" x "+AnsiString(handler->Rows());

   // find new size, allow is {4x4, 8x8, 16x16 ... 1024x1024}
   FindOptimalSize();
   scale_const = (double)(Image1->Width)/(double)(allow_size);
   fill_image = new GreyImage(allow_size, allow_size);
   fft_image = new GreyImage(allow_size, allow_size);
   filtered_image = new GreyImage(allow_size, allow_size);
   complete_image = new GreyImage(allow_size, allow_size);

   // is necessary change size ???
   NoResizeMaybe();

   // inicialize complex image
   options.ViewAs = magnitude;
   options.Inverse_kind = normal;
   complex_fft_image = new ComplexImage(allow_size, allow_size);
   complex_filtered_image = new ComplexImage(allow_size, allow_size);
   complex_complete_image = new ComplexImage(allow_size, allow_size);

   // For showing profile of fft image
   PictureForm->OtherHandler(true);

   HideShowingData();
   Image2->Enabled = false;
   Image3->Enabled = false;
}
//---------------------------------------------------------------------------
__fastcall TFftFiltrationForm::~TFftFiltrationForm()
{
  PictureForm->OtherHandler(false);
  delete (handler);
  delete (original_image);
  delete (fill_image);
  delete (fft_image);
  delete (filtered_image);
  delete (complete_image);
  delete (sized_image);
  delete (complex_fft_image);
  delete (complex_filtered_image);
  delete (complex_complete_image);
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();     
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::MethodComboBoxChange(TObject *Sender)
{
   switch(MethodComboBox->ItemIndex){
      case(0): MakeBlack(); break;
      case(1): MakeAverage(); break;
      case(2): MakeMirror(); break;
      case(3): MakePeriodic(); break;
      case(4): MakeRepetition(); break;
   }
   Scale scale(*fill_image, *sized_image, Image1->Width);
   handler->Assign(*sized_image);
   handler->Show(Image1);
   Image1->ShowHint = true;
   Button1->Enabled = true;
   PerformFftButton->Enabled = true;
   StatusBar->Panels->Items[0]->Text = " Click to Perform button on the Forward FFT panel. You can change type of view after.";
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::NoResizeMaybe()
{
   if ((allow_size == original_image->Cols())&&(allow_size == original_image->Rows())){
      Label6->Enabled = false;
      MethodComboBox->Enabled = false;
      handler->Set(*fill_image);
      Scale scale(*fill_image, *sized_image, Image1->Width);
      handler->Assign(*sized_image);
      handler->Show(Image1);
      PerformFftButton->Enabled = true;
      Image1->ShowHint = true;
      Button1->Enabled = true;
      StatusBar->Panels->Items[0]->Text = " Resize isn't requirement. Click to Perform button. You can change type of view after.";
   }else{
      StatusBar->Panels->Items[0]->Text = " Select method of resize.";
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::FindOptimalSize()
{
   allow_size = 2;
   int max_size = (handler->Cols() > handler->Rows()) ? handler->Cols() : handler->Rows();
   while(max_size > allow_size){
      allow_size *= 2;
   }
   Edit2->Text = AnsiString(allow_size)+" x "+AnsiString(allow_size);
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::MakeBlack()
{
   fill_image->ClearImageArea(BLACK, fill_image->Cols(), fill_image->Rows() ,0, 0);
   fill_image->WriteImageAreaFromImage(*original_image, 0, 0, original_image->Cols(),
                   original_image->Rows());
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::MakeAverage()
{
  // find mean value
  float sum = 0;
  float mean;
  for (int i=0; i < original_image->Cols(); i++){
     for (int j=0; j < original_image->Rows(); j++){
        sum += original_image->Data(i,j);
     }
  }
  mean = sum / (float)(original_image->Rows()*original_image->Cols());
  fill_image->ClearImageArea(ceil(mean), fill_image->Cols(), fill_image->Rows() ,0, 0);
  fill_image->WriteImageAreaFromImage(*original_image, 0, 0, original_image->Cols(),
                   original_image->Rows());
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::MakeMirror()
{
   // i must fill array between 'min_r to max_r' and 'min_c to max_c'
   int min_r = original_image->Rows();
   int min_c = original_image->Cols();
   int max_r = fill_image->Rows();
   int max_c = fill_image->Cols();
   int change_var;
   bool orientation;

   fill_image->WriteImageAreaFromImage(*original_image, 0, 0, original_image->Cols(),
                   original_image->Rows());

   // make mirroring in Rows
   change_var = min_r - 2;
   orientation = false;
   for (int i = min_r; i < max_r; i++){
      for(int j = 0; j < min_c; j++){
         fill_image->Data(j, i) = fill_image->Data(j, change_var);
      }
      if (orientation == false){
         change_var--;
         if (change_var == 0) {orientation = true;}
      }else{
         change_var++;
         if (change_var == min_r) {orientation = false;}
      }
   }

   // make mirroring in Cols
   change_var = min_c - 2;
   orientation = false;
   for (int j = min_c; j < max_c; j++){
      for (int i = 0; i < max_r; i++){
        fill_image->Data(j, i) = fill_image->Data(change_var, i);
      }
      if (orientation == false){
         change_var--;
         if (change_var == 0) {orientation = true;}
      }else{
         change_var++;
         if (change_var == min_c) {orientation = false;}
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::MakePeriodic()
{
   // i must fill array between 'min_r to max_r' and 'min_c to max_c'
   int min_r = original_image->Rows();
   int min_c = original_image->Cols();
   int max_r = fill_image->Rows();
   int max_c = fill_image->Cols();
   int change_var;

   fill_image->WriteImageAreaFromImage(*original_image, 0, 0, original_image->Cols(),
                   original_image->Rows());

   // make mirroring in Rows
   change_var = 0;
   for (int i = min_r; i < max_r; i++){
      for(int j = 0; j < min_c; j++){
         fill_image->Data(j, i) = fill_image->Data(j, change_var);
      }
      change_var++;
      if (change_var == min_r) {change_var = 0;}
   }

   // make mirroring in Cols
   change_var = 0;
   for (int j = min_c; j < max_c; j++){
      for (int i = 0; i < max_r; i++){
        fill_image->Data(j, i) = fill_image->Data(change_var, i);
      }
      change_var++;
      if (change_var == min_c) {change_var = 0;}
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::MakeRepetition()
{
   // i must fill array between 'min_r to max_r' and 'min_c to max_c'
   int min_r = original_image->Rows();
   int min_c = original_image->Cols();
   int max_r = fill_image->Rows();
   int max_c = fill_image->Cols();
   int change_var;

   fill_image->WriteImageAreaFromImage(*original_image, 0, 0, original_image->Cols(),
                   original_image->Rows());

   // make mirroring in Rows
   change_var = min_r - 2;
   for (int i = min_r; i < max_r; i++){
      for(int j = 0; j < min_c; j++){
         fill_image->Data(j, i) = fill_image->Data(j, change_var);
      }
   }

   // make mirroring in Cols
   change_var = min_c - 2;
   for (int j = min_c; j < max_c; j++){
      for (int i = 0; i < max_r; i++){
        fill_image->Data(j, i) = fill_image->Data(change_var, i);
      }
   }
}
//---------------------------------------------------------------------------
void
__fastcall TFftFiltrationForm::FFTForward(DFTOptions &options)
{
   complex_fft_image->ReImage(fill_image);
   DFT dft(complex_fft_image, ProgressBar1);
   dft.PerformDFT(1);
   SetDFTOptions(options);
}
//---------------------------------------------------------------------------
// Compute fft
void __fastcall TFftFiltrationForm::PerformFftButtonClick(TObject *Sender)
{
   ProgressBar1->Position = 0;
   ProgressBar1->Visible = true;
   FFTForward(options);
   ProgressBar1->Visible = false;
   ViewComboBox->ItemIndex = 0;
   ViewComboBox->Enabled = true;
   if (Image2->ShowHint == false) { ListBox2->Items->Add("Forward FFT"); }
   Image2->ShowHint = true;
   Image2->Enabled = true;
   Button2->Enabled = true;
   FilterComboBox->Enabled = true;
   PerformFftButton->Enabled = false;
   if (Image3->ShowHint == true) { PerformFilterButton->Enabled = true; }
   FftFiltrationForm->ActiveControl = ViewComboBox;
   ListBox1->Enabled = true;
   ListBox2->Enabled = true;
   StatusBar->Panels->Items[0]->Text = " Select the Type of filter, set parameters and click to Perform button on the Filtering panel.";
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::SetDFTOptions(DFTOptions &opt)
{
   switch(opt.ViewAs){
      case(0):
         *fft_image = complex_fft_image->MagImage(opt.Scale);
         break;
      case(1):
         *fft_image = complex_fft_image->ReImage(opt.Scale);
         break;
      case(2):
         *fft_image = complex_fft_image->ImImage(opt.Scale);
         break;
      case(3):
         *fft_image = complex_fft_image->PhaseImage();
         break;

   }
   Scale scale(*fft_image, *sized_image, Image2->Width);
   handler->Assign(*sized_image);
   handler->Show(Image2);
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::SetFilterOptions(DFTOptions &opt)
{
   switch(opt.ViewAs){
      case(0):
         *filtered_image = complex_filtered_image->MagImage(opt.Scale);
         break;
      case(1):
         *filtered_image = complex_filtered_image->ReImage(opt.Scale);
         break;
      case(2):
         *filtered_image = complex_filtered_image->ImImage(opt.Scale);
         break;
      case(3):
         *filtered_image = complex_filtered_image->PhaseImage();
         break;

   }
   Scale scale(*filtered_image, *sized_image, Image3->Width);
   handler->Assign(*sized_image);
   handler->Show(Image3);
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::ViewAsRadioGroupClick(TObject *Sender)
{
   // Set type of view fft
   switch(ViewComboBox->ItemIndex){
      case(0):
         options.ViewAs = magnitude;
         break;
      case(1):
         options.ViewAs = realpart;
         break;
      case(2):
         options.ViewAs = imaginary;
         break;
      case(3):
         options.ViewAs = phase;
         break;
   }
   SetDFTOptions(options);
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::ViewComboBoxChange(TObject *Sender)
{
   // Set type of view fft
   switch(ViewComboBox->ItemIndex){
      case(0):
         options.ViewAs = magnitude;
         break;
      case(1):
         options.ViewAs = realpart;
         break;
      case(2):
         options.ViewAs = imaginary;
         break;
      case(3):
         options.ViewAs = phase;
         break;
   }
   SetDFTOptions(options);
   if (Image3->ShowHint == true){SetFilterOptions(options);}
}
//---------------------------------------------------------------------------
// Visited box in filter options
void __fastcall TFftFiltrationForm::VisitedBox
                (AnsiString T1, AnsiString T2, AnsiString T3)
{
   // first box
   if (T1 != ""){
      LabelA->Caption = T1;
      LabelA->Visible = true;
      EditA->Visible = true;
   }else{
      LabelA->Visible = false;
      EditA->Visible = false;
   }

   // second box
   if (T2 != ""){
      LabelB->Caption = T2;
      LabelB->Visible = true;
      EditB->Visible = true;
   }else{
      LabelB->Visible = false;
      EditB->Visible = false;
   }

   //third box
   if (T3 != ""){
      LabelC->Caption = T3;
      LabelC->Visible = true;
      EditC->Visible = true;
   }else{
      LabelC->Visible = false;
      EditC->Visible = false;
   }
}
//--------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::FilterComboBoxChange(TObject *Sender)
{
   // Set type of filter
   switch(FilterComboBox->ItemIndex){
      case(0):
         VisitedBox("Frequence:", "Order:", "");
         EditA->Text = "50";
         EditB->Text = "3";
         break;
      case(1):
         VisitedBox("Frequence:", "Order:", "");
         EditA->Text = "50";
         EditB->Text = "3";
         break;
      case(2):
         VisitedBox("Multiple by:", "", "");
         EditA->Text = "0,5";
         break;
      case(3):
         VisitedBox("Angle", "Multiple by:", "");
         EditA->Text = "45";
         EditB->Text = "0";
         break;
      case(4):
         VisitedBox("Angle", "Multiple by:", "");
         EditA->Text = "45";
         EditB->Text = "0";
         break;
      case(5):
         VisitedBox("Freq.1:", "Freq.2:", "Multiple by:");
         EditA->Text = "10";
         EditB->Text = "30";
         EditC->Text = "0";
         break;
      case(6):
         VisitedBox("Freq.1:", "Freq.2:", "Multiple by:");
         EditA->Text = "10";
         EditB->Text = "30";
         EditC->Text = "0";
         break;
      case(7):
         VisitedBox("", "", "");
         EditA->Text = "1";
         EditB->Text = "50";
         break;
   }
   PerformFilterButton->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::PerformFilterButtonClick(TObject *Sender)
{
   switch(FilterComboBox->ItemIndex){
      case(0):
         PerformLowPass();
         break;
      case(1):
         PerformHighPass();
         break;
      case(2):
         PerformMultiple();
         break;
      case(3):
         PerformScissorsHorizont();
         break;
      case(4):
         PerformScissorsVertical();
         break;
      case(5):
         PerformBandPass();
         break;
      case(6):
         PerformBandStop();
         break;
      case(7):
         PerformHighPass();
         break;
   }
   PerformFilterButton->Enabled = false;
   PerformBackwardFftButton->Enabled = true;
   if (InverseRadioGroup->Enabled == false){
      options.Inverse_kind = normal;
      InverseRadioGroup->ItemIndex = 0;
      InverseRadioGroup->Enabled = true;
   }
   if (Image3->ShowHint == false) { ListBox2->Items->Add("Filtering"); }
   Image3->ShowHint = true;
   Image3->Enabled = true;
   Button3->Enabled = true;
   FftFiltrationForm->ActiveControl = FilterComboBox;
   StatusBar->Panels->Items[0]->Text = " You can click to Perform button on the Backward FFT panel.";
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::PerformLowPass()
{
   try{
      int cut_off = EditA->Text.ToInt();
      if (cut_off > fft_image->Cols()/2)
         throw Exception("Examined Image has only " +
                AnsiString(fft_image->Cols()/2) + "  coefficients in both direction");
      if (cut_off <= 0)
         throw Exception(" Frequency must be a non-zero positive integer number");
      int order = EditB->Text.ToInt();
      if ((order <= 0)||(order > 50))
         throw Exception("The Order must be integer between 1 and 50");
      *complex_filtered_image = *complex_fft_image;
      complex_filtered_image->CyrcularSymFilter(cut_off, order, LowPass);
      SetFilterOptions(options);
   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), " FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), " FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::PerformHighPass()
{
   try{
      int cut_off = EditA->Text.ToInt();
      if (cut_off > fft_image->Cols()/2)
         throw Exception("Examined Image has only " +
                AnsiString(fft_image->Cols()/2) + "  coefficients in both direction");
      if (cut_off <= 0)
         throw Exception(" Frequency must be a non-zero positive integer number");
      int order = EditB->Text.ToInt();
      if ((order <= 0)||(order > 50))
         throw Exception("The Order must be integer between 1 and 50");
      *complex_filtered_image = *complex_fft_image;
      complex_filtered_image->CyrcularSymFilter(cut_off, order, HighPass);
      SetFilterOptions(options);
   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), " FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), " FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::PerformMultiple()
{
   try{
      double mtp = EditA->Text.ToDouble();
      *complex_filtered_image = *complex_fft_image;
      complex_filtered_image->MultipleFrequencyMask(mtp);
      SetFilterOptions(options);
   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::PerformScissorsHorizont()
{
   const double pi = 3.141592654;
   try{
      int angle = EditA->Text.ToDouble();
      if((angle < 0)||(angle > 89))
         throw Exception("Angle must be a non-negative number between 0 and 89");

         double mtp = EditB->Text.ToDouble();
         *complex_filtered_image = *complex_fft_image;
          complex_filtered_image->ScissorsFrequencyMask(angle*pi/180, mtp, scissor_horizontal);
          //filtered_image->ScissorsFrequencyMask(angle*pi/180, mtp, scissor_vertical);
   SetFilterOptions(options);
   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::PerformScissorsVertical()
{
   const double pi = 3.141592654;
   try{
      int angle = EditA->Text.ToDouble();
      if((angle < 0)||(angle > 89))
         throw Exception("Angle must be a non-negative number between 0 and 89");

         double mtp = EditB->Text.ToDouble();
         *complex_filtered_image = *complex_fft_image;
          complex_filtered_image->ScissorsFrequencyMask(angle*pi/180, mtp, scissor_vertical);
   SetFilterOptions(options);
   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::PerformBandPass()
{

   try{
      int half = fft_image->Cols()/2 - 1;
      int freq_1 = EditA->Text.ToInt();
      if(freq_1 < 0)
         throw Exception("1. cut-off frequency must be a non-negative integer number");
      if(freq_1 > half)
         throw Exception("1. cut-off frequency must be a number between 0 and "+AnsiString(half-1));
      int freq_2 = EditB->Text.ToInt();
      if(freq_2 < 0)
         throw Exception("2. cut-off frequency must be a non-negative integer value");
      if(freq_2 > half)
         throw Exception("2. cut-off frequency must be a number between 0 and "+AnsiString(half-1));
      if(freq_1 >= freq_2)
         throw Exception("2. cut-off frequency must be of a bigger value than 1. cut-off frequency");

      // Multiple coef.
      double mtp = EditC->Text.ToDouble();
      *complex_filtered_image = *complex_fft_image;
      complex_filtered_image->RingFrequencyMask(freq_1, freq_2, mtp, ring_inverse);
      SetFilterOptions(options);

   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::PerformBandStop()
{

   try{
      int half = fft_image->Cols()/2 - 1;
      int freq_1 = EditA->Text.ToInt();
      if(freq_1 < 0)
         throw Exception("1. cut-off frequency must be a non-negative integer number");
      if(freq_1 > half)
         throw Exception("1. cut-off frequency must be a number between 0 and "+AnsiString(half-1));
      int freq_2 = EditB->Text.ToInt();
      if(freq_2 < 0)
         throw Exception("2. cut-off frequency must be a non-negative integer value");
      if(freq_2 > half)
         throw Exception("2. cut-off frequency must be a number between 0 and "+AnsiString(half-1));
      if(freq_1 >= freq_2)
         throw Exception("2. cut-off frequency must be of a bigger value than 1. cut-off frequency");

      // Multiple coef.
      double mtp = EditC->Text.ToDouble();
      *complex_filtered_image = *complex_fft_image;
      complex_filtered_image->RingFrequencyMask(freq_1, freq_2, mtp, ring_normal);
      SetFilterOptions(options);

   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::PerformBackwardFftButtonClick(
      TObject *Sender)
{
   try{
      ProgressBar2->Position = 0;
      ProgressBar2->Visible = true;

      switch(options.Inverse_kind){
      case(0):
          *complex_complete_image = *complex_filtered_image;
          break;
      case(1):
          *complex_complete_image = complex_filtered_image->ConstPhase();
          break;
      case(2):
          *complex_complete_image = complex_filtered_image->ConstMagnitude();
          break;
   }

      DFT dft(complex_complete_image, ProgressBar2);
      dft.PerformDFT(-1);
      ProgressBar2->Visible = false;

      *complete_image = complex_complete_image->MagImage();
      cut_complete_image->WriteImageAreaFromImage(*complete_image, 0, 0,
                        cut_complete_image->Cols(), cut_complete_image->Rows());

      // View cut or non-cut image
      switch (CheckBox1->Checked){
      case(false):
         ViewOutputImage();
         break;
      case(true):
         ViewCutOutputImage();
         break;
      }

      if (Image4->ShowHint == false) { ListBox2->Items->Add("Backward FFT"); }
      Image4->ShowHint = true;
      Button4->Enabled = true;
      CheckBox1->Enabled = true;
      PerformBackwardFftButton->Enabled = false;

      BitBtn1->Enabled = true;
      FftFiltrationForm->ActiveControl = BitBtn1;
      StatusBar->SimpleText = " Click to OK button. You can on/off cutting final image.";

   }catch(EConvertError& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }catch(Exception& E){
      Application->MessageBox(E.Message.c_str(), "FFT Filtering Tool Window" ,
            MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::EditAChange(TObject *Sender)
{
   PerformFilterButton->Enabled = true;  
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::EditBChange(TObject *Sender)
{
   PerformFilterButton->Enabled = true;         
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::EditCChange(TObject *Sender)
{
   PerformFilterButton->Enabled = true;          
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::Button1Click(TObject *Sender)
{
   type_of_view = 1;
   Application->CreateForm(__classid(TViewForm),
                          &ViewForm);
   ViewForm->ShowModal();
}
//---------------------------------------------------------------------------
BMPHandler* __fastcall TFftFiltrationForm::GetBMPHandler()
{
   switch (type_of_view){
      case(1):
         handler->Assign(*fill_image);
         break;
      case(2):
         handler->Assign(*fft_image);
         break;
      case(3):
         handler->Assign(*filtered_image);
         break;
      case(4):
         handler->Assign(*complete_image);
         break;
      case(5):
         handler->Assign(*cut_complete_image);
         break;
   }
   return handler;
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::Button2Click(TObject *Sender)
{
   type_of_view = 2;
   Application->CreateForm(__classid(TViewForm),
                          &ViewForm);
   ViewForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::Button3Click(TObject *Sender)
{
   type_of_view = 3;
   Application->CreateForm(__classid(TViewForm),
                          &ViewForm);
   ViewForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::Button4Click(TObject *Sender)
{
   switch (CheckBox1->Checked){
      case(false):
         type_of_view = 4;
         break;
      case(true):
         type_of_view = 5;
         break;
      }
   Application->CreateForm(__classid(TViewForm),
                          &ViewForm);
   ViewForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::BitBtn2Click(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::ViewOutputImage()
{
   Scale scale(*complete_image, *sized_image, Image4->Width);
   handler->Assign(*sized_image);
   handler->Show(Image4);
}
//---------------------------------------------------------------------------
void __fastcall TFftFiltrationForm::ViewCutOutputImage()
{
   Scale scale(*cut_complete_image, *sized_image, Image4->Width);
   handler->Assign(*sized_image);
   handler->Show(Image4);
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::CheckBox1Click(TObject *Sender)
{
   switch (CheckBox1->Checked){
      case(false):
         ViewOutputImage();
         break;
      case(true):
         ViewCutOutputImage();
         break;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::BitBtn1Click(TObject *Sender)
{
      switch (CheckBox1->Checked){
      case(false):
         PictureForm->AssignList(complete_image);
         break;
      case(true):
         PictureForm->AssignList(cut_complete_image);
         break;
   }
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::InverseRadioGroupChange(TObject *Sender)
{
   switch(InverseRadioGroup->ItemIndex){
      case(0):
          options.Inverse_kind = normal;
          break;
      case(1):
          options.Inverse_kind = const_phase;
          break;
      case(2):
          options.Inverse_kind = const_amplitude;
          break;
   }
   PerformBackwardFftButton->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::ListBox1Click(TObject *Sender)
{
  if (ListBox2->ItemIndex >= 0){
     Button5->Enabled = true;
  }
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::ListBox2Click(TObject *Sender)
{
  if (ListBox1->ItemIndex >= 0){
     Button5->Enabled = true;
  }
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::Button5Click(TObject *Sender)
{
   switch (ListBox2->ItemIndex){
      case(0):
         PictureForm->ChangeOtherHandler(fft_image);
         break;
      case(1):
         PictureForm->ChangeOtherHandler(filtered_image);
         break;
      case(2):
         PictureForm->ChangeOtherHandler(cut_complete_image);
         break;
   }

   switch (ListBox1->ItemIndex){
      case(0):
         PictureForm->MSaveAsClick(Sender);
         break;
      case(1):
         Application->CreateForm(__classid(TLineProfileForm),&LineProfileForm);
         LineProfileForm->ShowModal();
         break;
      case(2):
         Application->CreateForm(__classid(TCircleProfileForm),&CircleProfileForm);
         CircleProfileForm->ShowModal();
         break;
   }
}
//---------------------------------------------------------------------------



void __fastcall TFftFiltrationForm::Image2MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   // compute x-line
   pos_x = (double)(X)-((double)(Image1->Width)/(double)(2));
   pos_x = int(((double)(pos_x)) / scale_const);

   // compute y-line
   pos_y = (double)(Y)-((double)(Image1->Height)/(double)(2));
   pos_y = int(((double)(pos_y)) / scale_const);

   StatusBar->Panels->Items[1]->Text = "Row: "+AnsiString(pos_y);
   StatusBar->Panels->Items[2]->Text = "Column: "+AnsiString(pos_x);
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::Image3MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   // compute x-line
   pos_x = (double)(X)-((double)(Image1->Width)/(double)(2));
   pos_x = int(((double)(pos_x)) / scale_const);

   // compute y-line
   pos_y = (double)(Y)-((double)(Image1->Height)/(double)(2));
   pos_y = int(((double)(pos_y)) / scale_const);

   StatusBar->Panels->Items[1]->Text = "Row: "+AnsiString(pos_y);
   StatusBar->Panels->Items[2]->Text = "Column: "+AnsiString(pos_x);
}
//---------------------------------------------------------------------------

void TFftFiltrationForm::HideShowingData()
{
   StatusBar->Panels->Items[1]->Text = "Row: -";
   StatusBar->Panels->Items[2]->Text = "Column: -";
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::PanelBMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   HideShowingData();
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::PanelAMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   HideShowingData();        
}
//---------------------------------------------------------------------------


void __fastcall TFftFiltrationForm::PanelCMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   HideShowingData();        
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::PanelDMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   HideShowingData();        
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::FormMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   HideShowingData();        
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::ListBox1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   HideShowingData();        
}
//---------------------------------------------------------------------------

void __fastcall TFftFiltrationForm::ListBox2MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   HideShowingData();
}
//---------------------------------------------------------------------------

