/*
   freq_mask.cpp

   definin soubor tdy umoujc vybrn tvercov oblasti v okn
   pomoc kurzoru myi - Rectangle Frequency Mask

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "freq_mask.h"
#include "DFTUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
FreqMask::FreqMask()
:Image(NULL),rect(false),down(false)
{
}
//---------------------------------------------------------------------------
FreqMask::FreqMask(TImage* _Image)
: Image(_Image),rect(false),down(false)
{
}
//---------------------------------------------------------------------------
void
FreqMask::Draw(TPoint p1, TPoint p2)
{
   TCanvas* C = Image->Canvas;
   TPenMode  pm = C -> Pen -> Mode;
   TColor pc =  C->Pen->Color;
   TPenStyle ps = C->Pen->Style;
   TBrushStyle bs = C->Brush->Style;

   C->Pen->Color = clWhite;
   C->Pen->Style = psDot;
   C->Brush->Style = bsClear;
   C->Pen->Mode = pmXor;
   C->Rectangle(p1.x, p1.y, p2.x, p2.y);

   C->Pen->Color = pc;
   C->Pen->Style = ps;
   C->Brush->Style = bs;
   C->Pen->Mode = pm;
}
//---------------------------------------------------------------------------
void
FreqMask::MouseDown(TMouseButton Button, int X, int Y)
{
   switch (allow){
      case TRUE:
         if(Button==mbLeft){
            if(rect){
               Draw(P1,P2);
               Draw(PP1,PP2);
               rect = false;
            }
            down = true;
            P1.x = X;
            P1.y = Y;
            PP1 = CountPP(P1);
         }
         break;
   }
}
//---------------------------------------------------------------------------
void
FreqMask::MouseUp(TMouseButton Button, int X, int Y)
{
   if(allow)
      if(down){
         down = false;
      }
}
//---------------------------------------------------------------------------
void
FreqMask::MouseMove(int X, int Y)
{
   switch (allow){
      case TRUE:
         if(down){
            if(rect){
               Draw(P1,P2);
               Draw(PP1,PP2);
            }
            rect = true;
            int cols = DFTForm->GetDFTComplexImage()->Cols();
            int rows = DFTForm->GetDFTComplexImage()->Rows();
            P2.x = (X > cols) ? cols : X;
            P2.x = (X < 0) ? 0 : X;
            P2.y = (Y > rows) ? rows : Y;
            P2.y = (Y < 0) ? 0 : Y;
            PP2 = CountPP(P2);


            Draw(P1, P2);
            Draw(PP1,PP2);
         }
         break;
   }
}
//---------------------------------------------------------------------------
TPoint
FreqMask::CountPP(TPoint p1)
{
   int cols = DFTForm->GetDFTComplexImage()->Cols();
   int rows = DFTForm->GetDFTComplexImage()->Rows();
   TPoint pp;

   pp.x = p1.x - (p1.x-cols/2)*2;
   pp.y = p1.y + (rows/2-p1.y)*2;
   return pp;
}
//---------------------------------------------------------------------------
void
FreqMask::Clear()
{
   if(allow){
      if(rect){
         Draw(P1,P2);
         Draw(PP1,PP2);
      }
      rect = false;
      down = false;
   }   
}






