/*
   Kernel.h

   hlavikov soubor - jdrov funkce

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#ifndef kernelH
#define kernelH
//---------------------------------------------------------------------------
#include <fstream.h>
#include "assert.h"
#include "vexcept.h"
#include <iomanip.h>

//---------------------------------------------------------------------------
class Kernel {
private:
   double* kernel;
   int ker_cols;
   int ker_rows;
   double sum;
   double scale;
   bool absolute;
   AnsiString ker_name;
   void Resize(int _c,int _r);
public:
   // constructors
   Kernel();
   Kernel(int _kerl_cols, int _ker_rows);
   Kernel(const AnsiString& ker_filename);
   // copy constructur
   Kernel(const Kernel& init);
   ~Kernel();
   // const object member functions
   const int KerRows()    const { return ker_rows;}
   const int KerCols()    const { return ker_cols;}
   const AnsiString& Name() const { return ker_name;}
   const double& Sum()    const { return sum; }
   const bool& Absolute() const { return absolute; }
   const double& Scale()  const { return scale; }
   const double& KerData(int ker_col, int ker_row) const
         { return kernel[ker_col + ker_row*ker_cols]; }
   // non-const object member functions
   int KerRows()       { return ker_rows;}
   int KerCols()       { return ker_cols;}
   AnsiString& Name()  { return ker_name;}
   double& Sum()       { return sum; }
   bool& Absolute()    { return absolute; }
   double& Scale()     { return scale; }
   double& KerData(int ker_col, int ker_row)
         { return kernel[ker_col + ker_row*ker_cols]; }
   //---------------------------------------------------
   // member functions for assigning particular kernels
   //
   // ProImPro V. 3.1
   void FILTER_01();
   void FILTER_02();
   void FILTER_03();
   void FILTER_04();
   void FILTER_05();
   void FILTER_06();
   void FILTER_07();
   void FILTER_08();
   void FILTER_09();
   void FILTER_10();
   void FILTER_11();
   void FILTER_12();
   void FILTER_13();
   void FILTER_14();
   void FILTER_15();
   void FILTER_16();
   void FILTER_17();
   void FILTER_18();
   void FILTER_19();
   void FILTER_20();
   void FILTER_21();
   void FILTER_22();
   void FILTER_23();
   void FILTER_24();
   void FILTER_25();
   void FILTER_26();
   void FILTER_27();
   void FILTER_28();
   void FILTER_29();
   void FILTER_30();
   void FILTER_31();
   void FILTER_32();
   void FILTER_33();
   void FILTER_34();
   void FILTER_35();
   void FILTER_36();
   void FILTER_37();
   void FILTER_38();
   void FILTER_39();
   void FILTER_40();
   void FILTER_41A();
   void FILTER_41B();
   void FILTER_41C();
   void FILTER_41D();
   void FILTER_42A();
   void FILTER_42B();
   void FILTER_42C();
   void FILTER_42D();
   void FILTER_43A();
   void FILTER_43B();
   void FILTER_43C();
   void FILTER_43D();
   void FILTER_44A();
   void FILTER_44B();
   void FILTER_44C();
   void FILTER_44D();
   void FILTER_45A();
   void FILTER_45B();
   void FILTER_45C();
   void FILTER_45D();
   void FILTER_46A();
   void FILTER_46B();
   void FILTER_47A();
   void FILTER_47B();
   void FILTER_48A();
   void FILTER_48B();
   void FILTER_48C();
   void FILTER_48D();
   void FILTER_48E();
   void FILTER_48F();
   void FILTER_48G();
   void FILTER_48H();
   void FILTER_49A();
   void FILTER_49B();
   void FILTER_49C();
   void FILTER_49D();
   void FILTER_49E();
   void FILTER_49F();
   void FILTER_49G();
   void FILTER_49H();
   // Low-Pass Spatial Filters
   void LP1();
   void LP2();
   void LP3();
   // Hight-Pass Spatial Filters
   void HP1();
   void HP2();
   void HP3();
   //Shift and Difference Edge Enhancements
   void SDVerticalEdges();
   void SDHorizontalEdges();
   void SDHAndVEdges();
   // Matched Filter Edge Enhancements
   void MVerticalEdges();
   void MHorizontalEdges();
   // Gradient Diferenctional Edge Enhancemets
   void North();
   void NorthEast();
   void East();
   void SouthEast();
   void South();
   void SouthWest();
   void West();
   void NorthWest();
   // Laplace Edge Enhancemets
   void LAP1();
   void LAP2();
   void LAP3();
   void LAP4();
   // Line detection
   void LineHorizontal();
   void LineVertical();
   void Line_45();
   void Line_135();
   // Bluring Kernels
   void Blur3x3();
   void Blur5x5();
   // Sobel Masks
   void HorizontalSobel();
   void VerticalSobel();
   //---------------------------------------------------
   void SaveKernelAs(AnsiString& save_file);
   // overloaded operators
   Kernel& operator = (const Kernel& right);
}; // class Kernel

#endif
