/*
   Lut_base.h

   hlavikov soubor LUT tabulky - zkladn

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#ifndef lut_baseH
#define lut_baseH
//---------------------------------------------------------------------------
#include "lut.h"
#include "vpoint.h"

const int differ = 10;

class LUTBase
{
protected:
   TImage* Image, *ImageBack;
   TPanel* Panel;
   TCanvas* Canvas;
   LUT* lut;
   VPoint LeftTop, RightDown;
   //TLabel** labels;
   int width, lwidth;
   int height, lheight;
   bool down;
public:
   //constructors
   LUTBase();
   LUTBase(TImage* _Image, TPanel* _Panel);
   virtual ~LUTBase();
   // member functions
   int Width()  { return width;  }
   int Height() { return height; }
   // draws frame around the canvas
   void DrawFrame();
   void DrawWeb(TColor color=cl3DLight, TPenMode mode=pmCopy);
   void KeepInBorder(VPoint& point);
   void DrawLabels(TLabel** labels);
   // draws a single line from the point start to the point end
   void DrawLine(VPoint& start, VPoint& end,
                 TColor color = clWhite, int size = 2, TPenMode mode = pmXor,
                 TPenStyle style = psSolid);
   // draws a rectangle of the size sizexsize and fills it with color
   void DrawRect(VPoint& point, TColor color = clBlack, int size = 8);
   void MoveTo(int X, int Y);
   void LineTo(int X, int Y, TColor color = clBlack, int size = 2);
   // tests if the cursor of the mouse is beyond the line
   bool IsOnLine(VPoint& start, VPoint& end, int X, int Y, int tolerance);
   // tests if the cursor of the mouse is beyond the point 
   bool IsOnPoint(VPoint& point, int X, int Y, int tolerance);
   // returns the current cursor
   TCursor& Cursor()   { return Image->Cursor; }
   // returns LUT's value
   int& Lut(int index) { return lut->operator[](index); }
   bool& Down() { return down; }
   LUT& GetLUT() { return *lut; }
   // virtual functions
   virtual void DrawChart()=0;
   virtual void OnMouseDown(TMouseButton Button, int X, int Y)=0;
   virtual void OnMouseUp  (TMouseButton Button, int X, int Y)=0;
   virtual void OnMouseMove(int X, int Y)=0;
}; //class LUTBase
#endif
