/*
   histogram.h

   hlavikov soubor definujc zkladn operace pi prci s histogramem

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#ifndef histogramH
#define histogramH
//---------------------------------------------------------------------------
#include "image_grey.h"
#include "vpoint.h"


//---------------------------------------------------------------------------
// class Histogram for GreyLavels Images
// member functions for initialization, equalization, displaying
class Histogram
{
private:
   VPoint LeftTop, RightDown, Start, End;
   TCanvas* canvas;
   TImage* Image;
   long* hist;
   int levels;
   long max_value;
   int differ;
   TLabel**label;
   void DrawRect();
   void DrawLine(VPoint& start, VPoint& end, TColor color=clBlack,
                 int size=1, TPenMode mode=pmCopy);
   long MaxValue() { return max_value; }
public:
   // constructors
   Histogram();
   Histogram(int _levels);
   // destructor
   ~Histogram();
   // member function

   void InitFromGreyImage(const GreyImage& init, int col, int row,
                             int width, int height);
   void HistogramEqualization(const GreyImage& or_image,GreyImage& eq_image);
   void HistogramStretching(const GreyImage& or_image,GreyImage& st_image,float omission);
   void Display(TImage* Image);
   void DisplayEdge(TImage* Image);
   void DisplayWithComment(TImage* Image,TPanel* Panel);
   long Count(int _value) { return hist[_value]; }
   void LineOnly(TImage* _Image, int level);
   void CursorLine(TImage* _Image, int level);
private:
   // overloaded operators
   long& operator [] (int index) {
      if((index>=0)&&(index<MAX_QUANT_LEVELS)) return hist[index];
      return hist[0];
   }
}; // class Histogram
#endif
