/*
   list_image.h

   hlavikov soubor ablonovch td pro funkci Undo - list vzanch obraz

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#ifndef list_imageH
#define list_imageH
//---------------------------------------------------------------------------
const MAX_NUMBER_NODES = 10;
// class used for storing of a list of images - used for Undo
template<class T> class NodeI;
template<class T> class ListI;

//---------------------------------------------------------------------------
template<class T>
class NodeI
{
private:
   T            data;
   NodeI<T>*    next;
   NodeI<T>*    prev;
   NodeI();
   NodeI(const T& d);
   NodeI(const NodeI<T>& init);
public:
   friend ListI<T>;
}; // class NodeI

template<class T>
NodeI<T>::NodeI(const T& d)
{
   data = d;
}


template<class T>
NodeI<T>::NodeI(const NodeI<T>& init)
{
   data = init.data;
   next = init.next;
   prev = init.prev;
}

//---------------------------------------------------------------------------
template<class T>
class ListI
{
   int num_nodes;
   int max_number_nodes;
   NodeI<T>*   member;
public:
   // constructors
   ListI()
     : num_nodes(0), member(NULL),
       max_number_nodes(MAX_NUMBER_NODES) {}
   ListI(const ListI& init);
   ~ListI();
   //member functions
   T&   Member() { return member->data; }
   void Append(const T& obj);
   void Delete();
   void Reset();
   void Next();
   //void MaxNumber(int max_n);
   int  NumberNodes() { return num_nodes; }
   int& MaxNumberNodes() { return max_number_nodes; }
}; // class ListI

template<class T>
void ListI<T>::Append(const T& obj)
{
   NodeI<T>* ptr = new NodeI<T>(obj); // pointer to inserted node
   if(ptr==0)
      throw Exception("");
   if(num_nodes==0){
      ptr->next = ptr;
      ptr->prev = ptr;
      member = ptr;
      num_nodes++;
   }else if(num_nodes == max_number_nodes){
      ptr->prev = member;
      member->next->next->prev = ptr;
      ptr->next = member->next->next;
      delete (member->next);
      member->next = ptr;
      member = ptr;
   }else{
      ptr->prev = member;
      member->next->prev = ptr;
      ptr->next = member->next;
      member->next = ptr;
      member = ptr;
      num_nodes++;
   }
}

template<class T>
void ListI<T>::Delete()
{
   if(member->next!=member){
      member->prev->next = member->next;
      member->next->prev = member->prev;
      NodeI<T>* temp = member->prev;
      delete (member);
      member = temp;
      num_nodes--;
   }
}

template<class T>
void ListI<T>::Reset()
{
   while(num_nodes>1)
      Delete();
}

template<class T>
void ListI<T>::Next()
{
   if(member->next!=NULL){
      member = member->next;
   }
}

//template<class T>
//void ListI<T>::Maxnumber(int max_n)
//{
//   MAX_NUMBER_NODES = max_n;
//}

template<class T>
ListI<T>::~ListI()
{
   if(member!=NULL)
      while(member->next!=member){
         Delete();
      }
   delete(member);
}      
#endif
