/*
   point_image.h

   hlavikov soubor tdy s bodovmi operacemi

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#ifndef point_imageH
#define point_imageH
//---------------------------------------------------------------------------
#include "image_grey.h"
#include "vpoint.h"
#include "lut.h"
#include "histogram.h"
#include "brightness.h"

// Mostly uses histogram techniques
//---------------------------------------------------------------------------
class PointImage : public GreyImage
{
private:
   Histogram* histogram;
   Brightness* brightness;
public:
   PointImage()
     :GreyImage() {}
   PointImage(int _cols, int _rows);
   ~PointImage() {}
   // functions for drawing
   void DrawHLine(int col, int row,
                  int length, unsigned char grey);
   void DrawHLine(const VPoint& begin, const VPoint& end,
                  unsigned char grey);
   void DrawVLine(int col, int row,
                  int length, unsigned char grey);
   void DrawVLine(const VPoint& begin, const VPoint& end,
                  unsigned char grey);
   void DrawRect(const VPoint& begin, const VPoint& end,
                  unsigned char pixel_value);
   //  point transform functions
   void ApplyLUT(LUT& lut, int col, int row, int width, int height);
   void AdjImageBrightness(short brightness_factor, int col, int row,
                        int width, int height);
   void NegateImage(short treshold, int col, int row,
                        int width, int height);
   void TresholdImage(short treshold, int col, int row,
                        int width, int height);
   void GenHistogram(int col, int row, int width, int height);
   void DisplayHistogram(TImage* _Image);

   // overloaded functions
   PointImage& operator = (const GreyImage& right);
};
#endif
