/*
   GammaCorrectionUnit.cpp

   definin soubor okna pro gama korekci

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "GammaCorrectionUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TGammaCorrectionForm *GammaCorrectionForm;
//---------------------------------------------------------------------------
// Default constructor
__fastcall TGammaCorrectionForm::TGammaCorrectionForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   original_image = new GreyImage();
   sized_image = new GreyImage();
   modification_image = new GreyImage();

   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*original_image);
   Scale scale(*original_image, *sized_image, Image1->Width);

   handler->Assign(*sized_image);
   handler->Set(*modification_image);

   if (sized_image->Cols() > sized_image->Rows()){
     Image2->Top = Image2->Top + (sized_image->Cols()-sized_image->Rows())/2;
   }else if (sized_image->Rows() > sized_image->Cols()){
     Image2->Left = Image2->Left + (sized_image->Rows()-sized_image->Cols())/2;
   }

   handler->Show(Image1);

   in_1 = 0;
   in_2 = 255;
   out_1 = 0;
   out_2 = 255;
   gamma_level = 1.0;
   max_hist = 0;

   // Set transform to default, prepare histogram array
   for (int i = 0; i <= 255; i++){
     transform[i] = i;
     hist[i] = 0;
   }

   // Compute histogram
   for (int r = 0; r < sized_image->Rows(); r++){
     for (int  c = 0; c < sized_image->Cols(); c++){
       hist[sized_image->Data(c, r)] += 1;
     }
   }

   // find maximum size in histogram
   for (int i = 0; i <= 255; i++){
     max_hist = (max_hist < hist[i]) ? hist[i] : max_hist;
   }

   ComputePreview();

   StatusBar1->SimpleText = "You can change input, output and transfer function.";
}
//---------------------------------------------------------------------------
// Default destructor
__fastcall TGammaCorrectionForm::~TGammaCorrectionForm()
{
   delete (handler);
   delete (original_image);
   delete (sized_image);
   delete (modification_image);
}
//---------------------------------------------------------------------------
// Release memory before exit
void __fastcall TGammaCorrectionForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  Release();
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::UpDown1Click(TObject *Sender,
      TUDBtnType Button)
{
  Edit1->Text = UpDown1->Position;
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::UpDown2Click(TObject *Sender,
      TUDBtnType Button)
{
  Edit2->Text = UpDown2->Position;
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::UpDown3Click(TObject *Sender,
      TUDBtnType Button)
{
  Edit3->Text = UpDown3->Position;
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::UpDown4Click(TObject *Sender,
      TUDBtnType Button)
{
  Edit4->Text = UpDown4->Position;
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::TrackBar1Change(
      TObject *Sender)
{
  Timer1->Enabled = false;
  Edit5->Text = AnsiString((float)(TrackBar1->Position) / (float)(100));
  Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::CancelBitBtnClick(
      TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
// OK buttom
void __fastcall TGammaCorrectionForm::OKBitBtnClick(TObject *Sender)
{
  ComputeFinalImage();
  PictureForm->AssignList(original_image);
  Close();
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::Edit1Change(TObject *Sender)
{
   try{
      if(Edit1->Text=="") throw VEMsg("");

      in_1 = Edit1->Text.ToInt();

      if (in_1 < 0){
         Edit1->Text = 0;
         throw VEMsg("");
      }

      if (in_1 >= in_2){
         Edit1->Text = in_2;
         throw VEMsg("");
      }

      UpDown1->Position = Edit1->Text.ToInt();
      ComputeTransform();
      ComputePreview();

   }catch(EConvertError& e){
      MessageBox(Handle, e.Message.c_str(),
              "Histogram Modification", MB_ICONWARNING | MB_OK);
   }catch(VEMsg& e){
      ;// empty statemnet-just exit the function which is in a turn called
   }
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::Edit3Change(TObject *Sender)
{
      try{
      if(Edit3->Text=="") throw VEMsg("");

      out_1 = Edit3->Text.ToInt();

      if (out_1 < 0){
         Edit3->Text = 0;
         throw VEMsg("");
      }

      if (out_1 >= out_2){
         Edit3->Text = out_2;
         throw VEMsg("");
      }

      UpDown3->Position = Edit3->Text.ToInt();
      ComputeTransform();
      ComputePreview();

   }catch(EConvertError& e){
      MessageBox(Handle, e.Message.c_str(),
              "Histogram Modification", MB_ICONWARNING | MB_OK);
   }catch(VEMsg& e){
      ;// empty statemnet-just exit the function which is in a turn called
   }
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::Edit4Change(TObject *Sender)
{
      try{
      if(Edit4->Text=="") throw VEMsg("");

      out_2 = Edit4->Text.ToInt();

      if (out_2 > 255){
         Edit4->Text = 255;
         throw VEMsg("");
      }

      if (out_2 <= out_1){
         Edit4->Text = out_1;
         throw VEMsg("");
      }

      UpDown4->Position = Edit4->Text.ToInt();
      ComputeTransform();
      ComputePreview();

   }catch(EConvertError& e){
      MessageBox(Handle, e.Message.c_str(),
              "Histogram Modification", MB_ICONWARNING | MB_OK);
   }catch(VEMsg& e){
      ;// empty statemnet-just exit the function which is in a turn called
   }
}
//---------------------------------------------------------------------------
// Compute modification image
void __fastcall TGammaCorrectionForm::ComputePreview()
{
  for (int r = 0; r < sized_image->Rows(); r++){
     for (int  c = 0; c < sized_image->Cols(); c++){
        modification_image->Data(c, r)  = (unsigned char)(transform[sized_image->Data(c, r)]);
     }
  }

  handler->Assign(*modification_image);
  handler->Show(Image2);
  ShowHistogram();
  ShowModifHistogram();
}
//---------------------------------------------------------------------------
// Compute final image for movement to main window
void __fastcall TGammaCorrectionForm::ComputeFinalImage()
{
  for (int r = 0; r < original_image->Rows(); r++){
     for (int  c = 0; c < original_image->Cols(); c++){
        original_image->Data(c, r)  = (unsigned char)(transform[original_image->Data(c, r)]);
     }
  }
}
//---------------------------------------------------------------------------
// Make transformation
void __fastcall TGammaCorrectionForm::ComputeTransform()
{
   float step, actually, after_gamma;
   step = ((float)(out_2 - out_1)) / ((float)(in_2 - in_1));
   actually = (float)(0);

   // 0's before in_min
   for (int i = 0; i < in_1; i++){
      transform[i] = out_1;
   }

   // 255's after in_max
   for (int i = in_2 + 1; i <= 255; i++){
      transform[i] = out_2;
   }

   // between in_min and in_max
   for (int i = in_1; i <= in_2; i++){
      // pow - mocnina
      after_gamma = (float)(out_2 - out_1) *
              (float)pow(actually/(float)(out_2 - out_1), gamma_level);
      // movement
      transform[i] = (unsigned char)(after_gamma + out_1);
      actually += step;
   }
}
//---------------------------------------------------------------------------
// Show histogram and perform-path
void __fastcall TGammaCorrectionForm::ShowHistogram()
{
   // Clear
   Image3->Canvas->Pen->Color = clWhite;
   Image3->Canvas->Brush->Color = clWhite;
   Image3->Canvas->Rectangle(1, 1, Image3->Width, Image3->Height);

   // Show histogram
   Image3->Canvas->Pen->Color = clGray;
   Image3->Canvas->Pen->Width = 1;

   for (int i = 0; i <= 255; i++){
      Image3->Canvas->MoveTo(i+1,128);
      // Normalize and draw histogram
      Image3->Canvas->LineTo(i+1, 128 - ((float)(128 * ((float)(hist[i]) / (float)(max_hist)))));
   }

   // Show Specific Transform
   Image3->Canvas->Pen->Color = clBlack;
   Image3->Canvas->Pen->Width = 2;
   Image3->Canvas->MoveTo(1, 128 - transform[0]/2);

   for (int i = 2; i <= 256; i++){
       Image3->Canvas->LineTo(i, 128 - transform[i-1]/2);
   }
}
//-----------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::ResetButtonClick(
      TObject *Sender)
{
   Edit1->Text = 0;
   Edit2->Text = 255;
   Edit3->Text = 0;
   Edit4->Text = 255;
   TrackBar1->Position = 100;
}
//---------------------------------------------------------------------------
// Show modificated histogram
void __fastcall TGammaCorrectionForm::ShowModifHistogram()
{
   max_modif_hist = 0;

   // Prepare modificated histogram array
   for (int i = 0; i <= 255; i++){
     modif_hist[i] = 0;
   }

   // Compute modificated histogram
   for (int r = 0; r < sized_image->Rows(); r++){
     for (int  c = 0; c < sized_image->Cols(); c++){
       modif_hist[modification_image->Data(c, r)] += 1;
     }
   }

   // find maximum size in modificated histogram
   for (int i = 0; i <= 255; i++){
     max_modif_hist = (max_modif_hist < modif_hist[i]) ? modif_hist[i] : max_modif_hist;
   }

   // Clear
   Image4->Canvas->Pen->Color = clWhite;
   Image4->Canvas->Brush->Color = clWhite;
   Image4->Canvas->Rectangle(1, 1, Image3->Width, Image3->Height);

   // Show modificated histogram
   Image4->Canvas->Pen->Color = clGray;
   Image4->Canvas->Pen->Width = 1;

   for (int i = 0; i <= 255; i++){
      Image4->Canvas->MoveTo(i+1,128);
      // Normalize and draw modificated histogram
      Image4->Canvas->LineTo(i+1, 128 - ((float)(128 * ((float)(modif_hist[i])/(float)(max_modif_hist)))));
   }
}
//---------------------------------------------------------------------------
// delay 0.05s before compute transformation
void __fastcall TGammaCorrectionForm::Timer1Timer(TObject *Sender)
{
  Timer1->Enabled = false;
  gamma_level = (float)(TrackBar1->Position) / (float)(100);
  ComputeTransform();
  ComputePreview();        
}
//---------------------------------------------------------------------------
void __fastcall TGammaCorrectionForm::Edit2Change(TObject *Sender)
{
      try{
      if(Edit2->Text=="") throw VEMsg("");

      in_2 = Edit2->Text.ToInt();

      if (in_2 > 255){
         Edit2->Text = 255;
         throw VEMsg("");
      }

      if (in_2 <= in_1){
         Edit2->Text = in_1;
         throw VEMsg("");
      }

      UpDown2->Position = Edit2->Text.ToInt();
      ComputeTransform();
      ComputePreview();

   }catch(EConvertError& e){
      MessageBox(Handle, e.Message.c_str(),
              "Histogram Modification", MB_ICONWARNING | MB_OK);
   }catch(VEMsg& e){
      ;// empty statemnet-just exit the function which is in a turn called
   }        
}
//---------------------------------------------------------------------------

