/*
   HistogramEqualizationUnit.cpp

   definin soubor okna pro ekvalizaci histogramu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "HistogramEqualizationUnit.h"
#include "DrawUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
THistogramEqualizationForm *HistogramEqualizationForm;
//---------------------------------------------------------------------------
__fastcall THistogramEqualizationForm::THistogramEqualizationForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   original_image = new GreyImage();
   equailized_image = new GreyImage();
   sized_original_image = new GreyImage();
   sized_equailized_image = new GreyImage();

   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*original_image);

   // Original image
   Scale scale1(*original_image, *sized_original_image, Image1->Width);
   handler->Assign(*sized_original_image);
   handler->Show(Image1);

   // Original histogram
   histogram1.InitFromGreyImage(*original_image,0,0,original_image->Cols(),original_image->Rows());
   histogram1.Display(Image3);

   // Equalized histogram
   histogram2.InitFromGreyImage(*original_image,0,0,original_image->Cols(),original_image->Rows());
   histogram2.HistogramEqualization(*original_image,*equailized_image);
   histogram2.InitFromGreyImage(*equailized_image,0,0,equailized_image->Cols(),equailized_image->Rows());
   histogram2.Display(Image4);

   // Image after Equalization
   Scale scale2(*equailized_image, *sized_equailized_image, Image2->Width);
   handler->Assign(*sized_equailized_image);
   handler->Show(Image2);
   last1 = 0;
   last2 = 0;
}
//---------------------------------------------------------------------------
__fastcall THistogramEqualizationForm::~THistogramEqualizationForm()
{
   delete (handler);
   delete (original_image);
   delete (equailized_image);
   delete (sized_original_image);
   delete (sized_equailized_image);
}
//---------------------------------------------------------------------------
void __fastcall THistogramEqualizationForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  Release();
}
//---------------------------------------------------------------------------
void __fastcall THistogramEqualizationForm::BitBtnOkClick(TObject *Sender)
{
  PictureForm->AssignList(equailized_image);
  Close();
}
//---------------------------------------------------------------------------
void __fastcall THistogramEqualizationForm::Image3MouseDown(
      TObject *Sender, TMouseButton Button, TShiftState Shift, int X,
      int Y)
{
   DrawForm->OnMouseDown(Button, X, Y);        
}
//---------------------------------------------------------------------------
void __fastcall THistogramEqualizationForm::Image4MouseDown(
      TObject *Sender, TMouseButton Button, TShiftState Shift, int X,
      int Y)
{
   DrawForm->OnMouseDown(Button, X, Y);        
}
//---------------------------------------------------------------------------
void __fastcall THistogramEqualizationForm::Image3MouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   DrawForm->OnMouseUp(Button, X, Y);         
}
//---------------------------------------------------------------------------
void __fastcall THistogramEqualizationForm::Image4MouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   DrawForm->OnMouseUp(Button, X, Y);         
}
//---------------------------------------------------------------------------
void __fastcall THistogramEqualizationForm::Image3MouseMove(
      TObject *Sender, TShiftState Shift, int X, int Y)
{
  DrawForm->OnMouseMove(X, Y);
  histogram2.LineOnly(Image4, last2);
  if ((10 <= X)&&(X <= 265)&&(10 <= Y)&&(Y <= Panel4->Height-12)){
    histogram1.LineOnly(Image3, last1);
    histogram1.CursorLine(Image3, X-10);
    last1 = X-10;
    StatusBar1->Panels->Items[0]->Text = "Pixel value: " + AnsiString(X-10);
    StatusBar1->Panels->Items[1]->Text = "Count: " + AnsiString(histogram1.Count(X-10));
    StatusBar1->Panels->Items[2]->Text = "Original histogram";
  }else{
    histogram1.LineOnly(Image3, last1);
    StatusBar1->Panels->Items[0]->Text = "";
    StatusBar1->Panels->Items[1]->Text = "";
    StatusBar1->Panels->Items[2]->Text = "";
  }
}
//---------------------------------------------------------------------------
void __fastcall THistogramEqualizationForm::Image4MouseMove(
      TObject *Sender, TShiftState Shift, int X, int Y)
{
  DrawForm->OnMouseMove(X, Y);
  histogram1.LineOnly(Image3, last1);
  if ((10 <= X)&&(X <= 265)&&(10 <= Y)&&(Y <= Panel5->Height-12)){
    histogram2.LineOnly(Image4, last2);
    histogram2.CursorLine(Image4, X-10);
    last2 = X-10;
    StatusBar1->Panels->Items[0]->Text = "Pixel value: " + AnsiString(X-10);
    StatusBar1->Panels->Items[1]->Text = "Count: " + AnsiString(histogram2.Count(X-10));
    StatusBar1->Panels->Items[2]->Text = "Histogram after Equalization";
  }else{
    histogram2.LineOnly(Image4, last2);
    StatusBar1->Panels->Items[0]->Text = "";
    StatusBar1->Panels->Items[1]->Text = "";
    StatusBar1->Panels->Items[2]->Text = "";
  }
}
//---------------------------------------------------------------------------
void __fastcall THistogramEqualizationForm::Panel1MouseMove(
      TObject *Sender, TShiftState Shift, int X, int Y)
{
   histogram1.LineOnly(Image3, last1);
   histogram2.LineOnly(Image4, last2);
   StatusBar1->Panels->Items[0]->Text = "";
   StatusBar1->Panels->Items[1]->Text = "";
   StatusBar1->Panels->Items[2]->Text = "";       
}
//---------------------------------------------------------------------------

void __fastcall THistogramEqualizationForm::Image1MouseMove(
      TObject *Sender, TShiftState Shift, int X, int Y)
{
   histogram1.LineOnly(Image3, last1);
   histogram2.LineOnly(Image4, last2);
   StatusBar1->Panels->Items[0]->Text = "";
   StatusBar1->Panels->Items[1]->Text = "";
   StatusBar1->Panels->Items[2]->Text = ""; 
}
//---------------------------------------------------------------------------

void __fastcall THistogramEqualizationForm::Image2MouseMove(
      TObject *Sender, TShiftState Shift, int X, int Y)
{
   histogram1.LineOnly(Image3, last1);
   histogram2.LineOnly(Image4, last2);
   StatusBar1->Panels->Items[0]->Text = "";
   StatusBar1->Panels->Items[1]->Text = "";
   StatusBar1->Panels->Items[2]->Text = ""; 
}
//---------------------------------------------------------------------------




