/*
   HistogramModificationUnit.cpp

   definin soubor okna pro modifikaci histogramu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "HistogramModificationUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
THistogramModificationForm *HistogramModificationForm;
//---------------------------------------------------------------------------
__fastcall THistogramModificationForm::THistogramModificationForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   original_image = new GreyImage();
   sized_image = new GreyImage();
   modification_image = new GreyImage();

   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*original_image);
   Scale scale(*original_image, *sized_image, Image1->Width);

   handler->Assign(*sized_image);
   handler->Set(*modification_image);

   if (sized_image->Cols() > sized_image->Rows()){
     Image2->Top = Image2->Top + (sized_image->Cols()-sized_image->Rows())/2;
   }else if (sized_image->Rows() > sized_image->Cols()){
     Image2->Left = Image2->Left + (sized_image->Rows()-sized_image->Cols())/2;
   }

   handler->Show(Image1);

   // Prepare histogram array's
   for (int i = 0; i < 256; i++){
      hist_original[i] = 0;
   }

   // Compute input histogram
   for (int r = 0; r < sized_image->Rows(); r++){
     for (int  c = 0; c < sized_image->Cols(); c++){
       hist_original[sized_image->Data(c, r)] += 1;
     }
   }

   num_pixel = (long int)(sized_image->Rows()*sized_image->Cols());
   // Compute cumulative input histogram
   long int temp_int = 0;
   for (int i = 0; i < 256; i++){
       temp_int += (long int)(hist_original[i]);
       cum_hist_original[i] = (float)(temp_int) / (float)(num_pixel);
       cum_hist_original[i] = (cum_hist_original[i] <= 0.001)? (float)(0.001) : cum_hist_original[i];
       cum_hist_original[i] = (cum_hist_original[i] >= 0.999)? (float)(0.999) : cum_hist_original[i];
   }

   // find maximum size in input histogram
   max_hist = 0;
   for (int i = 0; i < 256; i++){
     max_hist = (max_hist < hist_original[i]) ? hist_original[i] : max_hist;
   }

   amp_index = 100;
   gmin = (float)(1);
   gmax = (float)(255);
   alfa = (float)(TrackBar1->Position) / (float)(amp_index);

   Edit5->Text = AnsiString((float)(TrackBar1->Position) / (float)(amp_index));
   ShowOriginalHistogram();
   StatusBar1->SimpleText = "Select Transfer function.";
}
//---------------------------------------------------------------------------
__fastcall THistogramModificationForm::~THistogramModificationForm()
{
   delete (handler);
   delete (original_image);
   delete (sized_image);
   delete (modification_image);
}
//---------------------------------------------------------------------------
void __fastcall THistogramModificationForm::ComboBox1Change(
      TObject *Sender)
{
   Timer1->Enabled = false;
   switch(ComboBox1->ItemIndex){
   case(0):
      Label1->Visible = false;
      Edit5->Visible = false;
      TrackBar1->Visible = false;
      UniformTransform();
      break;
   case(1):
      amp_index = 100;
      Edit5->Text = AnsiString((float)(TrackBar1->Position) / (float)(amp_index));
      Label1->Visible = true;
      Edit5->Visible = true;
      TrackBar1->Visible = true;
      ExponencialTransform();
      break;
   case(2):
      amp_index = 1;
      Edit5->Text = AnsiString((float)(TrackBar1->Position) / (float)(amp_index));
      Label1->Visible = true;
      Edit5->Visible = true;
      TrackBar1->Visible = true;
      RayleighTransform();
      break;
   case(3):
      Label1->Visible = false;
      Edit5->Visible = false;
      TrackBar1->Visible = false;
      CubeRootTransform();
      break;
   case(4):
      Label1->Visible = false;
      Edit5->Visible = false;
      TrackBar1->Visible = false;
      LogarithmizeTransform();
      break;
   }
   ComputePreviewImage();
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall THistogramModificationForm::UniformTransform()
{
  double reg_int;
  for (int i = 0; i <= 255; i++) {
     reg_int = (double)(gmax - gmin)*cum_hist_original[i] + (gmin);
     reg_int = (reg_int < 0)? 0 : reg_int;
     reg_int = (reg_int > 255)? 255 : reg_int;
     transform[i] = (int)(reg_int);
  }
}
//---------------------------------------------------------------------------
void __fastcall THistogramModificationForm::ExponencialTransform()
{
  double reg_int;
  for (int i = 0; i <= 255; i++) {
     reg_int = (gmin)-((double)(1)/alfa)*log(pow(((double)(1)-cum_hist_original[i]),(double)(1)/alfa));
     reg_int = (reg_int < 0)? 0 : reg_int;
     reg_int = (reg_int > 255)? 255 : reg_int;
     transform[i] = (unsigned char)(reg_int);
  }
}
//---------------------------------------------------------------------------
void __fastcall THistogramModificationForm::RayleighTransform()
{
  double reg_int;
  for (int i = 0; i <= 255; i++) {
     reg_int = (int)((double)(gmin) + pow(((double)(2)*pow(alfa,(double)(2))*(double)(log((double)(1)/((double)(1)-cum_hist_original[i])))),(double)(0.5)));
     reg_int = (reg_int < 0)? 0 : reg_int;
     reg_int = (reg_int > 255)? 255 : reg_int;
     transform[i] = (unsigned char)(reg_int);
  }
}
//---------------------------------------------------------------------------
void __fastcall THistogramModificationForm::CubeRootTransform()
{
  double reg_int;
  for (int i = 0; i <= 255; i++) {
     reg_int = pow((pow(gmax,0.333)-pow(gmin,0.333))*(cum_hist_original[i])+pow(gmin,0.333),3);
     reg_int = (reg_int < 0)? 0 : reg_int;
     reg_int = (reg_int > 255)? 255 : reg_int;
     transform[i] = (unsigned char)(reg_int);
  }
}
//---------------------------------------------------------------------------
void __fastcall THistogramModificationForm::LogarithmizeTransform()
{
  double reg_int;
  for (int i = 0; i <= 255; i++) {
     reg_int = gmin*pow((gmax/gmin),cum_hist_original[i]);
     reg_int = (reg_int < 0)? 0 : reg_int;
     reg_int = (reg_int > 255)? 255 : reg_int;
     transform[i] = (unsigned char)(reg_int);
  }
}
//---------------------------------------------------------------------------
void __fastcall THistogramModificationForm::ComputePreviewImage()
{
  for (int r = 0; r < sized_image->Rows(); r++){
     for (int  c = 0; c < sized_image->Cols(); c++){
        modification_image->Data(c, r)  = (unsigned char)(transform[sized_image->Data(c, r)]);
     }
  }
  handler->Assign(*modification_image);
  handler->Show(Image2);
  ShowModifHistogram();
  if (OKBitBtn->Enabled == false){
      OKBitBtn->Enabled = true;
  }    
}
//---------------------------------------------------------------------------
// Show modificated histogram
void __fastcall THistogramModificationForm::ShowModifHistogram()
{
   // Prepare modificated histogram array
   for (int i = 0; i <= 255; i++){
     hist_modif[i] = 0;
   }

   // Compute modificated histogram
   for (int r = 0; r < sized_image->Rows(); r++){
     for (int  c = 0; c < sized_image->Cols(); c++){
       hist_modif[modification_image->Data(c, r)] += 1;
     }
   }

   // find maximum size in modificated histogram
   max_modif_hist = 0;
   for (int i = 0; i <= 255; i++){
     max_modif_hist = (max_modif_hist < hist_modif[i]) ? hist_modif[i] : max_modif_hist;
   }

   // Clear
   Image4->Canvas->Pen->Color = clWhite;
   Image4->Canvas->Brush->Color = clWhite;
   Image4->Canvas->Rectangle(1, 1, Image3->Width, Image3->Height);

   // Show modificated histogram
   Image4->Canvas->Pen->Color = clGray;
   Image4->Canvas->Pen->Width = 1;

   for (int i = 0; i <= 255; i++){
      Image4->Canvas->MoveTo(i+1,128);
      // Normalize and draw modificated histogram
      Image4->Canvas->LineTo(i+1, 128 - ((float)(128 * ((float)(hist_modif[i])/(float)(max_modif_hist)))));
   }
}
//---------------------------------------------------------------------------
// Show original histogram
void __fastcall THistogramModificationForm::ShowOriginalHistogram()
{

   // Clear
   Image3->Canvas->Pen->Color = clWhite;
   Image3->Canvas->Brush->Color = clWhite;
   Image3->Canvas->Rectangle(1, 1, Image3->Width, Image3->Height);

   // Show modificated histogram
   Image3->Canvas->Pen->Color = clGray;
   Image3->Canvas->Pen->Width = 1;

   for (int i = 0; i <= 255; i++){
      Image3->Canvas->MoveTo(i+1,128);
      // Normalize and draw modificated histogram
      Image3->Canvas->LineTo(i+1, 128 - ((float)(128 * ((float)(hist_original[i])/(float)(max_hist)))));
   }
}
//---------------------------------------------------------------------------
void __fastcall THistogramModificationForm::TrackBar1Change(
      TObject *Sender)
{
   Timer1->Enabled = false;
   Edit5->Text = AnsiString((float)(TrackBar1->Position) / (float)(amp_index));
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
// Compute any function
void __fastcall THistogramModificationForm::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   alfa = (float)(TrackBar1->Position) / (float)(amp_index);

   switch(ComboBox1->ItemIndex){
   case(0):
      UniformTransform();
      break;
   case(1):
      amp_index = 100;
      ExponencialTransform();
      break;
   case(2):
      amp_index = 1;
      RayleighTransform();
      break;
   case(3):
      CubeRootTransform();
      break;
   case(4):
      LogarithmizeTransform();
      break;
   }
   
   ComputePreviewImage();
}
//---------------------------------------------------------------------------
// Release after Close
void __fastcall THistogramModificationForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  Release();        
}
//---------------------------------------------------------------------------
// OK Button
void __fastcall THistogramModificationForm::OKBitBtnClick(TObject *Sender)
{
  ComputeFinalImage();
  PictureForm->AssignList(original_image);
  Close();
}
//---------------------------------------------------------------------------
// Compute final image for movement to main window
void __fastcall THistogramModificationForm::ComputeFinalImage()
{
  for (int r = 0; r < original_image->Rows(); r++){
     for (int  c = 0; c < original_image->Cols(); c++){
        original_image->Data(c, r)  = (unsigned char)(transform[original_image->Data(c, r)]);
     }
  }
}

