/*
   HistogramSpecificationUnit.cpp

   definin soubor okna pro specifikaci histogramu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "HistogramSpecificationUnit.h"
#include "PictureUnit.h"
#include "TypeHistogramUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
THistogramSpecificationForm *HistogramSpecificationForm;
//---------------------------------------------------------------------------
// Default constructor
__fastcall THistogramSpecificationForm::THistogramSpecificationForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   original_lut = new LUT();

   original_image = new GreyImage();
   sized_image = new GreyImage();
   modification_image = new GreyImage();

   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*original_image);
   Scale scale(*original_image, *sized_image, Image1->Width);

   handler->Assign(*sized_image);
   handler->Set(*modification_image);

   if (sized_image->Cols() > sized_image->Rows()){
     Image2->Top = Image2->Top + (sized_image->Cols()-sized_image->Rows())/2;
   }else if (sized_image->Rows() > sized_image->Cols()){
     Image2->Left = Image2->Left + (sized_image->Rows()-sized_image->Cols())/2;
   }

   handler->Show(Image1);

   // Prepare histogram array's
   for (int i = 0; i < 256; i++){
      hist_original[i] = 0;
      hist_required[i] = 100;
   }
   NormalizeSurface();

   // Compute input histogram
   for (int r = 0; r < sized_image->Rows(); r++){
     for (int  c = 0; c < sized_image->Cols(); c++){
       hist_original[sized_image->Data(c, r)] += 1;
     }
   }

   // Compute cumulative input histogram
   long int temp_int = 0;
    for (int i = 0; i < 256; i++){
       temp_int += (long int)hist_original[i];
       // cum_hist_original[i] = (long int)(temp_int/(sized_image->Cols()*sized_image->Rows()));
       cum_hist_original[i] = (long int)temp_int;
   }

   // find maximum size in input histogram
   for (int i = 0; i < 256; i++){
     max_hist = (max_hist < hist_original[i]) ? hist_original[i] : max_hist;
   }

   StatusBar1->SimpleText = "Click to Required Histogram button.";

   ComputePreview();
}
//---------------------------------------------------------------------------
// Default destructor
__fastcall THistogramSpecificationForm::~THistogramSpecificationForm()
{
   delete (handler);
   delete (original_image);
   delete (sized_image);
   delete (modification_image);
   delete (original_lut);
}
//---------------------------------------------------------------------------
void __fastcall THistogramSpecificationForm::OKBitBtnClick(TObject *Sender)
{
  ComputeFinalImage();
  PictureForm->AssignList(original_image);
  Close();
}
//---------------------------------------------------------------------------

void __fastcall THistogramSpecificationForm::CancelBitBtnClick(TObject *Sender)
{
   Close();        
}
//---------------------------------------------------------------------------
// Compute modification image
void __fastcall THistogramSpecificationForm::ComputePreview()
{
  ComputeTransform();
  
  for (int r = 0; r < sized_image->Rows(); r++){
     for (int  c = 0; c < sized_image->Cols(); c++){
        modification_image->Data(c, r)  = (unsigned char)(transform[sized_image->Data(c, r)]);
     }
  }

  handler->Assign(*modification_image);
  handler->Show(Image2);
  ShowHistogram();
  ShowModifHistogram();
}
//---------------------------------------------------------------------------
// Show histogram and perform-path
void __fastcall THistogramSpecificationForm::ShowHistogram()
{
   // Clear
   Image3->Canvas->Pen->Color = clWhite;
   Image3->Canvas->Brush->Color = clWhite;
   Image3->Canvas->Rectangle(1, 1, Image3->Width, Image3->Height);

   // Show histogram
   Image3->Canvas->Pen->Color = clGray;
   Image3->Canvas->Pen->Width = 1;

   for (int i = 0; i <= 255; i++){
      Image3->Canvas->MoveTo(i+1,128);
      // Normalize and draw histogram
      Image3->Canvas->LineTo(i+1, 128 - ((float)(128 * ((float)(hist_original[i]) / (float)(max_hist)))));
   }
}
//---------------------------------------------------------------------------
// Show modificated histogram
void __fastcall THistogramSpecificationForm::ShowModifHistogram()
{
   max_modif_hist = 0;

   // Prepare modificated histogram array
   for (int i = 0; i <= 255; i++){
     hist_modif[i] = 0;
   }

   // Compute modificated histogram
   for (int r = 0; r < sized_image->Rows(); r++){
     for (int  c = 0; c < sized_image->Cols(); c++){
       hist_modif[modification_image->Data(c, r)] += 1;
     }
   }

   // find maximum size in modificated histogram
   for (int i = 0; i <= 255; i++){
     max_modif_hist = (max_modif_hist < hist_modif[i]) ? hist_modif[i] : max_modif_hist;
   }

   // Clear
   Image4->Canvas->Pen->Color = clWhite;
   Image4->Canvas->Brush->Color = clWhite;
   Image4->Canvas->Rectangle(1, 1, Image3->Width, Image3->Height);

   // Show modificated histogram
   Image4->Canvas->Pen->Color = clGray;
   Image4->Canvas->Pen->Width = 1;

   for (int i = 0; i <= 255; i++){
      Image4->Canvas->MoveTo(i+1,128);
      // Normalize and draw modificated histogram
      Image4->Canvas->LineTo(i+1, 128 - ((float)(128 * ((float)(hist_modif[i])/(float)(max_modif_hist)))));
   }

   // show specific histogram line
   Image4->Canvas->Pen->Color = clBlack;
   Image4->Canvas->Pen->Width = 2;
   Image4->Canvas->MoveTo(1, 128 - hist_required[0]/2);

   for (int i = 2; i <= 256; i++){
       Image4->Canvas->LineTo(i, 128 - hist_required[i-1]/2);
   }
}
//---------------------------------------------------------------------------
void __fastcall THistogramSpecificationForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
  Release();        
}
//---------------------------------------------------------------------------
// Compute final image for movement to main window
void __fastcall THistogramSpecificationForm::ComputeFinalImage()
{
  for (int r = 0; r < original_image->Rows(); r++){
     for (int  c = 0; c < original_image->Cols(); c++){
        original_image->Data(c, r)  = (unsigned char)(transform[original_image->Data(c, r)]);
     }
  }
}
//---------------------------------------------------------------------------
// Make transformation
void __fastcall THistogramSpecificationForm::ComputeTransform()
{
   // Compute cumulative required histogram
   long int temp_int = 0;
    for (int i = 0; i < 256; i++){
       temp_int += (long int)hist_required[i];
       // cum_hist_required[i] = (long int)(temp_int/(sized_image->Cols()*sized_image->Rows()));
       cum_hist_required[i] = (long int)temp_int;
   }

   // Compute required transformation array
   long int find_int;
   int reg_int;
   for (int t = 0; t < 256; t++){
      find_int =  (long int)cum_hist_original[t];
      reg_int = 0;
      while (cum_hist_required[reg_int] < cum_hist_original[t]){
         reg_int++;
      }
      reg_int = (reg_int < 0)? 0 : reg_int;
      reg_int = (reg_int > 255)? 255 : reg_int;
      transform[t] = reg_int;
   }
}
//---------------------------------------------------------------------------
void __fastcall THistogramSpecificationForm::Button1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TTypeHistogramForm),
                          &TypeHistogramForm);
   TypeHistogramForm->Top = Top + Height + 3;
   TypeHistogramForm->Left = Left;
   TypeHistogramForm -> ShowModal();
}
//---------------------------------------------------------------------------
// transform LUT -> array of integer
void __fastcall THistogramSpecificationForm::SetData(LUT* lut)
{
   *original_lut = *lut;
   int temp_i;
   for (int i = 0; i < 256; i++){
      hist_required[i] = original_lut->GetInt(i);
   }
   NormalizeSurface();
   ComputePreview();
}
//---------------------------------------------------------------------------
// Surface must be always rows*cols
void __fastcall THistogramSpecificationForm::NormalizeSurface()
{
   // Find surface
   int surface = 0;
   float const_surface;
   for (int i = 0; i < 256; i++){
      surface += hist_required[i];
   }

   // Compute proportion
   if (surface == 0){
      for (int i = 0; i < 256; i++){ hist_required[i] = 1; }
   }else{
      const_surface = ((float)(sized_image->Cols() * sized_image->Rows()) / (float)(surface));
   }

   // Recalculating
   for (int i = 0; i < 256; i++){
      hist_required[i] = (int)((float)(hist_required[i]) * const_surface);
   }
}



