/*
   HistogramStretchingUnit.cpp

   definin soubor okna pro funkci roztaen histogramu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "HistogramStretchingUnit.h"
#include "DrawUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
THistogramStrechingForm *HistogramStrechingForm;
//---------------------------------------------------------------------------
__fastcall THistogramStrechingForm::THistogramStrechingForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   original_image = new GreyImage();
   stretched_image = new GreyImage();
   sized_original_image = new GreyImage();
   sized_stretched_image = new GreyImage();

   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*original_image);

   // Original image
   Scale scale1(*original_image, *sized_original_image, Image1->Width);
   handler->Assign(*sized_original_image);
   handler->Show(Image1);

   if (sized_original_image->Cols() > sized_original_image->Rows()){
     Image2->Top=Image2->Top + (sized_original_image->Cols()-sized_original_image->Rows())/2;
   }else if (sized_original_image->Rows() > sized_original_image->Cols()){
     Image2->Left=Image2->Left + (sized_original_image->Rows()-sized_original_image->Cols())/2;
   }

   // Original histogram
   histogram1.InitFromGreyImage(*original_image,0,0,original_image->Cols(),original_image->Rows());
   histogram1.Display(Image3);

   last1 = -1;
   last2 = -1;
   ComputeIt();
   Image4->Enabled = true;
}
//---------------------------------------------------------------------------
__fastcall THistogramStrechingForm::~THistogramStrechingForm()
{
   delete (handler);
   delete (original_image);
   delete (stretched_image);
   delete (sized_original_image);
   delete (sized_stretched_image);
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::n(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  DrawForm->OnMouseDown(Button, X, Y);         
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::Image4MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   DrawForm->OnMouseDown(Button, X, Y);         
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::Image3MouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   DrawForm->OnMouseUp(Button, X, Y);         
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::Image4MouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   DrawForm->OnMouseUp(Button, X, Y);         
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::Image3MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  DrawForm->OnMouseMove(X, Y);
  if (last2 != -1) histogram2.LineOnly(Image4, last2);
  if ((10 <= X)&&(X <= 265)&&(10 <= Y)&&(Y <= Panel4->Height-12)){
    if (last1 != -1) histogram1.LineOnly(Image3, last1);
    histogram1.CursorLine(Image3, X-10);
    last1 = X-10;
    StatusBar1->Panels->Items[0]->Text = "Pixel value: " + AnsiString(X-10);
    StatusBar1->Panels->Items[1]->Text = "Count: " + AnsiString(histogram1.Count(X-10));
    StatusBar1->Panels->Items[2]->Text = "Original histogram";
  }else{
    if (last1 != -1) histogram1.LineOnly(Image3, last1);
    StatusBar1->Panels->Items[0]->Text = "";
    StatusBar1->Panels->Items[1]->Text = "";
    StatusBar1->Panels->Items[2]->Text = "";
  }        
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::Image3MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   DrawForm->OnMouseDown(Button, X, Y);         
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::Image4MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  DrawForm->OnMouseMove(X, Y);
  if (last2 != -1) histogram1.LineOnly(Image3, last1);
  if ((10 <= X)&&(X <= 265)&&(10 <= Y)&&(Y <= Panel5->Height-12)){
    if (last2 != -1) histogram2.LineOnly(Image4, last2);
    histogram2.CursorLine(Image4, X-10);
    last2 = X-10;
    StatusBar1->Panels->Items[0]->Text = "Pixel value: " + AnsiString(X-10);
    StatusBar1->Panels->Items[1]->Text = "Count: " + AnsiString(histogram2.Count(X-10));
    StatusBar1->Panels->Items[2]->Text = "Histogram after Stretching";
  }else{
    if (last2 != -1) histogram2.LineOnly(Image4, last2);
    StatusBar1->Panels->Items[0]->Text = "";
    StatusBar1->Panels->Items[1]->Text = "";
    StatusBar1->Panels->Items[2]->Text = "";
  }
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::BitBtnOkClick(TObject *Sender)
{
  PictureForm->AssignList(stretched_image);
  Close();
}
//---------------------------------------------------------------------------  
void __fastcall THistogramStrechingForm::Image1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   if (last1 != -1) histogram1.LineOnly(Image3, last1);
   if (last2 != -1) histogram2.LineOnly(Image4, last2);
   StatusBar1->Panels->Items[0]->Text = "";
   StatusBar1->Panels->Items[1]->Text = "";
   StatusBar1->Panels->Items[2]->Text = ""; 
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::Image2MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   if (last1 != -1) histogram1.LineOnly(Image3, last1);
   if (last2 != -1) histogram2.LineOnly(Image4, last2);
   StatusBar1->Panels->Items[0]->Text = "";
   StatusBar1->Panels->Items[1]->Text = "";
   StatusBar1->Panels->Items[2]->Text = "";
}
//---------------------------------------------------------------------------
__fastcall THistogramStrechingForm::ComputeIt()
{
   // Histogram after Stretching
   histogram2.HistogramStretching(*original_image,*stretched_image,omis_level);
   histogram2.InitFromGreyImage(*stretched_image,0,0,stretched_image->Cols(),stretched_image->Rows());
   histogram2.Display(Image4);

   // Resulting Image
   Scale scale2(*stretched_image, *sized_stretched_image, Image2->Width);
   handler->Assign(*sized_stretched_image);
   handler->Show(Image2);
}
//---------------------------------------------------------------------------
void __fastcall THistogramStrechingForm::TrackBarChange(TObject *Sender)
{
   Timer1->Enabled = false;
   Edit1->Text = FormatFloat("0.0", (float)(TrackBar->Position) / (float)(10));
   Edit1->Text = Edit1->Text + " %";
   omis_level = (float)(TrackBar->Position) / (float)(10);
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall THistogramStrechingForm::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   if (last1 != -1) histogram1.LineOnly(Image3, last1);
   if (last2 != -1) histogram2.LineOnly(Image4, last2);
   ComputeIt();
   Image2->Enabled = true;
   Image4->Enabled = true;
   BitBtnOk->Enabled = true;
}
//---------------------------------------------------------------------------

