/*
   TypeAdjHistogramUnit.cpp

   definin soubor okna pro monost zmny LUT pro specifikaci histogramu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TypeAdjHistogramUnit.h"
#include "TypeHistogramUnit.h"
#include "HistogramSpecificationUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTypeSpecificationHistogramForm *TypeSpecificationHistogramForm;
//---------------------------------------------------------------------------
__fastcall TTypeSpecificationHistogramForm::TTypeSpecificationHistogramForm(TComponent* Owner)
        : TForm(Owner)
{
   Left = HistogramSpecificationForm->Left + HistogramSpecificationForm->Width + 3;
   Top =  HistogramSpecificationForm->Top + + HistogramSpecificationForm->Height +
                                              TypeHistogramForm->Height + 3 - Height;
   width = Width;

   original_lut = new LUT();

   just_once = false;
   switch(TypeHistogramForm->WhichLUT()){
      case(0):
         lut_base = new LUTBrightness(Image,Panel1);
         break;
      case(1):
         lut_base = new LUTContrast(Image,Panel1);
         break;
      case(2):
         lut_base = new LUTThreshold(Image,Panel1);
         break;
      case(3):
         lut_base = new LUTEqual(Image,Panel1);
         just_once = true;
         break;
      case(4):
         lut_base = new LUTNegative(Image,Panel1);
         just_once = true;
         break;
      case(5):
         lut_base = new LUTEqualNegative(Image,Panel1);
         just_once = true;
         break;
      case(6):
         lut_base = new LUTFlyBelt(Image,Panel1);
         break;
      case(7):
         lut_base = new LUTBitField(Image,Panel1,1);
         just_once = true;
         break;
      case(8):
         lut_base = new LUTBitField(Image,Panel1,2);
         just_once = true;
         break;
      case(9):
         lut_base = new LUTBitField(Image,Panel1,3);
         just_once = true;
         break;
      case(10):
         lut_base = new LUTFree(Image,Panel1);
         break;
      case(11):
         lut_base = new LUT3LThreshold(Image,Panel1);
         break;
      case(12):
         lut_base = new LUTThresholdEqual_1(Image,Panel1);
         break;
      case(13):
         lut_base = new LUTThresholdEqual_2(Image,Panel1);
         break;
      case(14):
         lut_base = new LUTExp(Image,Panel1);
         just_once = true;
         break;
      case(15):
         lut_base = new LUTLn(Image,Panel1);
         just_once = true;
         break;

   }

   lut_base->DrawFrame();
   lut_base->DrawChart();

   if (just_once == true){ TransferData(); }

}
//---------------------------------------------------------------------------
__fastcall TTypeSpecificationHistogramForm::~TTypeSpecificationHistogramForm()
{
   delete (lut_base);
   delete (original_lut);
}
//---------------------------------------------------------------------------
void __fastcall TTypeSpecificationHistogramForm::ImageMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  lut_base->OnMouseDown(Button, X, Y);
}
//---------------------------------------------------------------------------
void __fastcall TTypeSpecificationHistogramForm::ImageMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   lut_base->OnMouseMove(X, Y);
   if(lut_base->Down()==true){
      LutTimer->Enabled = true;
      LutTimer->Tag = 1;
   }
   else{
      LutTimer->Enabled = false;
   }
}
//---------------------------------------------------------------------------
void __fastcall TTypeSpecificationHistogramForm::ImageMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   lut_base->OnMouseUp(Button, X, Y);
}
//---------------------------------------------------------------------------

void __fastcall TTypeSpecificationHistogramForm::LutTimerTimer(TObject *Sender)
{
   if(LutTimer->Tag==1){
      TransferData();
      LutTimer->Tag = 0;
   }
}
//---------------------------------------------------------------------------
void __fastcall TTypeSpecificationHistogramForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();
}
//---------------------------------------------------------------------------
void __fastcall TTypeSpecificationHistogramForm::TransferData()
{
   HistogramSpecificationForm->SetData(&(lut_base->GetLUT()));
}




