/*
   LUTAdjUnit.cpp

   definin soubor okna pro monost zmny LUT s nhledem na histogram

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "LUTAdjUnit.h"
#include "LUTListUnit.h"
#include "LUTUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TLUTAdjForm *LUTAdjForm;
//---------------------------------------------------------------------------
__fastcall TLUTAdjForm::TLUTAdjForm(TComponent* Owner)
        : TForm(Owner)
{

   //width_with_hist(609), width(308)
   //Width = width;
   Left = LUTForm->Left;
   Top =  LUTForm->Top + LUTForm->Height + 5;
   width_with_hist = Width;
   width = width_with_hist*0.505;
   Width = width;
   just_once = false;
   switch(LUTListForm->WhichLUT()){
      case(0):
         lut_base = new LUTBrightness(Image,Panel1);
         just_once = true;
         break;
      case(1):
         lut_base = new LUTContrast(Image,Panel1);
         just_once = true;
         break;
      case(2):
         lut_base = new LUTThreshold(Image,Panel1);
         just_once = true;
         break;
      case(3):
         lut_base = new LUTEqual(Image,Panel1);
         just_once = true;
         break;
      case(4):
         lut_base = new LUTNegative(Image,Panel1);
         just_once = true;
         break;
      case(5):
         lut_base = new LUTEqualNegative(Image,Panel1);
         just_once = true;
         break;
      case(6):
         lut_base = new LUTFlyBelt(Image,Panel1);
         break;
      case(7):
         lut_base = new LUTBitField(Image,Panel1,1);
         just_once = true;
         break;
      case(8):
         lut_base = new LUTBitField(Image,Panel1,2);
         just_once = true;
         break;
      case(9):
         lut_base = new LUTBitField(Image,Panel1,3);
         just_once = true;
         break;
      case(10):
         lut_base = new LUTFree(Image,Panel1);
         just_once = true;
         break;
      case(11):
         lut_base = new LUT3LThreshold(Image,Panel1);
         just_once = true;
         break;
      case(12):
         lut_base = new LUTThresholdEqual_1(Image,Panel1);
         just_once = true;
         break;
      case(13):
         lut_base = new LUTThresholdEqual_2(Image,Panel1);
         just_once = true;
         break;
      case(14):
         lut_base = new LUTExp(Image,Panel1);
         just_once = true;
         break;
      case(15):
         lut_base = new LUTLn(Image,Panel1);
         just_once = true;
         break;

   }
   lut_base->DrawFrame();
   lut_base->DrawChart();
   lut_image = new PointImage();
   apply_lut = new Brightness(*(LUTForm->GetLUTImage()), *lut_image);

   if(just_once==true){
      apply_lut->ApplyLUT(lut_base->GetLUT(), 0, 0,
                lut_image->Cols(),lut_image->Rows());
      LUTForm->SetBMPHandler(lut_image);
      LUTForm->SetLut(&(lut_base->GetLUT()));
      lut_histogram = new Histogram();
      lut_histogram->InitFromGreyImage(*lut_image, 0, 0,
                 lut_image->Cols(), lut_image->Rows());
      lut_histogram->Display(HistImage);
   }

}
//---------------------------------------------------------------------------
__fastcall TLUTAdjForm::~TLUTAdjForm()
{
   delete (lut_base);
   delete (lut_image);
   delete (apply_lut);
   delete (lut_histogram);
}
//---------------------------------------------------------------------------
void __fastcall TLUTAdjForm::ImageMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  lut_base->OnMouseDown(Button, X, Y);
}
//---------------------------------------------------------------------------
void __fastcall TLUTAdjForm::ImageMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
   lut_base->OnMouseMove(X, Y);
   if(lut_base->Down()==true){
      LutTimer->Enabled = true;
      LutTimer->Tag = 1;
   }
   else{
      LutTimer->Enabled = false;
   }

}
//---------------------------------------------------------------------------
void __fastcall TLUTAdjForm::ImageMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   lut_base->OnMouseUp(Button, X, Y);
}
//---------------------------------------------------------------------------

void __fastcall TLUTAdjForm::LutTimerTimer(TObject *Sender)
{
   if(LutTimer->Tag==1){
      apply_lut->ApplyLUT(lut_base->GetLUT(), 0, 0,
                lut_image->Cols(),lut_image->Rows());
      LUTForm->SetBMPHandler(lut_image);
      LUTForm->SetLut(&(lut_base->GetLUT()));
      if(CheckBoxHist->Checked==true){
         Width = width_with_hist;
         lut_histogram->InitFromGreyImage(*lut_image, 0, 0,
                  lut_image->Cols(), lut_image->Rows());
         lut_histogram->Display(HistImage);
      }
      LutTimer->Tag = 0;
   }
}
//---------------------------------------------------------------------------

void __fastcall TLUTAdjForm::CheckBoxHistClick(TObject *Sender)
{
   if(CheckBoxHist->Checked==true){
      Width = width_with_hist;
      lut_histogram = new Histogram();
      lut_histogram->InitFromGreyImage(*lut_image, 0, 0,
                 lut_image->Cols(), lut_image->Rows());
      lut_histogram->Display(HistImage);
   }
   else{
      Width = width;
   }
}
//---------------------------------------------------------------------------

void __fastcall TLUTAdjForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------

