/*
   FrameUnit.cpp

   definin soubor okna provdjcho aritmetick a logick operace

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "FrameUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFrameForm *FrameForm;
//---------------------------------------------------------------------------
__fastcall TFrameForm::TFrameForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   OpenDialog->Filter="Bitmap picture (*.bmp)|*.bmp";
   OpenDialog->DefaultExt = "bmp";

   original1_image = new GreyImage();
   sized1_image = new GreyImage();

   frame1_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   frame1_handler->Set(*original1_image);
   Scale scale(*original1_image, *sized1_image, Image1->Width);

   if (sized1_image->Cols() > sized1_image->Rows()){
     Image3->Top=Image3->Top + (sized1_image->Cols()-sized1_image->Rows())/2;
   }else if (sized1_image->Rows() > sized1_image->Cols()){
     Image3->Left=Image3->Left + (sized1_image->Rows()-sized1_image->Cols())/2;
   }

   frame1_handler->Assign(*sized1_image);
   frame1_handler->Show(Image1);
   image = NULL;
   frame_image = NULL;
   original2_image = NULL;
   sized2_image = NULL;
   frame_handler = NULL;
   frame2_handler = NULL;
   frame3_handler = NULL;
   Label4->Enabled = false;
   RadioButton1->Enabled = false;
   RadioButton2->Enabled = false;
   OKBitBtn->Enabled = false;

   percent = 50;
   constant = 10;
   method = 1; // Method of Overflow
   min_range = 0; // Min level of application
   max_range = 255; // Max level of application

   StatusBar->SimpleText = "Click to 'load Image2' and select second image.";
}
//---------------------------------------------------------------------------
__fastcall TFrameForm::~TFrameForm()
{
    delete (frame_handler);
    delete (frame1_handler);
    delete (frame2_handler);
    delete (frame3_handler);
    delete (sized1_image);
    delete (sized2_image);
    delete (image);
    delete (original1_image);
    delete (original2_image);
    delete (frame_image);
    delete (frame);
}
//---------------------------------------------------------------------------
void
TFrameForm::PerformFrameProcess(BitFunction combine_type, short scale)
{
   try{
      ClearCanvas(Image3->Canvas);
      frame ->CombineImages(0, 0, sized1_image->Cols(), sized1_image->Rows(),
              0, 0, combine_type, scale, constant, method, min_range, max_range);
      frame3_handler -> Assign(*frame_image);
      frame3_handler->Show(Image3);
   }catch(VEMsg &e){
      ShowMessage(e.msg);
   }
}
//---------------------------------------------------------------------------
void
TFrameForm::WeightImages(int percent)
{
   try{
      ClearCanvas(Image3->Canvas);
      frame ->WeightImages(0, 0, sized1_image->Cols(), sized1_image->Rows(),
                        0, 0, percent);
      frame3_handler -> Assign(*frame_image);
      frame3_handler->Show(Image3);
   }catch(VEMsg &e){
      ShowMessage(e.msg);
   }
}
//---------------------------------------------------------------------------
void
__fastcall TFrameForm::ComboBox2Change(TObject *Sender)
{
   Panel5->Visible = false;
   Panel8->Visible = false;
   Panel10->Visible = false;
   Panel11->Visible = false;
   Image3->Visible = true;
   method = 1;
   min_range = 0; // Min level of application
   max_range = 255; // Max level of application
   RadioButton3->Checked = true;
   if (RadioButton1->Checked == true){
      switch(ComboBox2->ItemIndex){
         case(0):
            PerformFrameProcess(AND,0);
            combine_type = AND;
            scale = 0;
            break;
         case(1):
            PerformFrameProcess(OR,0);
            combine_type = OR;
            scale = 0;
            break;
         case(2):
            PerformFrameProcess(XOR,0);
            combine_type = XOR;
            scale = 0;
            break;
      }
   }else{
      switch(ComboBox2->ItemIndex){
         case(0):
            PerformFrameProcess(InvAND,0);
            combine_type = InvAND;
            scale = 0;
            break;
         case(1):
            PerformFrameProcess(InvOR,0);
            combine_type = InvOR;
            scale = 0;
            break;
         case(2):
            PerformFrameProcess(InvXOR,0);
            combine_type = InvXOR;
            scale = 0;
            break;
      }
   }
   OKBitBtn->Enabled = true;
   StatusBar->SimpleText = "Click to OK button for use.";
   Label6->Enabled = false;
   RadioButton3->Enabled = false;
   RadioButton4->Enabled = false;
   RadioButton5->Enabled = false;
   RadioButton1->Enabled = true;
   RadioButton2->Enabled = true;
   Label4->Enabled = true;   
}
//---------------------------------------------------------------------------
void
TFrameForm::ClearCanvas(TCanvas* canvas)
{
   TRect rect;
   rect.Top = 0;
   rect.Left = 0;
   rect.Right = Image3->Width;
   rect.Bottom = Image3->Height;
   canvas->Brush->Color = clWhite;
   canvas->FillRect(rect);
}
//---------------------------------------------------------------------------
void __fastcall TFrameForm::OKBitBtnClick(TObject *Sender)
{
   try{
      frame = new FrameImage(*original1_image, *original2_image, *frame_image);
      if(ComboBox1->ItemIndex==8)
         frame ->WeightImages(0, 0, original1_image->Cols(),
                         original1_image->Rows(), 0, 0, percent);
      else
         frame ->CombineImages(0, 0, original1_image->Cols(),
                         original1_image->Rows(), 0, 0, combine_type, scale,
                         constant, method, min_range, max_range);
      PictureForm->AssignList(frame_image);
      Close();
   }catch(VEMsg &e){
      ShowMessage(e.msg);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFrameForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------


void __fastcall TFrameForm::PercentEditChange(TObject *Sender)
{
    try{
        if(PercentEdit->Text=="")
           throw VEMsg("");
        percent = PercentEdit->Text.ToInt();
        if(percent>100){
           PercentEdit->Text = 100;
           throw VEMsg("");
        }
        if(percent<0){
           PercentEdit->Text = 0;
           throw VEMsg("");
        }
        //StatusBar->SimpleText = "First Image "+AnsiString(percent);
        PercentUpDown->Position = (short)percent;
        ClearCanvas(Image3->Canvas);
        frame ->WeightImages(0, 0, sized1_image->Cols(), sized2_image->Rows(),
                        0, 0, percent);
        frame3_handler -> Assign(*frame_image);
        frame3_handler->Show(Image3);
    }catch(EConvertError& e){
         MessageBox(Handle, e.Message.c_str(),
            "Arithmetic and Logic operations", MB_ICONWARNING | MB_OK);
    }catch(VEMsg& e){
       ; // empty statemnet-just exit the function which is in a turn called
         // with a new value of PercentEdit->Text
    }
}
//---------------------------------------------------------------------------

void __fastcall TFrameForm::PercentUpDownClick(TObject *Sender,
      TUDBtnType Button)
{
   PercentEdit->Text = PercentUpDown->Position;
}
//---------------------------------------------------------------------------

void __fastcall TFrameForm::Button2Click(TObject *Sender)
{
   if(OpenDialog->Execute()){
      // artificial test of the same size
      delete frame_handler;
      delete image;
      frame_handler = new BMPHandler(OpenDialog->FileName);
      image = new GreyImage();
      frame_handler->Set(*image);
      file_name = ExtractFileName(OpenDialog->FileName);
      if(original1_image->Cols() != image->Cols()||
                 original1_image->Rows() != image->Rows()){
          AnsiString message = "Picture "+file_name+" isn't of the same size"
              +" .\n Required size is "+
              AnsiString(original1_image->Cols())+ " x "+
              AnsiString(original1_image->Rows());
          MessageBox(Handle, message.c_str(),
            "Arithmetic and Logic operations", MB_ICONQUESTION | MB_OK);
      }else{
         delete frame2_handler;
         delete original2_image;
         delete sized2_image;
         delete frame3_handler;

         frame2_handler = new BMPHandler(OpenDialog->FileName);
         original2_image = new GreyImage();
         frame2_handler->Set(*original2_image);
         file_name2 = ExtractFileName(OpenDialog->FileName);
         sized2_image = new GreyImage();
         Scale scale(*original2_image, *sized2_image, Image1->Width);
         frame2_handler->Assign(*sized2_image);
         ClearCanvas(Image2->Canvas);
         frame2_handler->Show(Image2);
         Image2->Visible = true;
         frame3_handler = new BMPHandler(*frame2_handler);
         frame_image = new GreyImage;
         frame = new FrameImage(*sized1_image, *sized2_image, *frame_image);
         Image3->Visible = false;
         Panel5->Visible = false;
         Panel8->Visible = false;
         Panel10->Visible = false;
         Panel11->Visible = false;
         ComboBox1->Enabled = true;
         ComboBox2->Enabled = true;
         Label2->Enabled = true;
         Label3->Enabled = true;
         method = 1;
         RadioButton3->Checked = true;
         StatusBar->SimpleText = "Select operation for one or both images.";
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TFrameForm::RadioButton2Click(TObject *Sender)
{
  Panel6->Visible = false;
  Panel5->Visible = false;
  ComboBox1->Enabled = true;
  ComboBox2->Enabled = false;
  ComboBox2->Text = "Select";
}
//---------------------------------------------------------------------------
void __fastcall TFrameForm::ComboBox1Change(TObject *Sender)
{
   Panel5->Visible = false;
   Panel8->Visible = false;
   Panel10->Visible = false;
   Panel11->Visible = false;
   Image3->Visible = true;
   min_range = 0; // Min level of application
   max_range = 255; // Max level of application
   switch(ComboBox1->ItemIndex){
      case(0):
         PerformFrameProcess(ADD,0);
         combine_type = ADD;
         scale = 0;
         break;
      case(1):
         PerformFrameProcess(SUB,0);
         combine_type = SUB;
         scale = 0;
         break;
      case(2):
         PerformFrameProcess(MULT,5);
         combine_type = MULT;
         scale = 5;
         break;
      case(3):
         PerformFrameProcess(DIV,-5);
         combine_type = DIV;
         scale = -5;
         break;
      case(4):
         PerformFrameProcess(MIN,0);
         combine_type = MIN;
         scale = 0;
         break;
      case(5):
         PerformFrameProcess(MAX,0);
         combine_type = MAX;
         scale = 0;
         break;
      case(6):
         PerformFrameProcess(AVE,0);
         combine_type = AVE;
         scale = 0;
         break;
      case(7):
         PerformFrameProcess(OVERLAY,0);
         combine_type = OVERLAY;
         scale = 0;
         break;
      case(8):
         Panel5->Visible = true;
         Panel8->Visible = true;
         Label6->Enabled = false;
         RadioButton3->Enabled = false;
         RadioButton4->Enabled = false;
         RadioButton5->Enabled = false;
         TFrameForm::WeightImages(percent);
         break;
   }
   OKBitBtn->Enabled = true;
   StatusBar->SimpleText = "Click to OK button for use.";
   if (ComboBox1->ItemIndex!=8){
      Label6->Enabled = true;
      RadioButton3->Enabled = true;
      RadioButton4->Enabled = true;
      RadioButton5->Enabled = true;
   }else{
      Label6->Enabled = false;
      RadioButton3->Enabled = false;
      RadioButton4->Enabled = false;
      RadioButton5->Enabled = false;
   }
   Label4->Enabled = false;
   RadioButton1->Enabled = false;
   RadioButton2->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TFrameForm::ComboBox3Change(TObject *Sender)
{
   if (frame2_handler==NULL){
      frame2_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
      original2_image = new GreyImage();
      frame2_handler->Set(*original2_image);
      sized2_image = new GreyImage();
      Scale scale(*original2_image, *sized2_image, Image1->Width);
      frame2_handler->Assign(*sized2_image);
      frame3_handler = new BMPHandler(*frame1_handler);
      frame3_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
      frame_image = new GreyImage;
      frame = new FrameImage(*sized1_image, *sized2_image, *frame_image);
   }
   Image3->Visible = true;
   Panel5->Visible = false;
   Panel8->Visible = false;
   scale = 0;
   switch(ComboBox3->ItemIndex){
   case(0):
      PerformFrameProcess(PLUS,scale);
      combine_type = PLUS;
      Panel10->Visible = true; // level range
      Panel11->Visible = true;
      break;
   case(1):
      PerformFrameProcess(MINUS,scale);
      combine_type = MINUS;
      Panel10->Visible = true; // level range
      Panel11->Visible = true;
      break;
   case(2):
      Panel10->Visible = false; // level range
      Panel11->Visible = false;
      min_range = 0; // Min level of application
      max_range = 255; // Max level of application
      PerformFrameProcess(MULTIP,scale);
      combine_type = MULTIP;
      break;
   case(3):
      Panel10->Visible = false; // level range
      Panel11->Visible = false;
      min_range = 0; // Min level of application
      max_range = 255; // Max level of application
      PerformFrameProcess(fDIV,scale);
      combine_type = fDIV;
      break;
   }
   OKBitBtn->Enabled = true;
   StatusBar->SimpleText = "Click to OK button for use.";
   Label6->Enabled = true;
   RadioButton3->Enabled = true;
   RadioButton4->Enabled = true;
   RadioButton5->Enabled = true;
   Label4->Enabled = false;
   RadioButton1->Enabled = false;
   RadioButton2->Enabled = false;
   ConstantEdit->Enabled = true;
   ConstantUpDown->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TFrameForm::ConstantEditChange(TObject *Sender)
{
   Panel5->Visible = false;
   Panel8->Visible = false;
   try{
      if(ConstantEdit->Text=="")
         throw VEMsg("");
      constant = ConstantEdit->Text.ToInt();
      if(constant<0){
         ConstantEdit->Text = 0;
         throw VEMsg("");
      }

      ConstantUpDown->Position = (short)constant;
      TFrameForm::ComboBox3Change(Sender);
   }catch(EConvertError& e){
         MessageBox(Handle, e.Message.c_str(),
            "Arithmetic and Logic operations", MB_ICONWARNING | MB_OK);
   }catch(VEMsg& e){
       ; // empty statemnet-just exit the function which is in a turn called
         // with a new value of PercentEdit->Text
   }
}
//---------------------------------------------------------------------------
void __fastcall TFrameForm::RadioButton3Click(TObject *Sender)
{
   if (method !=1){
      method = 1;
      PerformFrameProcess(combine_type,scale);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFrameForm::RadioButton4Click(TObject *Sender)
{
   if (method !=2){
      method = 2;
      PerformFrameProcess(combine_type,scale);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFrameForm::RadioButton5Click(TObject *Sender)
{
   if (method !=3){
      method = 3;
      PerformFrameProcess(combine_type,scale);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFrameForm::ConstantUpDownClick(TObject *Sender,
      TUDBtnType Button)
{
   ConstantEdit->Text = ConstantUpDown->Position;        
}
//---------------------------------------------------------------------------


void __fastcall TFrameForm::MinValueEditChange(TObject *Sender)
{
   try{
      if(MinValueEdit->Text=="")
         throw VEMsg("");
      min_range = MinValueEdit->Text.ToInt();
      if(min_range<0){
         MinValueEdit->Text = 0;
         throw VEMsg("");
      }

      MinUpDown->Position = (short)min_range;
      TFrameForm::ComboBox3Change(Sender);
   }catch(EConvertError& e){
         MessageBox(Handle, e.Message.c_str(),
            "Arithmetic and Logic operations", MB_ICONWARNING | MB_OK);
   }catch(VEMsg& e){
       ; // empty statemnet-just exit the function which is in a turn called
         // with a new value of PercentEdit->Text
   }
}
//---------------------------------------------------------------------------

void __fastcall TFrameForm::MinUpDownClick(TObject *Sender,
      TUDBtnType Button)
{
   MinValueEdit->Text = MinUpDown->Position;          
}
//---------------------------------------------------------------------------

void __fastcall TFrameForm::MaxUpDownClick(TObject *Sender,
      TUDBtnType Button)
{
   MaxValueEdit->Text = MaxUpDown->Position;
}
//---------------------------------------------------------------------------

void __fastcall TFrameForm::MaxValueEditChange(TObject *Sender)
{
   try{
      if(MaxValueEdit->Text=="")
         throw VEMsg("");
      max_range = MaxValueEdit->Text.ToInt();
      if(max_range<0){
         MaxValueEdit->Text = 0;
         throw VEMsg("");
      }

      MaxUpDown->Position = (short)max_range;
      TFrameForm::ComboBox3Change(Sender);
   }catch(EConvertError& e){
         MessageBox(Handle, e.Message.c_str(),
            "Arithmetic and Logic operations", MB_ICONWARNING | MB_OK);
   }catch(VEMsg& e){
       ; // empty statemnet-just exit the function which is in a turn called
         // with a new value of PercentEdit->Text
   }
}
//---------------------------------------------------------------------------






