/*
   asci.cpp

   definin soubor tdy pro Vstup/vstup formt ASCII

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "asci.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

//---------------------------------------------------------------------------
__fastcall VAsci::VAsci(AnsiString _file_path)
   :VInOut(_file_path)
{
      image = new GreyImage();
      suffix = "txt";
      CheckSuffix();
      AsciToGreyImage();
      handler = new BMPHandler();
}
//---------------------------------------------------------------------------
// destructor
__fastcall VAsci::~VAsci()
{
   delete (handler);
}
//---------------------------------------------------------------------------
void
__fastcall VAsci::AsciToGreyImage()
{
   unsigned long index = 0;
   TFileStream* fstream = new TFileStream(file_path, fmOpenRead);
   unsigned long size = fstream->Size;
   char* buff = new char[size];
   fstream->Read(buff,size);
   delete (fstream);
   int cols = CharToInt(buff, index, size);
   int rows = CharToInt(buff, index, size);
   image->Resize(cols, rows);
   for(register int r(0); r<image->Rows(); r++)
      for(register int c(0); c<image->Cols(); c++){
          image->Data(c,r) =
               (unsigned char)CharToInt(buff, index, size);
      }
   delete [] buff;
}
//---------------------------------------------------------------------------
void
__fastcall VAsci::SaveToFile(AnsiString _save_file_name)
{
   short int data;
   ofstream o_file(_save_file_name.c_str(),ios::binary);
   Assert(o_file!=0,VEMsg(AnsiString("Can't open file\n" + _save_file_name)));
   o_file << image->Cols() << " " << image->Rows()<< endl;
   for(register int r(0); r<image->Rows(); r++)
      for(register int c(0); c<image->Cols(); c++){
         data = image->Data(c,r);
         o_file << setw(4) << data;
      }
   o_file.close();
/*
   //--------------------------------------------------------------------------
   This rutine works in the same speed as the previous one!!!!
   //--------------------------------------------------------------------------

   ofstream o_file(_save_file_name.c_str(),ios::binary);
   Assert(o_file!=0,VEMsg(AnsiString("Can't open file\n" + _save_file_name)));
   o_file << " ";
   o_file.close();
   TFileStream* fstream = new TFileStream(_save_file_name, fmOpenWrite);
   fstream->Write(AnsiString(image->Cols()).c_str(),AnsiString(image->Cols()).Length());
   fstream->Write(AnsiString(" ").c_str(),AnsiString(" ").Length());
   fstream->Write(AnsiString(image->Rows()).c_str(),AnsiString(image->Cols()).Length());
   fstream->Write(AnsiString("\n").c_str(),AnsiString("\n").Length());
   for(register int r(0); r<image->Rows(); r++){
      for(register int c(0); c<image->Cols(); c++){
         fstream->Write(AnsiString(image->Data(c,r)).c_str(),AnsiString(image->Data(c,r)).Length());
         fstream->Write(AnsiString(" ").c_str(),AnsiString(" ").Length());
      }
      fstream->Write(AnsiString("\n").c_str(),AnsiString("\n").Length());
   }
   delete (fstream);   */
}
//---------------------------------------------------------------------------
inline int
__fastcall VAsci::CharToInt(char* buff, unsigned long& index,
                   unsigned long size)
{
   while(isdigit(buff[index])==0){
      index++;
      if(index>size)
         return (-1);
   }
   int number=0;
   int i=0;
   while(isdigit(buff[index])){
      index++;
      if(index>size)
         return (-1);
      i++;
   }
   int ind = index, ii=1;
   while(i!=0){
      number += (buff[--ind]-48)*ii;
      ii*=10;
      i--;
   }
   return number;
}
//---------------------------------------------------------------------------
void
__fastcall VAsci::Assign(BMPHandler* _handler)
{
   *handler = *_handler;
   handler->Set(*image);
}

