/*
   bmp.cpp

   definin soubor tdy pro Vstup/vstup formt BMP.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "bmp.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// constructor
__fastcall VBitmap::VBitmap(AnsiString _file_path)
   :VInOut(_file_path)
{
   image = new GreyImage();
   suffix = "bmp";
   CheckSuffix();
   CheckValidity(0);
   handler = new BMPHandler(_file_path, *image);
   color_handler = NULL;
}
//---------------------------------------------------------------------------
// constructor
__fastcall VBitmap::VBitmap(AnsiString _file_path, bool color_img)
   :VInOut(_file_path)
{
   color_image = new ColorImage();
   suffix = "bmp";
   CheckSuffix();
   CheckValidity(1);
   handler = NULL;
   color_handler = new BMPHandlerColor(_file_path, *color_image);
}
//---------------------------------------------------------------------------
__fastcall VBitmap::VBitmap(BMPHandler* _handler)
  :VInOut(_handler)
{
   handler = new BMPHandler(*_handler);
   color_handler = NULL;
}
//---------------------------------------------------------------------------
__fastcall VBitmap::VBitmap(BMPHandlerColor* _color_handler)
  :VInOut(_color_handler)
{
   color_handler = new BMPHandlerColor(*_color_handler);
   handler = NULL;
}
//---------------------------------------------------------------------------
// destructor
__fastcall VBitmap::~VBitmap()
{
}
//---------------------------------------------------------------------------
void
__fastcall VBitmap::SaveToFile(AnsiString _save_file_name)
{
   if (handler!=NULL){
      handler -> SaveToFile(_save_file_name);
   }else{
      color_handler -> SaveToFile(_save_file_name);
   }
}
//---------------------------------------------------------------------------
void
__fastcall VBitmap::ConvertToBMP(BMPHandler* _handler)
{
   *_handler = *handler;
}
//---------------------------------------------------------------------------
void
__fastcall VBitmap::ConvertToColorBMP(BMPHandlerColor* _color_handler)
{
   *_color_handler = *color_handler;
}
//---------------------------------------------------------------------------
void
__fastcall VBitmap::Assign(GreyImage& _image)
{
   handler -> Assign(_image);
}
//---------------------------------------------------------------------------
void
__fastcall VBitmap::Assign(BMPHandler* _handler)
{
   *handler = *_handler;
}
//---------------------------------------------------------------------------
void
__fastcall VBitmap::ColorAssign(BMPHandlerColor* _color_handler)
{
   *color_handler = *_color_handler;
}
//---------------------------------------------------------------------------
void
__fastcall VBitmap::CheckValidity(int type_image)
{
   char buff[54];
   bool valid = true;
   TFileStream* stream = new TFileStream(file_path,fmOpenRead);
   stream->Read(buff, 54);
   delete(stream);
   for (int index(0); index<2; index++){
      switch(index){
         case(0): valid = (buff[0]=='B') ? true : false; break;
         case(1): valid = (buff[1]=='M') ? true : false; break;
      }
   }
   Assert(valid==true,
        VECorruptedFile(AnsiString("Bitmap is not valid"), AnsiString("bmp")));

   switch(type_image){
      // test 256-colors image
      case(0):
         for (int index(0); index<2; index++){
            switch(index){
               case(0): valid = (buff[26]==0x01) ? true : false; break;
               case(1): valid = (buff[28]==0x08) ? true : false; break;
            }
         }
         Assert(valid==true,
              VECorruptedFile(AnsiString("This image isn't 8-bit Bitmap file."), AnsiString("bmp")));
         break;
      // test 16.7mil. colors image
      case(1):
         for (int index(0); index<2; index++){
            switch(index){
               case(0): valid = (buff[26]==0x01) ? true : false; break;
               case(1): valid = (buff[28]==0x18) ? true : false; break;
            }
         }
         Assert(valid==true,
              VECorruptedFile(AnsiString("This image isn't 24-bit Bitmap file."), AnsiString("bmp")));
         break;
   }
}



