/*
   contrast.cpp

   definin soubor tdy operace pro korekci kontrastu.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "contrast.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

// constructors + destructor
//---------------------------------------------------------------------------
// default constructor
Contrast::Contrast()
:col(0), row(0), height(0), width(0),
 sum(0), average(0)
{
   original_image = NULL;
   contrast_image   = NULL;
}

//---------------------------------------------------------------------------
Contrast::Contrast(const GreyImage& _original_image, GreyImage& _contrast_image,
                   int _col, int _row, int _width, int _height)
{
   original_image = &_original_image;
   contrast_image   = &_contrast_image;

   col = _col;
   row = _row;
   width = _width;
   height = _height;
   sum = 0;
   average = 0;

   Assert((col <= original_image->Cols())&&
         ((col+width)<=original_image->Cols())&&
         (row <= original_image->Rows())&&
         ((row+height)<=original_image->Rows()),VEMsg(AnsiString
         ("Contrast::Contrast:the portion goes out of image")));

   if((contrast_image->Rows()!=original_image->Rows())||
      (contrast_image->Cols()!=original_image->Cols()))
       contrast_image->Resize(original_image->Cols(), original_image->Rows());
   /* caunting of the avarage */
   for(register int c(col); c<col+width; c++)
      for(register int r(row); r<row+height; r++)
         sum += original_image->Data(c,r);

   average = (float)sum/(float)(height*width);
}

//-------------------------------------------------------------------------
// This function applies a transform previosly programmed into LookUpTable
// to the image or portion of image;
void
Contrast::ApplyLUT(LUT& lut)
{
   for(register int c(col); c<col+width; c++)
      for(register int r(row); r<row+height; r++)
        contrast_image->Data(c,r) =
           (unsigned char)lut[original_image->Data(c,r)];
}

//---------------------------------------------------------------------------
void
Contrast::ContrastCorrection(float contrast)
{

   register int new_value;
   for(register int index(0); index<256; index++)
   {
      new_value = contrast*((float)index - average) + average;
      new_value = (new_value < 0  ) ? 0   : new_value;
      new_value = (new_value > 255) ? 255 : new_value;
      contrast_lut[index] = new_value;
   }
   ApplyLUT(contrast_lut);
}
