/*
   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "kernel.h"


//---------------------------------------------------------------------------
#pragma package(smart_init)

// constructors
//---------------------------------------------------------------------------
// default constructor
Kernel::Kernel()
:ker_cols(0), ker_rows(0), sum(0), absolute(1),scale(1)
{
   kernel = NULL;
}

//---------------------------------------------------------------------------
// two arguments constructor
Kernel::Kernel(int _ker_cols, int _ker_rows)
:sum(0), absolute(true), scale(1)
{
   kernel = NULL;
   Resize(_ker_cols, _ker_rows);
}
//---------------------------------------------------------------------------
// function used for allocating mrmory for kernel array + exception handling
void
Kernel::Resize(int _c, int _r)
{
   delete [] kernel;
   kernel = new double[_c*_r];
   if(kernel==0)
      throw Exception("Kerlel::Resize : can't allocate enought memory");
   ker_cols = _c;
   ker_rows = _r;
}


//---------------------------------------------------------------------------
// one argument constructor that reads data from a text file *.ker*
Kernel::Kernel(const AnsiString& ker_filename)
{
   ifstream infile(ker_filename.c_str());
   Assert(infile!=0,VEMsg(AnsiString
     ("Kernel::Kernel(const AnsiString) : can't open file")));
   Name() = ExtractFileName(ker_filename);

   infile >> ker_cols >> ker_rows;
   kernel = new double[ker_cols*ker_rows];
   for( register index(0);index<ker_cols*ker_rows; index++)
      kernel[index] = 0;

   sum = 0;
   for (register int r(0); r< ker_rows; r++){
      for (register int c(0); c< ker_cols; c++){
         double na;
         infile >> na;
         kernel[c+r*ker_cols] = na;
         sum += na;
      }
   }
   Sum() = sum;
   Absolute() = true;
   Scale() = 1;
   infile.close();
}
//---------------------------------------------------------------------------
Kernel::Kernel(const Kernel& init)
{
   ker_cols = init.ker_cols;
   ker_rows = init.ker_rows;
   sum = init.sum;
   absolute = init.absolute;
   scale = init.scale;
   kernel = new double[ker_cols*ker_rows];
   for( register index(0);index<ker_cols*ker_rows; index++)
      kernel[index] = init.kernel[index];
}
//---------------------------------------------------------------------------
// destructor
Kernel::~Kernel()
{
   delete [] kernel;
}

// overloaded operators
//---------------------------------------------------------------------------
// overloaded operator = of assignment
Kernel&
Kernel::operator = (const Kernel& right)
{
   Assert(this!=&right,
      VESize(AnsiString
   ("Kernel::operator = (): Trying to assign the very same object")));
   if((ker_rows != right.ker_rows)||(ker_cols != right.ker_cols)){
      absolute = right.absolute;
      sum = right.sum;
      scale = right.scale;
      Resize(right.ker_cols,right.ker_rows);
   }
   for(register int i(0); i<ker_rows*ker_cols; i++)
      kernel[i] = right.kernel[i];
   return *this;
}

//---------------------------------------------------------------------------
void
Kernel::SaveKernelAs(AnsiString& save_file)
{
   Assert(kernel!=NULL,VEMsg(AnsiString
     ("Kernel::SaveKernelAs(AnsiString) : kernel wasn't allocated yet.")));

   ofstream out_file(save_file.c_str());
   Assert(out_file!=0,VEMsg(AnsiString
     ("Kernel::SaveKernelAs(AnsiString) : can't open file")));

   out_file << ker_cols << " ";
   out_file << ker_rows << endl;

   double na;

   for (register int r(0); r< ker_rows; r++){
      for (register int c(0); c< ker_cols; c++){
         na = kernel[c+r*ker_cols];
         out_file  << setw(6) << na;
      }
      out_file << endl;
   }
   out_file.close();
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_01()
{
   ker_name = AnsiString("Derivative North 1");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 0;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-2;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 0;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = -1;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_02()
{
   ker_name = AnsiString("Derivative North 2");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_03()
{
   ker_name = AnsiString("Derivative North East 1");
   Resize(3,3);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 2;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 0;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_04()
{
   ker_name = AnsiString("2nd Diagonal Detection");
   Resize(3,3);
   KerData(0,0) = (double) 2;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 2;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 2;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_05()
{
   ker_name = AnsiString("Derivative East 1");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-2;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 0;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 0;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 2;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_06()
{
   ker_name = AnsiString("Derivative East 2");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_07()
{
   ker_name = AnsiString("Derivated South East 1");
   Resize(3,3);
   KerData(0,0) = (double)-2;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 1;
   KerData(1,2) = (double) 0;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 2;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_08()
{
   ker_name = AnsiString("Derivative South East 2");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_09()
{
   ker_name = AnsiString("Derivative South 1");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double) 0;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-2;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 2;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double) 0;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_10()
{
   ker_name = AnsiString("Derivative South 2");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_11()
{
   ker_name = AnsiString("Derivative South West 1");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_12()
{
   ker_name = AnsiString("Derivative South West 2");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_13()
{
   ker_name = AnsiString("Derivative West 1");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 2;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 0;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 0;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-2;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_14()
{
   ker_name = AnsiString("Derivative West 2");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 3;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_15()
{
   ker_name = AnsiString("Derivative North West 1");
   Resize(3,3);
   KerData(0,0) = (double) 2;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-2;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_16()
{
   ker_name = AnsiString("Derivative North West 2");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_17()
{
   ker_name = AnsiString("Laplacian 1");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double) 0;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 0;
   KerData(1,1) = (double) 4;
   KerData(1,2) = (double) 0;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double) 0;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_18()
{
   ker_name = AnsiString("Laplacian 2");
   Resize(3,3);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 4;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 0;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_19()
{
   ker_name = AnsiString("Laplacian 3");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 8;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_20()
{
   ker_name = AnsiString("Laplacian 4");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-2;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double)-2;
   KerData(1,1) = (double)12;
   KerData(1,2) = (double)-2;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-2;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_21()
{
   ker_name = AnsiString("Enhancement (small)");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)16;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 8;
   scale = 8;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_22()
{
   ker_name = AnsiString("Enhancement (normal)");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)12;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 4;
   scale = 4;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_23()
{
   ker_name = AnsiString("Enhancement (strong)");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)10;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 2;
   scale = 2;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_24()
{
   ker_name = AnsiString("Enhancement (maximum)");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)9;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_25()
{
   ker_name = AnsiString("Enhancement Type 1");
   Resize(3,3);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 8;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 0;
   absolute = 0;
   sum = 4;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_26()
{
   ker_name = AnsiString("Enhancement Type 2");
   Resize(3,3);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 6;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 0;
   absolute = 0;
   sum = 2;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_27()
{
   ker_name = AnsiString("Enhancement Type 3");
   Resize(3,3);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 5;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 0;
   absolute = 0;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_28()
{
   ker_name = AnsiString("Enhancement Type 4");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double)-2;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-2;
   KerData(1,1) = (double) 5;
   KerData(1,2) = (double)-2;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double)-2;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_29()
{
   ker_name = AnsiString("Smoothing 1");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 1;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = false;
   sum = 9;
   scale = 9;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_30()
{
   ker_name = AnsiString("Smoothing 2");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = false;
   sum = 10;
   scale = 10;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_31()
{
   ker_name = AnsiString("Smoothing 3");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 3;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = false;
   sum = 11;
   scale = 11;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_32()
{
   ker_name = AnsiString("Smoothing 4");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 2;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 2;
   KerData(1,1) = (double) 4;
   KerData(1,2) = (double) 2;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 2;
   KerData(2,2) = (double) 1;
   absolute = false;
   sum = 16;
   scale = 16;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_33()
{
   ker_name = AnsiString("Derivative Horizontal 1");
   Resize(3,3);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double) 0;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 0;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 0;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_34()
{
   ker_name = AnsiString("Derivative Horizontal 2");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 0;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 0;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_35()
{
   ker_name = AnsiString("Derivative Vertical 1");
   Resize(3,3);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double) 0;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double) 0;
   KerData(2,2) = (double) 0;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_36()
{
   ker_name = AnsiString("Derivative Vertical 2");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double) 0;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double) 0;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_37()
{
   ker_name = AnsiString("Vertical Line Detect");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 2;
   KerData(1,1) = (double) 2;
   KerData(1,2) = (double) 2;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_38()
{
   ker_name = AnsiString("Horizontal Line Detect");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double) 2;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 2;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double) 2;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_39()
{
   ker_name = AnsiString("1st Diagonal Detection");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double) 2;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 2;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 2;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_40()
{
   ker_name = AnsiString("2nd Diagonal Detection");
   Resize(3,3);
   KerData(0,0) = (double) 2;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 2;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 2;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_41A()
{
   ker_name = AnsiString("3-level Maxabs (mask 1) 3x3");
   Resize(3,3);
   for(int r = 0; r<3; r++){
      KerData(0,r) =  (double) 1;
      KerData(1,r) =  (double) 0;
      KerData(2,r) =  (double)-1;
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_41B()
{
   ker_name = AnsiString("3-level Maxabs (mask 2) 3x3");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_41C()
{
   ker_name = AnsiString("3-level Maxabs( mask 3) 3x3");
   Resize(3,3);
   for(int c = 0; c<3; c++){
      KerData(c,0) =  (double) 1;
      KerData(c,1) =  (double) 0;
      KerData(c,2) =  (double)-1;
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_41D()
{
   ker_name = AnsiString("3-level Maxabs (mask 4) 3x3");
   Resize(3,3);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 0;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_42A()
{
   ker_name = AnsiString("3-level Maxabs (mask 1) 5x5");
   Resize(5,5);
   for(int r = 0; r<5; r++){
      KerData(0,r) = (double) 1;
      KerData(1,r) = (double) 1;
      KerData(2,r) = (double) 0;
      KerData(3,r) = (double)-1;
      KerData(4,r) = (double)-1;
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_42B()
{
   ker_name = AnsiString("3-level Maxabs (mask 2) 5x5");
   Resize(5,5);
   for(int c = 0; c<5; c++){
      KerData(c,4-c) = (double) 0;
      for (int r = 0; r<(4-c); r++){
          KerData(c,r) = (double) 1;
          KerData(4-c,4-r) = (double) -1;
      }
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_42C()
{
   ker_name = AnsiString("3-level Maxabs (mask 3) 5x5");
   Resize(5,5);
   for(int c = 0; c<5; c++){
      KerData(c,0) = (double) 1;
      KerData(c,1) = (double) 1;
      KerData(c,2) = (double) 0;
      KerData(c,3) = (double)-1;
      KerData(c,4) = (double)-1;
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_42D()
{
   ker_name = AnsiString("3-level Maxabs (mask 4) 5x5");
   Resize(5,5);
   for(int c = 0; c<5; c++){
      KerData(c,c) = (double) 0;
      for (int r = 4; r>c; r--){
          KerData(c,r) = (double)-1;
          KerData(4-c,4-r) = (double) 1;
      }
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_43A()
{
   ker_name = AnsiString("3-level Maxabs (mask 1) 7x7");
   Resize(7,7);
   for(int r = 0; r<7; r++){
      KerData(0,r) = (double) 1;
      KerData(1,r) = (double) 1;
      KerData(2,r) = (double) 1;
      KerData(3,r) = (double) 0;
      KerData(4,r) = (double)-1;
      KerData(5,r) = (double)-1;
      KerData(6,r) = (double)-1;
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_43B()
{
   ker_name = AnsiString("3-level Maxabs (mask 2) 7x7");
   Resize(7,7);
   for(int c = 0; c<7; c++){
      KerData(c,6-c) = (double) 0;
      for (int r = 0; r<(6-c); r++){
          KerData(c,r) = (double) 1;
          KerData(6-c,6-r) = (double) -1;
      }
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_43C()
{
   ker_name = AnsiString("3-level Maxabs (mask 3) 7x7");
   Resize(7,7);
   for(int c = 0; c<7; c++){
      KerData(c,0) = (double) 1;
      KerData(c,1) = (double) 1;
      KerData(c,2) = (double) 1;
      KerData(c,3) = (double) 0;
      KerData(c,4) = (double)-1;
      KerData(c,5) = (double)-1;
      KerData(c,6) = (double)-1;
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_43D()
{
   ker_name = AnsiString("3-level Maxabs (mask 4) 7x7");
   Resize(7,7);
   for(int c = 0; c<7; c++){
      KerData(c,c) = (double) 0;
      for (int r = 6; r>c; r--){
          KerData(c,r) = (double)-1;
          KerData(6-c,6-r) = (double) 1;
      }
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_44A()
{
   ker_name = AnsiString("3-level Maxabs (mask 1) 9x9");
   Resize(9,9);
   for(int r = 0; r<9; r++){
      KerData(0,r) = (double) 1;
      KerData(1,r) = (double) 1;
      KerData(2,r) = (double) 1;
      KerData(3,r) = (double) 1;
      KerData(4,r) = (double) 0;
      KerData(5,r) = (double)-1;
      KerData(6,r) = (double)-1;
      KerData(7,r) = (double)-1;
      KerData(8,r) = (double)-1;
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_44B()
{
   ker_name = AnsiString("3-level Maxabs (mask 2) 9x9");
   Resize(9,9);
   for(int c = 0; c<9; c++){
      KerData(c,8-c) = (double) 0;
      for (int r = 0; r<(8-c); r++){
          KerData(c,r) = (double) 1;
          KerData(8-c,8-r) = (double) -1;
      }
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_44C()
{
   ker_name = AnsiString("3-level Maxabs (mask 3) 9x9");
   Resize(9,9);
   for(int c = 0; c<9; c++){
      KerData(c,0) = (double) 1;
      KerData(c,1) = (double) 1;
      KerData(c,2) = (double) 1;
      KerData(c,3) = (double) 1;
      KerData(c,4) = (double) 0;
      KerData(c,5) = (double)-1;
      KerData(c,6) = (double)-1;
      KerData(c,7) = (double)-1;
      KerData(c,8) = (double)-1;
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_44D()
{
   ker_name = AnsiString("3-level Maxabs (mask 4) 9x9");
   Resize(9,9);
   for(int c = 0; c<9; c++){
      KerData(c,c) = (double) 0;
      for (int r = 8; r>c; r--){
          KerData(c,r) = (double)-1;
          KerData(8-c,8-r) = (double) 1;
      }
   }
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_45A()
{
   ker_name = AnsiString("5-level Maxabs (mask 1) 3x3");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 2;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 0;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 0;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-2;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_45B()
{
   ker_name = AnsiString("5-level Maxabs (mask 2) 3x3");
   Resize(3,3);
   KerData(0,0) = (double) 2;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 0;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 0;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-2;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_45C()
{
   ker_name = AnsiString("5-level Maxabs (mask 3) 3x3");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 0;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 2;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-2;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 0;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_45D()
{
   ker_name = AnsiString("5-level Maxabs (mask 4) 3x3");
   Resize(3,3);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-2;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 2;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 0;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_46A()
{
   ker_name = AnsiString("Roberts Mean-Square (mask 1)");
   Resize(2,2);
   KerData(0,0) = (double) 0;
   KerData(0,1) = (double) 1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 0;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_46B()
{
   ker_name = AnsiString("Roberts Mean-Square (mask 2)");
   Resize(2,2);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double) 0;
   KerData(1,0) = (double) 0;
   KerData(1,1) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_47A()
{
   ker_name = AnsiString("Sobel Mean-Square (mask 1)");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 2;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 0;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 0;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-2;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_47B()
{
   ker_name = AnsiString("Sobel Mean-Square (mask 2)");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 0;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 2;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-2;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 0;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_48A()
{
   ker_name = AnsiString("Compass Max (mask 1)");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_48B()
{
   ker_name = AnsiString("Compass Max (mask 2)");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_48C()
{
   ker_name = AnsiString("Compass Max (mask 3)");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double)-1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_48D()
{
   ker_name = AnsiString("Compass Max (mask 4)");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double)-1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_48E()
{
   ker_name = AnsiString("Compass Max (mask 5)");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,0) = (double) 1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_48F()
{
   ker_name = AnsiString("Compass Max (mask 6)");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double) 1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_48G()
{
   ker_name = AnsiString("Compass Max (mask 7)");
   Resize(3,3);
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double) 1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_48H()
{
   ker_name = AnsiString("Compass Max (mask 8)");
   Resize(3,3);
   KerData(0,0) = (double) 1;
   KerData(0,1) = (double) 1;
   KerData(0,2) = (double) 1;
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double) 1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   KerData(2,2) = (double) 1;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_49A()
{
   ker_name = AnsiString("Kirsch Max (mask 1)");
   Resize(3,3);
   KerData(0,0) = (double) 3;
   KerData(0,1) = (double) 3;
   KerData(0,2) = (double) 3;
   KerData(1,0) = (double) 3;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 3;
   KerData(2,0) = (double)-5;
   KerData(2,1) = (double)-5;
   KerData(2,2) = (double)-5;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_49B()
{
   ker_name = AnsiString("Kirsch Max (mask 2)");
   Resize(3,3);
   KerData(0,0) = (double) 3;
   KerData(0,1) = (double) 3;
   KerData(0,2) = (double) 3;
   KerData(1,0) = (double) 3;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-5;
   KerData(2,0) = (double) 3;
   KerData(2,1) = (double)-5;
   KerData(2,2) = (double)-5;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_49C()
{
   ker_name = AnsiString("Kirsch Max (mask 3)");
   Resize(3,3);
   KerData(0,0) = (double) 3;
   KerData(0,1) = (double) 3;
   KerData(0,2) = (double)-5;
   KerData(1,0) = (double) 3;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-5;
   KerData(2,0) = (double) 3;
   KerData(2,1) = (double) 3;
   KerData(2,2) = (double)-5;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_49D()
{
   ker_name = AnsiString("Kirsch Max (mask 4)");
   Resize(3,3);
   KerData(0,0) = (double) 3;
   KerData(0,1) = (double)-5;
   KerData(0,2) = (double)-5;
   KerData(1,0) = (double) 3;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double)-5;
   KerData(2,0) = (double) 3;
   KerData(2,1) = (double) 3;
   KerData(2,2) = (double) 3;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_49E()
{
   ker_name = AnsiString("Kirsch Max (mask 5)");
   Resize(3,3);
   KerData(0,0) = (double)-5;
   KerData(0,1) = (double)-5;
   KerData(0,2) = (double)-5;
   KerData(1,0) = (double) 3;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 3;
   KerData(2,0) = (double) 3;
   KerData(2,1) = (double) 3;
   KerData(2,2) = (double) 3;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_49F()
{
   ker_name = AnsiString("Kirsch Max (mask 6)");
   Resize(3,3);
   KerData(0,0) = (double)-5;
   KerData(0,1) = (double)-5;
   KerData(0,2) = (double) 3;
   KerData(1,0) = (double)-5;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 3;
   KerData(2,0) = (double) 3;
   KerData(2,1) = (double) 3;
   KerData(2,2) = (double) 3;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_49G()
{
   ker_name = AnsiString("Kirsch Max (mask 7)");
   Resize(3,3);
   KerData(0,0) = (double)-5;
   KerData(0,1) = (double) 3;
   KerData(0,2) = (double) 3;
   KerData(1,0) = (double)-5;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 3;
   KerData(2,0) = (double)-5;
   KerData(2,1) = (double) 3;
   KerData(2,2) = (double) 3;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::FILTER_49H()
{
   ker_name = AnsiString("Kirsch Max (mask 8)");
   Resize(3,3);
   KerData(0,0) = (double) 3;
   KerData(0,1) = (double) 3;
   KerData(0,2) = (double) 3;
   KerData(1,0) = (double)-5;
   KerData(1,1) = (double) 0;
   KerData(1,2) = (double) 3;
   KerData(2,0) = (double)-5;
   KerData(2,1) = (double)-5;
   KerData(2,2) = (double) 3;
   absolute = 0;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------

// Low-Pass Spatial Filtres
void
Kernel::LP1()
{
   ker_name = AnsiString("Low-Pass Spetial Filter LP1");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++)
         KerData(c,r) = (double)1/9;
   absolute = 1;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::LP2()
{
   ker_name = AnsiString("Low-Pass Spetial Filter LP2");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++)
         KerData(c,r) = (double)1/10;
   KerData(1,1) = (double)1/5;
   absolute = 1;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::LP3()
{
   ker_name = AnsiString("Low-Pass Spetial Filter LP3");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         if((r==1)||(c==1))
            KerData(c,r) = (double)1/8;
         else
            KerData(c,r) = (double)1/16;
      }
   KerData(1,1) = (double)1/4;
   absolute = 1;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
// High-Pass Spatial Filtres
void
Kernel::HP1()
{
   ker_name = AnsiString("High-Pass Spetial Filter HP1");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++)
         KerData(c,r) = (double)-1;
   KerData(1,1) = (double)9;
   absolute = 1;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::HP2()
{
   ker_name = AnsiString("High-Pass Spetial Filter HP2");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         if((r==1)||(c==1))
            KerData(c,r) = (double)-1;
         else
            KerData(c,r) = (double)0;
      }
   KerData(1,1) = (double)5;
   absolute = 1;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::HP3()
{
   ker_name = AnsiString("High-Pass Spetial Filter HP3");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         if((r==1)||(c==1))
            KerData(c,r) = (double)-2;
         else
            KerData(c,r) = (double)1;
      }
   KerData(1,1) = (double)5;
   absolute = 1;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
//Shift and Difference Edge Enhancements
void
Kernel::SDVerticalEdges()
{
   ker_name = AnsiString("Vertical Shift and Diff. Edge Ehanc.");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)0;
      }
   KerData(0,1) = (double)-1;
   KerData(1,1) = (double) 1;
   absolute = true;
   sum =0;
   scale = 0.2;
}
//---------------------------------------------------------------------------
void
Kernel::SDHorizontalEdges()
{
   ker_name = AnsiString("Horizontal Shift and Diff. Edge Ehanc.");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)0;
      }
   KerData(1,0) = (double)-1;
   KerData(1,1) = (double) 1;
   absolute = true;
   sum =0;
   scale = 0.2;
}
//---------------------------------------------------------------------------
void
Kernel::SDHAndVEdges()
{
   ker_name = AnsiString("Vert. and Horiz. Shift and Diff. Edge Ehanc.");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)0;
      }
   KerData(0,0) = (double)-1;
   KerData(1,1) = (double) 1;
   absolute = 1;
   sum =0;
   scale = 0.2;
}
//---------------------------------------------------------------------------
// Matched Filter Edge Enhancements
void
Kernel::MVerticalEdges()
{
   ker_name = AnsiString("Vertical Matched Filter Edge Enhanc.");
   Resize(3,5);
   for(int c(0); c<3; c++)
      for(int r(0); r<5; r++){
         if(c==0)
            KerData(c,r) = (double)-1;
         if(c==1)
            KerData(c,r) = (double) 0;
         if(c==2)
            KerData(c,r) = (double) 1;
      }
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::MHorizontalEdges()
{
   ker_name = AnsiString("Horizontal Matched Filter Edge Enhanc.");
   Resize(5,3);
   for(int c(0); c<5; c++)
      for(int r(0); r<3; r++){
         if(r==0)
            KerData(c,r) = (double)-1;
         if(r==1)
            KerData(c,r) = (double) 0;
         if(r==2)
            KerData(c,r) = (double) 1;
      }
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
// Gradient Diferenctional Edge Enhancemets
void
Kernel::North()
{
   ker_name = AnsiString("Gradient Diff Edge Enhanc. North");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         if(r==2)
            KerData(c,r) = (double)-1;
         else
            KerData(c,r) = (double) 1;
      }
   KerData(1,1) = (double)-2;
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::NorthEast()
{
   ker_name = AnsiString("Gradient Diff Edge Enhanc. NorthEast");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
            KerData(c,r) = (double) 1;
      }
   KerData(1,1) = (double)-2;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;
   KerData(1,2) = (double)-1;
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::East()
{
   ker_name = AnsiString("Gradient Diff Edge Enhanc. East");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
            KerData(c,r) = (double) 1;
      }
   KerData(1,1) = (double)-2;
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(0,2) = (double)-1;   KerData(1,1) = (double)-2;
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::SouthEast()
{
   ker_name = AnsiString("Gradient Diff Edge Enhanc. SouthEast");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
            KerData(c,r) = (double) 1;
      }
   KerData(1,1) = (double)-2;
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-1;
   KerData(1,0) = (double)-1;
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::South()
{
   ker_name = AnsiString("Gradient Diff Edge Enhanc. South");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         if(r==0)
            KerData(c,r) = (double) -1;
         else
            KerData(c,r) = (double)  1;
      }
   KerData(1,1) = (double)-2;
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::SouthWest()
{
   ker_name = AnsiString("Gradient Diff Edge Enhanc. SouthWest");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
            KerData(c,r) = (double) 1;
      }
   KerData(1,1) = (double)-2;
   KerData(1,0) = (double)-1;
   KerData(2,0) = (double)-1;
   KerData(2,1) = (double)-1;
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::West()
{
   ker_name = AnsiString("Gradient Diff Edge Enhanc. West");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         if(c==2)
            KerData(c,r) = (double) -1;
         else
            KerData(c,r) = (double) 1;
      }
   KerData(1,1) = (double) -2;
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::NorthWest()
{
   ker_name = AnsiString("Gradient Diff Edge Enhanc. NorthWest");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
            KerData(c,r) = (double) 1;
      }
   KerData(1,1) = (double)-2;
   KerData(1,2) = (double)-1;
   KerData(2,2) = (double)-1;
   KerData(2,1) = (double)-1;
   absolute = true;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
// Bluring Kernels
void
Kernel::Blur3x3()
{
   ker_name = AnsiString("Bluring Kernel 3x3");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
            KerData(c,r) = (double) 1;
      }
   absolute = true;
   sum = 9;
   scale = 9;
}
//---------------------------------------------------------------------------
void
Kernel::Blur5x5()
{
   ker_name = AnsiString("Bluring Kernel 5x5");
   Resize(5,5);
   for(int c(0); c<5; c++)
      for(int r(0); r<5; r++){
            KerData(c,r) = (double) 1;
      }
   absolute = true;
   sum = 25;
   scale = 25;

}
//---------------------------------------------------------------------------
// Laplace Edge Enhancemets
void
Kernel::LAP1()
{
   ker_name = AnsiString("Laplace Edge Enhacement LAP1");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         if((r==1)||(c==1))
            KerData(c,r) = (double)1;
         else
            KerData(c,r) = (double)0;
      }
   KerData(1,1) = (double)-4;
   absolute = 1;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::LAP2()
{
   ker_name = AnsiString("Laplace Edge Enhacement LAP2");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)-1;
      }
   KerData(1,1) = (double)8;
   absolute = 1;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::LAP3()
{
   ker_name = AnsiString("Laplace Edge Enhacement LAP3");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)-1;
      }
   KerData(1,1) = (double)9;
   absolute = 1;
   sum = 1;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::LAP4()
{
   ker_name = AnsiString("Laplace Edge Enhacement LAP4");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         if((r==1)||(c==1))
            KerData(c,r) = (double)-2;
         else
            KerData(c,r) = (double)1;
      }
   KerData(1,1) = (double)4;
   absolute = 1;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::HorizontalSobel()
{
   ker_name = AnsiString("Horizontal Sobel Filter");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)0;
      }
   KerData(0,0) = (double)-1;
   KerData(1,0) = (double)-2;
   KerData(2,0) = (double)-1;
   KerData(0,2) = (double)1;
   KerData(1,2) = (double)2;
   KerData(2,2) = (double)1;

   absolute = 1;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::VerticalSobel()
{
   ker_name = AnsiString("Vertical Sobel Filter");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)0;
      }
   KerData(0,0) = (double)-1;
   KerData(0,1) = (double)-2;
   KerData(0,2) = (double)-1;
   KerData(2,0) = (double)1;
   KerData(2,1) = (double)2;
   KerData(2,2) = (double)1;
   absolute = 1;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
// Line detection
void
Kernel::LineHorizontal()
{
   ker_name = AnsiString("Horizontal Line Detection");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)-1;
      }
   KerData(0,1) = (double)2;
   KerData(1,1) = (double)2;
   KerData(2,1) = (double)2;
   absolute = 1;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::LineVertical()
{
   ker_name = AnsiString("Vertical Line Detection");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)-1;
      }
   KerData(1,2) = (double)2;
   KerData(1,1) = (double)2;
   KerData(1,0) = (double)2;
   absolute = 1;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::Line_45()
{
   ker_name = AnsiString("45 Line Detection");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)-1;
      }
   KerData(0,2) = (double)2;
   KerData(1,1) = (double)2;
   KerData(2,0) = (double)2;
   absolute = 1;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------
void
Kernel::Line_135()
{
   ker_name = AnsiString("-45 Line Detection");
   Resize(3,3);
   for(int c(0); c<3; c++)
      for(int r(0); r<3; r++){
         KerData(c,r) = (double)-1;
      }
   KerData(0,0) = (double)2;
   KerData(1,1) = (double)2;
   KerData(2,2) = (double)2;
   absolute = 1;
   sum = 0;
   scale = 1;
}
//---------------------------------------------------------------------------

