/*
   dig.cpp

   definin soubor tdy pro Vstup/vstup formt Dig.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "dig.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)


//---------------------------------------------------------------------------
__fastcall VDig::VDig(AnsiString _file_path)
   :VInOut(_file_path)
{
      image = new GreyImage();
      suffix = "dig";
      CheckSuffix();
      DigToGreyImage();
      handler = new BMPHandler();
}
//---------------------------------------------------------------------------
// destructor
__fastcall VDig::~VDig()
{
   delete (handler);
}
//---------------------------------------------------------------------------
void
__fastcall VDig::DigToGreyImage()
{
   unsigned long index = 11;
   TFileStream* fstream = new TFileStream(file_path, fmOpenRead);
   long size = fstream->Size;
   unsigned char* buff = new unsigned char[size];
   int t = fstream->Read(buff,size);
   delete (fstream);
   //checking MAGIC which is always 474 (1DAx0) for SGI files
   // STORAGE  01h for RLE comprimation, 00h uncomprimated data
   // BPC 01h as 1 byte per pixel
   if(buff[0] != 0x50 || buff[1] != 0x48 || buff[2] != 0x53 ||
      buff[3] != 0x2D|| buff[4] != 0x41 || buff[5] != 0x72 ){
      delete [] buff;
      throw VECorruptedFile(AnsiString("DIF file is corrupted "),AnsiString("dig"));
   }
   if(buff[6] != 0x72){
      delete [] buff;
      throw VECorruptedFile(AnsiString("MIPS doesn't support compressed DIG files"),AnsiString("dig"));
   }
   int cols = (int)buff[7] + 256*(int)buff[8];
   int rows = (int)buff[9] + 256*(int)buff[10];
   // Checking size of the file
   if((cols*rows+11) != size){
      delete [] buff;
      throw VECorruptedFile(AnsiString("DIG file is corrupted."),AnsiString("dig"));
   }
   image->Resize(cols, rows);
   for(register int r(0); r<image->Rows(); r++)
      for(register int c(0); c<image->Cols(); c++){
          image->Data(c,r) = buff[index+c+r*cols];
      }
   delete [] buff;
}
//---------------------------------------------------------------------------
void
__fastcall VDig::SaveToFile(AnsiString _save_file_name)
{

   ofstream o_file(_save_file_name.c_str(),ios::binary);
   Assert(o_file!=0,VEMsg(AnsiString("Can't open file\n" + _save_file_name)));
   o_file << " ";
   o_file.close();
   TFileStream* fstream = new TFileStream(_save_file_name, fmOpenWrite);
   unsigned char header[11] = {
			0x50,0x48,0x53,0x2d,0x41,0x72,0x72,0,0,0,0};

//                         {80,72,83,45,65,114,114};

   SizeBMP_G sizes(image->Cols(), image->Rows());
   header[8] = sizes.width_h;
   header[7] = sizes.width_l;
   header[10] = sizes.height_h;
   header[9] = sizes.height_l;
   fstream->Write(header, 11);
   for(register int r(0); r<image->Rows(); r++){
      for(register int c(0); c<image->Cols(); c++){
         fstream->Write(&image->Data(c,r),1);
      }
   }
   delete (fstream);
}
//---------------------------------------------------------------------------
void
__fastcall VDig::Assign(BMPHandler* _handler)
{
   *handler = *_handler;
   handler->Set(*image);
}