/*
   image.h

   hlavikov soubor zkladn ablonov tdy pro alokaci matice obrazu.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#ifndef imageH
#define imageH
//---------------------------------------------------------------------------
#include "assert.h"
#include "vexcept.h"


template <class T>
class Image
{
protected:
   T* data;
   int rows;
   int cols;
public:
   void Resize(int _cols, int _rows);
   // constructors
   Image();
   Image(int _cols, int _rows);
   // copy constructor
   Image(const Image& right);
   // destructor
   virtual ~Image();
   // member functions
   // non-const object
   int       Rows()       {return rows;}
   int       Cols()       {return cols;}
   // const object
   const int Rows() const {return rows;}
   const int Cols() const {return cols;}
   // non-const object
   T&       Data(int col, int row)
       { return data[col + cols*row];}
   T&       Data(long unsigned index)
       { return data[index];}
   // const object
   const T& Data(int col, int row) const
       { return data[col + cols*row];}
   const T& Data(long unsigned index) const
       { return data[index];}
   // overloaded member functions
   Image& operator = (const Image& right);
};

//---------------------------------------------------------------------------
// member function declared as a private function, used for allocation
// image data array
template <class T>
void
Image<T>::Resize(int _cols, int _rows)
{
   if (data!=NULL)
      delete [] data;
   data = new T[_rows*_cols];
   Assert(data, VEMsg(AnsiString("low on memory")));
   rows = _rows;
   cols = _cols;
}

//---------------------------------------------------------------------------
// default constructor
template <class T>
Image<T>::Image()
:rows(0),cols(0)
{
   data = NULL;
}

//---------------------------------------------------------------------------
// 2 argument constructor
template <class T>
Image<T>::Image(int _cols, int _rows)
{
   data = NULL;
   Resize(_cols,_rows);
   rows = _rows;
   cols = _cols;
}

//---------------------------------------------------------------------------
// copy constructor
template <class T>
Image<T>::Image(const Image& right)
{
   data = NULL;
   Resize(right.cols, right.rows);
   rows = right.rows;
   cols = right.cols;
   for(register int i(0); i<rows*cols; i++)
      data[i] = right.data[i];
}

//---------------------------------------------------------------------------
// destructor
template <class T>
Image<T>::~Image()
{
   delete [] data;
}



// overloaded operators
//---------------------------------------------------------------------------
// overloaded operator of assignment
template <class T>
Image<T>&
Image<T>::operator = (const Image& right)
{
  /* Assert((rows==right.rows)&&(cols==right.cols),
      VESize(AnsiString("Image<T> objects must be of the same size.")));*/
   Resize(right.Cols(), right.Rows());   
   for(register int i(0); i<rows*cols; i++)
     data[i] = right.data[i];
   return *this;
}

#endif
