//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "lut_base.h"


//---------------------------------------------------------------------------
#pragma package(smart_init)

//---------------------------------------------------------------------------
// default constructor
LUTBase::LUTBase()
: width(0), height(0), Image(NULL), ImageBack(NULL), Panel(NULL),
  Canvas(NULL)
{
   lut = new LUT();
}
//---------------------------------------------------------------------------
// destructor
LUTBase::~LUTBase()
{
   delete (lut);
  /* for(int i(0); i<10; i++)
     delete (labels[i]);*/
   //delete [] labels;
}

//---------------------------------------------------------------------------
// main constructor
LUTBase::LUTBase(TImage* _Image, TPanel* _Panel)
 :Image(_Image), Panel(_Panel)
{
   width = Image->ClientWidth;
   height = Image->ClientHeight;
   lwidth = width-2*differ;
   lheight = height-2*differ;
   LeftTop.X() = differ;
   LeftTop.Y() = differ;
   RightDown.X() = width-differ;
   RightDown.Y() = height-differ;
   Canvas = Image->Canvas;
   lut = new LUT();

   //labels = new TLabel*[10];
}
//---------------------------------------------------------------------------
// DrawFrame() draws a frame around all Canvas + web + all Labels.
void
LUTBase::DrawFrame()
{
   Canvas->Brush->Color=clWhite;
   Canvas->FillRect(TRect(0,0,Image->Width,Image->Height));
   Canvas->Pen->Color = clBlack;
   Canvas->Pen->Width = 4;
   Canvas->Rectangle(0,0,width+1,height+1);
   DrawWeb();
   //DrawLabels();
}
//---------------------------------------------------------------------------
// DrawWeb() draws a web of four vertical and four horizontal lines
void
LUTBase::DrawWeb(TColor color, TPenMode mode)
{
   DrawLine(VPoint(LeftTop.X(),LeftTop.Y()),
            VPoint(LeftTop.X(), RightDown.Y()), color,1,mode);
   DrawLine(VPoint(LeftTop.X(), RightDown.Y()),
            VPoint(RightDown.X(), RightDown.Y()), color,1,mode);
   DrawLine(VPoint(RightDown.X(), RightDown.Y()),
            VPoint(RightDown.X(), LeftTop.Y()), color,1,mode);
   DrawLine(VPoint(RightDown.X(), LeftTop.Y()),
            VPoint(LeftTop.X(), LeftTop.Y()), color,1,mode);
   float wdif = (width-2*differ+2)/4;
   float hdif = (height-2*differ+2)/4;
   for(int i(1); i<4; i++)
      DrawLine(VPoint(LeftTop.X(), LeftTop.Y()+hdif*i),
               VPoint(RightDown.X(),LeftTop.Y()+hdif*i), color,1,mode);
   for(int i(1); i<4; i++)
      DrawLine(VPoint(LeftTop.X()+wdif*i,LeftTop.Y()),
               VPoint(LeftTop.X()+wdif*i,RightDown.Y()), color,1,mode);
}
//---------------------------------------------------------------------------
void
LUTBase::DrawLabels(TLabel** labels)
{
   float wdif = (width-2*differ+2)/4;
   float hdif = (height-2*differ+2)/4;
   for(int i(0); i<5; i++){
      labels[i] = new TLabel(Image);
      labels[i]->Parent = Panel;
      labels[i]->Color = clWhite;
      labels[i]->Font->Color = clGray;
      labels[i]->Left = LeftTop.X() + i*wdif-i*3;
      labels[i]->Alignment = taCenter;
      labels[i]->Top = RightDown.Y() + 6;
      labels[i]->Caption = i*64;
      labels[i]->Visible = true;
      labels[i]->Show();
   }
   for(int i(5); i<10; i++){
      labels[i] = new TLabel(Image);
      labels[i]->Parent = Panel;
      labels[i]->Color = clWhite;
      labels[i]->Font->Color = clGray;
      labels[i]->Width = 12;
      labels[i]->Left = LeftTop.X()-17;
      labels[i]->Alignment = taRightJustify;
      labels[i]->Top = RightDown.Y() - (i-5)*hdif - 5;
      labels[i]->Caption = (i-5)*64;
      labels[i]->Visible = true;
      labels[i]->Show();
   }
}
//---------------------------------------------------------------------------
void
LUTBase::DrawLine(VPoint& start, VPoint& end,
                 TColor color, int size, TPenMode mode, TPenStyle style)
{
   TPenMode  pm = Canvas -> Pen -> Mode;
   TColor pc =  Canvas->Pen->Color;
   int psize = Canvas->Pen->Width;
   TPenStyle pstyle = Canvas->Pen->Style;

   Canvas->Pen->Color = color;
   Canvas->Pen->Width = size;
   Canvas->Pen->Mode = mode;
   Canvas->Pen->Style = style;
   Canvas->MoveTo(start.X(), start.Y());
   Canvas->LineTo(end.X(), end.Y());
   Canvas->Pen->Color = pc;
   Canvas->Pen->Width = psize;
   Canvas->Pen->Mode = pm;
   Canvas->Pen->Style = pstyle;

}
//---------------------------------------------------------------------------
void
LUTBase::DrawRect(VPoint& point, TColor color, int size)
{
   TPenMode  pm = Canvas -> Pen -> Mode;
   TRect rect = Rect(point.X()-size/2, point.Y()-size/2,
                     point.X()+size/2, point.Y()+size/2);
   TColor pc =  Canvas->Pen->Color;
   Canvas->Pen->Mode = pmXor;
   Canvas->Brush->Color = color;
   Canvas->Pen->Width = size;
   Canvas->FillRect(rect);
   Canvas->Brush->Color = pc;
   Canvas->Pen->Mode = pm;
}
//---------------------------------------------------------------------------
bool
LUTBase::IsOnLine(VPoint& start, VPoint& end, int X, int Y, int tolerance)
{
   VPoint n_start, n_end;
   if(start.X()>end.X()){
      n_end = start;
      n_start = end;
   }
   else{
      n_end = end;
      n_start = start;
   }
   if((X>n_end.X() + tolerance)||(X < start.X()- tolerance))
      return false;

   if(n_end.Y()>n_start.Y()){
      if((Y>n_end.Y() + tolerance)||(Y < start.Y()- tolerance))
         return false;
   }
   else{
      if((Y>n_start.Y() + tolerance)||(Y < end.Y()- tolerance))
         return false;
   }


   int delta_x = n_end.X() - n_start.X();
   int delta_y = n_start.Y() - n_end.Y();
   double k;
   if(delta_y==0)
      k=-2;
   else
      k = - (double)delta_x/(double)delta_y;
   if(delta_x==0)
      if(delta_y==0)
         return false;
   if(k<-1){
      k = -(double)delta_y/(double)delta_x;
      int q = n_start.Y() - k*(double)n_start.X();
      if ((Y <= (int)(k*(double)X + (double)q) + tolerance)&&
          (Y >= (int)(k*(double)X + (double)q) - tolerance))
         return true;
      else
         return false;
   }
   else{
      int q = n_start.X() - k*(double)n_start.Y();
      if ((X <= (int)(k*(double)Y + (double)q + tolerance))&&
          (X >= (int)(k*(double)Y + (double)q - tolerance)))
         return true;
      else
         return false;
   }
}
//---------------------------------------------------------------------------
bool
LUTBase::IsOnPoint(VPoint& point, int X, int Y, int tolerance)
{
   if((X > point.X() - tolerance)&&(X < point.X() + tolerance)&&
      (Y > point.Y() - tolerance)&&(Y < point.Y() + tolerance))
      return true;
   else
      return false;
}
//---------------------------------------------------------------------------
void
LUTBase::MoveTo(int X, int Y)
{
   Canvas->MoveTo(X,Y);
}
//---------------------------------------------------------------------------
void
LUTBase::LineTo(int X, int Y, TColor color, int size)
{
   TPenMode  pm = Canvas -> Pen -> Mode;
   TColor pc =  Canvas->Pen->Color;
   int psize = Canvas->Pen->Width;

   Canvas->Pen->Color = color ;
   Canvas->Pen->Width = size;
   //Canvas->Pen->Mode = pmXor;
   Canvas->LineTo(X, Y);
   Canvas->Pen->Color = pc;
   Canvas->Pen->Width = psize;
   Canvas->Pen->Mode = pm;
}
//---------------------------------------------------------------------------
void
LUTBase::KeepInBorder(VPoint& point)
{
   if(point.X() < LeftTop.X())
      point.X() = LeftTop.X();
   if(point.X() > RightDown.X())
      point.X() = RightDown.X();
   if(point.Y() < LeftTop.Y())
      point.Y() = LeftTop.Y();
   if(point.Y() > RightDown.Y())
      point.Y() = RightDown.Y();
}


