//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "lut_3l_threshold.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)



// constructor - sets the left-down and right-top points
//---------------------------------------------------------------------------
LUT3LThreshold::LUT3LThreshold(TImage* _Image, TPanel* _Panel)
      : LUTBase(_Image,_Panel)
{
    L.X() = LeftTop.X();
    L.Y() = LM1.Y() = LeftTop.Y()+lheight*3/4;
    R.X() = RightDown.X();
    R.Y() = RM1.Y() = LeftTop.Y()+lheight*3/4;
    LM1.X() = LM2.X() = LeftTop.X()+lwidth/4;
    RM1.X() = RM2.X() = LeftTop.X()+lwidth*3/4;
    LM2.Y() = RM2.Y() = LeftTop.Y()+lheight/4;
    Down() = false;
}

//---------------------------------------------------------------------------
// virtual function - just calls the DrawLine() function
void
LUT3LThreshold::DrawChart()
{
   DrawLine(L,LM1);
   DrawLine(LM1,LM2);
   DrawLine(LM2,RM2);
   DrawLine(RM2,RM1);
   DrawLine(RM1,R);
   DrawRect(LM1);
   DrawRect(LM2);
   DrawRect(RM1);
   DrawRect(RM2);
}

//---------------------------------------------------------------------------
// until the left button of the mose is pressed (i.e. down is equal to true)
// this function only changes the Cursor of the mouse. If the left button is
// pressed OnMouseMove() clears the old line and redraws the new one.
void
LUT3LThreshold::OnMouseMove(int X, int Y)
{
    if(Down()==false){
      Cursor() = crDefault;
      if(IsOnLine(L, LM1, X, Y, 3) == true)
         Cursor() = crSizeNS;
      if(IsOnLine(LM2, RM2, X, Y, 3) == true)
         Cursor() = crSizeNS;
      if(IsOnLine(RM1, R, X, Y, 3) == true)
         Cursor() = crSizeNS;
      if(IsOnLine(LM1, LM2, X, Y, 3) == true)
         Cursor() = crSizeWE;
      if(IsOnLine(RM1, RM2, X, Y, 3) == true)
         Cursor() = crSizeWE;
      if(IsOnPoint(LM1, X, Y, 5) == true)
         Cursor() = crCross;
      if(IsOnPoint(RM1, X, Y, 5) == true)
         Cursor() = crCross;
      if(IsOnPoint(LM2, X, Y, 5) == true)
         Cursor() = crCross;
      if(IsOnPoint(RM2, X, Y, 5) == true)
         Cursor() = crCross;
   }
   else{
      DrawLine(L,LM1);
      DrawLine(LM1,LM2);
      DrawLine(LM2,RM2);
      DrawLine(RM2,RM1);
      DrawLine(RM1,R);
      DrawRect(LM1,clWhite);
      DrawRect(LM2,clWhite);
      DrawRect(RM1,clWhite);
      DrawRect(RM2,clWhite);
      // set new positions
      SetNewPoints(X, Y, L, LM1, LM2, R, RM1, RM2);
      DrawWeb();
      DrawLine(L,LM1);
      DrawLine(LM1,LM2);
      DrawLine(LM2,RM2);
      DrawLine(RM2,RM1);
      DrawLine(RM1,R);
      DrawRect(LM1);
      DrawRect(LM2);
      DrawRect(RM1);
      DrawRect(RM2);
   }
}

//---------------------------------------------------------------------------
void
LUT3LThreshold::OnMouseDown(TMouseButton Button, int X, int Y)
{
   if(Button == mbLeft){
      if(IsOnLine(L, LM1, X, Y, 3) == true){
         Cursor() = crSizeNS;
         Down() = true;
         which_drag = _L_LM1;
      }
      if(IsOnLine(LM1, LM2, X, Y, 3) == true){
         Cursor() = crSizeWE;
         Down() = true;
         which_drag = _LM1_LM2;
      }
      if(IsOnLine(LM2, RM2, X, Y, 3) == true){
         Cursor() = crSizeNS;
         Down() = true;
         which_drag = _LM2_RM2;
      }
      if(IsOnLine(RM1, R, X, Y, 3) == true){
         Cursor() = crSizeNS;
         Down() = true;
         which_drag = _RM1_R;
      }
      if(IsOnLine(LM1, LM2, X, Y, 3) == true){
         Cursor() = crSizeWE;
         Down() = true;
         which_drag = _LM1_LM2;
      }
      if(IsOnLine(RM1, RM2, X, Y, 3) == true){
         Cursor() = crSizeWE;
         Down() = true;
         which_drag = _RM2_RM1;
      }
      if(IsOnPoint(LM1, X, Y, 4) == true){
         Cursor() = crCross;
         Down() = true;
         which_drag = _LM1;
      }
      if(IsOnPoint(LM2, X, Y, 4) == true){
         Cursor() = crCross;
         Down() = true;
         which_drag = _LM2;
      }
      if(IsOnPoint(RM1, X, Y, 4) == true){
         Cursor() = crCross;
         Down() = true;
         which_drag = _RM1;
      }
      if(IsOnPoint(RM2, X, Y, 4) == true){
         Cursor() = crCross;
         Down() = true;
         which_drag = _RM2;
      }
   }
}

//---------------------------------------------------------------------------
void
LUT3LThreshold::OnMouseUp(TMouseButton Button, int X, int Y)
{
  if(Down()){
      Down() = false;
      Cursor() = crDefault;
   }
}
//---------------------------------------------------------------------------
// This function checks if the cursor isn't out of border!
// This function is renewing the LUT as well.
void
LUT3LThreshold::SetNewPoints(int X, int Y, VPoint& L, VPoint& LM1, VPoint& LM2,
                                           VPoint& R, VPoint& RM1, VPoint& RM2)
{
  switch(which_drag){
      case(0):           //_LM1
         LM1.X() = LM2.X() = X;
         LM1.Y() = L.Y() = Y;
         KeepInBorder(LM1);
         KeepInBorder(LM2);
         KeepInBorder(L);
         if(LM1.X() > RM1.X()-8)
            LM1.X() = RM1.X()-8;
         LM2.X() = LM1.X();
         break;
      case(1):           //_LM2
         LM2.X() = X;
         LM2.Y() = Y;
         KeepInBorder(LM2);
         RM2.Y() = LM2.Y();
         LM1.X() = LM2.X();
         if(LM2.X() > RM2.X()-8)
            LM2.X() = RM2.X()-8;
         LM1.X() = LM2.X();
         break;
      case(2):           //_RM1
         RM1.X() = X;
         RM1.Y() = Y;
         KeepInBorder(RM1);
         R.Y() = RM1.Y();
         if(RM1.X() < LM1.X()+8)
            RM1.X() = LM1.X()+8;
         RM2.X() = RM1.X();
         break;
      case(3):           //_RM2
         RM2.X() = X;
         RM2.Y() = Y;
         KeepInBorder(RM2);
         LM2.Y() = RM2.Y();
         if(RM2.X() < LM2.X()+8)
            RM2.X() = LM2.X()+8;
         RM1.X() = RM2.X();
         break;
      case(4):           //L_LM1
         L.Y() = Y;
         KeepInBorder(L);
         LM1.Y() = L.Y();
         break;
      case(5):           //_LM1_LM2
         LM1.X() = X;
         KeepInBorder(LM1);
         if(LM1.X() > RM1.X()-8)
            LM1.X() = RM1.X()-8;
         LM2.X() = LM1.X();
         break;
      case(6):           //_LM2_RM2
         LM2.Y() = Y;
         KeepInBorder(LM2);
         RM2.Y() = LM2.Y();
         break;
      case(7):           //_RM2_RM1
         RM2.X() = X;
         KeepInBorder(RM2);
         if(RM2.X() < LM2.X()+8)
            RM2.X() = LM2.X()+8;
         RM1.X() = RM2.X();
         break;
      case(8):           //_RM1_R
         RM1.Y() = Y;
         KeepInBorder(RM1);
         R.Y() = RM1.Y();
         break;
   }
   for(int index(0); index<256; index++){
      if(index<=(LM1.X()-LeftTop.X())*256/lwidth)
         Lut(index) = (RightDown.Y()-LM1.Y())*255/lheight;
      if(((LM1.X()-LeftTop.X())*256/lwidth<index)&&
         (index<=(RM1.X()-LeftTop.X())*256/lwidth))
         Lut(index) = (RightDown.Y()-RM2.Y())*255/lheight;
      if(((RM1.X()-LeftTop.X())*256/lwidth<index))
         Lut(index) = (RightDown.Y()-RM1.Y())*255/lheight;
   }
}
//---------------------------------------------------------------------------
void
LUT3LThreshold::KeepInBorder(VPoint& point)
{
   if(point.X() < LeftTop.X())
      point.X() = LeftTop.X();
   if(point.X() > RightDown.X())
      point.X() = RightDown.X();
   if(point.Y() < LeftTop.Y())
      point.Y() = LeftTop.Y();
   if(point.Y() > RightDown.Y())
      point.Y() = RightDown.Y();
}
