//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "lut_brightness.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

// constructor - sets the left-down and right-top points
//---------------------------------------------------------------------------
LUTBrightness::LUTBrightness(TImage* _Image, TPanel* _Panel)
      : LUTBase(_Image, _Panel)
{
    LD.X() = LeftTop.X();
    LD.Y() = RightDown.Y();
    RT.X() = RightDown.X();
    RT.Y() = LeftTop.Y();
    Down() = false;
}

//---------------------------------------------------------------------------
// virtual function - just calls the DrawLine() function
void
LUTBrightness::DrawChart()
{
   DrawLine(VPoint(LeftTop.X(),RightDown.Y()),LD);
   DrawLine(VPoint(RightDown.X(),LeftTop.Y()),RT);
   DrawLine(LD,RT);
   DrawRect(LD,clBlack,2);
   DrawRect(RT,clBlack,2);
}

//---------------------------------------------------------------------------
// until the left button of the mose is pressed (i.e. down is equal to true)
// this function only changes the Cursor of the mouse. If the left button is
// pressed OnMouseMove() clears the old line and redraws the new one.
void
LUTBrightness::OnMouseMove(int X, int Y)
{
   if(Down()==false){
      if(IsOnLine(LD, RT, X, Y, 3) == true)
         Cursor() = crSizeNWSE;
      else
         Cursor() = crDefault;
   }
   else{
      Cursor() = crSizeNWSE;
      DrawLine(VPoint(LeftTop.X(),RightDown.Y()),LD);
      DrawLine(VPoint(RightDown.X(),LeftTop.Y()),RT);
      DrawLine(LD,RT);
      DrawRect(LD,clWhite,2);
      DrawRect(RT,clWhite,2);
      SetNewPoints(X, Y, LD, RT);
      DrawWeb();
      DrawLine(VPoint(LeftTop.X(),RightDown.Y()),LD);
      DrawLine(VPoint(RightDown.X(),LeftTop.Y()),RT);
      DrawLine(LD,RT);
      DrawRect(LD,clBlack,2);
      DrawRect(RT,clBlack,2);


   }
}

//---------------------------------------------------------------------------
void
LUTBrightness::OnMouseDown(TMouseButton Button, int X, int Y)
{
   if(Button == mbLeft){
      if(IsOnLine(LD, RT, X, Y, 3) == true)
         Down() = true;
   }
}

//---------------------------------------------------------------------------
void
LUTBrightness::OnMouseUp(TMouseButton Button, int X, int Y)
{
   if(Down()){
      Down() = false;
      Cursor() = crDefault;
   }
}
//---------------------------------------------------------------------------
// This function checks if the cursor isn't out of border!
// This function is renewing the LUT as well.
void
LUTBrightness::SetNewPoints(int X, int Y, VPoint& LD, VPoint& RT)
{
   int q = X + Y;
   if (q < LeftTop.X()+LeftTop.Y()+10){
      LD.X() = LeftTop.X();
      LD.Y() = LeftTop.Y()+10;
      RT.Y() = LeftTop.Y();
      RT.X() = LeftTop.X()+10;
      for(int i(0); i<256; i++)
         Lut(i) = 255;
   }
   else{
      if(q > RightDown.X()+RightDown.Y()-10){
         LD.Y() = RightDown.Y();
         LD.X() = RightDown.X()-10;
         RT.Y() = RightDown.Y()-10;
         RT.X() = RightDown.X();
         for(int i(0); i<256; i++)
            Lut(i) = 0;
      }
      else{
         if (q <= RightDown.X()+LeftTop.Y()){
            LD.X() = LeftTop.X();
            LD.Y() = q - LeftTop.Y();
            RT.Y() = LeftTop.Y();
            RT.X() = q - LeftTop.Y();
            for(int i(0); i<256; i++){
               if(i < (q - 2*LeftTop.X())*255/lwidth)
                  Lut(i) = (256 - (q - 2*LeftTop.X())*255/lwidth) +  i;
               else
                  Lut(i) = 255;
            }
         }
         else{
            if(q > RightDown.X()+LeftTop.Y()){
               LD.X() = q - RightDown.X();
               LD.Y() = RightDown.Y();
               RT.Y() = q - RightDown.X();
               RT.X() = RightDown.X();
               for(int i(0); i<256; i++){
                  if(i < (q - RightDown.X()-LeftTop.Y())*255/lheight)
                     Lut(i) = 0;
                  else
                     Lut(i) = -(q- RightDown.X()-LeftTop.Y())*255/lheight + i ;
               }
            }
         }
      }
   }
}

