//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "lut_fly_belt.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
// constructor - sets the left-down and right-top points
//---------------------------------------------------------------------------
LUTFlyBelt::LUTFlyBelt(TImage* _Image, TPanel* _Panel)
      : LUTBase(_Image, _Panel)
{
    LD.X() = LeftTop.X();
    LD.Y() = RightDown.Y();
    RT.X() = RightDown.X();
    RT.Y() = LeftTop.Y();
    LM.X() = LeftTop.X()+lwidth/3;
    LM.Y() = RightDown.Y()-lheight/5;
    RM.X() = RightDown.X()-lwidth/3;
    RM.Y() = LeftTop.Y()+lheight/5;
    Down() = false;
}

//---------------------------------------------------------------------------
// virtual function - just calls the DrawLine() function
void
LUTFlyBelt::DrawChart()
{
   DrawLine(LD,LM);
   DrawLine(LM,RM);
   DrawLine(RM,RT);
   DrawRect(LM);
   DrawRect(RM);
}

//---------------------------------------------------------------------------
// until the left button of the mose is pressed (i.e. down is equal to true)
// this function only changes the Cursor of the mouse. If the left button is
// pressed OnMouseMove() clears the old line and redraws the new one.
void
LUTFlyBelt::OnMouseMove(int X, int Y)
{
   if(Down()==false){
      Cursor() = crDefault;
      if(IsOnPoint(LM, X, Y, 4) == true)
         Cursor() = crCross;
      if(IsOnPoint(RM, X, Y, 4) == true)
         Cursor() = crCross;
   }
   else{
      Cursor() = crCross;
      switch(which_point){
         case(0): //_LM
            DrawLine(LD,LM);
            DrawLine(LM,RM);
            DrawLine(RM,RT);
            DrawRect(LM,clWhite);
            DrawRect(RM,clWhite);
            // new positions
            SetNewPoints(X, Y, LM, RM);
            DrawWeb();
            DrawLine(LD,LM);
            DrawLine(LM,RM);
            DrawLine(RM,RT);
            DrawRect(LM);
            DrawRect(RM);
            break;
         case(1): //_RM
            DrawLine(LD,LM);
            DrawLine(LM,RM);
            DrawLine(RM,RT);
            DrawRect(LM,clWhite);
            DrawRect(RM,clWhite);
            // new positions
            SetNewPoints(X, Y, LM, RM);
            DrawWeb();
            DrawLine(LD,LM);
            DrawLine(LM,RM);
            DrawLine(RM,RT);
            DrawRect(LM);
            DrawRect(RM);
            break;
      }
   }
}

//---------------------------------------------------------------------------
void
LUTFlyBelt::OnMouseDown(TMouseButton Button, int X, int Y)
{
   if(Button == mbLeft){
      if(IsOnPoint(LM, X, Y, 4) == true){
         Down() = true;
         which_point = _LM;
      }
      if(IsOnPoint(RM, X, Y, 4) == true){
         Down() = true;
         which_point = _RM;
      }
   }
}

//---------------------------------------------------------------------------
void
LUTFlyBelt::OnMouseUp(TMouseButton Button, int X, int Y)
{
   if(Down()){
      Down() = false;
      Cursor() = crDefault;
   }
}
//---------------------------------------------------------------------------
// This function checks if the cursor isn't out of border!
// This function is renewing the LUT as well.
void
LUTFlyBelt::SetNewPoints(int X, int Y, VPoint& LM, VPoint& RM)
{
   double corec_w = (double)255/(double)lwidth;
   double corec_h = (double)255/(double)lheight;
   switch(which_point){
      case(0):
         LM.X() = X;
         LM.Y() = Y;
         KeepInBorder(LM);
         if(LM.X() > RM.X())
            LM.X() = RM.X();
         break;
      case(1):
         RM.X() = X;
         RM.Y() = Y;
         KeepInBorder(RM);
         if(LM.X() > RM.X())
            RM.X() = LM.X();
         break;
   }
   for(register int index(0); index<256; index++){
      if(index< LM.X() - LeftTop.X())
         Lut(index) = index*corec_w*(double)(RightDown.Y()-LM.Y())/
                              (corec_h*(double)(LM.X()-LeftTop.X()));
      if(RM.X()!=LM.X())
         if((LM.X()-LeftTop.X()<index)&&(index<=RM.X()-LeftTop.X()))
            Lut(index)=(index-(LM.X()-LeftTop.X())*corec_w)*(double)(LM.Y()-RM.Y())/
                       (RM.X()-LM.X())+ (RightDown.Y()-LM.Y())*corec_h;
      if(RM.X()-LeftTop.X()<index)
         if(RightDown.X()!=RM.X())
            Lut(index)= ((double)(index-(RM.X()-LeftTop.X())*corec_w)*(double)(RM.Y()-LeftTop.Y())/
                       (double)(RightDown.X()-RM.X())+(RightDown.Y()-RM.Y())*corec_h);
   }
}

//---------------------------------------------------------------------------

void
LUTFlyBelt::KeepInBorder(VPoint& point)
{
   if(point.X() < LeftTop.X())
      point.X() = LeftTop.X();
   if(point.X() > RightDown.X())
      point.X() = RightDown.X();
   if(point.Y() < LeftTop.Y())
      point.Y() = LeftTop.Y();
   if(point.Y() > RightDown.Y())
      point.Y() = RightDown.Y();
}
