//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "lut_threshold.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)


// constructor - sets the left-down and right-top points
//---------------------------------------------------------------------------
LUTThreshold::LUTThreshold(TImage* _Image, TPanel* _Panel)
      : LUTBase(_Image, _Panel)
{
    LD.X() = LeftTop.X();
    LD.Y() = RightDown.Y();
    RT.X() = RightDown.X();
    RT.Y() = LeftTop.Y();
    D.X() = LeftTop.X()+lwidth/2;
    D.Y() = RightDown.Y();
    T.X() = LeftTop.X()+lwidth/2;
    T.Y() = LeftTop.Y();
    Down() = false;
}

//---------------------------------------------------------------------------
// virtual function - just calls the DrawLine() function
void
LUTThreshold::DrawChart()
{
   DrawLine(LD,D);
   DrawLine(D,T);
   DrawLine(T,RT);
   DrawRect(D,clBlack,2);
   DrawRect(T,clBlack,2);
}

//---------------------------------------------------------------------------
// until the left button of the mose is pressed (i.e. down is equal to true)
// this function only changes the Cursor of the mouse. If the left button is
// pressed OnMouseMove() clears the old line and redraws the new one.
void
LUTThreshold::OnMouseMove(int X, int Y)
{
   if(Down()==false){
      if(IsOnLine(D, T, X, Y, 3) == true)
         Cursor() = crSizeWE;
      else
         Cursor() = crDefault;
   }
   else{
      Cursor() = crSizeWE;
      DrawLine(LD,D);
      DrawLine(D,T);
      DrawLine(T,RT);
      DrawRect(D,clWhite,2);
      DrawRect(T,clWhite,2);
      SetNewPoints(X, Y, D, T);
      DrawWeb();
      DrawLine(LD,D);
      DrawLine(D,T);
      DrawLine(T,RT);
      DrawRect(D,clBlack,2);
      DrawRect(T,clBlack,2);
   }
}

//---------------------------------------------------------------------------
void
LUTThreshold::OnMouseDown(TMouseButton Button, int X, int Y)
{
   if(Button == mbLeft){
      if(IsOnLine(D, T, X, Y, 3) == true)
         Down() = true;
   }
}

//---------------------------------------------------------------------------
void
LUTThreshold::OnMouseUp(TMouseButton Button, int X, int Y)
{
   if(Down()){
      Down() = false;
      Cursor() = crDefault;
   }
}
//---------------------------------------------------------------------------
// This function checks if the cursor isn't out of border!
// This function is renewing the LUT as well.
void
LUTThreshold::SetNewPoints(int X, int Y, VPoint& D, VPoint& T)
{
   if(X < LeftTop.X())
      X = LeftTop.X();
   if(X > RightDown.X())
      X = RightDown.X();
   D.X() = T.X() = X;
   for(register int i(0); i<256; i++){
      if(i < (X-LeftTop.X())*256/(RightDown.X()-LeftTop.X()))
        Lut(i) = 0;
      else
        Lut(i) = 255;
   }
}

