//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "lut_threshold_equal_1.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

// constructor - sets the left-down and right-top points
//---------------------------------------------------------------------------
LUTThresholdEqual_1::LUTThresholdEqual_1(TImage* _Image, TPanel* _Panel)
      : LUTBase(_Image, _Panel)
{
   LD.X() = LeftTop.X();
   LD.Y() = RightDown.Y();
   RT.X() = RightDown.X();
   RT.Y() = LeftTop.Y();
   LM.X() = LeftTop.X() + lwidth/3;
   LM.Y() = RightDown.Y()-(LM.X()-LeftTop.X())*lheight/lwidth;
   LMT.X() = LM.X();
   LMT.Y() = LeftTop.Y();
   RM.X() = LeftTop.X() + 2*lwidth/3;
   RM.Y() = RightDown.Y()-(RM.X()-LeftTop.X())*lheight/lwidth;
   RMT.X() = RM.X();
   RMT.Y() = LeftTop.Y();
   RMD.Y() = LMD.Y() = RightDown.Y();
   RMD.X() = RM.X();
   LMD.X() = LM.X();
   Down() = false;
}

//---------------------------------------------------------------------------
// virtual function - just calls the DrawLine() function
void
LUTThresholdEqual_1::DrawChart()
{
   DrawWeb();
   DrawLine(LD,LM);
   DrawLine(LM,LMT);
   DrawLine(LMT,RMT);
   DrawLine(RMT,RM);
   DrawLine(RM,RT);
   DrawLine(RMD,RM,clBlack,1,pmCopy,psDot);
   DrawLine(LMD,LM,clBlack,1,pmCopy,psDot);
   DrawRect(LM,clBlack,2);
   DrawRect(LMT,clBlack,2);
   DrawRect(RMT,clBlack,2);
   DrawRect(RM,clBlack,2);
}

//---------------------------------------------------------------------------
// until the left button of the mose is pressed (i.e. down is equal to true)
// this function only changes the Cursor of the mouse. If the left button is
// pressed OnMouseMove() clears the old line and redraws the new one.
void
LUTThresholdEqual_1::OnMouseMove(int X, int Y)
{
   if(Down()==false){
      Cursor() = crDefault;
      if((LM.X()<X)&&(X<RM.X()))
         Cursor() = crCross;
      if(IsOnLine(LMD, LMT, X, Y, 3) == true)
         Cursor() = crSizeWE;
      if(IsOnLine(RMD, RMT, X, Y, 3) == true)
         Cursor() = crSizeWE;
   }
   else{
      DrawLine(LD,LM);
      DrawLine(LM,LMT);
      DrawLine(LMT,RMT);
      DrawLine(RMT,RM);
      DrawLine(RM,RT);
      DrawLine(RMD,RM,clWhite,1,pmCopy);
      DrawLine(LMD,LM,clWhite,1,pmCopy);
      DrawRect(LM,clWhite,2);
      DrawRect(LMT,clWhite,2);
      DrawRect(RMT,clWhite,2);
      DrawRect(RM,clWhite,2);
      // set new positions
      SetNewPoints(X, Y);
      DrawWeb();
      DrawLine(LD,LM);
      DrawLine(LM,LMT);
      DrawLine(LMT,RMT);
      DrawLine(RMT,RM);
      DrawLine(RM,RT);
      DrawLine(RMD,RM,clWhite,1,pmCopy,psDot);
      DrawLine(LMD,LM,clWhite,1,pmCopy,psDot);
      DrawRect(LM,clBlack,2);
      DrawRect(LMT,clBlack,2);
      DrawRect(RMT,clBlack,2);
      DrawRect(RM,clBlack,2);
   }
}

//---------------------------------------------------------------------------
void
LUTThresholdEqual_1::OnMouseDown(TMouseButton Button, int X, int Y)
{
   if(Button == mbLeft){
      if((LM.X()<X)&&(X<RM.X())){
         Cursor() = crCross;
         RMold = RM;
         LMold = LM;
         start = X;
         Down() = true;
         which_drag = _M;
      }
      if(IsOnLine(LMD, LMT, X, Y, 3) == true){
         Cursor() = crSizeWE;
         Down() = true;
         which_drag = _L;
      }
      if(IsOnLine(RMD, RMT, X, Y, 3) == true){
         Cursor() = crSizeWE;
         Down() = true;
         which_drag = _R;
      }
   }
}

//---------------------------------------------------------------------------
void
LUTThresholdEqual_1::OnMouseUp(TMouseButton Button, int X, int Y)
{
   if(Down()){
      Down() = false;
      Cursor() = crDefault;
   }
}
//---------------------------------------------------------------------------
// This function checks if the cursor isn't out of border!
// This function is renewing the LUT as well.
void
LUTThresholdEqual_1::SetNewPoints(int X, int Y)
{
   switch(which_drag){
      case(0):           //_L
         LM.X() = X;
         LM.Y() = RightDown.Y()-(LM.X()-LeftTop.X())*lheight/lwidth;
         KeepInBorder(LM);
         if(LM.X() > RM.X()-8)
            LM.X() = RM.X()-8;
         LMT.X() = LMD.X() = LM.X();
         LM.Y() = RightDown.Y()-(LM.X()-LeftTop.X())*lheight/lwidth;
         break;
      case(1):
         diff = RM.X() - LM.X();
         RM.X() = RMold.X() +  (X-start);
         RM.Y() = RightDown.Y()-(RM.X()-LeftTop.X())*lheight/lwidth;
         LM.X() = LMold.X() + (X-start);
         LM.Y() = RightDown.Y()-(LM.X()-LeftTop.X())*lheight/lwidth;
         //KeepInBorder(RM);
         //KeepInBorder(LM);
         if(RM.X() > RightDown.X()){
            RM.X() = RightDown.X();
            LM.X() = RM.X()-diff;
         }
         if(LM.X() < LeftTop.X()){
            LM.X() = LeftTop.X();
            RM.X() = LM.X()+diff;
         }
         LMT.X() = LMD.X() = LM.X();
         LM.Y() = RightDown.Y()-(LM.X()-LeftTop.X())*lheight/lwidth;
         RMT.X() = RMD.X() = RM.X();
         RM.Y() = RightDown.Y()-(RM.X()-LeftTop.X())*lheight/lwidth;

         break;
      case(2):           //_R
         RM.X() = X;
         RM.Y() = RightDown.Y()-(RM.X()-LeftTop.X())*lheight/lwidth;
         KeepInBorder(RM);
         if(RM.X() < LM.X()+8)
            RM.X() = LM.X()+8;
         RMT.X() = RMD.X() = RM.X();
         RM.Y() = RightDown.Y()-(RM.X()-LeftTop.X())*lheight/lwidth;
         break;
   }
   for(register int index(0); index<256; index++){
      if(index< LM.X() - LeftTop.X())
         Lut(index) = index*lwidth/lheight;
      if((LM.X()-LeftTop.X()<index)&&(index<=RM.X()-LeftTop.X()))
            Lut(index)=255;
      if(RM.X()-LeftTop.X()<=index)
         Lut(index) = index*lwidth/lheight;
   }
}
