/*
   sgi.cpp

   definin soubor tdy pro Vstup/vstup formt SGI.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "sgi.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

//---------------------------------------------------------------------------
__fastcall VSgi::VSgi(AnsiString _file_path)
   :VInOut(_file_path)
{
      image = new GreyImage();
      suffix = "sgi";
      CheckSuffix();
      SgiToGreyImage();
      handler = new BMPHandler();
}
//---------------------------------------------------------------------------
// destructor
__fastcall VSgi::~VSgi()
{
   delete (handler);
}
//---------------------------------------------------------------------------
void
__fastcall VSgi::SgiToGreyImage()
{
   unsigned long index = 512;
   TFileStream* fstream = new TFileStream(file_path, fmOpenRead);
   long size = fstream->Size;
   unsigned char* buff = new unsigned char[size];
   int t = fstream->Read(buff,size);
   delete (fstream);
   //checking MAGIC which is always 474 (1DAx0) for SGI files
   // STORAGE  01h for RLE comprimation, 00h uncomprimated data
   // BPC 01h as 1 byte per pixel
   if(buff[0] != 1 || buff[1] != 218 || buff[2] != 0 ||
      buff[3] != 1|| buff[4] != 0 || buff[5] != 3){
      delete [] buff;
      throw VECorruptedFile(AnsiString("SGI file is corrupted or not supported by MIPS."),AnsiString("sgi"));
   }
   int cols = (int)buff[7] + 256*(int)buff[6];
   int rows = (int)buff[9] + 256*(int)buff[8];
   // Checking size of the file
   if((cols*rows+512)!= size){
      delete [] buff;
      throw VECorruptedFile(AnsiString("SGI file is corrupted."),AnsiString("sgi"));
   }
   image->Resize(cols, rows);
   for(register int r(0); r<image->Rows(); r++)
      for(register int c(0); c<image->Cols(); c++){
          image->Data(c,image->Rows()-1-r) = buff[index+c+r*cols];
      }
   delete [] buff;
}
//---------------------------------------------------------------------------
void
__fastcall VSgi::SaveToFile(AnsiString _save_file_name)
{

   ofstream o_file(_save_file_name.c_str(),ios::binary);
   Assert(o_file!=0,VEMsg(AnsiString("Can't open file\n" + _save_file_name)));
   o_file << " ";
   o_file.close();
   TFileStream* fstream = new TFileStream(_save_file_name, fmOpenWrite);
   unsigned char header[512] = {
			1,218,0,1,0,3,0,0,0,0,0,1,0,0,0,0,255,255,255,255,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
			0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

   SizeBMP_G sizes(image->Cols(), image->Rows());
   header[6] = sizes.width_h;
   header[7] = sizes.width_l;
   header[8] = sizes.height_h;
   header[9] = sizes.height_l;
   fstream->Write(header, 512);
   AnsiString buffer(ImageName()+" written by MIPS on "+TDateTime().CurrentDateTime().DateTimeString());
   fstream->Position = 24;
   fstream->Write(buffer.c_str(), buffer.Length());
   fstream->Position = 512;
   for(register int r(0); r<image->Rows(); r++){
      for(register int c(0); c<image->Cols(); c++){
         fstream->Write(&image->Data(c,image->Rows()-1-r),1);
      }
   }
   delete (fstream);
}
//---------------------------------------------------------------------------
void
__fastcall VSgi::Assign(BMPHandler* _handler)
{
   *handler = *_handler;
   handler->Set(*image);
}
