/*
   vmask.cpp

   definin soubor tdy konvolun masky

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "vmask.h"


//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
VMask::VMask()
 : data(NULL), cols(0), rows(0)
{
}
//---------------------------------------------------------------------------
VMask::VMask(int _c, int _r)
{
   data = new int[_c*_r];
   if(data==0)
      throw EMORPHMemory("Not enought memory for a mask");
   cols = _c;
   rows = _r;
}
//---------------------------------------------------------------------------
VMask::VMask(MaskOptions init)
{
   switch(init.Size()){
      case(0):
         data = new int [3*3];
         if(data==0)
            throw EMORPHMemory("Not enought memory for a mask");
         cols = 3;
         rows = 3;
         break;
      case(1):
         data = new int [5*5];
         if(data==0)
            throw EMORPHMemory("Not enought memory for a mask");
         cols = 5;
         rows = 5;
         break;
   }
   switch(init.Shape()){
      case(0): Full();  break;
      case(1): Round(); break;
      case(2): Cross(); break;
      case(3): Vertical(); break;
      case(4): Horizontal(); break;
   }
}
//---------------------------------------------------------------------------
VMask::~VMask()
{
  delete [] data;
}
//---------------------------------------------------------------------------
VMask&
VMask::operator=(const VMask& init)
{
   if(this!=&init){
      delete (data);
      cols = init.cols;
      rows = init.rows;
      data = new int[cols*rows];
      for(int i(0); i< cols*rows; i++)
         data [i] = init.data[i];
   }
   return *this;
}
//---------------------------------------------------------------------------
void VMask::Full(int i)
{
   for(int c(0); c<cols; c++)
      for(int r(0); r<rows; r++){
         Data(c,r) = i;
      }
}
//---------------------------------------------------------------------------
void VMask::Cross(int i)
{
   for(int c(0); c<cols; c++)
      for(int r(0); r<rows; r++){
         if((c==cols/2)||(r==rows/2))
            Data(c,r) = i;
         else
            Data(c,r) = 0;
      }
}
//---------------------------------------------------------------------------
void VMask::Round(int i)
{
   for(int c(0); c<cols; c++)
      for(int r(0); r<rows; r++){
         Data(c,r) = i;
      }
   Data(0,0) = 0;
   Data(cols-1,0) = 0;
   Data(0,rows-1) = 0;
   Data(cols-1,rows-1) = 0;
}
//---------------------------------------------------------------------------
void VMask::Vertical(int i)
{
   for(int c(0); c<cols; c++)
      for(int r(0); r<rows; r++){
         if(c==cols/2)
            Data(c,r) = i;
         else
            Data(c,r) = 0;
      }
}
//---------------------------------------------------------------------------
void VMask::Horizontal(int i)
{
   for(int c(0); c<cols; c++)
      for(int r(0); r<rows; r++){
         if(r==rows/2)
            Data(c,r) = i;
         else
            Data(c,r) = 0;
      }
}



