/*
   bmp_handler.cpp

   definin soubor tdy operace s bmp files.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "bmp_handler.h"


#pragma package(smart_init)
// constructors and destructors
//---------------------------------------------------------------------------
// default constructor
//---------------------------------------------------------------------------
BMPHandler::BMPHandler()
: image(NULL), bitmap(NULL)
{
}
//---------------------------------------------------------------------------
// one argument constructor
// this constructor loads the data from file saved on disc and instantiates
// the bitmap object
BMPHandler::BMPHandler(const AnsiString _file_name)
: image(NULL), file_name(_file_name)
{
   bitmap = new Graphics::TBitmap();
   bitmap->LoadFromFile(file_name);
}
//---------------------------------------------------------------------------
BMPHandler::BMPHandler(TMemoryStream* memory_stream)
: image(NULL), file_name(NULL)
{
   bitmap = new Graphics::TBitmap();
   bitmap->LoadFromStream(memory_stream);
}
//---------------------------------------------------------------------------
BMPHandler::BMPHandler(Graphics::TBitmap* _bitmap)
:file_name(NULL),image(NULL)
{
   bitmap = new Graphics::TBitmap();
   bitmap->Assign(_bitmap);
   bitmap->Dormant();
   bitmap->FreeImage();
}
//---------------------------------------------------------------------------
void BMPHandler::Assign(Graphics::TBitmap* _bitmap)
{
   delete(bitmap);
   bitmap = new Graphics::TBitmap();
   bitmap->Assign(_bitmap);
   bitmap->Dormant();
   bitmap->FreeImage();
}
//---------------------------------------------------------------------------
// two argument constructor
// this constructor instantiate bitmap object + instantiate the GreyImage
// object image

BMPHandler::BMPHandler(const AnsiString _file_name, GreyImage _image)
:file_name(_file_name)
{
   bitmap = new Graphics::TBitmap();
   bitmap->LoadFromFile(file_name);
   image = new GreyImage(_image);
   Byte *tmp;
   // resize the GreyImage object if it's of a different size from the size
   // of the bitmap
   if((image->Cols()!=bitmap->Width)||(image->Rows()!=bitmap->Height))
         image->Resize(bitmap->Width,bitmap->Height);
   for (register int r = 0; r < bitmap->Height; r++){
      tmp = (Byte*)bitmap->ScanLine[r];
      for (register int c = 0; c < bitmap->Width; c++){
         image->Data(c,r) = tmp[c];
      }
   }
}

//---------------------------------------------------------------------------
// copy constructor
BMPHandler::BMPHandler(const BMPHandler& init)
:file_name(init.file_name),image(NULL)
{
   bitmap = new Graphics::TBitmap();
   bitmap->Assign(init.bitmap);
   bitmap->Dormant();
   bitmap->FreeImage();
}

//---------------------------------------------------------------------------
// destructor
BMPHandler::~BMPHandler()
{
   delete (bitmap);
   delete (image);
}

//---------------------------------------------------------------------------
// function uses Image to draw a bitmap on it
void
BMPHandler::Show(TImage* image)
{
   image->Width  = bitmap->Width;
   image->Height = bitmap->Height;
   image->Picture->Graphic = bitmap;

}
//---------------------------------------------------------------------------
// function Assign() serves for assigning the bitmap data by the _image
// this function can't be called after default constructor! bitmap doesn't
// have the palette + info_header
void
BMPHandler::Assign(GreyImage _image)
{
   delete (image);
   image = new GreyImage(_image);
   bitmap->Width = image->Cols();
   bitmap->Height = image->Rows();
   Byte* tmp;
   for (int r = 0; r < bitmap->Height; r++)
   {
      tmp = (Byte*)bitmap->ScanLine[r];
      for (int c = 0; c < bitmap->Width; c++){
         tmp[c] = image->Data(c,r);
      }
   }
}
//---------------------------------------------------------------------------
void
BMPHandler::Assign(TMemoryStream* memory_stream)
{
   delete (bitmap);
   bitmap = new Graphics::TBitmap();
   bitmap->LoadFromStream(memory_stream);
}
//---------------------------------------------------------------------------
// Set() function assigns image loaded in the bitmap file to the _image
void
BMPHandler::Set(GreyImage& _image)
{
   Byte *tmp;
   if((_image.Cols()!=bitmap->Width)||(_image.Rows()!=bitmap->Height)){
         _image.Resize(bitmap->Width,bitmap->Height);
   }      
   for (int r = 0; r < bitmap->Height; r++)
   {
      tmp = (Byte*)bitmap->ScanLine[r];
      for (int c = 0; c < bitmap->Width; c++){
         _image.Data(c,r) = tmp[c];
      }
   }
}
//---------------------------------------------------------------------------
void
BMPHandler::Set(Graphics::TBitmap* _bitmap)
{
   _bitmap = bitmap;
}
//---------------------------------------------------------------------------
// overloaded operator of assignment
BMPHandler&
BMPHandler::operator = (BMPHandler& right)
{
   if(this!=&right){
        delete (bitmap);
        delete (image);
        file_name = right.file_name;
        bitmap = new Graphics::TBitmap();
        bitmap->Assign(right.bitmap);
        bitmap->Dormant();
        bitmap->FreeImage();
        image = NULL;
    }
    return *this;
}
//---------------------------------------------------------------------------
void
BMPHandler::SaveToFile(AnsiString _file_name)
{
   file_name = _file_name;
   bitmap->SaveToFile(_file_name);
}
//---------------------------------------------------------------------------
void
BMPHandler::CreateBitmapAndAssign(GreyImage* _image)
{
   mstream = new TMemoryStream();
   SizeBMP_G sizes(_image->Cols(), _image->Rows());
   unsigned char* buffer;
   buffer = new unsigned char[54];
   // writing a head of the bmp file
   for (register int i=0; i<54; i++) {
      switch (i) {
         case 0 :  buffer[i] = 0x42;              break;
	 case 1 :  buffer[i] = 0x4d;              break;
	 case 2 :  buffer[i] = sizes.size_file_l; break;
	 case 3 :  buffer[i] = sizes.size_file_h; break;
         case 4 :  buffer[i] = sizes.size_file_hh;break;
	 case 18 : buffer[i] = sizes.width_l;     break;
	 case 19 : buffer[i] = sizes.width_h;     break;
	 case 22 : buffer[i] = sizes.height_l;    break;
	 case 23 : buffer[i] = sizes.height_h;    break;
	 case 34 : buffer[i] = sizes.size_bmp_l;  break;
 	 case 35 : buffer[i] = sizes.size_bmp_h;  break;
	 case 36 : buffer[i] = sizes.size_bmp_hh; break;
	 case 10 : buffer[i] = 0x36; break;
 	 case 11 : buffer[i] = 0x04; break;
	 case 14 : buffer[i] = 0x28; break;
	 case 26 : buffer[i] = 0x01; break;
	 case 28 : buffer[i] = 0x08; break;
   	 default : buffer[i] = 0;    break;
      }
   }
   mstream->Write(buffer, 54);
   delete [] buffer;
   buffer = new unsigned char[4*256];
   // LUTable of BMP file
   unsigned long index = 0;
   for(register int i=0; i<256; i++){
      buffer[index++] = ((unsigned char) i);
      buffer[index++] = ((unsigned char) i);
      buffer[index++] = ((unsigned char) i);
      buffer[index++] = ((unsigned char) 0);
   }
   mstream->Write(buffer, 4*256);
   delete [] buffer;
   int rest = _image->Cols()%4;
   int new_size = _image->Rows()*(_image->Cols()+4-rest);
   if(rest!=0)
      buffer = new unsigned char[new_size];
   else
      buffer = new unsigned char[_image->Rows()*_image->Cols()];
   // data
   index = 0;
   int rest_tmp;
   for(register int r(_image->Rows()-1); r>-1; r--){
      for(register int c(0); c<_image->Cols(); c++){
         buffer[index++] = _image->Data(c,r);
      }
      if(rest!=0){
         rest_tmp = rest;
         while((4-rest_tmp)!=0){
            buffer[index++] = 0;
            rest_tmp++;
         }
      }
   }
   if(rest!=0){
      mstream->Write(buffer, new_size);
   }else
      mstream->Write(buffer, _image->Rows()*_image->Cols());
   mstream->Position = 0;
   Assign(mstream);

   delete (mstream);
   delete [] buffer;

 //  delete (image);
 // image = new GreyImage(*_image);
 //  bitmap->Width = image->Cols();
 //  bitmap->Height = image->Rows();
 // Byte* tmp;
 //  for (int r = 0; r < bitmap->Height; r++)
 //  {
 //     tmp = (Byte*)bitmap->ScanLine[r];
 //     for (int c = 0; c < bitmap->Width; c++){
 //        image->Data(c,r) = tmp[c];
 //     }
 //  }
}
//---------------------------------------------------------------------------



