/*
   bmp_handler.cpp

   definin soubor tdy operace s bmp files.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "bmp_handler_color.h"


#pragma package(smart_init)
// constructors and destructors
//---------------------------------------------------------------------------
// default constructor
//---------------------------------------------------------------------------
BMPHandlerColor::BMPHandlerColor()
: color_image(NULL), bitmap(NULL)
{
}
//---------------------------------------------------------------------------
BMPHandlerColor::~BMPHandlerColor()
{
   delete (bitmap);
   delete (color_image);
}
//---------------------------------------------------------------------------
// copy constructor
BMPHandlerColor::BMPHandlerColor(const BMPHandlerColor& init)
:file_name(init.file_name),color_image(NULL)
{
   bitmap = new Graphics::TBitmap();
   bitmap->Assign(init.bitmap);
   bitmap->Dormant();
   bitmap->FreeImage();
}
//---------------------------------------------------------------------------
// two argument constructor
// this constructor instantiate bitmap object + instantiate the GreyImage
// object image

BMPHandlerColor::BMPHandlerColor(const AnsiString _file_name, ColorImage _color_image)
:file_name(_file_name)
{
   bitmap = new Graphics::TBitmap();
   bitmap->LoadFromFile(file_name);
   color_image = new ColorImage(_color_image);
   Byte *tmp;
   Byte b,ir,ig,ib;
   // resize the ColorImage object if it's of a different size from the size
   // of the bitmap
   if((color_image->Cols()!=bitmap->Width)||(color_image->Rows()!=bitmap->Height))
         color_image->Resize(bitmap->Width,bitmap->Height);
   for (register int r = 0; r < bitmap->Height; r++){
      tmp = (Byte*)bitmap->ScanLine[r];
      b = 0;
      for (register int c = 0; c < bitmap->Width; c++){
         ir = tmp[b]; b++;
         ig = tmp[b]; b++;
         ib = tmp[b]; b++;
         color_image->Data(c,r) = 65536*ir + 256*ig + ib;
      }
   }
}
//---------------------------------------------------------------------------
void
BMPHandlerColor::SaveToFile(AnsiString _file_name)
{
   file_name = _file_name;
   bitmap->SaveToFile(_file_name);
}
//---------------------------------------------------------------------------
void
BMPHandlerColor::Set(ColorImage& _color_image)
{
   Byte *tmp;
   Byte b,ir,ig,ib;
   if((_color_image.Cols()!=bitmap->Width)||(_color_image.Rows()!=bitmap->Height)){
         _color_image.Resize(bitmap->Width,bitmap->Height);
   }
   for (int r = 0; r < bitmap->Height; r++)
   {
      tmp = (Byte*)bitmap->ScanLine[r];
      b = 0;
      for (int c = 0; c < bitmap->Width; c++){
         ir = (Byte)tmp[b]; b++;
         ig = (Byte)tmp[b]; b++;
         ib = (Byte)tmp[b]; b++;
         _color_image.Data(c,r) = (65536*ir + 256*ig + ib);
      }
   }
}
//---------------------------------------------------------------------------
// function Assign() serves for assigning the bitmap data by the _image
// this function can't be called after default constructor! bitmap doesn't
// have the palette + info_header
void
BMPHandlerColor::Assign(ColorImage _color_image)
{
   delete (color_image);
   color_image = new ColorImage(_color_image);
   bitmap->Width = color_image->Cols();
   bitmap->Height = color_image->Rows();
   Byte* tmp;
   Byte b;
   for (int r = 0; r < bitmap->Height; r++)
   {
      tmp = (Byte*)bitmap->ScanLine[r];
      b = 0;
      for (int c = 0; c < bitmap->Width; c++){
         tmp[b++] = floor(color_image->Data(c,r)/65536);
         tmp[b++] = floor((color_image->Data(c,r) - (65536*tmp[b-1]))/256);
         tmp[b++] = color_image->Data(c,r) - (65536*tmp[b-2])-(256*tmp[b-1]);
      }
   }
}
//---------------------------------------------------------------------------
void
BMPHandlerColor::Assign(TMemoryStream* memory_stream)
{
   delete (bitmap);
   bitmap = new Graphics::TBitmap();
   bitmap->LoadFromStream(memory_stream);
}
//---------------------------------------------------------------------------
// function uses Image to draw a bitmap on it
void
BMPHandlerColor::Show(TImage* image)
{
   image->Width  = bitmap->Width;
   image->Height = bitmap->Height;
   image->Picture->Graphic = bitmap;

}
//---------------------------------------------------------------------------
void
BMPHandlerColor::CreateBitmapAndAssign(ColorImage* _color_image)
{
   mstream = new TMemoryStream();
   SizeBMP_G sizes(_color_image->Cols(), _color_image->Rows());
   unsigned char* buffer;
   buffer = new unsigned char[54];
   // writing a head of the bmp file
   for (register int i=0; i<54; i++) {
      switch (i) {
         case 0 :  buffer[i] = 0x42;                    break;
	 case 1 :  buffer[i] = 0x4d;                    break;
	 case 2 :  buffer[i] = sizes.size_color_file_l; break;
	 case 3 :  buffer[i] = sizes.size_color_file_h; break;
         case 4 :  buffer[i] = sizes.size_color_file_hh;break;
	 case 18 : buffer[i] = sizes.width_l;           break;
	 case 19 : buffer[i] = sizes.width_h;           break;
	 case 22 : buffer[i] = sizes.height_l;          break;
	 case 23 : buffer[i] = sizes.height_h;          break;
	 case 34 : buffer[i] = sizes.size_bmp_l;        break;
 	 case 35 : buffer[i] = sizes.size_bmp_h;        break;
	 case 36 : buffer[i] = sizes.size_bmp_hh;       break;
	 case 10 : buffer[i] = 0x36; break;
 	 case 11 : buffer[i] = 0x00; break;
	 case 14 : buffer[i] = 0x28; break;
	 case 26 : buffer[i] = 0x01; break;
	 case 28 : buffer[i] = 0x18; break;
   	 default : buffer[i] = 0;    break;
      }
   }
   mstream->Write(buffer, 54);

   delete [] buffer;
   int rest = _color_image->Cols()%4;
   int new_size = 3*_color_image->Rows()*(_color_image->Cols()+4-rest);
   if(rest!=0)
      buffer = new unsigned char[new_size];
   else
      buffer = new unsigned char[3*_color_image->Rows()*_color_image->Cols()];
   // data
   int index = 0;
   int rest_tmp;
   for(register int r(_color_image->Rows()-1); r>-1; r--){
      for(register int c(0); c<_color_image->Cols(); c++){
         buffer[index++] = floor(_color_image->Data(c,r)/65536);
         buffer[index++] = floor((_color_image->Data(c,r) - (65536*buffer[index-1]))/256);
         buffer[index++] = _color_image->Data(c,r) - (65536*buffer[index-2])-(256*buffer[index-1]);
      }
      if(rest!=0){
         rest_tmp = rest;
         while((4-rest_tmp)!=0){
            buffer[index++] = 0;
            buffer[index++] = 0;  //??
            buffer[index++] = 0;  //??
            rest_tmp++;
         }
      }
   }
   if(rest!=0){
      mstream->Write(buffer, new_size);
   }else
      mstream->Write(buffer, 3*_color_image->Rows()*_color_image->Cols());
   mstream->Position = 0;
   Assign(mstream);

   delete (mstream);
   delete [] buffer;

   delete (color_image);
   color_image = new ColorImage(*_color_image);
   bitmap->Width = color_image->Cols();
   bitmap->Height = color_image->Rows();
   Byte* tmp;
   Byte b;
   for (int r = 0; r < bitmap->Height; r++)
   {
      b=0;
      tmp = (Byte*)bitmap->ScanLine[r];
      for (int c = 0; c < bitmap->Width; c++){
         tmp[b++] = floor(color_image->Data(c,r)/65536);
         tmp[b++] = floor((color_image->Data(c,r) - (65536*tmp[b-1]))/256);
         tmp[b++] = color_image->Data(c,r) - (65536*tmp[b-2])-(256*tmp[b-1]);
      }
   }

}
