/*
   frame_image.cpp

   definin soubor tdy pro aritmetick a logick operace

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "frame_image.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

// constructors
//---------------------------------------------------------------------------
__fastcall
FrameImage::FrameImage()
{
   s_image = NULL;
   d_image = NULL;
   r_image = NULL;
}

//---------------------------------------------------------------------------
__fastcall
FrameImage::FrameImage(const GreyImage& _s_image, const GreyImage& _d_image,
                               GreyImage& _r_image)
{
   s_image = &_s_image;
   d_image = &_d_image;
   r_image = &_r_image;
   if((r_image->Rows()!=d_image->Rows())||
      (r_image->Cols()!=d_image->Cols()))
       r_image->Resize(d_image->Cols(), d_image->Rows());
}

//---------------------------------------------------------------------------
void __fastcall
FrameImage::ApplicationFunction(int s_col, int s_row, int s_width,
                          int s_height, int d_col, int d_row,
                          BitFunction combine_type, short scale,
                          int constant, int method_type, int min_level_range,
                          int max_level_range, bool proc_type){

   register int s_image_col, s_image_row, dest_col, dest_row;
   int s_data, d_data;
   int s_cols_extent = s_col + s_width;
   int s_rows_extent = s_row + s_height;

   dest_row = d_row;
   for (s_image_row = s_row; s_image_row < s_rows_extent; s_image_row++){
      dest_col = d_col;
      for (s_image_col = s_col; s_image_col < s_cols_extent; s_image_col++){
       s_data = s_image->Data(s_image_col, s_image_row);
       d_data = d_image->Data(dest_col, dest_row);
       if ((s_data >= min_level_range) && (s_data <= max_level_range)){
         switch(combine_type){
            case AND:
              d_data &= s_data;
              break;
            case OR:
              d_data |= s_data;
              break;
            case XOR:
              d_data ^= s_data;
              break;
            case ADD:
              d_data += s_data;
              break;
            case SUB:
              d_data -= s_data;
              break;
            case MULT:
              d_data *= s_data;
              break;
            case DIV:
              if(s_data != 0)
                 d_data /= s_data;
              break;
            case MIN:
              d_data = min(s_data, d_data);
              break;
            case MAX:
              d_data = max(s_data, d_data);
              break;
            case AVE:
              d_data = (s_data+d_data)/2;
              break;
            case OVERLAY:
              d_data = s_data;
              break;
            case InvAND:
               d_data = fnot(d_data);
               s_data = fnot(s_data);
               d_data &= s_data;
               d_data = fnot(d_data);
               break;
            case InvOR:
               d_data = fnot(d_data);
               s_data = fnot(s_data);
               d_data |= s_data;
               d_data = fnot(d_data);
               break;
            case InvXOR:
               d_data = fnot(d_data);
               s_data = fnot(s_data);
               d_data ^= s_data;
               d_data = fnot(d_data);
               break;
            case PLUS:
               d_data = s_data + constant;
               break;
            case MINUS:
               d_data = s_data - constant;
               break;
            case MULTIP:
               d_data = s_data * constant;
               break;
            case fDIV:
               d_data = s_data / constant;
               break;
         }
       }else{
         d_data = s_data;
       }
         if (scale < 0)
            d_data <<= abs(scale);
         else if (scale > 0)
            d_data >>= scale;

         if (proc_type == false){
            switch(method_type){
               case 1:
                  // range limitation
                  d_data = (d_data < MIN_SAMPLE) ? MIN_SAMPLE : d_data;
                  d_data = (d_data > MAX_SAMPLE) ? MAX_SAMPLE : d_data;
                  break;
               case 2:
                  /// find min and max values for standardization
                  if (d_data > maxi) maxi = d_data;
                  if (mini > d_data) mini = d_data;
                  break;
               case 3:
                  /// overflow
                  while (d_data>255) {
                     d_data -=256;
                  }
                  break;
            }
         }else{
            /// standardization
            d_data = (255*(d_data-mini))/(maxi-mini);
         }

         r_image->Data(dest_col++, dest_row) = (unsigned char)d_data;
         }
      dest_row++;
   }
}
//---------------------------------------------------------------------------
void __fastcall
FrameImage::CombineImages(int s_col, int s_row, int s_width,
                          int s_height, int d_col, int d_row,
                          BitFunction combine_type, short scale,
                          int constant, int method_type, int min_level_range,
                          int max_level_range){
   Assert((s_row < s_image->Rows())&&(s_row >=0 )&&
          (s_col < s_image->Cols())&&(s_col >=0 )&&
          (s_row + s_height <= s_image->Rows())&&
          (s_col + s_width <= s_image->Cols()),VEMsg(AnsiString
          ("FrameImage::CombineImages: partition of the source image ")+
           AnsiString(" goes out of border1")));


   Assert((d_row < d_image->Rows())&&(d_row >=0 )&&
          (d_col < d_image->Cols())&&(d_col >=0 )&&
          (d_row + s_height <= d_image->Rows())&&
          (d_col + s_width <= d_image->Cols()),VEMsg(AnsiString
          ("FrameImage::CombineImages: partition of the destination image ")+
           AnsiString(" goes out of border2")));

   r_image->ClearImageArea(WHITE, r_image->Cols(),r_image->Rows());

   maxi = 1;
   mini = 999;

   ApplicationFunction(s_col, s_row, s_width, s_height, d_col, d_row,
                       combine_type, scale, constant, method_type,
                       min_level_range, max_level_range, false);
   /// standardization
   if (method_type == 2){
      if (maxi == mini){
         if (mini+1 <= MAX_SAMPLE){ maxi = mini+1;}
         else{mini = maxi-1;}
      }
      ApplicationFunction(s_col, s_row, s_width, s_height, d_col, d_row,
                          combine_type, scale, constant, method_type,
                          min_level_range, max_level_range, true);
   }
}
//---------------------------------------------------------------------------
void __fastcall  FrameImage::WeightImages(int s_col, int s_row, int s_width,
                                 int s_height, int d_col, int d_row,
                                 int percent_of_s)
{
   Assert((s_row < s_image->Rows())&&(s_row >=0 )&&
          (s_col < s_image->Cols())&&(s_col >=0 )&&
          (s_row + s_height <= s_image->Rows())&&
          (s_col + s_width <= s_image->Cols()),VEMsg(AnsiString
          ("FrameImage::CombineImages: partition of the source image ")+
           AnsiString(" goes out of border")));

   Assert(0<=percent_of_s && percent_of_s <=100, VEMsg(AnsiString
          ("FrameImage::CombineImages: unproper percentage ")));

   Assert((d_row < d_image->Rows())&&(d_row >=0 )&&
          (d_col < d_image->Cols())&&(d_col >=0 )&&
          (d_row + s_height <= d_image->Rows())&&
          (d_col + s_width <= d_image->Cols()),VEMsg(AnsiString
          ("FrameImage::CombineImages: partition of the destination image ")+
           AnsiString(" goes out of border")));

   r_image->ClearImageArea(WHITE, r_image->Cols(),r_image->Rows());


   register int s_image_col, s_image_row, dest_col, dest_row;
   int s_data, d_data, r_data;
   int s_cols_extent = s_col + s_width;
   int s_rows_extent = s_row + s_height;

   dest_row = d_row;
   for (s_image_row = s_row; s_image_row < s_rows_extent; s_image_row++)
   {
      dest_col = d_col;
      for (s_image_col = s_col; s_image_col < s_cols_extent; s_image_col++)
      {
         s_data = s_image->Data(s_image_col, s_image_row);
         d_data = d_image->Data(dest_col, dest_row);
         r_data = (double)s_data*(double)percent_of_s/100 +
                  (double)d_data*(double)(100-percent_of_s)/100;

         r_data = (r_data < MIN_SAMPLE) ? MIN_SAMPLE : r_data;
         r_data = (r_data > MAX_SAMPLE) ? MAX_SAMPLE : r_data;

         r_image->Data(dest_col++, dest_row) = (unsigned char)r_data;
      }
      dest_row++;
   }
}
//---------------------------------------------------------------------------
int max(int value1, int value2)
{
    return ( (value1 > value2) ? value1 : value2);
}

//---------------------------------------------------------------------------
int min(int value1, int value2)
{
    return ( (value1 > value2) ? value2 : value1);
}
//---------------------------------------------------------------------------
int fnot(int value1)
{
    return (abs(255 - value1));
}
//---------------------------------------------------------------------------



