/*
   histogram.cpp

   definin soubor tdy definujc zkladn operace s histogramem

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "histogram.h"
const int num_labels = 7;

//---------------------------------------------------------------------------
#pragma package(smart_init)

// constructors
//---------------------------------------------------------------------------
// default constructor
Histogram::Histogram()
:levels(MAX_QUANT_LEVELS), max_value(0), differ(10)
{
   hist = new long[levels];
   for(register int index(0); index < MAX_QUANT_LEVELS; index++)
      hist[index] = 0;
}
//---------------------------------------------------------------------------
// 1 argument constructor
Histogram::Histogram(int _levels)
:levels(_levels), max_value(0)
{
   hist = new long[levels];
   for(register int index(0); index < MAX_QUANT_LEVELS; index++)
      hist[index] = 0;
}
//---------------------------------------------------------------------------
// destructor
Histogram::~Histogram()
{
   delete [] hist;
}
//---------------------------------------------------------------------------
// This function calculate the histogram of any portion of an image.
void
Histogram::InitFromGreyImage(const GreyImage& init, int col, int row,
                             int width, int height)
{
   Assert((col <= init.Cols())&&((col+width)<=init.Cols())&&
      (row <= init.Rows())&&((row+height)<=init.Rows()),VEMsg(AnsiString
      ("Histogram::InitFromGreyImage:the portion goes out of image")));

   for(register index(0); index<MAX_QUANT_LEVELS; index++)
      hist[index] = 0;

   for(register int c(col); c < col+width; c++)
      for(register int r(row); r < row+height; r++)
         hist[init.Data(c,r)]+=1;
}
//---------------------------------------------------------------------------
// before calling this function Initialization must have been called!
void
Histogram::HistogramEqualization(const GreyImage& or_image,GreyImage& eq_image)
{
  long sum;
  long total_pixels = or_image.Cols()*or_image.Rows();

   // in case of different sizes of the images resize the st_image
  if(or_image.Cols()!=eq_image.Cols()||or_image.Rows()!=eq_image.Rows())
     eq_image.Resize(or_image.Cols(), or_image.Rows());

  unsigned char histeq[MAX_QUANT_LEVELS];
  long histor[MAX_QUANT_LEVELS];

  // initialize the histograms with zeroes
  for(register int index(0); index < MAX_QUANT_LEVELS; index++){
     histeq[index] = 0;
     histor[index] = 0;
  }
  // initialize the original image
  for(register int c(0); c < or_image.Cols(); c++)
      for(register int r(0); r < or_image.Rows(); r++)
         histor[or_image.Data(c,r)]+=1;

  // equalization
  for(int i(0); i < MAX_QUANT_LEVELS; i++){
     sum = 0.0;
     for(int j(0); j <= i; j++)
        sum += histor[j];
     histeq[i]=(double)255*(double)sum/(double)total_pixels;
  }

  for(int c(0); c < or_image.Cols(); c++)
      for(int r(0); r < or_image.Rows(); r++)
         eq_image.Data(c,r) = histeq[or_image.Data(c,r)];

}
//---------------------------------------------------------------------------
// before calling this function Initialization must have been called!
void
Histogram::HistogramStretching(const GreyImage& or_image,GreyImage& st_image,float omission)
{
  long sum;
  long total_pixels = or_image.Cols()*or_image.Rows();

  // in case of different sizes of the images resize the st_image
  if(or_image.Cols()!=st_image.Cols()||or_image.Rows()!=st_image.Rows())
     st_image.Resize(or_image.Cols(), or_image.Rows());

  unsigned char histst[MAX_QUANT_LEVELS];
  long histor[MAX_QUANT_LEVELS];

  // initialize the histograms with zeroes
  for(register int index(0); index < MAX_QUANT_LEVELS; index++){
     histor[index] = 0;
  }

  // initialize the original image
  for(register int c(0); c < or_image.Cols(); c++){
      for(register int r(0); r < or_image.Rows(); r++)
         histor[or_image.Data(c,r)]+=1;
  }

  // find maximal number points with the same brightness
  long max_points = 0;
  for(register int index(0); index < MAX_QUANT_LEVELS; index++){
     max_points = (histor[index] > max_points) ? histor[index] : max_points;
  }

  // omis = absolutely omission level
  long omis;
  omis = omission*((double)(max_points)/(double)(100));

  // find min and max levels
  int min_level = 255;
  int max_level = 0;
  for(register index(0); index<MAX_QUANT_LEVELS; index++){
      min_level = ((histor[index] > omis)&&(index < min_level)) ? index : min_level;
      max_level = ((histor[index] > omis)&&(index > max_level)) ? index : max_level;
  }

  // transform values in original image
  omis_image = new GreyImage(or_image.Cols(), or_image.Rows());
  for(int c(0); c < or_image.Cols(); c++){
     for(int r(0); r < or_image.Rows(); r++){
        omis_image->Data(c,r) = (or_image.Data(c,r) > min_level) ? or_image.Data(c,r) : min_level;
        omis_image->Data(c,r) = (or_image.Data(c,r) < max_level) ? or_image.Data(c,r) : max_level;
     }
  }

  // second initialize the histograms with zeroes
  for(register int index(0); index < MAX_QUANT_LEVELS; index++){
     histst[index] = 0;
     histor[index] = 0;
  }

   // second initialize the original image
  for(register int c(0); c < or_image.Cols(); c++){
      for(register int r(0); r < or_image.Rows(); r++)
         histor[or_image.Data(c,r)]+=1;
  }
  // stretching
  if (min_level != max_level){
     for(int i(min_level); i <= max_level; i++)
        histst[i] =  (int)((MAX_QUANT_LEVELS-1)*(i-min_level)/(max_level-min_level));

     for(int c(0); c < or_image.Cols(); c++)
         for(int r(0); r < or_image.Rows(); r++)
            st_image.Data(c,r) = histst[omis_image->Data(c,r)];
  }
  delete (omis_image);
}
//---------------------------------------------------------------------------
//
void
Histogram::Display(TImage* _Image)
{
   Image = _Image;
   canvas = Image->Canvas;
   LeftTop.X() = differ;
   LeftTop.Y() = differ;
   RightDown.X() = Image->ClientWidth  - differ;
   RightDown.Y() = Image->ClientHeight - differ;
   DrawRect();
   max_value=0;
   Start.Y() = RightDown.Y();
   for(register index(0); index<MAX_QUANT_LEVELS; index++)
      max_value = (max_value > hist[index]) ? max_value : hist[index];

   float height_koef = 0.95*(RightDown.Y()-differ)/ max_value;
   for(register int index(0); index < MAX_QUANT_LEVELS; index++){
      Start.X() = End.X() = differ+index;
      End.Y() = RightDown.Y() - height_koef*hist[index];
      DrawLine(Start,End);
   }
}
//---------------------------------------------------------------------------
//
void
Histogram::DisplayEdge(TImage* _Image)
{
   Image = _Image;
   canvas = Image->Canvas;
   LeftTop.X() = differ;
   LeftTop.Y() = differ;
   RightDown.X() = Image->ClientWidth  - differ;
   RightDown.Y() = Image->ClientHeight - differ;
//   canvas->FillRect(TRect(0,0,Image->ClientWidth, Image->ClientHeight));
   DrawRect();
   max_value=0;
   Start.Y() = RightDown.Y();
   for(register index(0); index<MAX_QUANT_LEVELS; index++)
      max_value = (max_value > hist[index]) ? max_value : hist[index];

   float height_koef = 0.95*(RightDown.Y()-differ)/ max_value;
   for(register int index(0); index < MAX_QUANT_LEVELS; index++){
      Start.X() = End.X() = differ+index;
      End.Y() = RightDown.Y() - height_koef*hist[index];
      DrawLine(Start,End);
      Start.Y() = End.Y();
   }
}
//---------------------------------------------------------------------------
void
Histogram::DisplayWithComment(TImage* _Image,TPanel* Panel)
{
   int differ_com = 45;
   Image = _Image;
   canvas = Image->Canvas;
   LeftTop.X() = differ_com;
   LeftTop.Y() = differ_com;
   RightDown.X() = Image->ClientWidth  - differ_com;
   RightDown.Y() = Image->ClientHeight - differ_com;
   DrawRect();
   max_value=0;
   Start.Y() = RightDown.Y();
   for(register index(0); index<MAX_QUANT_LEVELS; index++)
      max_value = (max_value > hist[index]) ? max_value : hist[index];

   float height_koef = 0.95*(RightDown.Y()-differ_com)/ max_value;
   for(register int index(0); index < MAX_QUANT_LEVELS; index++){
      Start.X() = End.X() = differ_com+index;
      End.Y() = RightDown.Y() - height_koef*hist[index];
      DrawLine(Start,End);
   }

   label = new TLabel*[num_labels];
   for(int i(0); i<3; i++){   //horizontal
      label[i] = new TLabel(Panel);
      label[i]->Parent = Panel;
      label[i]->Top = RightDown.Y()+5;
      label[i]->Color = clWhite;
      label[i]->Caption = i*128;
      label[i]->Left = LeftTop.X()+i*128-label[i]->Width/2;
      label[i]->Visible = true;
      label[i]->Show();
   }
   for(int i(3); i<5; i++){   //vertical
      label[i] = new TLabel(Panel);
      label[i]->Parent = Panel;
      if(i==3){
         label[i]->Caption = "0";
         label[i]->Top = RightDown.Y()-label[i]->Width/2-2;
         label[i]->Color = clWhite;
      }else{
         label[i]->Caption = max_value;
         label[i]->Top = RightDown.Y()-height_koef*max_value;
         label[i]->Color = clWhite;
      }
      label[i]->Left = LeftTop.X()-label[i]->Width-2;
      label[i]->Visible = true;
      label[i]->Show();
   }

   for(int i(5); i<7; i++){   //coment
      label[i] = new TLabel(Panel);
      label[i]->Parent = Panel;
      if(i==5){
         label[i]->Caption = "Pixel value";
         label[i]->Top = RightDown.Y()+18;
         label[i]->Color = clWhite;
         label[i]->Left = LeftTop.X()+128-label[i]->Width/2;
      }else{
         label[i]->Caption = "Count";
         label[i]->Top = LeftTop.X()-20;
         label[i]->Color = clWhite;
         label[i]->Left = 10;
      }
      label[i]->Visible = true;
      label[i]->Show();
   }

}
//---------------------------------------------------------------------------
void
Histogram::DrawLine(VPoint& start, VPoint& end,
                 TColor color, int size, TPenMode mode)
{
   TPenMode  pm = canvas -> Pen -> Mode;
   TColor pc =  canvas->Pen->Color;
   int psize = canvas->Pen->Width;

   canvas->Pen->Color = color;
   canvas->Pen->Width = size;
   canvas->Pen->Mode = mode;
   canvas->MoveTo(start.X(), start.Y());
   canvas->LineTo(end.X(), end.Y());
   canvas->Pen->Color = pc;
   canvas->Pen->Width = psize;
   canvas->Pen->Mode = pm;
}
//---------------------------------------------------------------------------
void
Histogram::DrawRect()
{
   TColor color=clBlack;
   TPenMode mode = pmCopy;
   canvas->Pen->Color = clBlack;
   canvas->Pen->Width = 4;
   canvas->Rectangle(0,0,Image->Width+1,Image->Height+1);
   color=cl3DLight;
   DrawLine(VPoint(LeftTop.X()-1 , LeftTop.Y()),
            VPoint(LeftTop.X()-1 , RightDown.Y()+1), color,1,mode);
   DrawLine(VPoint(LeftTop.X()-1 , RightDown.Y()+1),
            VPoint(RightDown.X() , RightDown.Y()+1), color,1,mode);
   DrawLine(VPoint(RightDown.X() , RightDown.Y()+1),
            VPoint(RightDown.X() , LeftTop.Y()), color,1,mode);
   DrawLine(VPoint(RightDown.X() , LeftTop.Y()),
            VPoint(LeftTop.X()-1 , LeftTop.Y()), color,1,mode);
}
//---------------------------------------------------------------------------
// draw one black-line after drawing red-cursor
void Histogram::CursorLine(TImage* _Image, int level)
{
   Image = _Image;
   canvas = Image->Canvas;
   LeftTop.X() = differ;
   LeftTop.Y() = differ;
   RightDown.X() = Image->ClientWidth  - differ;
   RightDown.Y() = Image->ClientHeight - differ;
   //DrawRect();
   max_value=0;
   Start.Y() = RightDown.Y();
   for(register index(0); index<MAX_QUANT_LEVELS; index++)
      max_value = (max_value > hist[index]) ? max_value : hist[index];

   float height_koef = 0.95*(RightDown.Y()-differ)/ max_value;

      Start.X() = End.X() = differ+level;
      End.Y() = differ;
      DrawLine(Start, End, clRed);

}
//---------------------------------------------------------------------------
// draw red-cursor
void Histogram::LineOnly(TImage* _Image, int level)
{
   Image = _Image;
   canvas = Image->Canvas;
   LeftTop.X() = differ;
   LeftTop.Y() = differ;
   RightDown.X() = Image->ClientWidth  - differ;
   RightDown.Y() = Image->ClientHeight - differ;
   //DrawRect();
   max_value=0;
   Start.Y() = RightDown.Y();
   for(register index(0); index<MAX_QUANT_LEVELS; index++)
      max_value = (max_value > hist[index]) ? max_value : hist[index];

   float height_koef = 0.95*(RightDown.Y()-differ)/ max_value;

      Start.X() = End.X() = differ+level;
      End.Y() = RightDown.Y() - height_koef*hist[level];
      DrawLine(Start, End, clBlack);
      Start.Y() = End.Y();
      End.Y() = differ;
      DrawLine(Start, End, clWhite);
}

