/*
   image_color.cpp

   definin soubor tdy pro zkladn tdu 8bitovch obraz.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "image_color.h"

#pragma package(smart_init)
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Split color image to red, green and blue images
void __fastcall ColorImage::SplitToRGB(GreyImage* R_component,
                GreyImage* G_component, GreyImage* B_component)
{
   //R_component->Resize(Cols(), Rows());
   //G_component->Resize(Cols(), Rows());
   //B_component->Resize(Cols(), Rows());

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){
         B_component->Data(c,r) = (unsigned char)(floor(Data(c,r)/65536));
         G_component->Data(c,r) = (unsigned char)(floor((Data(c,r) - (65536*B_component->Data(c,r)))/256));
         R_component->Data(c,r) = (unsigned char)(Data(c,r) - (65536*B_component->Data(c,r)) - (256*G_component->Data(c,r)));
      }
   }
}
//---------------------------------------------------------------------------
// Combine red, green and blue imeges to color image
void __fastcall ColorImage::CombineFromRGB(GreyImage* R_component,
                GreyImage* G_component, GreyImage* B_component)
{
   Byte ir,ig,ib;
   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){
         ir = (Byte)(R_component->Data(c,r));
         ig = (Byte)(G_component->Data(c,r));
         ib = (Byte)(B_component->Data(c,r));
         Data(c,r) = (65536*ib + 256*ig + ir);
      }
   }
}
//---------------------------------------------------------------------------
// Grey scale conversation (from color image made greyscale image)
void __fastcall ColorImage::GreyScale(int type_conversion, GreyImage* Y)
{
   // type_conversation = 1 -> linear  [Y=0.2126R+0.7152G+0.0722B]
   // type_conversation = 1 -> nonlinear  [Y=0.299R+0.5876G+0.114B]
   // type_conversation = 1 -> average  [Y=(R+G+B)/3]

   unsigned char R,G,B;

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){
         B = (floor(Data(c,r)/65536));
         G = (floor((Data(c,r) - (65536*B))/256));
         R = (Data(c,r) - (65536*B) - (256*G));
         switch (type_conversion) {
            case 1: Y->Data(c,r) = (unsigned char)(0.2126*R + 0.7152*G + 0.0722*B); break;
            case 2: Y->Data(c,r) = (unsigned char)(0.299*R + 0.587*G + 0.114*B); break;
            case 3: Y->Data(c,r) = (unsigned char)(0.3333*R + 0.3333*G + 0.3333*B); break;
         }
      }
   }
}
//---------------------------------------------------------------------------
// Split color image to hue, saturation and value images
void __fastcall ColorImage::RGBtoHSV(FloatImage* H, FloatImage* S, FloatImage* V)
{
   byte R, G, B;
   float var_R, var_G, var_B, var_Min, var_Max, del_Max, del_R, del_G, del_B;

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){
         B = (floor(Data(c,r)/65536));
         G = (floor((Data(c,r) - (65536*B))/256));
         R = (Data(c,r) - (65536*B) - (256*G));

         var_R = ( (float)R / 255 ); // RGB values = From 0 to 255
         var_G = ( (float)G / 255 );
         var_B = ( (float)B / 255 );

         var_Min = min(min( var_R, var_G), var_B ); // Min. value of RGB
         var_Max = max(max( var_R, var_G), var_B ); // Max. value of RGB
         del_Max = var_Max - var_Min; // Delta RGB value

         V->Data(c,r) = var_Max;

         if ( del_Max == 0 ) // This is a gray, no chroma...
         {
            H->Data(c,r) = 0; // HSV results = From 0 to 1
            S->Data(c,r) = 0;
         }
         else // Chromatic data...
         {
            S->Data(c,r) = del_Max / var_Max;

            del_R = ( ( ( var_Max - var_R ) / 6 ) + ( del_Max / 2 ) ) / del_Max;
            del_G = ( ( ( var_Max - var_G ) / 6 ) + ( del_Max / 2 ) ) / del_Max;
            del_B = ( ( ( var_Max - var_B ) / 6 ) + ( del_Max / 2 ) ) / del_Max;

            if      ( var_R == var_Max ) { H->Data(c,r) = del_B - del_G;              }
            else if ( var_G == var_Max ) { H->Data(c,r) = ( (float)(1) / (float)(3) ) + del_R - del_B;  }
            else if ( var_B == var_Max ) { H->Data(c,r) = ( (float)(2) / (float)(3) ) + del_G - del_R;  }

            if ( H->Data(c,r) < 0 ) { H->Data(c,r) += 1; }
            if ( H->Data(c,r) > 1 ) { H->Data(c,r) -= 1; }
         }
      }
   }
}
//---------------------------------------------------------------------------
// Combine hue, saturation and value images to color image
void __fastcall ColorImage::HSVtoRGB(FloatImage* H, FloatImage* S, FloatImage* V)
{
   byte R, G, B;
   float var_h, var_i, var_1, var_2, var_3, var_r, var_g, var_b;

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){
         if ( S->Data(c,r) == 0 ) // HSV values = From 0 to 1
         {
            R = (V->Data(c,r) * 255); // RGB results = From 0 to 255
            G = (V->Data(c,r) * 255);
            B = (V->Data(c,r) * 255);
         }
         else
         {
            var_h = H->Data(c,r) * 6;
            var_i = (float)((int)( var_h ));
            var_1 = V->Data(c,r) * ( 1 - S->Data(c,r) );
            var_2 = V->Data(c,r) * ( 1 - S->Data(c,r) * ( var_h - var_i ));
            var_3 = V->Data(c,r) * ( 1 - S->Data(c,r) * ( 1 - ( var_h - var_i )));

            if      ( var_i == 0 ) { var_r = V->Data(c,r); var_g = var_3;        var_b = var_1;        }
            else if ( var_i == 1 ) { var_r = var_2;        var_g = V->Data(c,r); var_b = var_1;        }
            else if ( var_i == 2 ) { var_r = var_1;        var_g = V->Data(c,r); var_b = var_3;        }
            else if ( var_i == 3 ) { var_r = var_1;        var_g = var_2;        var_b = V->Data(c,r); }
            else if ( var_i == 4 ) { var_r = var_3;        var_g = var_1;        var_b = V->Data(c,r); }
            else                   { var_r = V->Data(c,r); var_g = var_1;        var_b = var_2;        }

            R = (var_r * 255); // RGB results = From 0 to 255
            G = (var_g * 255);
            B = (var_b * 255);
         }
         Data(c,r) = (65536*(long unsigned)B + 256*(long unsigned)G + (long unsigned)R);
      }
   }
}
//---------------------------------------------------------------------------
// Split color image to hue, saturation and lightness images
void __fastcall ColorImage::RGBtoHSL(FloatImage* H, FloatImage* S, FloatImage* L)
{
   byte R, G, B;
   float var_R, var_G, var_B, var_Min, var_Max, del_Max, del_R, del_G, del_B;

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){
         B = (floor(Data(c,r)/65536));
         G = (floor((Data(c,r) - (65536*B))/256));
         R = (Data(c,r) - (65536*B) - (256*G));

         var_R = ( (float)R / 255 ); // RGB values = From 0 to 255
         var_G = ( (float)G / 255 );
         var_B = ( (float)B / 255 );

         var_Min = min(min( var_R, var_G), var_B ); // Min. value of RGB
         var_Max = max(max( var_R, var_G), var_B ); // Max. value of RGB
         del_Max = var_Max - var_Min; // Delta RGB value

         L->Data(c,r) = ( var_Max + var_Min ) / (float)(2);

         if ( del_Max == 0 ) // This is a gray, no chroma...
         {
            H->Data(c,r) = 0; // HSL results = From 0 to 1
            S->Data(c,r) = 0;
         }
         else  // Chromatic data...
         {
            if ( L->Data(c,r) < 0.5 ) { S->Data(c,r) = del_Max / ( var_Max + var_Min ); }
            else                      { S->Data(c,r) = del_Max / (( float)(2) - var_Max - var_Min);}

            del_R = ( ( ( var_Max - var_R ) / (float)(6) ) + ( del_Max / (float)(2) ) ) / del_Max;
            del_G = ( ( ( var_Max - var_G ) / (float)(6) ) + ( del_Max / (float)(2) ) ) / del_Max;
            del_B = ( ( ( var_Max - var_B ) / (float)(6) ) + ( del_Max / (float)(2) ) ) / del_Max;

            if      ( var_R == var_Max ) { H->Data(c,r) = del_B - del_G; }
            else if ( var_G == var_Max ) { H->Data(c,r) = ( (float)(1) / (float)(3) ) + del_R - del_B; }
            else if ( var_B == var_Max ) { H->Data(c,r) = ( (float)(2) / (float)(3) ) + del_G - del_R; }

            if ( H->Data(c,r) < 0 ) { H->Data(c,r) += 1; }
            if ( H->Data(c,r) > 1 ) { H->Data(c,r) -= 1; }
         }
      }
   }
}
//---------------------------------------------------------------------------
// Combine hue, saturation and lightness images to color image
void __fastcall ColorImage::HSLtoRGB(FloatImage* H, FloatImage* S, FloatImage* L)
{
   byte R, G, B;
   float var_1, var_2;

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){
         B = (floor(Data(c,r)/65536));
         G = (floor((Data(c,r) - (65536*B))/256));
         R = (Data(c,r) - (65536*B) - (256*G));

         if ( S->Data(c,r) == 0 ) // HSL values = From 0 to 1
         {
            R = L->Data(c,r) * 255; // RGB results = From 0 to 255
            G = L->Data(c,r) * 255;
            B = L->Data(c,r) * 255;
         }
         else
         {
            if ( L->Data(c,r) < 0.5 ) { var_2 = L->Data(c,r) * ( 1 + S->Data(c,r) );}
            else                      { var_2 = ( L->Data(c,r) + S->Data(c,r) ) - ( S->Data(c,r) * L->Data(c,r) );}

            var_1 = (float)(2) * L->Data(c,r) - var_2;

            R = (byte)(255 * Hue_2_RGB( var_1, var_2, H->Data(c,r) + ( (float)(1) / (float)(3) )));
            G = (byte)(255 * Hue_2_RGB( var_1, var_2, H->Data(c,r) ));
            B = (byte)(255 * Hue_2_RGB( var_1, var_2, H->Data(c,r) - ( (float)(1) / (float)(3) )));
         }
         Data(c,r) = (65536*(long unsigned)B + 256*(long unsigned)G + (long unsigned)R);
      }
   }
}
//---------------------------------------------------------------------------
//Function Hue_2_RGB  for  'CombineFromHSL' function
float ColorImage::Hue_2_RGB(float v1, float v2, float vH)
{
   if ( vH < 0 ) { vH += (float)(1); }
   if ( vH > 1 ) { vH -= (float)(1); }
   if ( ( (float)(6) * vH ) < 1 ) return ( v1 + ( v2 - v1 ) * (float)(6) * vH );
   if ( ( (float)(2) * vH ) < 1 ) return ( v2 );
   if ( ( (float)(3) * vH ) < 2 ) return ( v1 + ( v2 - v1 ) * ( ( (float)(2) / (float)(3) ) - vH ) * (float)(6) );
   return ( v1 );
}
//---------------------------------------------------------------------------
// Convert RGB to CMY
void __fastcall ColorImage::RGBtoCMY(FloatImage* C, FloatImage* M, FloatImage* Y)
{
   byte R, G, B;

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){
         //RGB values = From 0 to 255
         B = (floor(Data(c,r)/65536));
         G = (floor((Data(c,r) - (65536*B))/256));
         R = (Data(c,r) - (65536*B) - (256*G));

         C->Data(c,r) = (float)(1) - ( (float)(R) / (float)(255) );
         M->Data(c,r) = (float)(1) - ( (float)(G) / (float)(255) );
         Y->Data(c,r) = (float)(1) - ( (float)(B) / (float)(255) );
      }
   }
}
//---------------------------------------------------------------------------
// Convert CMY to RGB
void __fastcall ColorImage::CMYtoRGB(FloatImage* C, FloatImage* M, FloatImage* Y)
{
   byte R, G, B;

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){
         //CMY values = From 0 to 1
         R = ( (float)(1) - C->Data(c,r) ) * (float)(255);
         G = ( (float)(1) - M->Data(c,r) ) * (float)(255);
         B = ( (float)(1) - Y->Data(c,r) ) * (float)(255);

         Data(c,r) = (65536*(long unsigned)B + 256*(long unsigned)G + (long unsigned)R);
      }
   }
}
//---------------------------------------------------------------------------
// Convert RGB to CMYK
void __fastcall ColorImage::RGBtoCMYK(FloatImage* C, FloatImage* M, FloatImage* Y, FloatImage* K)
{
   byte R, G, B;
   float _C, _M, _Y;
   float var_K;

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){

         // RBG to CMY
         B = (floor(Data(c,r)/65536));
         G = (floor((Data(c,r) - (65536*B))/256));
         R = (Data(c,r) - (65536*B) - (256*G));

         _C = (float)(1) - ( (float)(R) / (float)(255) );
         _M = (float)(1) - ( (float)(G) / (float)(255) );
         _Y = (float)(1) - ( (float)(B) / (float)(255) );

         // CMY to CMYK

         var_K = (float)(1);

         if ( _C < var_K )  { var_K = _C; }
         if ( _M < var_K )  { var_K = _M; }
         if ( _Y < var_K )  { var_K = _Y; }

         if ( var_K !=1 ){
            C->Data(c,r) = ( _C - var_K ) / ( (float)(1) - var_K );
            M->Data(c,r) = ( _M - var_K ) / ( (float)(1) - var_K );
            Y->Data(c,r) = ( _Y - var_K ) / ( (float)(1) - var_K );
         }else{
           C->Data(c,r) = 0;
           M->Data(c,r) = 0;
           Y->Data(c,r) = 0;
         }
         K->Data(c,r) = var_K;
      }
   }
}
//---------------------------------------------------------------------------
// Convert CMYK to RGB
void __fastcall ColorImage::CMYKtoRGB(FloatImage* C, FloatImage* M, FloatImage* Y, FloatImage* K)
{
   byte R, G, B;
   float _C, _M, _Y;

   for (register int r = 0; r < Rows(); r++){
      for (register int c = 0; c < Cols(); c++){

         // CMYK to CMY
         _C = ( C->Data(c,r) * ( (float)(1) - K->Data(c,r) ) + K->Data(c,r) );
         _M = ( M->Data(c,r) * ( (float)(1) - K->Data(c,r) ) + K->Data(c,r) );
         _Y = ( Y->Data(c,r) * ( (float)(1) - K->Data(c,r) ) + K->Data(c,r) );

         //CMY to RGB
         R = (byte)(((float)(1) - _C) * (float)(255));
         G = (byte)(((float)(1) - _M) * (float)(255));
         B = (byte)(((float)(1) - _Y) * (float)(255));

         Data(c,r) = (65536*(long unsigned)B + 256*(long unsigned)G + (long unsigned)R);
      }
   }
}

