/*
   image_complex.cpp

   definin soubor tdy pro tdu komplexnch obraz (po FFT)

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "image_complex.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)


//---------------------------------------------------------------------------
// overloaded assignment operator
ComplexImage&
ComplexImage::operator = (const ComplexImage& init)
{
   if(this != &init){
      Resize(init.cols, init.rows);
      for(register int c(0); c<cols; c++)
         for(register int r(0); r<rows; r++)
            data[c+r*cols] = init.data[c+r*cols];
      return *this;
   }
   return *this;
}
//---------------------------------------------------------------------------
GreyImage
ComplexImage::ReImage(int scale)
{
   double temp;
   GreyImage tmp(cols, rows);
   for(register int c(0); c<cols; c++)
      for(register int r(0); r<rows; r++){
         temp = (Data(c,r).real()>0) ? Data(c,r).real() :
                                       -Data(c,r).real();
         if (scale!=1)
            temp = scale*log(temp+1);
         tmp.Data(c,r) = (temp > 255) ? 255 : temp;
      }
   return tmp;
}
//---------------------------------------------------------------------------
GreyImage
ComplexImage::MagImage(int scale)
{
   double temp;
   GreyImage tmp(cols, rows);
   for(int c(0); c<cols; c++)
      for(int r(0); r<rows; r++){
         temp = abs(Data(c,r));
         if (scale!=1)
           temp = scale*log(temp+1);
         tmp.Data(c,r) = (temp > 255) ? 255 : temp;
      }
   return tmp;
}
//---------------------------------------------------------------------------
GreyImage
ComplexImage::ImImage(int scale)
{
   double temp;
   GreyImage tmp(cols, rows);
   for(int c(0); c<cols; c++)
      for(int r(0); r<rows; r++){
         temp = (Data(c,r).imag()>0) ? Data(c,r).imag() :
                                       -Data(c,r).imag();
         if (scale!=1)
            temp = scale*log(temp+1);
         tmp.Data(c,r) = (temp > 255) ? 255 : temp;
      }
   return tmp;
}
//---------------------------------------------------------------------------
GreyImage
ComplexImage::PhaseImage()
{
   double pi = 3.14159276;
   int temp;
   GreyImage tmp(cols, rows);
   for(int c(0); c<cols; c++){
      for(int r(0); r<rows; r++){
         if (arg(Data(c,r)) != 0){
            temp = (int)((double)(pi + arg(Data(c,r)))*(double)(255)/((double)(2)*pi));
            temp = (temp < 0) ? 0 : temp;
            temp = (temp > 255) ? 255 : temp;
         }else{
            temp = 0;
         }
         tmp.Data(c,r) = temp;
      }
   }
   return tmp;
}
//---------------------------------------------------------------------------
void
ComplexImage::ReImage(GreyImage* image, bool center)
{
   Assert((image->Cols()==Cols())&&(image->Rows()==Rows()),
      VEMsg(AnsiString("the two images aren't of the same size")));
   for(register int c(0); c<cols; c++)
      for(register int r(0); r<rows; r++){
         if(center==true)
            Data(c,r) = vcomplex(image->Data(c,r)*pow(-1,c+r),
                               Data(c,r).imag());
         else
            Data(c,r) = vcomplex(image->Data(c,r),
                               Data(c,r).imag());
      }
}
//---------------------------------------------------------------------------
void
ComplexImage::ImImage(GreyImage* image)
{
   Assert((image->Cols()==Cols())&&(image->Rows()==Rows()),
      VEMsg(AnsiString("the two images aren't of the same size")));
   for(register int c(0); c<cols; c++)
      for(register int r(0); r<rows; r++)
         Data(c,r) = vcomplex(Data(c,r).real(),image->Data(c,r));
}

//---------------------------------------------------------------------------
void
ComplexImage::HighWindowFilter(int c_coef, int r_coef)
{
   int center = rows/2;
   for(register int c(center-c_coef); c<center+c_coef-1; c++)
      for(register int r(center-r_coef); r<center+r_coef-1; r++)
         Data(c,r) =  vcomplex(0,0);
}
//---------------------------------------------------------------------------
void
ComplexImage::LowWindowFilter(int c_coef, int r_coef)
{
   int center = rows/2;
   for(register int c(0); c<cols; c++)
      for(register int r(0); r<rows; r++){
         if(!((center-c_coef<c)&&(c<center+c_coef-1)&&
            (center-r_coef<r)&&(r<center+r_coef-1)))
            Data(c,r) =  vcomplex(0,0);
      }
}
//---------------------------------------------------------------------------
// Assumes that the frequency spectrum is centered!
void
ComplexImage::CyrcularSymFilter(double F, int N, CircularKind circular_kind)
{
   double Huv;
   int cc, rr; // frequency koefficients
   for(register int c(0); c<cols; c++)
      for(register int r(0); r<rows; r++){
         cc = (c < cols/2) ? cols/2 - c : c - cols/2;
         rr = (r < rows/2) ? rows/2 - r : r - rows/2;
         switch (circular_kind){
           case LowPass:
              Huv = 1.0/(1.0 + pow(sqrt(cc*cc +rr*rr)/F,2*N));
              break;
           case HighPass:
              if(cc*cc +rr*rr==0)
                 Huv = 0;
              else
                 Huv = 1.0/(1.0 + pow(F/sqrt(cc*cc +rr*rr),2*N));
              break;
         }
         Data(c,r)*= Huv;
      }
}
//---------------------------------------------------------------------------
void
ComplexImage::RingFrequencyMask(int freq_1, int freq_2, double mtp, RingKind ring_kind)
{
   double Huv;
   int cc, rr; // frequency koefficients
   for(register int c(0); c<cols; c++)
      for(register int r(0); r<rows; r++){
         cc = (c < cols/2) ? cols/2 - c : c - cols/2;
         rr = (r < rows/2) ? rows/2 - r : r - rows/2;
         switch (ring_kind){
           case ring_normal:
              if (freq_1 <= sqrt(cc*cc +rr*rr) && sqrt(cc*cc +rr*rr)<freq_2)
                 Huv = mtp;
              else
                 Huv = 1;
              break;
           case ring_inverse:
              if (freq_1 > (int)sqrt(cc*cc +rr*rr) || (int)sqrt(cc*cc +rr*rr) > freq_2)
                 Huv = mtp;
              else
                 Huv = 1;
              break;
         }
         Data(c,r)*= Huv;
      }
}
//---------------------------------------------------------------------------
void
ComplexImage::RectFrequencyMask(TPoint p1, TPoint p2, double mtp)
{
   TPoint pp1, pp2;
   TPoint temp1, temp2;
   temp1.x  = (p2.x > p1.x) ? p1.x : p2.x;
   temp1.y  = (p2.y > p1.y) ? p1.y : p2.y;
   temp2.x  = (p2.x < p1.x) ? p1.x : p2.x;
   temp2.y  = (p2.y < p1.y) ? p1.y : p2.y;

   p1 = temp1;
   p2 = temp2;

   pp1.x = p1.x - (p1.x-cols/2)*2;
   pp1.y = p1.y + (rows/2-p1.y)*2;
   pp2.x = p2.x - (p2.x-cols/2)*2;
   pp2.y = p2.y + (rows/2-p2.y)*2;

   for(register int c(0); c<cols; c++)
      for(register int r(0); r<rows; r++){
         if ((p1.x < c) && (c < p2.x) && (p1.y < r) && (r < p2.y))
             Data(c,r)*= mtp;
         else
            if ((pp2.x < c) && (c < pp1.x) && (pp2.y < r) && (r < pp1.y))
               Data(c,r)*= mtp;
      }
}
//---------------------------------------------------------------------------
void
ComplexImage::MultipleFrequencyMask(double mtp)
{
   for(register int c(0); c<cols; c++){
      for(register int r(0); r<rows; r++){
         Data(c,r)*= mtp;
      }
   }
}
//---------------------------------------------------------------------------
void
ComplexImage::ScissorsFrequencyMask(double angle, double mtp, ScissorsKind kind)
{
   switch(kind){
      case scissor_vertical:
         for(register int c(0); c<cols; c++)
            for(register int r(0); r<rows; r++){
               if(abs(rows/2 - r) > abs(cols/2 - c)*tan(angle))
                  Data(c,r) *= mtp;
            }
         break;
      case scissor_horizontal:
         for(register int c(0); c<cols; c++)
            for(register int r(0); r<rows; r++){
               if(abs(rows/2 - r) < abs(cols/2 - c)*tan(angle))
                  Data(c,r) *= mtp;
            }
         break;
   }
}
//---------------------------------------------------------------------------
ComplexImage operator * (const ComplexImage& left,
                                           const ComplexImage& right)
{
   Assert((left.Cols()==right.Cols())&&(left.Rows()==right.Rows()),
      VEMsg(AnsiString("the two images aren't of the same size")));
   ComplexImage complex_image(left.Cols(), left.Rows());
   for(register int c(0); c<left.Cols(); c++)
      for(register int r(0); r<right.Rows(); r++){
         complex_image.Data(c,r) = left.Data(c,r)*right.Data(c,r);
      }
   return complex_image;
}
//---------------------------------------------------------------------------
ComplexImage operator / (const ComplexImage& left,
                                           const ComplexImage& right)
{
   Assert((left.Cols()==right.Cols())&&(left.Rows()==right.Rows()),
      VEMsg(AnsiString("the two images aren't of the same size")));
   ComplexImage complex_image(left.Cols(), left.Rows());
   for(register int c(0); c<left.Cols(); c++)
      for(register int r(0); r<right.Rows(); r++){
         if(abs(right.Data(c,r))<0.05)
            complex_image.Data(c,r) = vcomplex(left.Data(c,r).real(),
                                               left.Data(c,r).imag());
         else
            complex_image.Data(c,r) = left.Data(c,r)/right.Data(c,r);
      }
   return complex_image;
}
//---------------------------------------------------------------------------
ComplexImage
ComplexImage::Divide(const ComplexImage& right, double bound)
{
   Assert((this->Cols()==right.Cols())&&(this->Rows()==right.Rows()),
      VEMsg(AnsiString("the two images aren't of the same size")));
   for(register int c(0); c<Cols(); c++)
      for(register int r(0); r<Rows(); r++){
         if(abs(right.Data(c,r))< bound)
            this->Data(c,r) = vcomplex(this->Data(c,r).real(),
                                               this->Data(c,r).imag());
         else
            this->Data(c,r) = this->Data(c,r)/right.Data(c,r);
      }
   return *this;
}
//---------------------------------------------------------------------------
ComplexImage
ComplexImage::ConstPhase()
{
   ComplexImage temp(Cols(), Rows());
   for(register int c(0); c<Cols(); c++)
      for(register int r(0); r<Rows(); r++){
         temp.Data(c,r) = vcomplex(abs(Data(c,r)),0);
      }
   return temp;
}
//---------------------------------------------------------------------------
ComplexImage
ComplexImage::ConstMagnitude()
{
   ComplexImage temp(Cols(), Rows());
   for(register int c(0); c<Cols(); c++){
      for(register int r(0); r<Rows(); r++){
         if (abs(Data(c,r)) == 0){
            temp.Data(c,r) = vcomplex(0,0);
         }else{
            temp.Data(c,r) = vcomplex(Data(c,r).real()/abs(Data(c,r)),
                             Data(c,r).imag()/abs(Data(c,r)));
         }
      }
   }
   return temp;
}
//---------------------------------------------------------------------------
ComplexImage&
ComplexImage::operator *= (double m)
{
   for(register int c(0); c<Cols(); c++)
      for(register int r(0); r<Rows(); r++)
         Data(c,r) *= m;
   return *this;
}
//---------------------------------------------------------------------------
ComplexImage&
ComplexImage::operator /= (double d)
{
   for(register int c(0); c<Cols(); c++)
      for(register int r(0); r<Rows(); r++)
         Data(c,r) /= d;
   return *this;
}
