/*
   image_complex.h

   hlavikov soubor tdy pro tdu komplexnch obraz (po FFT)

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#ifndef image_complexH
#define image_complexH
//---------------------------------------------------------------------------
#include "image.h"
#include "image_const.h"
#include "image_grey.h"
#include <math.h>
#include <complex>
using namespace std;
typedef complex<double> vcomplex;

// basic class for greyscale images
enum CircularKind {LowPass, HighPass};
enum ScissorsKind {scissor_vertical, scissor_horizontal};
enum RingKind {ring_normal, ring_inverse};

class ComplexImage : public Image<vcomplex>
{
private:
public:
   // constructors
   ComplexImage()
     :Image<vcomplex>()  {}
   ComplexImage(int _cols, int _rows)
     :Image<vcomplex>(_cols,_rows) {}
   // destructor
   virtual ~ComplexImage() {}
   // member functions
   int       Rows()       {return rows;}
   int       Cols()       {return cols;}
   // const object
   const int Rows() const {return rows;}
   const int Cols() const {return cols;}
   // non-const object
   vcomplex&        Data(int col, int row)
       { return data[col + cols*row];}
   // const object
   const vcomplex&  Data(int col, int row) const
       { return data[col + cols*row];}
   // functions for getting and setting real and imag parts
   // of image
   GreyImage ReImage(int scale=1);
   GreyImage ImImage(int scale=1);
   GreyImage MagImage(int scale=1);
   GreyImage PhaseImage();
   void      ReImage(GreyImage* init, bool center = true);
   void      ImImage(GreyImage* init);
   // filtering functions
   // Cyrculary Symmetric Filter
   void      CyrcularSymFilter(double F, int N, CircularKind circular_kind);
   void      LowWindowFilter(int c_coef, int r_coef);
   void      HighWindowFilter(int c_coef, int r_coef);
   void      RectFrequencyMask(TPoint p1, TPoint p2, double mtp=0);
   void      MultipleFrequencyMask(double mtp);
   void      ScissorsFrequencyMask(double angle, double mtp, ScissorsKind kind);
   void      RingFrequencyMask(int freq_1, int freq_2, double mtp, RingKind kind);
   // functions for const Phase and const Magnitude
   ComplexImage      ConstPhase();
   ComplexImage      ConstMagnitude();   
   // overloaded operator
   ComplexImage& operator = (const ComplexImage& init);
   ComplexImage& operator *= (double m);
   ComplexImage& operator /= (double d);
   // friend functions
   friend ComplexImage operator * (const ComplexImage& left,
                                           const ComplexImage& right);
   friend ComplexImage operator / (const ComplexImage& left,
                                           const ComplexImage& right);
   ComplexImage Divide(const ComplexImage& right, double bound = 0.05);

}; // class ComplexImage
#endif
