/*
   image_grey.cpp

   definin soubor tdy pro zkladn tdu 8bitovch obraz.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "image_grey.h"

//---------------------------- -----------------------------------------------
#pragma package(smart_init)

// member functions
//---------------------------------------------------------------------------
// This function is used to copy a portion(or all) of an image.
// the dimensions of the portion of the image to copy are described
// by row, col, width, height
void
GreyImage::ReadImageAreaToImage(GreyImage& buff,int col, int row,
                             int width, int height)
{
   unsigned long PixelBuff = 0;
   Assert((row<rows)&&(row>=0)&&(col<cols)&&(col>=0)&&
          (row+height<rows)&&(col+width<cols),VEMsg(AnsiString
          ("GreyImage::ReadImageAreaToImage : buffer goes\n out of border")));

   buff.Resize(width,height);
   for(register int r(row); r < row+height; r++)
      for(register int c(col); c < col+width; c++)
         buff.Data(PixelBuff++) = Data(c,r);
}

//--------------------------------------------------------------------------
// This function performs a complementary operation to the
// ReadImageAreaToImage function defined above
void
GreyImage::WriteImageAreaFromImage(GreyImage& buff,int col, int row,
                             int width, int height)
{
   Assert((row<=rows)&&(row>=0)&&(col<=cols)&&(col>=0)&&
      (row+height<=rows)&&(col+width<=cols),VEMsg(AnsiString
      ("GreyImage::WriteImageAreaFromImage : buffer goes\n out of border")));


   for(register int r(0); r < height; r++)
      for(register int c(0); c < width; c++)
         Data(col+c, row+r) = buff.Data(c,r);
}
//--------------------------------------------------------------------------
// This function sets an area of an image specified by row, col, width and
// height to the pixel_value specified
void
GreyImage::ClearImageArea(unsigned char pixel_value, int width,
                          int height ,int col, int row)
{
  Assert((row<=rows)&&(row>=0)&&(col<=cols)&&(col>=0)&&
      (row+height<=rows)&&(col+width<=cols),VEMsg(AnsiString
      ("GreyImage::ClearImageArea : buffer goes\n out of border")));

   for(register int r(row); r < row+height; r++)
      for(register int c(col); c < col+width; c++)
         Data(c, r) = pixel_value;
}
//---------------------------------------------------------------------------
void
GreyImage::BitField(GreyImage& image, int b, int width,
                       int height, int col, int row)
{
   Assert((row<Rows())&&(row>=0)&&(col<cols)&&(col>=0)&&
      (row+height<=rows)&&(col+width<=cols),VEMsg(AnsiString
      ("GreyImage::BitField : buffer goes\n out of border")));

   if(this->Cols()!=image.Cols()||this->Rows()!=image.Rows())
      image.Resize(this->Cols(),this->Rows());
   int data;
   for(register int r(row); r < row+height; r++)
      for(register int c(col); c < col+width; c++){
         switch(b){
            case 0: data = ((Data(c,r)&1)==1)    ? 255 : 0; break;
            case 1: data = ((Data(c,r)&2)==2)    ? 255 : 0; break;
            case 2: data = ((Data(c,r)&4)==4)    ? 255 : 0; break;
            case 3: data = ((Data(c,r)&8)==8)    ? 255 : 0; break;
            case 4: data = ((Data(c,r)&16)==16)  ? 255 : 0; break;
            case 5: data = ((Data(c,r)&32)==32)  ? 255 : 0; break;
            case 6: data = ((Data(c,r)&64)==64)  ? 255 : 0; break;
            case 7: data = ((Data(c,r)&128)==128)? 255 : 0; break;
            default : data = 0;
         }
         image.Data(c,r) = (unsigned char)data;
      }
}
//--------------------------------------------------------------------------
// draw circle
void
GreyImage::DrawCircle(int ic, int ir, int irad, unsigned char pixel_value)
{
   for(register int r = 0; r < Rows(); r++)
      for(register int c = 0; c < Cols(); c++)
         if (irad == (int)sqrt((c-ic)*(c-ic)+(r-ir)*(r-ir)))
             Data(c, r) = pixel_value;

}
//--------------------------------------------------------------------------
// draw line
void
GreyImage::Drawline(int p1r, int p1c, int p2r, int p2c, unsigned char pixel_value)
{
   int number_r, number_c;
   float step_r, step_c, r, c;
   
   number_r = p2r - p1r;
   number_c = p2c - p1c;

   step_r = ((float)(number_r)) / (float)(150);
   step_c = ((float)(number_c)) / (float)(150);

   r = (float)(p1r);
   c = (float)(p1c);

   for (int i = 0; i < 150; i++){
     r += step_r;
     c += step_c;
     Data((int)(c), (int)(r)) = pixel_value;
   }
}
//--------------------------------------------------------------------------
// draw cross
void
GreyImage::DrawCross(int ic, int ir, unsigned char pixel_value)
{
   for(int r = 0; r < Rows(); r++)
      Data(ic, r) = pixel_value;
   for(int c = 0; c < Cols(); c++)
      Data(c, ir) = pixel_value;
}




