/*
   In_out.cpp

   definin soubor tdy pro prci se soubory

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "in_out.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// constructors
__fastcall VInOut::VInOut(AnsiString _file_path)
   :file_path(_file_path), image(NULL), color_image(NULL), mstream(NULL)
{
}
//---------------------------------------------------------------------------
__fastcall VInOut::VInOut(BMPHandler* handler)
   :file_path(handler->FileName()), mstream(NULL)
{
   image = new GreyImage;
   handler->Set(*image);
}
//---------------------------------------------------------------------------
__fastcall VInOut::VInOut(BMPHandlerColor* color_handler)
   :file_path(color_handler->FileName()), mstream(NULL)
{
   color_image = new ColorImage;
   color_handler->Set(*color_image);
}
//---------------------------------------------------------------------------
// destructor
__fastcall VInOut::~VInOut()
{
   delete (image);
   delete (color_image);
}
//---------------------------------------------------------------------------
void
__fastcall VInOut::CheckSuffix()
{
   int suffix_size = 0;
   suffix_size = file_path.Length() - file_path.LastDelimiter(".");
   Assert(suffix_size!=0,VEMsg(AnsiString("Suffix is not valid.")));
   AnsiString tmp_suf = file_path.SubString(file_path.LastDelimiter(".")+1,
                                              suffix_size);
   Assert(suffix.AnsiCompare(tmp_suf.LowerCase())==0,
         VEMsg(AnsiString("Suffix is not valid.")));

}
//---------------------------------------------------------------------------
AnsiString
__fastcall VInOut::ImageName()
{
   int image_name_size = file_path.Length() - file_path.LastDelimiter("\\");
   return (file_path.SubString(file_path.LastDelimiter("\\")+1,
                                              image_name_size));
}

//---------------------------------------------------------------------------
// this function converts GreyImage (created when reading the image in the
// ancestors constructor) to the bitmap
void
__fastcall VInOut::ConvertToBMP(BMPHandler* handler)
{
   handler->FileName() = file_path;
   mstream = new TMemoryStream();
   SizeBMP_G sizes(image->Cols(), image->Rows());
   unsigned char* buffer;
   buffer = new unsigned char[54];
   // writing a head of the bmp file
   for (register int i=0; i<54; i++) {
      switch (i) {
         case 0 :  buffer[i] = 0x42;              break;
	 case 1 :  buffer[i] = 0x4d;              break;
	 case 2 :  buffer[i] = sizes.size_file_l; break;
	 case 3 :  buffer[i] = sizes.size_file_h; break;
         case 4 :  buffer[i] = sizes.size_file_hh;break;
	 case 18 : buffer[i] = sizes.width_l;     break;
	 case 19 : buffer[i] = sizes.width_h;     break;
	 case 22 : buffer[i] = sizes.height_l;    break;
	 case 23 : buffer[i] = sizes.height_h;    break;
	 case 34 : buffer[i] = sizes.size_bmp_l;  break;
 	 case 35 : buffer[i] = sizes.size_bmp_h;  break;
	 case 36 : buffer[i] = sizes.size_bmp_hh; break;
	 case 10 : buffer[i] = 0x36; break;
 	 case 11 : buffer[i] = 0x04; break;
	 case 14 : buffer[i] = 0x28; break;
	 case 26 : buffer[i] = 0x01; break;
	 case 28 : buffer[i] = 0x08; break;
   	 default : buffer[i] = 0;    break;
      }
   }
   mstream->Write(buffer, 54);
   delete [] buffer;
   buffer = new unsigned char[4*256];
   // LUTable of BMP file
   unsigned long index = 0;
   for(register int i=0; i<256; i++){
      buffer[index++] = ((unsigned char) i);
      buffer[index++] = ((unsigned char) i);
      buffer[index++] = ((unsigned char) i);
      buffer[index++] = ((unsigned char) 0);
   }
   mstream->Write(buffer, 4*256);
   delete [] buffer;
   int rest = image->Cols()%4;
   int new_size = image->Rows()*(image->Cols()+4-rest);
   if(rest!=0)
      buffer = new unsigned char[new_size];
   else
      buffer = new unsigned char[image->Rows()*image->Cols()];
   // data
   index = 0;
   int rest_tmp;
   for(register int r(image->Rows()-1); r>-1; r--){
      for(register int c(0); c<image->Cols(); c++){
         buffer[index++] = image->Data(c,r);
      }
      if(rest!=0){
         rest_tmp = rest;
         while((4-rest_tmp)!=0){
            buffer[index++] = 0;
            rest_tmp++;
         }
      }
   }
   if(rest!=0){
      mstream->Write(buffer, new_size);
   }else
      mstream->Write(buffer, image->Rows()*image->Cols());
   mstream->Position = 0;
   handler->Assign(mstream);

   delete (mstream);
   delete [] buffer;
}
//---------------------------------------------------------------------------
// this function converts ColorImage (created when reading the image in the
// ancestors constructor) to the bitmap
void
__fastcall VInOut::ConvertToColorBMP(BMPHandlerColor* color_handler)
{
   color_handler->FileName() = file_path;
   mstream = new TMemoryStream();
   SizeBMP_G sizes(color_image->Cols(), color_image->Rows());
   unsigned char* buffer;
   buffer = new unsigned char[54];
   // writing a head of the bmp file
   for (register int i=0; i<54; i++) {
      switch (i) {
         case 0 :  buffer[i] = 0x42;                    break;
	 case 1 :  buffer[i] = 0x4d;                    break;
	 case 2 :  buffer[i] = sizes.size_color_file_l; break;
	 case 3 :  buffer[i] = sizes.size_color_file_h; break;
         case 4 :  buffer[i] = sizes.size_color_file_hh;break;
	 case 18 : buffer[i] = sizes.width_l;           break;
	 case 19 : buffer[i] = sizes.width_h;           break;
	 case 22 : buffer[i] = sizes.height_l;          break;
	 case 23 : buffer[i] = sizes.height_h;          break;
	 case 34 : buffer[i] = sizes.size_bmp_l;        break;
 	 case 35 : buffer[i] = sizes.size_bmp_h;        break;
	 case 36 : buffer[i] = sizes.size_bmp_hh;       break;
	 case 10 : buffer[i] = 0x36; break;
 	 case 11 : buffer[i] = 0x00; break;
	 case 14 : buffer[i] = 0x28; break;
	 case 26 : buffer[i] = 0x01; break;
	 case 28 : buffer[i] = 0x18; break;
   	 default : buffer[i] = 0;    break;
      }
   }
   mstream->Write(buffer, 54);
   delete [] buffer;
   int rest = color_image->Cols()%4;
   int new_size = 3*color_image->Rows()*(color_image->Cols()+4-rest);
   if(rest!=0)
      buffer = new unsigned char[new_size];
   else
      buffer = new unsigned char[3*color_image->Rows()*color_image->Cols()];
   // data
   int index = 0;
   int rest_tmp;
   for(register int r(color_image->Rows()-1); r>-1; r--){
      for(register int c(0); c<color_image->Cols(); c++){
         buffer[index++] = color_image->Data(c,r);
      }
      if(rest!=0){
         rest_tmp = rest;
         while((4-rest_tmp)!=0){
            buffer[index++] = 0;
            rest_tmp++;
         }
      }
   }
   if(rest!=0){
      mstream->Write(buffer, new_size);
   }else
      mstream->Write(buffer, 3*color_image->Rows()*color_image->Cols());
   mstream->Position = 0;
   color_handler->Assign(mstream);

   delete (mstream);
   delete [] buffer;
}
//---------------------------------------------------------------------------
void
__fastcall VInOut::Assign(GreyImage& _image)
{
   image = new GreyImage(_image);
}
//---------------------------------------------------------------------------
void
__fastcall VInOut::SaveToBMP(AnsiString _save_file_name)
{
   mstream = new TMemoryStream();
   SizeBMP_G sizes(image->Cols(), image->Rows());
   unsigned char* buffer;
   buffer = new unsigned char[54];
   // writing a head of the bmp file
   for (register int i=0; i<54; i++) {
      switch (i) {
         case 0 :  buffer[i] = 0x42;              break;
	 case 1 :  buffer[i] = 0x4d;              break;
	 case 2 :  buffer[i] = sizes.size_file_l; break;
	 case 3 :  buffer[i] = sizes.size_file_h; break;
         case 4 :  buffer[i] = sizes.size_file_hh;break;
	 case 18 : buffer[i] = sizes.width_l;     break;
	 case 19 : buffer[i] = sizes.width_h;     break;
	 case 22 : buffer[i] = sizes.height_l;    break;
	 case 23 : buffer[i] = sizes.height_h;    break;
	 case 34 : buffer[i] = sizes.size_bmp_l;  break;
 	 case 35 : buffer[i] = sizes.size_bmp_h;  break;
	 case 36 : buffer[i] = sizes.size_bmp_hh; break;
	 case 10 : buffer[i] = 0x36; break;
 	 case 11 : buffer[i] = 0x04; break;
	 case 14 : buffer[i] = 0x28; break;
	 case 26 : buffer[i] = 0x01; break;
	 case 28 : buffer[i] = 0x08; break;
   	 default : buffer[i] = 0;    break;
      }
   }
   mstream->Write(buffer, 54);
   buffer = new unsigned char[4*256];
   // LUTable of BMP file
   unsigned long index = 0;
   for(register int i=0; i<256; i++){
      buffer[index++] = ((unsigned char) i);
      buffer[index++] = ((unsigned char) i);
      buffer[index++] = ((unsigned char) i);
      buffer[index++] = ((unsigned char) 0);
   }
   mstream->Write(buffer, 4*256);
   int rest = image->Cols()%4;
   if(rest!=0)
      buffer = new unsigned char[image->Rows()*(image->Cols()+4-rest)];
   else
      buffer = new unsigned char[image->Rows()*image->Cols()];
   // data
   index = 0;
   int rest_tmp;
   for(register int r(image->Rows()); r>0; r--){
      for(register int c(0); c<image->Cols(); c++){
         buffer[index++] = image->Data(c,r);
      }
      if(rest!=0){
         rest_tmp = rest;
         while((4-rest_tmp)!=0){
            buffer[index++] = 0;
            rest_tmp++;
         }
      }
   }
   if(rest!=0)
      mstream->Write(buffer, image->Rows()*(image->Cols()+4-rest));
   else
      mstream->Write(buffer, image->Rows()*image->Cols());
   mstream->Position = 0;

   Graphics::TBitmap * bitmap = new Graphics::TBitmap();
   bitmap->LoadFromStream(mstream);
   bitmap->SaveToFile(_save_file_name);
   delete (mstream);
   delete (buffer);
   delete (bitmap);
}
//---------------------------------------------------------------------------
void
__fastcall VInOut::ChangePath(AnsiString _file_path)
{
  file_path = _file_path;
}
//---------------------------------------------------------------------------
// Save data
void
__fastcall VInOut::SaveProfile(TPoint points[], AnsiString file_path)
{
   ofstream _file(file_path.c_str());
   for (int i = 0; i < 360; i++){
      _file << 273 - points[i].y << endl;
   }
   _file.close();
}
