/*
   point_image.cpp

   definin soubor tdy s bodovmi operacemi

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "point_image.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

// constructors
//---------------------------------------------------------------------------
PointImage::PointImage(int _cols, int _rows)
:GreyImage(_cols,_rows)
{
}

// functions for drawing
//---------------------------------------------------------------------------
// bool DrawHLine function
void
PointImage::DrawHLine(int col, int row,
                      int length, unsigned char grey)
{
   Assert((col <= cols)&&((col+length)<=cols)&&
      (row <= rows),VEMsg(AnsiString
      ("PointImage::DrawHLine:the two points don't share the same row ")));
   {
      length++;
      while(length--)
         Data(row,col++) = grey;
   }
}

//---------------------------------------------------------------------------
// bool DrawHLine function
void
PointImage::DrawHLine(const VPoint& begin, const VPoint& end,
                      unsigned char grey)
{
   Assert((begin.Col() <= cols)&&((end.Col())<=cols)&&
         (begin.Row() == end.Row())&&(end.Row() <= rows),
          VEMsg(AnsiString
          ("PointImage::DrawHLine:the two points don't share the same row ")));

   {
     if(begin.Col()<end.Col())
       for(int c(begin.Col()); c<end.Col(); c++)
         Data(begin.Row(),c) = grey;
     else
       for(int c(end.Col()); c<begin.Col(); c++)
         Data(begin.Row(),c) = grey;

   }
}

//---------------------------------------------------------------------------
// void DrawVLine function
void
PointImage::DrawVLine(int col, int row,
                      int length, unsigned char grey)
{
   Assert((col <= cols)&&((row+length)<=rows)&&
      (row <= rows),VEMsg(AnsiString
          ("PointImage::DrawVLine:the two points don't share the same colmn ")));

   {
      length++;
      while(length--)
         Data(row++,col) = grey;
   }
}

//---------------------------------------------------------------------------
// void DrawVLine function
void
PointImage::DrawVLine(const VPoint& begin, const VPoint& end,
                      unsigned char grey)
{
   Assert((begin.Row() <= rows)&&((end.Row())<= rows)&&
         (begin.Col() == end.Col())&&(end.Col() <= cols),
          VEMsg(AnsiString
          ("PointImage::DrawVLine:the two points don't share the same colmn")));

   {
     if(begin.Row()<end.Row())
       for(int r(begin.Row()); r<end.Row(); r++)
         Data(r,begin.Col()) = grey;
     else
       for(int r(end.Row()); r<begin.Row(); r++)
         Data(r,begin.Col()) = grey;
   }
}

//-------------------------------------------------------------------------
void
PointImage::DrawRect(const VPoint& begin, const VPoint& end,
                     unsigned char pixel_value)
{
   Assert((begin.Row() <= rows)&&((end.Row())<= rows)&&
         (begin.Col() <= cols)&&(end.Col() <= cols),
          VEMsg(AnsiString
          ("PointImage::DrawRect:one of the point is out of image")));

   VPoint top_left, top_right, bottom_left, bottom_right;
   if(begin.Row()<end.Row()){
      bottom_left.Y() = bottom_right.Y() = begin.Row();
      top_left.Y() = top_right.Y() = end.Row();
   }
   else{
      bottom_left.Y() = bottom_right.Y() = end.Row();
      top_left.Y() = top_right.Y() = begin.Row();
   }
   if(begin.Col()<end.Col()){
      bottom_left.X() = top_left.X() = begin.Col();
      bottom_right.X() = top_right.X() = end.Col();
   }
   else{
      bottom_left.X() = top_left.X() = end.Col();
      bottom_right.X() = top_right.X() = begin.Col();
   }
   DrawVLine(bottom_left,top_left,pixel_value);
   DrawVLine(bottom_right,top_right,pixel_value);
   DrawHLine(bottom_left,bottom_right,pixel_value);
   DrawHLine(top_left,top_right,pixel_value);
}

// point transform functions
//-------------------------------------------------------------------------
// This function applies a transform previosly programmed into LookUpTable
// to the image or portion of image;
void
PointImage::ApplyLUT(LUT& lut, int col, int row, int width, int height)
{
   Assert((col <= cols)&&((col+width)<=cols)&&
      (row <= rows)&&((row+height)<=rows),VEMsg(AnsiString
      ("PointImage::ApplyLUT:the portion goes out of image")));
   for(register int c(col); c<col+width; c++)
      for(register int r(row); r<row+height; r++)
        Data(c,r) = (unsigned char)lut[Data(c,r)];
}

//---------------------------------------------------------------------------
// This functoin adjust the brightness of an image; it goes through an image
// or portion of an image and adds brightness_factor to each pixel it
// encounteres. brightness_factor can be positive or negative. positive value
// lighten the image, whereas negative value darken it
void
PointImage::AdjImageBrightness(short brightness_factor, int col, int row,
                        int width, int height)
{
   /*LUT bright_lut;
   register int new_value;
   for(register int index(0); index<256; index++)
   {
      new_value = index + brightness_factor;
      new_value = (new_value < 0  ) ? 0   : new_value;
      new_value = (new_value > 255) ? 255 : new_value;
      bright_lut[index] = new_value;
   }
   ApplyLUT(bright_lut, col, row, width, height);*/
}

//---------------------------------------------------------------------------
// This function will negate an image pixel by pixel. treshold is value of
// image data where the negation begins. If treshold is 0, all pixels value
// are negated. that is, pixel 0 becomes 255. If treshold is grater than 0,
// thi pixel value in the range 0..trashold-1 are left alone while pixel
// values treshold..255 ar negated
void
PointImage::NegateImage(short treshold, int col, int row,
                        int width, int height)
{
   LUT negate_lut;
   for(register int index(treshold); index<MAX_QUANT_LEVELS; index++)
      negate_lut[index] = MAX_SAMPLE - index;
   ApplyLUT(negate_lut, col, row, width, height);
}

//---------------------------------------------------------------------------
// This function converts a grey scale image into a binary image with
// each pixel either WHITE or BLACK. Cut off is made by trashold. Pixels
// in range 0..trashold-1 become black while pixel values between trashold..
// 256 become white;
void
PointImage::TresholdImage(short treshold, int col, int row,
                        int width, int height)
{
   LUT treshold_lut;
   for(register int index(0); index < treshold; index++)
       treshold_lut[index] = BLACK;

   for(register int index(treshold); index < MAX_QUANT_LEVELS; index++)
       treshold_lut[index] = WHITE;

   ApplyLUT(treshold_lut, col, row, width, height);
}

//---------------------------------------------------------------------------
// This function calculates the histogram of any portion of an image
void
PointImage::GenHistogram(int col, int row, int width,
                         int height)
{
   histogram = new Histogram();
   histogram->InitFromGreyImage(*this,col,row,width,height);
}

//---------------------------------------------------------------------------
// functoin for drawing an histogram on canvas
void
PointImage::DisplayHistogram(TImage* _Image)
{
   histogram->Display(_Image);
}

// overloaded member functions
//-------------------------------------------------------------------------
PointImage&
PointImage::operator = (const GreyImage& right)
{
   if(!(rows==right.Rows())&&(cols==right.Cols()))
   Resize(right.Rows(),right.Cols());
   
   for(int i(0); i<rows*cols; i++)
      data[i] = right.Data(i);
   return *this;
}
