/*
   pseudo_color.cpp

   definin soubor definujc funkce pro pseudobarevn operace

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "pseudo_color.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

// constructors
//---------------------------------------------------------------------------
// default constructor
PseudoColor::PseudoColor()
{
}
//---------------------------------------------------------------------------
void
__fastcall PseudoColor::Transform(GreyImage* image1,
           TMemoryStream* memory_stream, int palette[3][256])
{
   SizeBMP_G sizes(image1->Cols(), image1->Rows());
   buffer = new unsigned char[54];
   // writing a head of the bmp file
   for (register int i=0; i<54; i++) {
      switch (i) {
         case 0 :  buffer[i] = 0x42;              break;
	 case 1 :  buffer[i] = 0x4d;              break;
	 case 2 :  buffer[i] = sizes.size_file_l; break;
	 case 3 :  buffer[i] = sizes.size_file_h; break;
         case 4 :  buffer[i] = sizes.size_file_hh;break;
	 case 18 : buffer[i] = sizes.width_l;     break;
	 case 19 : buffer[i] = sizes.width_h;     break;
	 case 22 : buffer[i] = sizes.height_l;    break;
	 case 23 : buffer[i] = sizes.height_h;    break;
	 case 34 : buffer[i] = sizes.size_bmp_l;  break;
 	 case 35 : buffer[i] = sizes.size_bmp_h;  break;
	 case 36 : buffer[i] = sizes.size_bmp_hh; break;
	 case 10 : buffer[i] = 0x36; break;
 	 case 11 : buffer[i] = 0x04; break;
	 case 14 : buffer[i] = 0x28; break;
	 case 26 : buffer[i] = 0x01; break;
	 case 28 : buffer[i] = 0x08; break;
   	 default : buffer[i] = 0;    break;
      }
   }
   memory_stream->Write(buffer, 54);
   delete [] buffer;
   buffer = new unsigned char[4*256];
   // LUTable of BMP file
   unsigned long index = 0;
   for(register int i=0; i<256; i++){
      buffer[index++] = ((unsigned char) palette[0][i]);
      buffer[index++] = ((unsigned char) palette[1][i]);
      buffer[index++] = ((unsigned char) palette[2][i]);
      buffer[index++] = ((unsigned char) 0);
   }

   memory_stream->Write(buffer, 4*256);
   delete [] buffer;
   int rest = image1->Cols()%4;
   int new_size = image1->Rows()*(image1->Cols()+4-rest);
   if(rest!=0)
      buffer = new unsigned char[new_size];
   else
      buffer = new unsigned char[image1->Rows()*image1->Cols()];
     // data
   index = 0;
   int rest_tmp;
   for(register int r(image1->Rows()-1); r>-1; r--){
      for(register int c(0); c<image1->Cols(); c++){
         buffer[index++] = image1->Data(c,r);
      }
      if(rest!=0){
         rest_tmp = rest;
         while((4-rest_tmp)!=0){
            buffer[index++] = 0;
            rest_tmp++;
         }
      }
   }
   if(rest!=0){
      memory_stream->Write(buffer, new_size);
   }else
      memory_stream->Write(buffer, image1->Rows()*image1->Cols());
   memory_stream->Position = 0;
}
//---------------------------------------------------------------------------
void
__fastcall PseudoColor::ChangePalette(int palette[3][256],
           Palette_Type pal)
{
   int a=0; int b=0; int c=255;
   int L1[3]={0, 0, 0};
   int L2[3]={255, 255, 255};
   switch(pal){
      case GREY: // grey scale
         for(int i=0; i<256; i++){
            palette[0][i] = i;
            palette[1][i] = i;
            palette[2][i] = i;
         }
         break;
      case BONE:
         L2[2]=74; L2[1]=74; L2[0]=106; //order R-G-B
         Gradient(0, 85, L1, L2, palette);
         L1[2]=163; L1[1]=195; L1[0]=195;
         Gradient(86, 170, L2, L1, palette);
         L2[2]=255; L2[1]=255; L2[0]=255;
         Gradient(171, 255, L1, L2, palette);
         break;
      case COPPER:
         L2[2]=255; L2[1]=159; L2[0]=101;
         Gradient(0, 207, L1, L2, palette);
         L1[2]=255; L1[1]=199; L1[0]=127;
         Gradient(208, 255, L2, L1, palette);
         break;
      case HOT:
         L2[2]=255; L2[1]=0; L2[0]=0;
         Gradient(0, 85, L1, L2, palette);
         L1[2]=255; L1[1]=255; L1[0]=0;
         Gradient(86, 170, L2, L1, palette);
         L2[2]=255; L2[1]=255; L2[0]=255;
         Gradient(171, 255, L1, L2, palette);
         break;
      case JET:
         L1[2]=0; L1[1]=0; L1[0]=143;
         L2[2]=0; L2[1]=0; L2[0]=255;
         Gradient(0, 32, L1, L2, palette);
         L1[2]=0; L1[1]=255; L1[0]=255;
         Gradient(33, 96, L2, L1, palette);
         L2[2]=255; L2[1]=255; L2[0]=0;
         Gradient(97, 159, L1, L2, palette);
         L1[2]=255; L1[1]=0; L1[0]=0;
         Gradient(160, 223, L2, L1, palette);
         L2[2]=128; L2[1]=0; L2[0]=0;
         Gradient(224, 255, L1, L2, palette);
         break;
      case HSV:
         L1[2]=255; L1[1]=0; L1[0]=0;
         L2[2]=0; L2[1]=255; L2[0]=64;
         Gradient(0, 85, L1, L2, palette);
         L1[2]=128; L1[1]=0; L1[0]=255;
         Gradient(86, 170, L2, L1, palette);
         L2[2]=255; L2[1]=0; L2[0]=96;
         Gradient(171, 255, L1, L2, palette);
         break;
      case FLAG: // red|white|blue|black..
         for(int i=0; i<256; i++){
            palette[0][i] = a;
            palette[1][i] = b;
            palette[2][i] = c;
            if ((i%8==0) && (i!=0)){
              if ((b==0) && (c==255)){a=255; b=255; c=255;}else{
              if ((a==255) && (b==255)){a=255; b=0; c=0;}else{
              if ((a==255) && (c==0)){a=0; b=0; c=0;}else{
              a=0; b=0; c=255;}}}
            }
         }
         break;
   }
}
//---------------------------------------------------------------------------
void
__fastcall PseudoColor::CreateImageFromPalette(int palette[3][256],
                   TMemoryStream* memory_stream)
{
   SizeBMP_G sizes(256,1);
   buffer = new unsigned char[54];
   // writing a head of the bmp file
   for (register int i=0; i<54; i++) {
      switch (i) {
         case 0 :  buffer[i] = 0x42;              break;
	 case 1 :  buffer[i] = 0x4d;              break;
	 case 2 :  buffer[i] = sizes.size_file_l; break;
	 case 3 :  buffer[i] = sizes.size_file_h; break;
         case 4 :  buffer[i] = sizes.size_file_hh;break;
	 case 18 : buffer[i] = sizes.width_l;     break;
	 case 19 : buffer[i] = sizes.width_h;     break;
	 case 22 : buffer[i] = sizes.height_l;    break;
	 case 23 : buffer[i] = sizes.height_h;    break;
	 case 34 : buffer[i] = sizes.size_bmp_l;  break;
 	 case 35 : buffer[i] = sizes.size_bmp_h;  break;
	 case 36 : buffer[i] = sizes.size_bmp_hh; break;
	 case 10 : buffer[i] = 0x36; break;
 	 case 11 : buffer[i] = 0x04; break;
	 case 14 : buffer[i] = 0x28; break;
	 case 26 : buffer[i] = 0x01; break;
	 case 28 : buffer[i] = 0x08; break;
   	 default : buffer[i] = 0;    break;
      }
   }
   memory_stream->Write(buffer, 54);
   delete [] buffer;
   buffer = new unsigned char[4*256];
   // LUTable of BMP file
   unsigned long index = 0;
   for(register int i=0; i<256; i++){
      buffer[index++] = ((unsigned char) palette[0][i]);
      buffer[index++] = ((unsigned char) palette[1][i]);
      buffer[index++] = ((unsigned char) palette[2][i]);
      buffer[index++] = ((unsigned char) 0);
   }

   memory_stream->Write(buffer, 4*256);
   delete [] buffer;
   buffer = new unsigned char[256];
   // data
   for (int i=0; i<256; i++){
     buffer[i] = i;
   }
   memory_stream->Write(buffer, 256);
   memory_stream->Position = 0;
}
//---------------------------------------------------------------------------
// destructor
PseudoColor::~PseudoColor()
{
   delete buffer;
}

//---------------------------------------------------------------------------
// translation: "number" (0-16.777.215) to RGB values (0-255)
void
__fastcall PseudoColor::IntToRBG(int NotRGB, int RGB[3])
{
   RGB[0] = floor(NotRGB/65536);
   RGB[1] = floor((NotRGB - (65536*RGB[0]))/256);
   RGB[2] = NotRGB - (65536*RGB[0])-(256*RGB[1]);
}
//---------------------------------------------------------------------------
void
__fastcall PseudoColor::Gradient(int first_level, int second_level, int first_color[3],
           int second_color[3], int palette[3][256])
{
   float step[3];
   float actually[3];

   for (int i=0; i<3; i++){
       step[i] = ((float)second_color[i]-(float)first_color[i])/
                 ((float)second_level-(float)first_level+(float)1);
       actually[i] = (float)first_color[i];
   }
   for (int i=first_level; i<=second_level; i++){
      palette[0][i] = floor(actually[0]);
      palette[1][i] = floor(actually[1]);
      palette[2][i] = floor(actually[2]);
      actually[0] += step[0];
      actually[1] += step[1];
      actually[2] += step[2];
   }
}
//---------------------------------------------------------------------------
int  PseudoColor::RoundToInt(float f)
{
   int i = abs(f-floor(f)) < abs(f-ceil(f)) ? floor(f):ceil(f);
   return i;
}
//---------------------------------------------------------------------------
