/*
   scale_color.cpp

   definin soubor pro zmnu velikosti barevnho obrazu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "scale_color.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

// constructors + destructor
//---------------------------------------------------------------------------
// default constructor
Scale_color::Scale_color()
{
   original_image = NULL;
   scaled_image   = NULL;
}

//---------------------------------------------------------------------------
Scale_color::Scale_color(const ColorImage& _original_image, ColorImage& _scaled_image)
{
   original_image = &_original_image;
   scaled_image   = &_scaled_image;
}
//---------------------------------------------------------------------------
Scale_color::Scale_color(const ColorImage& _original_image, ColorImage& _scaled_image,
         int new_size)
{
   original_image = &_original_image;
   scaled_image   = &_scaled_image;

   red_image = new GreyImage(original_image->Cols(), original_image->Rows());
   green_image = new GreyImage(original_image->Cols(), original_image->Rows());
   blue_image = new GreyImage(original_image->Cols(), original_image->Rows());

   scaled_red_image = new GreyImage();
   scaled_green_image = new GreyImage();
   scaled_blue_image = new GreyImage();

   //split
   original_image->SplitToRGB(red_image, green_image, blue_image);

   // resize
   Scale scaleR(*red_image, *scaled_red_image, new_size);
   Scale scaleG(*green_image, *scaled_green_image, new_size);
   Scale scaleB(*blue_image, *scaled_blue_image, new_size);

   //combine
   delete(scaled_image);
   scaled_image = new ColorImage(scaled_red_image->Cols(),scaled_red_image->Rows());
   scaled_image->CombineFromRGB(scaled_red_image, scaled_green_image, scaled_blue_image);

}
//---------------------------------------------------------------------------
Scale_color::~Scale_color()
{
   delete (red_image);
   delete (green_image);
   delete (blue_image);
   delete (scaled_red_image);
   delete (scaled_green_image);
   delete (scaled_blue_image);
}











