/*
   translate.cpp

   definin soubor - pesun

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "translate.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)



// constructors + destructor
//---------------------------------------------------------------------------
// default constructor
Translate::Translate()
{
   original_image = NULL;
   translated_image   = NULL;
}

//---------------------------------------------------------------------------
Translate::Translate(const GreyImage& _original_image,
                     GreyImage& _translated_image)
{
   original_image = &_original_image;
   translated_image   = &_translated_image;
   if((translated_image->Rows()!=original_image->Rows())||
      (translated_image->Cols()!=original_image->Cols()))
       translated_image->Resize(original_image->Cols(), original_image->Rows());
}


//---------------------------------------------------------------------------
void
Translate::PerformTranslation(int s_col, int s_row, int s_width, int s_height,
                              int d_col, int d_row, bool erase_flag)
{
   Assert((s_row < original_image->Rows())&&(s_row >=0 )&&
          (s_col < original_image->Cols())&&(s_col >=0 )&&
          (s_row + s_height <= original_image->Rows())&&
          (s_col + s_width <=  original_image->Cols()),VEMsg(AnsiString
          ("Translate::PerformTranslation: partition of the source image ")+
           AnsiString(" goes out of border")));

   Assert((d_row < original_image->Rows())&&(d_row >=0 )&&
          (d_col < original_image->Cols())&&(d_col >=0 )&&
          (d_row + s_height <= original_image->Rows())&&
          (d_col + s_width <=  original_image->Cols()),VEMsg(AnsiString
          ("Translate::PerformTranslation: partition of the source image ")+
           AnsiString(" goes out of border")));

   /*
    translated_image->ClearImageArea(WHITE, translated_image->Cols(),
                                translated_image->Rows());
   */
   *translated_image = *original_image;
   register int s_image_col, s_image_row;
   register int dest_col, dest_row;
   int s_col_extent, s_row_extent;

   s_col_extent = s_col + s_width;
   s_row_extent = s_row + s_height;

   dest_row = d_row;
   if(erase_flag)
      translated_image->ClearImageArea(BLACK, s_width, s_height, s_col,
                                          s_row);
   for(s_image_row = s_row; s_image_row < s_row_extent; s_image_row++){
      dest_col = d_col;
      for(s_image_col = s_col; s_image_col < s_col_extent; s_image_col++){
         translated_image->Data(dest_col++, dest_row) =
              original_image->Data(s_image_col, s_image_row);
      }
      dest_row++;
   }
}

//---------------------------------------------------------------------------
void
Translate::MirrorImage(int s_col, int s_row, int s_width, int s_height,
                       int d_col, int d_row, MirrorType which_mirror)
{
   Assert((s_row < original_image->Rows())&&(s_row >=0 )&&
          (s_col < original_image->Cols())&&(s_col >=0 )&&
          (s_row + s_height <= original_image->Rows())&&
          (s_col + s_width <=  original_image->Cols()),VEMsg(AnsiString
          ("Translate::MirrorImage: partition of the source image ")+
           AnsiString(" goes out of border")));

   *translated_image = *original_image;        

   register int s_image_col, s_image_row;
   register int dest_col, dest_row;
   int s_col_extent, s_row_extent;

   s_col_extent = s_col + s_width;
   s_row_extent = s_row + s_height;

   switch(which_mirror)
   {
      case HorizMirror:
         dest_row = d_row;
         for(s_image_row = s_row; s_image_row < s_row_extent; s_image_row++){
            dest_col = d_col + s_width;
            for(s_image_col = s_col; s_image_col < s_col_extent; s_image_col++){
                translated_image->Data(--dest_col, dest_row) =
                   original_image->Data(s_image_col, s_image_row);
            }
            dest_row++;
         }
         break;
      case VertMirror:
         dest_row = d_row + (s_height -1);
         for(s_image_row = s_row; s_image_row < s_row_extent; s_image_row++){
            dest_col = d_col;
            for(s_image_col = s_col; s_image_col < s_col_extent; s_image_col++){
                translated_image->Data(dest_col++, dest_row) =
                   original_image->Data(s_image_col, s_image_row);
            }
            dest_row--;
         }
         break;
   }
}   






