/*
   MORPHUnit.cpp

   definin soubor hlavnho okna morfologie

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "MORPHUnit.h"
#include "MORPHMaskUnit.h"
#include "PictureUnit.h"
#include "MORPHThreshUnit.h"
#include "..\Headers\list_image.h"
#include "MORPHUsersMaskUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

TMORPHForm *MORPHForm;
//---------------------------------------------------------------------------
__fastcall TMORPHForm::TMORPHForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+50;
   Left = PictureForm->Left +50;
   
   list = new ListI<GreyImage>();

//----------------------------------------------------------------------------
   mask = new VMask(mask_options);
   AssignStatusBar(mask_options);
   handler = new BMPHandler(*PictureForm->GetBMPHandler());
   original_image = new GreyImage();
   before_image = new GreyImage();
   after_image = new GreyImage();
   handler->Set(*original_image);
   switch(PictureForm->GetMorphKind()){
      case(0):       // binary
         if(!OnlyBinaryValues(original_image)){
            Application->CreateForm(__classid(TMORPHThreshForm),
                          &MORPHThreshForm);
            MORPHThreshForm->ShowModal();
         }
         PictureForm->GetBMPHandler()->Set(*before_image);
         list->Append(*before_image);
         morpholog = new BinMorpholog(before_image, after_image, mask);
         break;
      case(1):       // gerylevel
         handler->Set(*before_image);
         list->Append(*before_image);         
         morpholog = new GreyMorpholog(before_image, after_image, mask);
         break;
   }

}
//---------------------------------------------------------------------------
__fastcall TMORPHForm::~TMORPHForm()
{
   delete (handler);
   delete (mask);
   delete (before_image);
   delete (after_image);
   delete (original_image);
   delete (list);
}
//---------------------------------------------------------------------------
// this function tests if the image consists of any pixel different from
// 0 or 255.
bool
__fastcall TMORPHForm::OnlyBinaryValues(GreyImage* image)
{
   for(register int c(0); c<image->Cols(); c++){
      for(register int r(0); r<image->Rows(); r++){
         if((image->Data(c,r)!=255)&&
            (image->Data(c,r)!=0))
            return false;
      }
   }
   return true;
}
//---------------------------------------------------------------------------
void __fastcall TMORPHForm::CancelBitBtnClick(TObject *Sender)
{
   PictureForm->GetBMPHandler()->Assign(*original_image);
   PictureForm->GetBMPHandler()->Show(PictureForm->Image);
}
//---------------------------------------------------------------------------
void __fastcall TMORPHForm::PerformBitBtnClick(TObject *Sender)
{
   switch(FiltersRadioGroup->ItemIndex){
      case(0):                 //Close
          morpholog->Close();
          break;
      case(1):                 //Open
          morpholog->Open();
          break;
      case(2):                 //Dilation
          morpholog->Dilation();
          break;
      case(3):                 //Erosion
          morpholog->Erosion();
          break;
      case(4):                 //Outline
          morpholog->Outline();
          break;
      case(5):                 //Filter
          morpholog->Open();
          *before_image = *after_image;
          morpholog->Close();
          break;
      case(6):                 //Conditional Dilation
          morpholog->ConditionalDilatation();
          break;
   }
   *before_image = *after_image;
   list->Append(*after_image);
   PictureForm->GetBMPHandler()->Assign(*before_image);
   PictureForm->GetBMPHandler()->Show(PictureForm->Image);
}
//---------------------------------------------------------------------------

void __fastcall TMORPHForm::MIPSMaskClick(TObject *Sender)
{
   Application->CreateForm(__classid(TMORPHMaskForm),
                          &MORPHMaskForm);
   MORPHMaskForm->ShowModal();
   delete (mask);
   mask = new VMask(mask_options);
   morpholog->NewMask(mask);
   AssignStatusBar(mask_options);
}
//---------------------------------------------------------------------------
void
__fastcall TMORPHForm::AssignStatusBar(MaskOptions init)
{
   switch(init.Size()){
      case(0): MORPHForm->StatusBar->Panels->Items[0]->Text="3X3"; break;
      case(1): MORPHForm->StatusBar->Panels->Items[0]->Text="5X5"; break;
      case(2): AnsiString text =  AnsiString(init.Cols())+
                   AnsiString("x")+AnsiString(init.Rows());
               MORPHForm->StatusBar->Panels->Items[0]->Text= text.c_str();
               break;
   }
   switch(init.Shape()){
      case(0): MORPHForm->StatusBar->Panels->Items[1]->Text="Full"; break;
      case(1): MORPHForm->StatusBar->Panels->Items[1]->Text="Round"; break;
      case(2): MORPHForm->StatusBar->Panels->Items[1]->Text="Cross"; break;
      case(3): MORPHForm->StatusBar->Panels->Items[1]->Text="Vertical"; break;
      case(4): MORPHForm->StatusBar->Panels->Items[1]->Text="Horizontal"; break;
      case(5): MORPHForm->StatusBar->Panels->Items[1]->Text="User's"; break;
   }
}
void __fastcall TMORPHForm::MUndoClick(TObject *Sender)
{
   list->Delete();
   *before_image = (list->Member());
   PictureForm->GetBMPHandler()->Assign(*before_image);
   PictureForm->GetBMPHandler()->Show(PictureForm->Image);
}
//---------------------------------------------------------------------------

void __fastcall TMORPHForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------

void __fastcall TMORPHForm::MUsersMaskClick(TObject *Sender)
{
   Application->CreateForm(__classid(TMORPHUsersMaskForm),
                          &MORPHUsersMaskForm);
   MORPHUsersMaskForm->ShowModal();

   morpholog->NewMask(mask);
   AssignStatusBar(mask_options);
}
//---------------------------------------------------------------------------


void __fastcall TMORPHForm::OKBitBtnClick(TObject *Sender)
{
   PictureForm->AssignList(after_image);
}
//---------------------------------------------------------------------------




