/*
   MORPHUserMaskUnit.cpp

   definin soubor okna vytvoen nov masky.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "MORPHUsersMaskUnit.h"
#include "MORPHNewSizeMaskUnit.h"
#include "MORPHUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMORPHUsersMaskForm *MORPHUsersMaskForm;
//---------------------------------------------------------------------------
__fastcall TMORPHUsersMaskForm::TMORPHUsersMaskForm(TComponent* Owner)
        : TForm(Owner), mask(NULL)
{
   Top = MORPHForm->Top +20;
   Left = MORPHForm->Left +20;
   form_width = Width;
   form_height = Height;
   SetSize(5,5);
}
//---------------------------------------------------------------------------
__fastcall TMORPHUsersMaskForm::~TMORPHUsersMaskForm()
{
   delete (mask);
}
//---------------------------------------------------------------------------
void TMORPHUsersMaskForm::SetSize(int _cols, int _rows)
{
   if(!(_cols>=1)&&(_cols<=15)&&(_rows>=1)&&(_rows<=15))
      throw Exception("Number of rows and columns must be between 0 and 15");
   cols_num = _cols;
   rows_num = _rows;

   if(cols_num<5)
      Width = form_width;
   else
      Width = form_width + (cols_num-5)*diff_left;
   if(rows_num<5)
      Height = form_height;
   else
      Height = form_height + (rows_num-5)*diff_top;

   // when move from ViewKernel to modify the original values can be modified

   box = new TCheckBox*[cols_num*rows_num];
   for(int r(0); r<rows_num; r++){
      for(int c(0); c<cols_num; c++){
         box[c+r*cols_num] = new TCheckBox(Panel);
         box[c+r*cols_num]->Visible = false;
         box[c+r*cols_num]->Parent = Panel;
         box[c+r*cols_num]->Width = 17;
         box[c+r*cols_num]->Top = box_top + r*diff_top;
         box[c+r*cols_num]->Left = box_left + c*diff_left;

         box[c+r*cols_num]->Show();
         box[c+r*cols_num]->Visible = true;
      }
   }
}
//---------------------------------------------------------------------------
void
TMORPHUsersMaskForm::DeleteBox()
{
   for(int i(0); i<cols_num*rows_num; i++)
      delete box[i];
   delete [] box;
}
//---------------------------------------------------------------------------
void __fastcall TMORPHUsersMaskForm::M3x3Click(TObject *Sender)
{
   DeleteBox();
   SetSize(3,3);
}
//---------------------------------------------------------------------------


void __fastcall TMORPHUsersMaskForm::M5x5Click(TObject *Sender)
{
   DeleteBox();
   SetSize(5,5);
}
//---------------------------------------------------------------------------


void __fastcall TMORPHUsersMaskForm::FormResize(TObject *Sender)
{
   OKButton->Left = ClientWidth - buttons_differ;
   CancelBitBtn->Left = ClientWidth - buttons_differ;
   Set1BitBtn->Left = ClientWidth - buttons_differ;
   ResetBitBtn->Left = ClientWidth - buttons_differ;
   
}
//---------------------------------------------------------------------------
void __fastcall TMORPHUsersMaskForm::SetNewSizeMask(int _c, int _r)
{
   DeleteBox();
   cols_num = _c;
   rows_num = _r;
   SetSize(cols_num ,rows_num);
}
void __fastcall TMORPHUsersMaskForm::MAnotherClick(TObject *Sender)
{
   Application->CreateForm(__classid(TMORPHNewSizeMaskForm),
                          &MORPHNewSizeMaskForm);
   MORPHNewSizeMaskForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TMORPHUsersMaskForm::Set1BitBtnClick(TObject *Sender)
{
   for(int i(0); i<cols_num*rows_num; i++)
      box[i]->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TMORPHUsersMaskForm::ResetBitBtnClick(TObject *Sender)
{
   for(int i(0); i<cols_num*rows_num; i++)
      box[i]->Checked = false;
}
//---------------------------------------------------------------------------

void __fastcall TMORPHUsersMaskForm::OKButtonClick(TObject *Sender)
{
   mask = new VMask(cols_num, rows_num);
   for(int c(0); c<cols_num; c++)
      for(int r(0); r<rows_num; r++){
         if(box[c+r*cols_num]->Checked == true)
            mask->Data(c,r) = 1;
         else
            mask->Data(c,r) = 0;
      }
   MORPHForm->GetMask() = *mask;
   MORPHForm->GetMaskOptions().Cols() = cols_num;
   MORPHForm->GetMaskOptions().Rows() = rows_num;
   MORPHForm->GetMaskOptions().Shape() = shape_user;
   MORPHForm->GetMaskOptions().Size() = size_user;
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TMORPHUsersMaskForm::N7x71Click(TObject *Sender)
{
   DeleteBox();
   SetSize(7,7);        
}
//---------------------------------------------------------------------------

