/*
   morpholog_binary.cpp

   definin soubor tdy BinMorpholog pro morfologick operace nad
   binrnmi obrazy.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "morpholog_binary.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

//---------------------------------------------------------------------------
void
BinMorpholog::Dilation()
{
   MORPHForm->ProgressBar->Max = orig_image->Cols()-mask->Cols();
   MORPHForm->ProgressBar->Step = 1;
   unsigned char max;
   for(int c(mask->Cols()/2); c<orig_image->Cols()-mask->Cols()/2; c++){
      for(int r(mask->Rows()/2); r<orig_image->Rows()-mask->Rows()/2; r++){
         max = 0;
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++){
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               if(mask->Data(i+mask->Cols()/2,j+mask->Rows()/2)==1){
                   if(orig_image->Data(c+i,r+j)>max)
                      max =  orig_image->Data(c+i,r+j);
               }
            }
         }
         max = (unsigned char)((max>128)? 255 : 0);
         morph_image->Data(c,r) = max;
      }
      MORPHForm->ProgressBar->StepIt();
   }
   MORPHForm->ProgressBar->Position=0;
}
//---------------------------------------------------------------------------
void
BinMorpholog::Erosion()
{
   MORPHForm->ProgressBar->Max = orig_image->Cols()-mask->Cols();
   MORPHForm->ProgressBar->Step = 1;
   unsigned char min;
   for(int c(mask->Cols()/2); c<orig_image->Cols()-mask->Cols()/2; c++){
      for(int r(mask->Rows()/2); r<orig_image->Rows()-mask->Rows()/2; r++){
         min = 255;
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++){
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               if(mask->Data(i+mask->Cols()/2,j+mask->Rows()/2)==1){
                   if(orig_image->Data(c+i,r+j)<min)
                      min =  orig_image->Data(c+i,r+j);
               }
            }
         }
         min = (unsigned char)((min>128)? 255 : 0);
         morph_image->Data(c,r) = min;
      }
      MORPHForm->ProgressBar->StepIt();
   }
   MORPHForm->ProgressBar->Position=0;
}
//---------------------------------------------------------------------------
void
BinMorpholog::Close()
{
   Dilation();
   GreyImage tmp(orig_image->Cols(),orig_image->Rows());
   tmp = *orig_image;
   *orig_image = *morph_image;
   Erosion();
   *orig_image = tmp;
}   
//---------------------------------------------------------------------------
void
BinMorpholog::Open()
{
   Erosion();
   GreyImage tmp(orig_image->Cols(),orig_image->Rows());
   tmp = *orig_image;
   *orig_image = *morph_image;
   Dilation();
   *orig_image = tmp;
}
//---------------------------------------------------------------------------
void
BinMorpholog::Outline()
{
   Erosion();
   for(register int c(0); c<orig_image->Cols(); c++)
      for(register int r(0); r<orig_image->Rows(); r++)
         morph_image->Data(c,r) = (unsigned char)(orig_image->Data(c,r) -
                        morph_image->Data(c,r));
}
//---------------------------------------------------------------------------
void
BinMorpholog::Skeleton()
{
   bool pixel_on = true;
   int pixel;
   GreyImage filter(orig_image->Cols(), orig_image->Rows());
   GreyImage filter1(orig_image->Cols(), orig_image->Rows());
   GreyImage temp_orig(*orig_image);
   morph_image->ClearImageArea(0,morph_image->Cols(), morph_image->Rows());
   while(pixel_on == true){
      pixel_on = false;
      BinMorpholog mor1(&temp_orig,&filter,mask);
      mor1.Erosion();
      BinMorpholog mor2(&filter,&filter1,mask);
      mor2.Dilation();
      for(int c(mask->Cols()/2); c<orig_image->Cols()-mask->Cols()/2; c++){
         for(int r(mask->Rows()/2); r<orig_image->Rows()-mask->Rows()/2; r++){
            pixel =  temp_orig.Data(c,r) - filter1.Data(c,r);
            morph_image->Data(c,r) = (unsigned char)(morph_image->Data(c,r) | pixel);
            if(pixel==255)
              pixel_on = true;
            temp_orig.Data(c,r) = filter.Data(c,r);
         }
      }
   }
}
//---------------------------------------------------------------------------
 // Conditional Dilatation
void BinMorpholog::ConditionalDilatation()
{
   int min_size, min_r, max_r, min_c, max_c, step, last_step , background_value, threshold_value;
   long max_histogram;
   int histogram[256];
   bool no_object;

   min_c = 1;
   min_r = 1;
   max_c = orig_image->Cols();
   max_r = orig_image->Rows();

   step = 0;
   last_step = -1;
   threshold_value = 128;

   // find smaller size
   min_size = (max_c < max_r) ? max_c : max_r;

   // find size of array which will be dilationed
   while((step != last_step)&&(step < ceil(min_size/2))){
      last_step = step;
      no_object = true;
      // find object in up-line
      for (int i = min_c - 1; i < max_c; i++){
         if (orig_image->Data(i, step) > threshold_value) {no_object = false;}
      }
      // find object in down-line
      for (int i = min_c - 1; i < max_c; i++){
         if (orig_image->Data(i, orig_image->Rows()-1-step) > threshold_value) {no_object = false;}
      }
      // find object in left-line
      for (int i = min_r; i < max_r-1; i++){
         if (orig_image->Data(step, i) > threshold_value) {no_object = false;}
      }
      // find object in right-line
      for (int i = min_r; i < max_r-1; i++){
         if (orig_image->Data(orig_image->Cols()-1-step, i) > threshold_value) {no_object = false;}
      }
      min_c++;
      min_r++;
      max_c--;
      max_r--;
      if (no_object != true) {step++;}
   }

    // find background -  maximum level in histogram
   // prepare empty histogram
   for (int i = 0; i < 256; i++){
      histogram[i] = 0;
   }

   // compute histogram
   for(int c = 0; c < orig_image->Cols(); c++){
      for(int r = 0; r < orig_image->Rows(); r++){
         histogram[orig_image->Data(c, r)]++;
      }
   }

   // find maximum in histogram (from '0' to 'threshold_value')
   max_histogram = (long)(0);
   background_value = 0;
   for (int i = 0; i < threshold_value; i++){
      if (histogram[i] > max_histogram){
         max_histogram = (long)histogram[i];
         background_value = i;
      }
   }

   if (step < ceil(min_size/2) - 3){
      // clear out image
      for (int c = 0; c < orig_image->Cols(); c++){
         for (int r = 0; r < orig_image->Rows(); r++){
             morph_image->Data(c,r) = (unsigned char)background_value;
         }
      }

      // copy unchanged part from original image into out_image
      for (int c = step; c < orig_image->Cols() - step; c++){
         for (int r = step; r < orig_image->Rows() - step; r++){
             morph_image->Data(c,r) = (unsigned char)orig_image->Data(c,r);
         }
      }
   }else{ // it is imposible perform this function (bad define object in image?)
      for (int c = 0; c < orig_image->Cols(); c++){
         for (int r = 0; r < orig_image->Rows(); r++){
             morph_image->Data(c,r) = (unsigned char)background_value;
         }
      }
   }
}




