/*
   morpholog_grey.cpp

   definin soubor tdy GreyMorpholog pro morfologick operace nad
   edotnovmi obrazy.

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "morpholog_grey.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

//---------------------------------------------------------------------------
// DILATATION
void
GreyMorpholog::Dilation()
{
   MORPHForm->ProgressBar->Max = orig_image->Cols()-mask->Cols();
   MORPHForm->ProgressBar->Step = 1;
   VMask array(mask->Cols(), mask->Rows());
   int max;
   for(int c(mask->Cols()/2); c<orig_image->Cols()-mask->Cols()/2; c++){
      for(int r(mask->Rows()/2); r<orig_image->Rows()-mask->Rows()/2; r++){
         max = 0;
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++){
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               array.Data(i+array.Cols()/2,j+array.Rows()/2) =
                      orig_image->Data(c+i,r+j) +
                      mask->Data(i+mask->Cols()/2,j+mask->Rows()/2);
            }
         }
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++){
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               if(array.Data(i+array.Cols()/2,j+array.Rows()/2)>max){
                  max = array.Data(i+array.Cols()/2,j+array.Rows()/2);
               }
            }
         }
         if(max>255)
         max=255;
         morph_image->Data(c,r) = (unsigned char)max;
      }
      MORPHForm->ProgressBar->StepIt();
   }
   MORPHForm->ProgressBar->Position=0;
}
//---------------------------------------------------------------------------
// EROSION
void
GreyMorpholog::Erosion()
{
   MORPHForm->ProgressBar->Max = orig_image->Cols()-mask->Cols();
   MORPHForm->ProgressBar->Step = 1;
   VMask array(mask->Cols(), mask->Rows());
   int min;
   for(int c(mask->Cols()/2); c<orig_image->Cols()-mask->Cols()/2; c++){
      for(int r(mask->Rows()/2); r<orig_image->Rows()-mask->Rows()/2; r++){
         min = 255;
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++)
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               array.Data(i+array.Cols()/2,j+array.Rows()/2) =
                      orig_image->Data(c+i,r+j) -
                      mask->Data(i+mask->Cols()/2,j+mask->Rows()/2);
            }
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++)
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               if(array.Data(i+array.Cols()/2,j+array.Rows()/2)<min)
                  min = array.Data(i+array.Cols()/2,j+array.Rows()/2);
            }
         if(min<0)
            min=0;
         morph_image->Data(c,r) = (unsigned char)min;
      }
      MORPHForm->ProgressBar->StepIt();
   }
   MORPHForm->ProgressBar->Position=0;
}
//---------------------------------------------------------------------------
// CLOSE
void
GreyMorpholog::Close()
{
   Dilation();
   GreyImage tmp(orig_image->Cols(),orig_image->Rows());
   tmp = *orig_image;
   *orig_image = *morph_image;
   Erosion();
   *orig_image = tmp;
}
//---------------------------------------------------------------------------
// OPEN
void
GreyMorpholog::Open()
{
   Erosion();
   GreyImage tmp(orig_image->Cols(),orig_image->Rows());
   tmp = *orig_image;
   *orig_image = *morph_image;
   Dilation();
   *orig_image = tmp;
}
//---------------------------------------------------------------------------
// TOP HAT
void
GreyMorpholog::TopHat()
{
   GreyImage temp_image(orig_image->Cols(), orig_image->Rows());
   temp_image = *orig_image;
   int temp;
   Open();
   for(register int c(0); c<temp_image.Cols(); c++){
      for(register int r(0); r<temp_image.Rows(); r++){
          temp = temp_image.Data(c,r) - morph_image->Data(c,r);
          if(temp<0)
             temp = 0;
          morph_image->Data(c,r) = (unsigned char)temp;
      }
   }
}
//---------------------------------------------------------------------------
// FIND OUTLINE
void
GreyMorpholog::Outline()
{
   // Erosion
   MORPHForm->ProgressBar->Max = orig_image->Cols()-mask->Cols();
   MORPHForm->ProgressBar->Step = 1;
   VMask array(mask->Cols(), mask->Rows());
   int min, max;
   for(int c(mask->Cols()/2); c<orig_image->Cols()-mask->Cols()/2; c++){
      for(int r(mask->Rows()/2); r<orig_image->Rows()-mask->Rows()/2; r++){
         min = 255;
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++)
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               array.Data(i+array.Cols()/2,j+array.Rows()/2) =
                      orig_image->Data(c+i,r+j) -
                      mask->Data(i+mask->Cols()/2,j+mask->Rows()/2);
            }
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++)
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               if(array.Data(i+array.Cols()/2,j+array.Rows()/2)<min)
                  min = array.Data(i+array.Cols()/2,j+array.Rows()/2);
            }
         if(min<0)
            min=0;
         morph_image->Data(c,r) = (unsigned char)min;
      }
      MORPHForm->ProgressBar->StepIt();
   }

   //Dilation
   for(int c(mask->Cols()/2); c<orig_image->Cols()-mask->Cols()/2; c++){
      for(int r(mask->Rows()/2); r<orig_image->Rows()-mask->Rows()/2; r++){
         max = 0;
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++){
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               array.Data(i+array.Cols()/2,j+array.Rows()/2) =
                      orig_image->Data(c+i,r+j) +
                      mask->Data(i+mask->Cols()/2,j+mask->Rows()/2);
            }
         }
         for(int i(-mask->Cols()/2); i<=mask->Cols()/2; i++){
            for(int j(-mask->Rows()/2); j<=mask->Rows()/2; j++){
               if(array.Data(i+array.Cols()/2,j+array.Rows()/2)>max){
                  max = array.Data(i+array.Cols()/2,j+array.Rows()/2);
               }
            }
         }
         if(max>255)
         max=255;
         morph_image->Data(c,r) = morph_image->Data(c,r) - (unsigned char)max;
      }
      MORPHForm->ProgressBar->StepIt();
   }
   MORPHForm->ProgressBar->Position=0;
}
//---------------------------------------------------------------------------
 // Conditional Dilatation
void GreyMorpholog::ConditionalDilatation()
{
   int min_size, min_r, max_r, min_c, max_c, step, last_step , background_value, threshold_value;
   long max_histogram;
   int histogram[256];
   bool no_object;

   min_c = 1;
   min_r = 1;
   max_c = orig_image->Cols();
   max_r = orig_image->Rows();

   step = 0;
   last_step = -1;
   threshold_value = 128;

   // find smaller size
   min_size = (max_c < max_r) ? max_c : max_r;

   // find size of array which will be dilationed
   while((step != last_step)&&(step < ceil(min_size/2))){
      last_step = step;
      no_object = true;
      // find object in up-line
      for (int i = min_c - 1; i < max_c; i++){
         if (orig_image->Data(i, step) > threshold_value) {no_object = false;}
      }
      // find object in down-line
      for (int i = min_c - 1; i < max_c; i++){
         if (orig_image->Data(i, orig_image->Rows()-1-step) > threshold_value) {no_object = false;}
      }
      // find object in left-line
      for (int i = min_r; i < max_r-1; i++){
         if (orig_image->Data(step, i) > threshold_value) {no_object = false;}
      }
      // find object in right-line
      for (int i = min_r; i < max_r-1; i++){
         if (orig_image->Data(orig_image->Cols()-1-step, i) > threshold_value) {no_object = false;}
      }
      min_c++;
      min_r++;
      max_c--;
      max_r--;
      if (no_object != true) {step++;}
   }

    // find background -  maximum level in histogram
   // prepare empty histogram
   for (int i = 0; i < 256; i++){
      histogram[i] = 0;
   }

   // compute histogram
   for(int c = 0; c < orig_image->Cols(); c++){
      for(int r = 0; r < orig_image->Rows(); r++){
         histogram[orig_image->Data(c, r)]++;
      }
   }

   // find maximum in histogram (from '0' to 'threshold_value')
   max_histogram = (long)(0);
   background_value = 0;
   for (int i = 0; i < threshold_value; i++){
      if (histogram[i] > max_histogram){
         max_histogram = (long)histogram[i];
         background_value = i;
      }
   }

   if (step < ceil(min_size/2) - 3){
      // clear out image
      for (int c = 0; c < orig_image->Cols(); c++){
         for (int r = 0; r < orig_image->Rows(); r++){
             morph_image->Data(c,r) = (unsigned char)background_value;
         }
      }

      // copy unchanged part from original image into out_image
      for (int c = step; c < orig_image->Cols() - step; c++){
         for (int r = step; r < orig_image->Rows() - step; r++){
             morph_image->Data(c,r) = (unsigned char)orig_image->Data(c,r);
         }
      }
   }else{ // it is imposible perform this function (bad define object in image?)
      for (int c = 0; c < orig_image->Cols(); c++){
         for (int r = 0; r < orig_image->Rows(); r++){
             morph_image->Data(c,r) = (unsigned char)background_value;
         }
      }
   }
}





