/*
   noise.cpp

   definin soubor tdy pro generaci umu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "noise.h"

#include "NoiseUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
// constructors
//---------------------------------------------------------------------------
// default constr
Noise::Noise()
{
   original_image = NULL;
   noise_image    = NULL;
}

//---------------------------------------------------------------------------
// 2 argument constructor
Noise::Noise(const GreyImage& _original_image,
                  GreyImage& _noise_image)
{
   original_image = &_original_image;
   noise_image = &_noise_image;
   if((noise_image->Rows()!=original_image->Rows())||
      (noise_image->Cols()!=original_image->Cols()))
       noise_image->Resize(original_image->Cols(), original_image->Rows());
}

//---------------------------------------------------------------------------
// Salt and Pepper noise that is named acording to the white and black appe-
// rance it adds to images
void
Noise::SaltAndPepper(int col, int row, int width, int height,
                      double probability)
{
   Assert((row < original_image->Rows())&&(row >=0 )&&
          (col < original_image->Cols())&&(col >=0 )&&
          (row + height <= original_image->Rows())&&
          (col + width <= original_image->Cols()),VEMsg(AnsiString
          ("Noise::SaltAndPepper: partition of an image ")+
           AnsiString(" goes out of border")));
   Assert((probability <= 1)&&(probability >= 0),
           VEMsg(AnsiString
          ("Noise::SaltAndPepper: probability must be between 0 an 1")));

   noise_image->ClearImageArea(WHITE, noise_image->Cols(),
                                 noise_image->Rows());


   register int image_row, image_col, data;
   int data1, data2;
   register int cols_extent;
   register int rows_extent;
   cols_extent = col + width;
   rows_extent = row + height;
   data  = (int)(probability*32768/2);
   data1 = data + 16384;
   data2 = 16384 - data;

   NoiseForm->ProgressBar->Step = 1;
   NoiseForm->ProgressBar->Max = rows_extent;



   for (image_row = row; image_row < rows_extent; image_row++)
   {
      for (image_col = col; image_col < cols_extent; image_col++)
      {
         data = rand();
         if((data >= 16384) && (data < data1))
            noise_image->Data(image_col, image_row) = 0;
         else{
            if((data>=data2)&&(data<16384))
               noise_image->Data(image_col, image_row) = 255;
            else{
               noise_image->Data(image_col, image_row) =
                    original_image->Data(image_col, image_row);
            }
         }
      }
      NoiseForm->ProgressBar->StepIt();
   }
   NoiseForm->ProgressBar->Position = 0;
}

//---------------------------------------------------------------------------
void
Noise::GaussianNoise(int col, int row, int width, int height,
                      double var, double mean)
{
   Assert((row < original_image->Rows())&&(row >=0 )&&
          (col < original_image->Cols())&&(col >=0 )&&
          (row + height <= original_image->Rows())&&
          (col + width <= original_image->Cols()),VEMsg(AnsiString
          ("Noise::GaussianNoise: partition of an image ")+
           AnsiString(" goes out of border")));

   register double noise, theta;
   register int image_row, image_col;
   register int cols_extent;
   register int rows_extent;
   register int data;
   cols_extent = col + width;
   rows_extent = row + height;

   NoiseForm->ProgressBar->Step = 1;
   NoiseForm->ProgressBar->Max = rows_extent;


   for (image_row = row; image_row < rows_extent; image_row++)
   {
      for (image_col = col; image_col < cols_extent; image_col++)
      {
         noise = sqrt(-2*var*log(1.0-(float)rand() / 32767.1));
         theta = (float)rand() * 1.9175345e-4 - 3.14159265;
         noise = noise * cos(theta);
         noise = noise + mean;

         data = noise + original_image->Data(image_col, image_row);

         if(data > 255)
            data = 255;
         if(data < 0)
            data = 0;

         noise_image->Data(image_col, image_row) =
                (unsigned char)(data);
      }
      NoiseForm->ProgressBar->StepIt();
   }
   NoiseForm->ProgressBar->Position = 0;
}
//---------------------------------------------------------------------------
void Noise::GammaNoise(int col, int row, int width, int height,
                      double var, int alpha)
{
   Assert((row < original_image->Rows())&&(row >=0 )&&
          (col < original_image->Cols())&&(col >=0 )&&
          (row + height <= original_image->Rows())&&
          (col + width <= original_image->Cols()),VEMsg(AnsiString
          ("Noise::GaussianNoise: partition of an image ")+
           AnsiString(" goes out of border")));

   register double noise, theta, rx, ry, A, noise1;
   register int image_row, image_col;
   register int cols_extent;
   register int rows_extent;
   register double data;
   cols_extent = col + width;
   rows_extent = row + height;

   NoiseForm->ProgressBar->Step = 1;
   NoiseForm->ProgressBar->Max = rows_extent;

   A=sqrt((double)var/(double)alpha)/2;

   for (image_row = row; image_row < rows_extent; image_row++)
   {
      for (image_col = col; image_col < cols_extent; image_col++)
      {
         noise1 = 0;
         for(int i(0); i<=alpha; i++)
         {
            noise = sqrt(-2*A*log(1.0-(float)rand() / 32767.1));
            theta = (float)rand() * 1.9175345e-4 - 3.14159265;
            rx = noise*cos(theta);
            ry = noise*sin(theta);
            noise = rx*rx + ry*ry;
            noise1 = noise1 + noise;
         }
         data = noise1 + original_image->Data(image_col, image_row);
         if(data > 255)
            data = 255;
         if(data < 0)
            data = 0;
         noise_image->Data(image_col, image_row) =
                (unsigned char)(data);
      }
      NoiseForm->ProgressBar->StepIt();
   }
   NoiseForm->ProgressBar->Position = 0;
}
//---------------------------------------------------------------------------
void Noise::NegativeExponentialNoise(int col, int row, int width, int height,
                                                          double var)
{
   Assert((row < original_image->Rows())&&(row >=0 )&&
          (col < original_image->Cols())&&(col >=0 )&&
          (row + height <= original_image->Rows())&&
          (col + width <= original_image->Cols()),VEMsg(AnsiString
          ("Noise::GaussianNoise: partition of an image ")+
           AnsiString(" goes out of border")));

   register double noise, theta, A, rx, ry;
   register int image_row, image_col;
   register int cols_extent;
   register int rows_extent;
   register int data;
   cols_extent = col + width;
   rows_extent = row + height;

   NoiseForm->ProgressBar->Step = 1;
   NoiseForm->ProgressBar->Max = rows_extent;

   for (image_row = row; image_row < rows_extent; image_row++)
   {
      for (image_col = col; image_col < cols_extent; image_col++)
      {
         A = sqrt((double)var)/2;
         noise = sqrt(-2*A*log(1.0-(float)rand() / 32767.1));
         theta = (float)rand() * 1.9175345e-4 - 3.14159265;
         rx = noise * cos(theta);
         ry  =noise * sin(theta);
         noise = rx*rx+ry*ry;
         data = noise+0.5 + original_image->Data(image_col, image_row);

         if(data > 255)
            data = 255;
         if(data < 0)
            data = 0;

         noise_image->Data(image_col, image_row) =
                (unsigned char)(data);
      }
      NoiseForm->ProgressBar->StepIt();
   }
   NoiseForm->ProgressBar->Position = 0;
}
//---------------------------------------------------------------------------
void Noise::RayleighNoise(int col, int row, int width, int height,
                      double var)
{
   register double noise, A;
   register int image_row, image_col;
   register int cols_extent;
   register int rows_extent;
   register int data;
   cols_extent = col + width;
   rows_extent = row + height;

   NoiseForm->ProgressBar->Step = 1;
   NoiseForm->ProgressBar->Max = rows_extent;

   for (image_row = row; image_row < rows_extent; image_row++)
   {
      for (image_col = col; image_col < cols_extent; image_col++)
      {
         A = 2.3299 * var;
         noise = sqrt(-2*A*log(1.0-(float)rand() / 32767.1));
         data = noise+0.5 + original_image->Data(image_col, image_row);
         if(data > 255)
            data = 255;
         if(data < 0)
            data = 0;
         noise_image->Data(image_col, image_row) =
                (unsigned char)(data);
      }
      NoiseForm->ProgressBar->StepIt();
   }
   NoiseForm->ProgressBar->Position = 0;
}
//---------------------------------------------------------------------------
void Noise::UniformNoise(int col, int row, int width, int height,
                                    double var, double mean)
{
   register double noise;
   register int image_row, image_col;
   register int cols_extent;
   register int rows_extent;
   register int data;
   cols_extent = col + width;
   rows_extent = row + height;

   NoiseForm->ProgressBar->Step = 1;
   NoiseForm->ProgressBar->Max = rows_extent;

   for (image_row = row; image_row < rows_extent; image_row++)
   {
      for (image_col = col; image_col < cols_extent; image_col++)
      {
         noise = sqrt((double)var)*1.057192e-4*(float)rand()+
                 mean-sqrt((double)var)*1.7320508;
         data = noise + original_image->Data(image_col, image_row);
         if(data > 255)
            data = 255;
         if(data < 0)
            data = 0;
         noise_image->Data(image_col, image_row) =
                (unsigned char)(data);
      }
      NoiseForm->ProgressBar->StepIt();
   }
   NoiseForm->ProgressBar->Position = 0;
}

















