/*
   NoiseUnit.cpp

  definin soubor hlavnho okna pro generaci umu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "NoiseUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNoiseForm *NoiseForm;
//---------------------------------------------------------------------------
__fastcall TNoiseForm::TNoiseForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   orig_image = new GreyImage();
   image = new GreyImage();
   handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   handler->Set(*orig_image);
   handler->Set(*image);
   noise = new Noise(*orig_image,*image);
   NoiseRadioGroupClick(this);
   StatusBar->SimpleText = "Select type of noise and click to Perform button.";
}
//---------------------------------------------------------------------------
__fastcall TNoiseForm::~TNoiseForm()
{
   delete (handler);
   delete (image);
   delete (orig_image);
   delete (noise);
}
//---------------------------------------------------------------------------
void __fastcall TNoiseForm::PerformBitBtnClick(TObject *Sender)
{
   try{
      Perform();
      PictureForm->AssignImage(image);
      StatusBar->SimpleText = "Click to OK button for use performed image.";
   }catch(EConvertError& e){
      Application->MessageBox(e.Message.c_str(),
         "Noise Tool Window",MB_ICONHAND|MB_OK);
   }catch(Exception& e){
      Application->MessageBox(e.Message.c_str(),
         "Noise Tool Window",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TNoiseForm::Perform()
{
      if(Edit1->Text == "")
         Edit1->Text = 0;
      double temp;
      int itemp;
      switch(NoiseRadioGroup->ItemIndex){
         case 0:          // Salt and Pepper
            temp = Edit1->Text.ToDouble();
            if(temp<0)
               throw Exception("non-negative value is required");
            if(temp<0){temp = 0; Edit1->Text = 0;}
            if(temp>1){temp = 1; Edit1->Text = 1;}
            noise_options.salt_and_pepper.probability = temp;
            break;
         case 1:          // Gaussian
            temp = Edit1->Text.ToDouble();
            if(temp<0)
               throw Exception("non-negative value is required");
            noise_options.gaussian.mean = temp;
            break;
         case 2:          // Gamma
            itemp = Edit1->Text.ToInt();
            if(temp<0)
               throw Exception("non-negative value is required");
            if(itemp<1) {itemp = 1;Edit1->Text = 1;}
            noise_options.gamma.alpha = itemp;
            break;
         case 3:          // Negative Exponential noise
            temp = Edit1->Text.ToDouble();
            if(temp<0)
               throw Exception("non-negative value is required");
            noise_options.negative_exp.var = temp;
            break;
         case 4:          // Rayleigh
            temp = Edit1->Text.ToDouble();
            if(temp<0)
               throw Exception("non-negative value is required");
            noise_options.rayleigh.var = temp;
            break;
         case 5:          // Uniform
            temp = Edit1->Text.ToDouble();
            if(temp<0)
               throw Exception("non-negative value is required");
            noise_options.uniform.mean = temp;
            break;
      }
      if(Edit2->Text == "")
         Edit2->Text = 0;

      switch(NoiseRadioGroup->ItemIndex){
         case 1:          // Gaussian
            temp = Edit2->Text.ToDouble();
            if(temp<0)
               throw Exception("non-negative value is required");
            noise_options.gaussian.var = temp;
            break;
         case 2:          // Gamma
            temp = Edit2->Text.ToDouble();
            if(temp<0)
               throw Exception("non-negative value is required");
            noise_options.gamma.var = temp;
            break;
         case 5:          // Uniform
            temp = Edit2->Text.ToDouble();
            if(temp<0)
               throw Exception("non-negative value is required");
            noise_options.uniform.var = temp;
            break;
      }

      // calling of the noise functions
      switch(NoiseRadioGroup->ItemIndex){
         case 0 :     // Salt and Pepper
             noise->SaltAndPepper(0, 0, orig_image->Cols(),orig_image->Rows(),
                                  noise_options.salt_and_pepper.probability);
             break;
         case 1 :     // Gaussian
             noise->GaussianNoise(0, 0, orig_image->Cols(),orig_image->Rows(),
                     noise_options.gaussian.var, noise_options.gaussian.mean);
             break;
         case 2 :     // Gamma
             noise->GammaNoise(0, 0, orig_image->Cols(),orig_image->Rows(),
                     noise_options.gamma.var, noise_options.gamma.alpha);
             break;
         case 3 :     // Negative Exponential
             noise->NegativeExponentialNoise(0, 0, orig_image->Cols(),
                     orig_image->Rows(), noise_options.negative_exp.var);
             break;
         case 4 :     // Rayleigh
             noise->RayleighNoise(0, 0, orig_image->Cols(),orig_image->Rows(),
                     noise_options.rayleigh.var);
             break;
         case 5 :     // Uniform
             noise->UniformNoise(0, 0, orig_image->Cols(),orig_image->Rows(),
                     noise_options.uniform.var, noise_options.uniform.mean);
             break;

      }
      changed = true;

}
//---------------------------------------------------------------------------
void __fastcall TNoiseForm::NoiseRadioGroupClick(TObject *Sender)
{
   switch(NoiseRadioGroup->ItemIndex){
      case 0:       // Salt and Pepper
         Label1->Caption = "Probability";
         Edit1->Text = FormatFloat(
             "0.00",noise_options.salt_and_pepper.probability);
         Label2->Visible = false;
         Edit2->Visible = false;
         break;
      case 1:        // Gaussian
         Label1->Caption = "Mean";
         Edit1->Text = FormatFloat(
             "0.00",noise_options.gaussian.mean);
         Label2->Caption = "Variance";
         Edit2->Text = FormatFloat(
             "0.00",noise_options.gaussian.var);
         Label2->Visible = true;
         Edit2->Visible = true;
         break;
      case 2:          // Gamma noise
         Label1->Caption = "Alpha";
         Edit1->Text = FormatFloat(
             "0",noise_options.gamma.alpha);
         Label2->Caption = "Variance";
         Edit2->Text = FormatFloat(
             "0.00",noise_options.gamma.var);
         Label2->Visible = true;
         Edit2->Visible = true;
         break;
      case 3:       // Negative Exponential Noise
         Label1->Caption = "Variance";
         Edit1->Text = FormatFloat(
             "0.00",noise_options.negative_exp.var);
         Label2->Visible = false;
         Edit2->Visible = false;
         break;
      case 4:        // Rayleigh Noise
         Label1->Caption = "Variance";
         Edit1->Text = FormatFloat(
             "0.00",noise_options.rayleigh.var);
         Label2->Visible = false;
         Edit2->Visible = false;
         break;
      case 5:        // Uniform
         Label1->Caption = "Mean";
         Edit1->Text = FormatFloat(
             "0.00",noise_options.uniform.mean);
         Label2->Caption = "Variance";
         Edit2->Text = FormatFloat(
             "0.00",noise_options.uniform.var);
         Label2->Visible = true;
         Edit2->Visible = true;
         break;
   }
}
//---------------------------------------------------------------------------
void __fastcall TNoiseForm::ResetBitBtnClick(TObject *Sender)
{
   PictureForm->AssignImage(orig_image);
   changed = false;
}
//---------------------------------------------------------------------------

void __fastcall TNoiseForm::BitBtn2Click(TObject *Sender)
{
   PictureForm->AssignImage(orig_image);
}
//---------------------------------------------------------------------------

void __fastcall TNoiseForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------

void __fastcall TNoiseForm::BitBtn1Click(TObject *Sender)
{
   if(changed)
      PictureForm->AssignList(image);
}
//---------------------------------------------------------------------------

