/*
   PictureUnit.cpp

   definin soubor hlavnho okna programu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "PictureUnit.h"
#include "CutUnit.h"
#include "TranslateUnit.h"
#include "ColoBoxUnit.h"
#include "DrawNewSizeUnit.h"
#include "DrawUnit.h"
#include "RotateUnit.h"
#include "ResizeUnit.h"
#include "ConvolutionUnit.h"
#include "HistogramUnit.h"
#include "noise.h"
#include "NoiseUnit.h"
#include "LUTUnit.h"
#include "DFTUnit.h"
#include "DFTSizeUnit.h"
#include "BrightnessUnit.h"
#include "ContrastUnit.h"
#include "dft.h"
#include "NolnlinearFiltersUnit.h"
#include "MORPHUnit.h"
#include "FrameUnit.h"
#include "AboutUnit.h"
#include "LicenceUnit.h"
#include "AverageUnit.h"
#include "PseudoUnit.h"
#include "BrightnessProfileUnit.h"
#include "RGBComponentUnit.h"
#include "CentroidUnit.h"
#include "FlatFieldUnit.h"
#include "StatisticsUnit.h"
#include "HistogramEqualizationUnit.h"
#include "HistogramStretchingUnit.h"
#include "ShowTwoImagesUnit.h"
#include "ShowFourImagesUnit.h"
#include "ShowSixImagesUnit.h"
#include "GreyScaleUnit.h"
#include "HSVToolUnit.h"
#include "HSLToolUnit.h"
#include "CMYToolUnit.h"
#include "CMYKToolUnit.h"
#include "MagnifyingGlassUnit.h"
#include "CircleProfileUnit.h"
#include "NonUniformIlluminationUnit.h"
#include "LineProfileUnit.h"
#include "GammaCorrectionUnit.h"
#include "HistogramSpecificationUnit.h"
#include "HistogramModificationUnit.h"
#include "SnrPsnrUnit.h"
#include "FftFiltrationUnit.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPictureForm *PictureForm;
//---------------------------------------------------------------------------
__fastcall TPictureForm::TPictureForm(TComponent* Owner)
        : TForm(Owner),handler(NULL),color_handler(NULL),file(NULL),scale_const(1.0),
          nf_cols(3), nf_rows(3)
{
   image = new GreyImage();
   color_image = new ColorImage();

   list = new ListI<GreyImage>();
   color_list = new ListI<ColorImage>();

   Caption  = AppName;
   MEdit->Visible=false;

   MSave->Visible=false;
   MSaveAs->Visible=false;
   MImage->Visible=true;
   MNoise1->Visible=false;
   MContrast->Visible = false;
   MBrightness->Visible = false;
   MProfile->Visible = false;
   MHistogram1->Visible=false;
   MConvolution1->Visible=false;
   LUT1->Visible=false;
   DFT1->Visible=false;
   MLogicArith1->Visible=false;
   MObject1->Visible=false;
   MMorpholog1->Visible=false;
   MNonlinearFiltres1->Visible=false;
   MBitFields1->Visible=false;
   MSobelFilters1->Visible=false;
   MCharacteristics->Visible = false;
   MCorrection->Visible = false;
   MAveraging->Visible=true;

   MView->Visible=true;
   MActualView1->Visible = false;
   MZoomIn1->Visible = false;
   MZoomOut1->Visible = false;
   MShowMoreImage->Visible = true;
   MMagnifyingGlass->Visible = false;

   other_handler = new BMPHandler();

   MColor->Visible = false;
   show_profile = false;
   RGB_component = 0;
   scale_const = 1;
   min_height = PictureForm->ClientHeight;
   min_width = PictureForm->ClientWidth;
   undo_level = 0;
   undo_level_color = 0;
   measuring = false;
   get_other_handler = false;
}
//---------------------------------------------------------------------------
__fastcall TPictureForm::~TPictureForm()
{
   //delete (handler);
   //delete (other_handler)
   //delete (color_handler);
   //delete (file);
   //delete (list);
   //delete (image);
   //delete (color_image);
}
//---------------------------------------------------------------------------
// open file - use open dialog
void __fastcall TPictureForm::Open1Click(TObject *Sender)
{
 if(OpenDialog->Execute()){
      try{
         if (handler!=NULL) {delete (handler); handler = NULL;}
         if (color_handler!=NULL) {delete (color_handler); color_handler=NULL;}
         ////
               //delete (file);      ???
         ////
         delete (list);
         delete (color_list);
         // sets visibility of Menu items
         MSave->Visible = true;
         MSaveAs->Visible = true;
         list = new ListI<GreyImage>();
         color_list = new ListI<ColorImage>();
         switch(OpenDialog->FilterIndex){
            // Bitmap BMP file - 256 colors
            case(1):
               color_handler = NULL;
               undo_level = -1;
               InitialPicture(OpenDialog->FileName, bmp);
               break;
            // Bitmap BMP file - 16.7 million colors
            case(2):
               handler = NULL;
               undo_level_color = -1;
               InitialColorPicture(OpenDialog->FileName, bmp);
               break;
            }
      }catch(VEMsg &E){
         Application->MessageBox(E.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
      }catch(VECorruptedFile &E){
         Application->MessageBox(E.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
         file = NULL;
         switch (OpenDialog->FilterIndex){
            case(1):
               delete(handler);
               handler = NULL;
               break;
            case(2):
               delete(color_handler);
               color_handler = NULL;
               break;
         }
         MSave->Visible = false;
         MSaveAs->Visible = false;
      }
   }
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::AssignImage(GreyImage* image)
{
   handler->Assign(*image);
   handler->Set(*PictureForm->image);
   StatusBar1->Panels->Items[0]->Text = AnsiString(handler->Cols()/scale_const)+" x "+AnsiString(handler->Rows()/scale_const);
   Panel->Width = handler->Cols()+2;
   Panel->Height = handler->Rows()+2;
   Panel->Left = MainPanel->Width/2 - Panel->Width/2;
   Panel->Top = MainPanel->Height/2 - Panel->Height/2;
   handler->Show(Image);

   if (RGB_component!=0){
      switch (RGB_component){
         case(1):
            RGBComponentsForm->AssignChangedImage(*PictureForm->image, 1);
            break;
         case(2):
            RGBComponentsForm->AssignChangedImage(*PictureForm->image, 2);
            break;
         case(3):
            RGBComponentsForm->AssignChangedImage(*PictureForm->image, 3);
            break;
      }
   }

   SizeAdaptation();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::AssignColorImage(ColorImage* color_image)
{
   color_handler->Assign(*color_image);
   color_handler->Set(*PictureForm->color_image);
   StatusBar1->Panels->Items[0]->Text = AnsiString(color_handler->Cols())+" x "+AnsiString(color_handler->Rows());
   Panel->Width = color_handler->Cols()+2;
   Panel->Height = color_handler->Rows()+2;
   Panel->Left = MainPanel->Width/2 - Panel->Width/2;
   Panel->Top = MainPanel->Height/2 - Panel->Height/2;
   color_handler->Show(Image);

   SizeAdaptation();
}
//---------------------------------------------------------------------------
bool __fastcall TPictureForm::IfFileExistsOverWrite(AnsiString file_path)
{
     if(FileExists(file_path))
        if(Application->MessageBox(
            ("File "+file_name+" exsits\nDo you want to overwrite it?").c_str(),
                "SaveDialog Message", MB_YESNO|MB_ICONQUESTION)==mrYes)
           return  true;
        else
           return  false;
     return true;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::CreateGreyImage(GreyImage* image)
{
   if (color_handler!=NULL){
      delete color_handler;
      color_handler = NULL;
   }
   delete handler;
   delete list;
   handler = new BMPHandler();
   list = new ListI<GreyImage>();
   filter_index = 0;
   ext = extention[0];
   handler->CreateBitmapAndAssign(image);
   list->Append(*image);
   ////
   handler->Set(*PictureForm->image);
   handler->Set(*image);
   ////
   Panel->Height = handler->Height()+2;
   Panel->Width = handler->Width()+2;
   Panel->Left = MainPanel->Width/2 - Panel->Width/2;
   Panel->Top = MainPanel->Height/2 - Panel->Height/2;
   handler->Show(Image);

   file = new VBitmap(handler);
   //file_name = OpenDialog->FileName;
   ////
      file_name = "";
   ////
   file->ChangePath(file_name);

   undo_level = 0;
   MUndo1->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::CreateColorImage(ColorImage* color_image)
{
   if (handler!=NULL){
      delete handler;
      handler = NULL;
    }
   delete color_handler;
   color_handler = new BMPHandlerColor();
   filter_index = 0;
   ext = extention[0];
   color_handler->CreateBitmapAndAssign(color_image);
   color_list->Append(*color_image);
   color_handler->Set(*PictureForm->color_image);
   Panel->Height = color_handler->Height()+2;
   Panel->Width = color_handler->Width()+2;
   Panel->Left = MainPanel->Width/2 - Panel->Width/2;
   Panel->Top = MainPanel->Height/2 - Panel->Height/2;
   color_handler->Show(Image);

   file = new VBitmap(color_handler);
   file->ChangePath(OpenDialog->FileName);

   undo_level_color++;
   undo_level_color = (undo_level_color <= 9) ? undo_level_color : 9;
   MUndo1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::AssignList(GreyImage* image)
{
   list->Append(*image);
   AssignImage(image);
   undo_level++;
   undo_level = (undo_level < 9) ? undo_level : 9;
   if (undo_level!=0) MUndo1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::AssignColorList(ColorImage* color_image)
{
   color_list->Append(*color_image);
   AssignColorImage(color_image);
   undo_level_color++;
   if (undo_level_color!=0) MUndo1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::InitialColorPicture(AnsiString _file_path,
                                             ImageType _image_type)
{
   color_handler = new BMPHandlerColor();
   file = new VBitmap(_file_path, true);
   file->ConvertToColorBMP(color_handler);
   StatusBar1->Panels->Items[4]->Text = file->ImageName();
   filter_index = 0;
   ext = extention[0];
   color_handler->Set(*color_image);
   AssignColorList(color_image);

   MView->Visible=true;
   MActualView1->Visible = false;
   MZoomIn1->Visible = false;
   MZoomOut1->Visible = false;
   MShowMoreImage->Visible=true;
   MMagnifyingGlass->Visible = false;

   MEdit->Visible = true;
   MUndo1->Visible = true;
   MCut1->Visible = false;
   MResize1->Visible = false;
   MTranslate->Visible = false;
   MFFTSize->Visible = false;
   MRotate1->Visible = false;

   MImage->Visible = true;
   MNoise1->Visible = false;
   MContrast->Visible = false;
   MBrightness->Visible = false;
   MProfile->Visible = false;
   MHistogram1->Visible = false;
   MConvolution1->Visible = false;
   LUT1->Visible = false;
   DFT1->Visible = false;
   MLogicArith1->Visible = false;
   MObject1->Visible = false;
   MMorpholog1->Visible = false;
   MNonlinearFiltres1->Visible = false;
   MBitFields1->Visible = false;
   MSobelFilters1->Visible = false;
   MCharacteristics->Visible = false;
   MCorrection->Visible = false;
   MAveraging->Visible = true;

   MColor->Visible=true;
   MPseudoColor->Visible = false;
   MSplitToRGB->Visible = true;
   MGreyScale->Visible = true;
   MAdjust->Visible = true;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::InitialPicture(AnsiString _file_path,
                                        ImageType _image_type)
{
   handler = new BMPHandler();
   file = new VBitmap(_file_path);
   file->ConvertToBMP(handler);
   StatusBar1->Panels->Items[4]->Text = file->ImageName();
   filter_index = 0;
   ext = extention[0];
   handler->Set(*image);
   AssignList(image);
   
   MView->Visible=true;
   MActualView1->Visible = false;
   MZoomIn1->Visible = true;
   MZoomOut1->Visible = true;
   MShowMoreImage->Visible = true;
   MMagnifyingGlass->Visible = true;

   MEdit->Visible = true;
   MUndo1->Visible = true;
   MCut1->Visible = true;
   MResize1->Visible = true;
   MTranslate->Visible = true;
   MFFTSize->Visible = true;
   MRotate1->Visible = true;

   MImage->Visible = true;
   MNoise1->Visible = true;
   MContrast->Visible = true;
   MBrightness->Visible = true;
   MProfile->Visible = true;
   MHistogram1->Visible = true;
   MConvolution1->Visible = true;
   LUT1->Visible = true;
   DFT1->Visible = true;
   MLogicArith1->Visible = true;
   MObject1->Visible = true;
   MMorpholog1->Visible = true;
   MNonlinearFiltres1->Visible = true;
   MBitFields1->Visible = true;
   MSobelFilters1->Visible = true;
   MCharacteristics->Visible = true;
   MCorrection->Visible = true;
   MAveraging->Visible = true;

   MColor->Visible = true;
   MPseudoColor->Visible = true;
   MSplitToRGB->Visible = false;
   MGreyScale->Visible = false;
   MAdjust->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::ImageMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  // for measuring od distance
  pos_x = int(((double)(X)) / scale_const);
  pos_y = int(((double)(Y)) / scale_const);
  if (measuring == true){
     StatusBar1->Panels->Items[5]->Text = "Distance: "
     +AnsiString((int)(sqrt((pos_x-diff_x)*(pos_x-diff_x)+(pos_y-diff_y)*(pos_y-diff_y))));
  }
  int rpoint, gpoint, bpoint;
  DrawForm->OnMouseMove(X, Y);
  if(handler!=NULL){
     Image->Cursor = crCross;
     X = pos_x;
     Y = pos_y;
     X = (X<0) ? 0 : X;
     Y = (Y<0) ? 0 : Y;
     X = (X>handler->Cols()-1) ? handler->Cols()-1  : X;
     Y = (Y>handler->Rows()-1) ? handler->Rows()-1  : Y;
     StatusBar1->Panels->Items[0]->Text = AnsiString(handler->Cols())+" x "+AnsiString(handler->Rows());
     StatusBar1->Panels->Items[1]->Text = "Row: "+AnsiString(Y);
     StatusBar1->Panels->Items[2]->Text = "Column: "+AnsiString(X);
     StatusBar1->Panels->Items[3]->Text = "Value: "+AnsiString(image->Data(X*scale_const,Y*scale_const));
     // show brightness profile
     if (show_profile == true){
        BrightnessProfileForm->ShowGraph(X, Y);
     }
  }else{
     if(color_handler!=NULL){
        Image->Cursor = crCross;
        X = (X<0) ? 0 : X;
        Y = (Y<0) ? 0 : Y;
        X = (X>color_image->Cols()-1) ? color_image->Cols()-1  : X;
        Y = (Y>color_image->Rows()-1) ? color_image->Rows()-1  : Y;
        StatusBar1->Panels->Items[0]->Text = AnsiString(color_handler->Cols())+" x "+AnsiString(color_handler->Rows());
        StatusBar1->Panels->Items[1]->Text = "Row: "+AnsiString(Y);
        StatusBar1->Panels->Items[2]->Text = "Column: "+AnsiString(X);

        bpoint = floor(color_image->Data(X,Y)/65536);
        gpoint = floor((color_image->Data(X,Y) - (65536*bpoint))/256);
        rpoint = color_image->Data(X,Y) - (65536*bpoint)-(256*gpoint);
        StatusBar1->Panels->Items[3]->Text = "R:"+IntToStr(rpoint)+" G:"+IntToStr(gpoint)+" B:"+IntToStr(bpoint);

        // show brightness profile
        if (show_profile == true){
           BrightnessProfileForm->ShowGraph(X, Y);
        }
     }else{
        StatusBar1->Panels->Items[0]->Text = "";
        StatusBar1->Panels->Items[1]->Text = "";
        StatusBar1->Panels->Items[2]->Text = "";
        StatusBar1->Panels->Items[3]->Text = "";
     }   
  }

}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::FormMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  Image->Cursor = crDefault;
  if(handler!=NULL){
     StatusBar1->Panels->Items[1]->Text = "Row: -";
     StatusBar1->Panels->Items[2]->Text = "Column: -";
     StatusBar1->Panels->Items[3]->Text = "Value: -";
  }else{
     if(color_handler!=NULL){
        StatusBar1->Panels->Items[0]->Text = AnsiString(color_handler->Cols())+" x "+AnsiString(color_handler->Rows());
        StatusBar1->Panels->Items[1]->Text = "Row: -";
        StatusBar1->Panels->Items[2]->Text = "Column: -";
        StatusBar1->Panels->Items[3]->Text = "R:- G:- B:-";
     }else{
        StatusBar1->Panels->Items[0]->Text = "";
        StatusBar1->Panels->Items[1]->Text = "";
        StatusBar1->Panels->Items[2]->Text = "";
        StatusBar1->Panels->Items[3]->Text = "";
     }
  }
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MCut1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TCutForm),
                          &CutForm);
   CutForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MUndo1Click(TObject *Sender)
{
   if (handler!=NULL){
      list->Delete();
      *image = (list->Member());
      AssignImage(image);
      undo_level--;
      if (undo_level == 0)
         MUndo1->Enabled = false;
      
   }else{
      color_list->Delete();
      *color_image = (color_list->Member());
      AssignColorImage(color_image);
      undo_level_color--;
      if (undo_level_color == 0)
         MUndo1->Enabled = false;
   }
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MTranslateClick(TObject *Sender)
{
   Application->CreateForm(__classid(TTranslateForm),
                          &TranslateForm);
   TranslateForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::ImageMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   DrawForm->OnMouseDown(Button, X, Y);            
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::ImageMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   DrawForm->OnMouseUp(Button, X, Y);              
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MObject1Click(TObject *Sender)
{
   MEdit->Visible = false;
   MImage->Visible = false;
   MView->Visible = false;
   Open1->Visible = false;
   MSave->Visible = false;
   MSaveAs->Visible = false;
   MColor->Visible = false;
   DrawForm->Top = PictureForm->Top+30;
   DrawForm->Left = PictureForm->Left + PictureForm->Width - 30 - DrawForm->Width;
   DrawForm->AssignImage();
   DrawForm->Show();

}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::RotateFreely1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TRotateForm),
                          &RotateForm);
   RotateForm -> Show();
   MEdit->Visible = false;
   Open1->Visible = false;
   MSave->Visible = false;
   MSaveAs->Visible = false;
   MImage->Visible = false;
   MView->Visible = false;
   MColor->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MResize1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TResizeForm),
                          &ResizeForm);
   ResizeForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MSaveClick(TObject *Sender)
{
  if (handler!=NULL){
     file->Assign(handler);
  }else{
     file->ColorAssign(color_handler);
  }
  file->SaveToFile(file->FilePath());
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MSaveAsClick(TObject *Sender)
{
   if (get_other_handler == false){
      file_name = file->ImageName();
      SaveDialog->FileName = file->ImageName();
   }else{
      SaveDialog->FileName = "";
   }
   SaveDialog->DefaultExt = ext;
   SaveDialog->FilterIndex = filter_index;
   if(SaveDialog->Execute()){
      try{
         switch(SaveDialog->FilterIndex){
            case(1):
               // Bitmap file
               //delete(file);
               //file = NULL;
               file_name = SaveDialog->FileName;
               file->ChangePath(file_name);
               file_name = ChangeFileExt(file_name, extention[0]);
               if(!IfFileExistsOverWrite(file_name))
                   return;
               if (handler!=NULL){
                  //file = new VBitmap(handler);
                  if (get_other_handler == false){
                     file->Assign(handler);
                  }else{
                     file->Assign(other_handler);
                  }
               }else{
                  //file = new VBitmap(color_handler);
                  file->ColorAssign(color_handler);
               }
               file->SaveToFile(file_name);
               if (get_other_handler == false){
                  MSave->Visible=true;
                  StatusBar1->Panels->Items[4]->Text = file->ImageName();
               }
               break;
          }
      }catch(VEMsg &E){
         Application->MessageBox(E.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
      }
   }
}
//---------------------------------------------------------------------------
// Flip Horizontally
void __fastcall TPictureForm::Horizontally1Click(TObject *Sender)
{
  try{
      GreyImage orig, mirror;
      handler -> Set(orig);
      Translate trans(orig, mirror);
      trans.MirrorImage(0, 0, orig.Cols(), orig.Rows(), 0, 0,
                                HorizMirror);
      AssignList(&mirror);
   }catch(VEMsg &e){
      Application->MessageBox(e.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
// Flip Vertically
void __fastcall TPictureForm::Vertically1Click(TObject *Sender)
{
 try{
      GreyImage orig, mirror;
      handler -> Set(orig);
      Translate trans(orig, mirror);
      trans.MirrorImage(0, 0, orig.Cols(), orig.Rows(), 0, 0,
                                VertMirror);
      AssignList(&mirror);
   }catch(VEMsg &e){
      Application->MessageBox(e.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
// Rotate Left 90
void __fastcall TPictureForm::RotateLeft1Click(TObject *Sender)
{
  try{
      GreyImage orig, rot;
      handler -> Set(orig);
      Rotate rotate(orig, rot);
      rotate.ChangeSizeRotation(90, 1, clBlack);
      AssignList(&rot);
   }catch(VEMsg &e){
      Application->MessageBox(e.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
// Rotate Right 90
void __fastcall TPictureForm::RotateRight1Click(TObject *Sender)
{
  try{
      GreyImage orig, rot;
      handler -> Set(orig);
      Rotate rotate(orig, rot);
      rotate.ChangeSizeRotation(270, 1, clBlack);
      AssignList(&rot);
   }catch(VEMsg &e){
      Application->MessageBox(e.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
// Rotate Right 180
void __fastcall TPictureForm::Rotate1Click(TObject *Sender)
{
  try{
      GreyImage orig, rot;
      handler -> Set(orig);
      Rotate rotate(orig, rot);
      rotate.PerformRotation(0, 0, orig.Cols(), orig.Rows(), 180, 1, clBlack);
      AssignList(&rot);
   }catch(VEMsg &e){
      Application->MessageBox(e.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
// Actual size
void __fastcall TPictureForm::MActualView1Click(TObject *Sender)
{
   scale_const = 1;
   StatusBar1->Panels->Items[0]->Text = AnsiString(handler->Cols())+" x "+AnsiString(handler->Rows());
   handler->Set(*image);
   Panel->Width = image->Cols()+2;
   Panel->Height = image->Rows()+2;
   Panel->Left = MainPanel->Width/2 - Panel->Width/2;
   Panel->Top = MainPanel->Height/2 - Panel->Height/2;
   handler->Show(Image);
   MEdit->Visible = true;
   MImage->Visible = true;
   MFile->Visible = true;
   MColor->Visible = true;
   MActualView1->Visible = false;
   MMagnifyingGlass->Visible = true;

   SizeAdaptation();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MZoomIn1Click(TObject *Sender)
{
   scale_const *=  (double)(1.5);
   GreyImage image, scaled_image;
   handler->Set(image);
   Scale scale(image, scaled_image);
   int cols = image.Cols()*scale_const;
   int rows = image.Rows()*scale_const;
  // if((cols>MAX_COLS)||(rows>MAX_ROWS)){
  //    cols = (int) ((double) (cols) / (double)(1.5));
  //    rows = (int) ((double) (rows) / (double)(1.5));
  //    scale_const /=  1.5;
  // }
   scale.NewSize(cols, rows, false);
   AssignImage(&scaled_image);
   handler->Assign(image);
   MEdit->Visible = false;
   MImage->Visible = false;
   MFile->Visible = false;
   MColor->Visible = false;
   MActualView1->Visible = true;
   MMagnifyingGlass->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MZoomOut1Click(TObject *Sender)
{
   scale_const /=  (double)(1.5);
   GreyImage image, scaled_image;
   handler->Set(image);
   Scale scale(image, scaled_image);
   scale.NewSize((int)((double)(image.Cols()-1)*scale_const), (int)((double)(image.Rows()-1)*scale_const), false);
   AssignImage(&scaled_image);
   handler->Assign(image);
   MEdit->Visible = false;
   MImage->Visible = false;
   MFile->Visible = false;
   MColor->Visible = false;
   MActualView1->Visible = true;
   MMagnifyingGlass->Visible = false;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MConvolution1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TConvolutionForm),
                          &ConvolutionForm);
   ConvolutionForm->ShowModal();              
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MNoise1Click(TObject *Sender)
{
 Application->CreateForm(__classid(TNoiseForm),
                       &NoiseForm);
   NoiseForm->ShowModal();                   
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::LUT1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TLUTForm),
                          &LUTForm);
   LUTForm->Top = Top + 50;
   LUTForm->Left = Left + 50;
   LUTForm -> ShowModal();
}
//---------------------------------------------------------------------------
bool __fastcall TPictureForm::CheckSize(int cols, int rows)
{
   int max_size = (cols>rows)? cols : rows;
   if(max_size>1024)
      throw EDFTTooLargeImage(AnsiString("Image is too large!\n\
      The longer size can't be bigger than 1024"));
   int pow_of2=1;
   int which_pow_of2=0;
   while (max_size>pow_of2){
       pow_of2 = pow(2, ++which_pow_of2);
       if((max_size==pow_of2)&&(max_size==cols)&&(max_size==rows))
          return true;
   }
   return false;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MFFTSizeClick(TObject *Sender)
{
   if(!CheckSize(handler->Cols(),handler->Rows())){
      Application->CreateForm(__classid(TDFTSizeForm),
                             &DFTSizeForm);
      DFTSizeForm->ShowModal();
   }else
      Application->MessageBox("This Image has proper size for FFT",
         "MIPS",MB_ICONINFORMATION|MB_OK);
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MContrastClick(TObject *Sender)
{
   Application->CreateForm(__classid(TContrastForm),
                       &ContrastForm);
   ContrastForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MVerticalSobelClick(TObject *Sender)
{
 GreyImage orig_image, sobel_image;
   handler->Set(orig_image);
   EdgeDetector edge_detector(orig_image, sobel_image);
   if(Sender==MVerticalSobel)
      edge_detector.VerticalSobel();
   else if(Sender == MHorizontalSobel)
      edge_detector.HorizontalSobel();
   else if(Sender == MEdgeMagnitude)
      edge_detector.MagnitudeSobel();
   else if(Sender == MEdgeMagnitudePure)
      edge_detector.PureMagnitudeSobel();
   else if(Sender == MEdgePhase)
      edge_detector.PhaseSobel();

   AssignList(&sobel_image);               
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MBit1Click(TObject *Sender)
{
  *image = list->Member();
   GreyImage bit_image;
   if (Sender==MBit1){
      image->BitField(bit_image, 0, image->Cols(),image->Rows());
   }else if(Sender==MBit2){
      image->BitField(bit_image, 1, image->Cols(),image->Rows());
   }else if(Sender==MBit3){
      image->BitField(bit_image, 2, image->Cols(),image->Rows());
   }else if(Sender==MBit4){
      image->BitField(bit_image, 3, image->Cols(),image->Rows());
   }else if(Sender==MBit5){
      image->BitField(bit_image, 4, image->Cols(),image->Rows());
   }else if(Sender==MBit6){
      image->BitField(bit_image, 5, image->Cols(),image->Rows());
   }else if(Sender==MBit7){
      image->BitField(bit_image, 6, image->Cols(),image->Rows());
   }else if(Sender==MBit8){
      image->BitField(bit_image, 7, image->Cols(),image->Rows());
   }
   AssignList(&bit_image);        
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MMaximumClick(TObject *Sender)
{
   GreyImage filtered_image;
   GreyImage orig_image;
   handler->Set(orig_image);
   nonlinear_filter = new NonlinearFilter(orig_image,filtered_image);
   if(Sender == MMaximum){
      nonlinear_filter->MaximumFilter(0, 0, orig_image.Cols(),
                     orig_image.Rows(),nf_cols,nf_rows);
   }else if(Sender == MMedian){
      nonlinear_filter->MedianFilter(0, 0, orig_image.Cols(),
                     orig_image.Rows(),nf_cols,nf_rows);
   }else if(Sender == MMinimum){
      nonlinear_filter->MinimumFilter(0, 0, orig_image.Cols(),
                     orig_image.Rows(),nf_cols,nf_rows);
   }else if(Sender == MHarmonicMean){
      nonlinear_filter->HarmonicMeanFilter(0, 0, orig_image.Cols(),
                     orig_image.Rows(),nf_cols,nf_rows);
   }else if(Sender == MMidpoint){
      nonlinear_filter->MidpointFilter(0, 0, orig_image.Cols(),
                     orig_image.Rows(),nf_cols,nf_rows);
   }else if(Sender == MGeometricMean){
      nonlinear_filter->GeometricMeanFilter(0, 0, orig_image.Cols(),
                     orig_image.Rows(),nf_cols,nf_rows);
   }
   AssignList(&filtered_image);        
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::SizeMedianClick(TObject *Sender)
{
   Application->CreateForm(__classid(TNonlinearFiltersForm),
                          &NonlinearFiltersForm);
   NonlinearFiltersForm->ColsEdit->Text = IntToStr(nf_cols);
   NonlinearFiltersForm->RowsEdit->Text = IntToStr(nf_rows);
   try{
      if (NonlinearFiltersForm->ShowModal() != IDCANCEL){
         nf_cols = NonlinearFiltersForm->ColsEdit->Text.ToInt();
         nf_rows = NonlinearFiltersForm->RowsEdit->Text.ToInt();
      }
   }catch(EConvertError& e){
      MessageBox(Handle, e.Message.c_str(),
      "Nonlinear Filters", MB_ICONWARNING | MB_OK);
   }                
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MMorphBinary1Click(TObject *Sender)
{
   morph_kind = 0;
   Application->CreateForm(__classid(TMORPHForm),
                          &MORPHForm);
   MORPHForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MMorphGreylevel1Click(TObject *Sender)
{
   morph_kind = 1;
   Application->CreateForm(__classid(TMORPHForm),
                          &MORPHForm);
   MORPHForm->ShowModal();             
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MLogicArith1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TFrameForm),
                          &FrameForm);
   FrameForm->ShowModal();               
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
    Release();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::Exit1Click(TObject *Sender)
{
   Close();        
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MViewHistogramClick(TObject *Sender)
{
   Application->CreateForm(__classid(THistogramForm),
                       &HistogramForm);
   HistogramForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::AboutMIPS1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TAboutForm),
                       &AboutForm);
   AboutForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::Licence1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TLicenceForm),
                       &LicenceForm);
   LicenceForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MAveragingClick(TObject *Sender)
{
   Application->CreateForm(__classid(TAverageForm),
                       &AverageForm);
   AverageForm->ShowModal();
}
//---------------------------------------------------------------------------


void __fastcall TPictureForm::MPseudoColorClick(TObject *Sender)
{
   Application->CreateForm(__classid(TPseudoColorsForm),
                       &PseudoColorsForm);
   PseudoColorsForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::ChangeColorPalette(Graphics::TBitmap* _bitmap)
{
   handler->Assign(_bitmap);
   handler->Show(Image);
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MBrightnessClick(TObject *Sender)
{
   Application->CreateForm(__classid(TBrightnessForm),
                       &BrightnessForm);
   BrightnessForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::Profile1Click(TObject *Sender)
{
   show_profile = true;
   Open1->Visible = false;
   MSave->Visible = false;
   MSaveAs->Visible = false;
   MView->Visible = false;
   MEdit->Visible = false;
   MImage->Visible = false;
   MColor->Visible = false;
   Application->CreateForm(__classid(TBrightnessProfileForm),
                       &BrightnessProfileForm);
   BrightnessProfileForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::ShowProfileOff()
{
   show_profile = false;
}
//---------------------------------------------------------------------------
void __fastcall TPictureForm::MSplitToRGBClick(TObject *Sender)
{
   Open1->Visible = false;
   MSave->Visible = false;
   MSaveAs->Visible = false;
   MView->Visible = false;
   MEdit->Visible = false;
   MImage->Visible = false;
   MColor->Visible = false;
   Application->CreateForm(__classid(TRGBComponentsForm),
                       &RGBComponentsForm);
   RGBComponentsForm->Show();
}
//---------------------------------------------------------------------------
 void __fastcall TPictureForm::Centroid1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TCentroidForm),
                       &CentroidForm);
   CentroidForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::MFlatFieldClick(TObject *Sender)
{
   Application->CreateForm(__classid(TFlatFieldForm),
                       &FlatFieldForm);
   FlatFieldForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::MStatisticsClick(TObject *Sender)
{
   Application->CreateForm(__classid(TStatisticsForm),
                       &StatisticsForm);
   StatisticsForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::MEqualizationHistogramClick(TObject *Sender)
{
   Application->CreateForm(__classid(THistogramEqualizationForm),
                       &HistogramEqualizationForm);
   HistogramEqualizationForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::Stretching1Click(TObject *Sender)
{
   Application->CreateForm(__classid(THistogramStrechingForm),
                       &HistogramStrechingForm);
   HistogramStrechingForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::N2Images1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TShowTwoImagesForm),
                       &ShowTwoImagesForm);
   ShowTwoImagesForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::N4Images1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TShowFourImagesForm),
                       &ShowFourImagesForm);
   ShowFourImagesForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::N6Images1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TShowSixImagesForm),
                       &ShowSixImagesForm);
   ShowSixImagesForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::MGreyScaleClick(TObject *Sender)
{
   Application->CreateForm(__classid(TGreyScaleForm),
                       &GreyScaleForm);
   GreyScaleForm->ShowModal();
}
//---------------------------------------------------------------------------
       
void __fastcall TPictureForm::MHSVClick(TObject *Sender)
{
   Application->CreateForm(__classid(THSVToolForm),
                       &HSVToolForm);
   HSVToolForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::MHSLClick(TObject *Sender)
{
   Application->CreateForm(__classid(THSLToolForm),
                       &HSLToolForm);
   HSLToolForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::RGB1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TCMYToolForm),
                       &CMYToolForm);
   CMYToolForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::CMYK1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TCMYKToolForm),
                       &CMYKToolForm);
   CMYKToolForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::MMagnifyingGlassClick(TObject *Sender)
{
   Application->CreateForm(__classid(TMagnifyingGlassForm),
                       &MagnifyingGlassForm);
   MagnifyingGlassForm->ShowModal();
}
//---------------------------------------------------------------------------


void __fastcall TPictureForm::Profile3Click(TObject *Sender)
{
   Application->CreateForm(__classid(TCircleProfileForm),
                       &CircleProfileForm);
   CircleProfileForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::MNonUniformIlluminationClick(TObject *Sender)
{
   Application->CreateForm(__classid(TNonUniformIlluminationForm),
                       &NonUniformIlluminationForm);
   NonUniformIlluminationForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::Profile2Click(TObject *Sender)
{
   Application->CreateForm(__classid(TLineProfileForm),
                       &LineProfileForm);
   LineProfileForm->ShowModal();
}
//---------------------------------------------------------------------------


void __fastcall TPictureForm::PSNR1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TSnrPsnrForm),
                       &SnrPsnrForm);
   SnrPsnrForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::Specification1Click(TObject *Sender)
{
   Application->CreateForm(__classid(THistogramSpecificationForm),
                       &HistogramSpecificationForm);
   HistogramSpecificationForm->Top = Top + 50;
   HistogramSpecificationForm->Left = Left + 50;
   HistogramSpecificationForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::DFT11Click(TObject *Sender)
{
   try{
      if(!CheckSize(handler->Cols(),handler->Rows())){
         throw EDFTTooLargeImage("Only Images with sizes of power to 2 are supported");
      }else{
         Application->CreateForm(__classid(TDFTForm),
                          &DFTForm);
         MEdit->Visible = false;
         Open1->Visible = false;
         MSave->Visible = false;
         MSaveAs->Visible = false;
         MImage->Visible = false;
         MView->Visible = false;
         MColor->Visible = false;
      }
   }catch(EDFTTooLargeImage &E){
      Application->MessageBox(E.Message.c_str(),
         "Discrete Fourier Transform",MB_ICONQUESTION|MB_OK);
   }        
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::DFT12Click(TObject *Sender)
{
   Application->CreateForm(__classid(TFftFiltrationForm),
                       &FftFiltrationForm);
   FftFiltrationForm->ShowModal();
}
//---------------------------------------------------------------------------
// Change showing scrollbars
void __fastcall TPictureForm::SizeAdaptation()
{
   // resize window
   PictureForm->ClientHeight = (min_height > Panel->Height+40) ? min_height : Panel->Height+40;
   PictureForm->ClientWidth = (min_width > Panel->Width+20) ? min_width : Panel->Width+20;

   // alignment image to centre of window
   Panel->Left = MainPanel->Width/2 - Panel->Width/2;
   Panel->Top = MainPanel->Height/2 - Panel->Height/2;

   // show or hide scrollbars
   if (Panel->Width > MainPanel->Width){
      ScrollBarHorizontal->Visible = true;
      ScrollBarHorizontal->Position = 50;
   }else{
      ScrollBarHorizontal->Visible = false;
   }
   if (Panel->Height > MainPanel->Height){
      ScrollBarVertical->Visible = true;
      ScrollBarVertical->Position = 50;
   }else{
      ScrollBarVertical->Visible = false;
   }
   Panel->Left = MainPanel->Width/2 - Panel->Width/2;
   Panel->Top = MainPanel->Height/2 - Panel->Height/2;
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::FormResize(TObject *Sender)
{
   Panel->Left = MainPanel->Width/2 - Panel->Width/2;
   Panel->Top = MainPanel->Height/2 - Panel->Height/2;
   if (Panel->Width > MainPanel->Width){
      ScrollBarHorizontal->Visible = true;
   }else{
      ScrollBarHorizontal->Visible = false;
      ScrollBarHorizontal->Position = 50;
   }
   if (Panel->Height > MainPanel->Height){
      ScrollBarVertical->Visible = true;
      ScrollBarVertical->Position = 50;
   }else{
      ScrollBarVertical->Visible = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::MainPanelMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  Image->Cursor = crDefault;
  if(handler!=NULL){
     StatusBar1->Panels->Items[0]->Text = AnsiString(handler->Cols())+" x "+AnsiString(handler->Rows());
     StatusBar1->Panels->Items[1]->Text = "Row: -";
     StatusBar1->Panels->Items[2]->Text = "Column: -";
     StatusBar1->Panels->Items[3]->Text = "Value: -";
  }else{
     if(color_handler!=NULL){
        StatusBar1->Panels->Items[0]->Text = AnsiString(color_handler->Cols())+" x "+AnsiString(color_handler->Rows());
        StatusBar1->Panels->Items[1]->Text = "Row: -";
        StatusBar1->Panels->Items[2]->Text = "Column: -";
        StatusBar1->Panels->Items[3]->Text = "R:- G:- B:-";
     }else{
        StatusBar1->Panels->Items[0]->Text = "";
        StatusBar1->Panels->Items[1]->Text = "";
        StatusBar1->Panels->Items[2]->Text = "";
        StatusBar1->Panels->Items[3]->Text = "";
     }
  }        
}
//---------------------------------------------------------------------------
// Horizontal scrollbar
void __fastcall TPictureForm::ScrollBarHorizontalChange(TObject *Sender)
{
   int size_overlap = (MainPanel->Width - Panel->Width);
   Panel->Left = int((float)(ScrollBarHorizontal->Position)*((float)(size_overlap)/(float)(100)));
}
//---------------------------------------------------------------------------
// Vertical scrollbar
void __fastcall TPictureForm::ScrollBarVerticalChange(TObject *Sender)
{
   int size_overlap = (MainPanel->Height - Panel->Height);
   Panel->Top = int((float)(ScrollBarVertical->Position)*((float)(size_overlap)/(float)(100)));
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::GammaAdjustment1Click(TObject *Sender)
{
   Application->CreateForm(__classid(TGammaCorrectionForm),
                       &GammaCorrectionForm);
   GammaCorrectionForm->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TPictureForm::HistogramAdjustment1Click(TObject *Sender)
{
   Application->CreateForm(__classid(THistogramModificationForm),
                       &HistogramModificationForm);
   HistogramModificationForm->ShowModal();
}
//---------------------------------------------------------------------------
// Start showing distance after click to mouse button
void __fastcall TPictureForm::ImageClick(TObject *Sender)
{
  diff_x = pos_x;
  diff_y = pos_y;
  measuring = true;
}
//---------------------------------------------------------------------------
// if any function call handler from different form
BMPHandler* __fastcall TPictureForm::GetBMPHandler()
{
   if (get_other_handler == false){
      return handler;
   }else{
      return other_handler;
   }
}

