/*
   ColorMapEditorUnit.cpp

   definin soubor okna pro definici barevn palety pro pseudobarevn
   obohacen obrazu

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "ColormapEditorUnit.h"
#include "PseudoUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TColorMapForm *ColorMapForm;
//---------------------------------------------------------------------------
// constructor
__fastcall TColorMapForm::TColorMapForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PseudoColorsForm->Top+30;
   Left = PseudoColorsForm->Left + PseudoColorsForm->Width - 30 - Width;
   Edit1->Text = "0";
   Edit2->Text = "255";
   start_level = 0;
   end_level = 255;
   CheckBox1->Checked = false;
   Panel4->Color = StringToColor("clBlack");
   Panel2->Color = StringToColor("clSilver");
   Label4->Enabled = false;
   Panel2->Enabled = false;
   Button2->Enabled = false;

   for (int i=0; i<256; i++){
      palette[0][i] = PseudoColorsForm->custom_palette[0][i];
      palette[1][i] = PseudoColorsForm->custom_palette[1][i];
      palette[2][i] = PseudoColorsForm->custom_palette[2][i];
   }

   stream = new TMemoryStream;
   pseudo = new PseudoColor();
   pseudo->CreateImageFromPalette(palette, stream);
   PaletteImage->Picture->Bitmap->LoadFromStream(stream);
   StatusBar1->SimpleText = "Select 'start level','end level','color' and click to Perform button.";
}
//---------------------------------------------------------------------------
// destructor
__fastcall TColorMapForm::~TColorMapForm()
{
   delete (stream);
   delete (pseudo);
}
//---------------------------------------------------------------------------
// use created palette
void __fastcall TColorMapForm::BitBtn1Click(TObject *Sender)
{
   for (int i=0; i<256; i++){
      PseudoColorsForm->custom_palette[0][i] = palette[0][i];
      PseudoColorsForm->custom_palette[1][i] = palette[1][i];
      PseudoColorsForm->custom_palette[2][i] = palette[2][i];
   }
  Close();
}
//---------------------------------------------------------------------------
// close form
void __fastcall TColorMapForm::BitBtn2Click(TObject *Sender)
{
  Close();
}
//---------------------------------------------------------------------------
// change color palette (0-255)
void __fastcall TColorMapForm::Reset_ButtonClick(TObject *Sender)
{
   Edit1->Text = "0";
   Edit2->Text = "255";
   start_level = 0;
   end_level = 255;
   CheckBox1->Checked = false;
   Panel4->Color = StringToColor("clBlack");
   Panel2->Color = StringToColor("clSilver");
   Label4->Enabled = false;
   Panel2->Enabled = false;
   Button2->Enabled = false;

   delete stream;
   delete pseudo;
   stream = new TMemoryStream;
   pseudo = new PseudoColor();
   pseudo->ChangePalette(palette, GREY);
   pseudo->CreateImageFromPalette(palette, stream);
   PaletteImage->Picture->Bitmap->LoadFromStream(stream);
   StatusBar1->SimpleText = "Select 'start level','end level','color' and click to Perform button.";
}
//---------------------------------------------------------------------------
// type of color - gradient or one color
void __fastcall TColorMapForm::CheckBox1Click(TObject *Sender)
{
   if (Label4->Enabled == false){
      Panel2->Color = StringToColor("clWhite");
      Label4->Enabled = true;
      Panel2->Enabled = true;
      Button2->Enabled = true;
   }else{
      Panel2->Color = StringToColor("clSilver");
      Label4->Enabled = false;
      Panel2->Enabled = false;
      Button2->Enabled = false;
   }
}
//---------------------------------------------------------------------------
// start color
void __fastcall TColorMapForm::Button1Click(TObject *Sender)
{
   if (ColorDialog->Execute()){
     Panel4->Color = ColorDialog->Color;
   }
}
//---------------------------------------------------------------------------
// end color
void __fastcall TColorMapForm::Button2Click(TObject *Sender)
{
   if (ColorDialog->Execute()){
     Panel2->Color = ColorDialog->Color;
   }
}
//---------------------------------------------------------------------------
// apply new color palette to original image
void __fastcall TColorMapForm::Perform_ButtonClick(TObject *Sender)
{
   // this variable representative RGB values
   int array1[3];
   int array2[3];
   // if end_level > start_level, then turn start_level and end_level over
   if (end_level<start_level){
      start_level = Edit2->Text.ToInt();
      end_level = Edit1->Text.ToInt();
      Edit1->Text = start_level;
      Edit2->Text = end_level;
   }
   pseudo->IntToRBG(ColorToRGB(Panel4->Color),array1); // color to RGB values
   pseudo->IntToRBG(ColorToRGB(Panel2->Color),array2); // color to RGB values
   if (CheckBox1->Checked == false){
      for (int i=start_level; i<=end_level; i++){
         palette[0][i] = array1[0];
         palette[1][i] = array1[1];
         palette[2][i] = array1[2];
      }
   }else{
     pseudo->Gradient(start_level, end_level, array1,
                      array2, palette);
   }
   delete stream;
   stream = new TMemoryStream;
   pseudo->CreateImageFromPalette(palette, stream);
   PaletteImage->Picture->Bitmap->LoadFromStream(stream);
   StatusBar1->SimpleText = " Click to OK for using palette. You can continue with a next operation.";
}
//---------------------------------------------------------------------------
// start changed level
void __fastcall TColorMapForm::Edit1Change(TObject *Sender)
{
    try{
       if(Edit1->Text=="")
       throw VEMsg("");
       start_level = Edit1->Text.ToInt();
       if(start_level<0){
           Edit1->Text = 0;
           throw VEMsg("");
        }
       if(start_level>255){
           Edit1->Text = 255;
           throw VEMsg("");
        }
    }catch(EConvertError& e){
         MessageBox(Handle, e.Message.c_str(),
            "ColorMap Tool Window", MB_ICONWARNING | MB_OK);
    }catch(VEMsg& e){

    }
}
//---------------------------------------------------------------------------
// end changed level
void __fastcall TColorMapForm::Edit2Change(TObject *Sender)
{
    try{
       if(Edit2->Text=="")
       throw VEMsg("");
       end_level = Edit2->Text.ToInt();
       if(end_level<0){
           Edit2->Text = 0;
           throw VEMsg("");
        }
       if(end_level>255){
           Edit2->Text = 255;
           throw VEMsg("");
        }
    }catch(EConvertError& e){
         MessageBox(Handle, e.Message.c_str(),
            "ColorMap Tool Window", MB_ICONWARNING | MB_OK);
    }catch(VEMsg& e){

    }
}
//---------------------------------------------------------------------------
// close form
void __fastcall TColorMapForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release(); 
}
//---------------------------------------------------------------------------

