/*
   ResizeUnit.cpp

   definin soubor okna pro zmnu velikosti okna

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "ResizeUnit.h"
#include "PictureUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TResizeForm *ResizeForm;
//---------------------------------------------------------------------------
__fastcall TResizeForm::TResizeForm(TComponent* Owner)
        : TForm(Owner)
{
   Top = PictureForm->Top+30;
   Left = PictureForm->Left + PictureForm->Width - 30 - Width;

   StandardComboBox->ItemIndex = 0;
   UnitComboBox->ItemIndex = 0;
   original_image = new GreyImage;
   resized_image = new GreyImage;
   proc_image = new GreyImage;
   handler = new BMPHandler(*PictureForm->GetBMPHandler());
   handler->Set(*original_image);
   WidthLabel->Caption = AnsiString("Width:  ")+AnsiString(original_image->Cols());
   HeightLabel->Caption = AnsiString("Height: ")+AnsiString(original_image->Rows());
   handler->Set(*proc_image);
   SetProcSize();
   scale = new Scale(*proc_image, *resized_image);

   width = original_image->Cols();
   height = original_image->Rows();
   WidthUpDown->Position = (short)width;
   HeightUpDown->Position = (short)height;
   WidthEdit->Text  =  width;
   HeightEdit->Text = height;
}
//---------------------------------------------------------------------------
__fastcall TResizeForm::~TResizeForm()
{
   delete (handler);
   delete (original_image);
   delete (resized_image);
   delete (proc_image);
   delete (scale);
}
//---------------------------------------------------------------------------
void
__fastcall TResizeForm::SetSize(int _new_cols, int _new_rows)
{
   try{
      int new_cols = (double)_new_cols*(double)proc_image->Cols()/
                                             (double)original_image->Cols();
      int new_rows = (double)_new_rows*(double)proc_image->Rows()/
                                             (double)original_image->Rows();
      scale->NewSize(new_cols, new_rows, true);
      handler->Assign(*resized_image);
      handler->Show(Image);
   }catch(VEMsg& E){
      MessageBox(Handle, E.msg.c_str(), "Resize",
                        MB_ICONQUESTION | MB_OK);
   }
}
//---------------------------------------------------------------------------
void
__fastcall TResizeForm::SetProcSize()
{
   GreyImage temp;
   int larger_size_of_image;
   larger_size_of_image = (proc_image->Cols()>proc_image->Rows()) ?
                 proc_image->Cols() : proc_image->Rows();
   int larger = larger_size_of_image*ResizePanel->Width/LARGER_SIZE_OF_PAPER;
   Scale scale(*proc_image, temp, larger);
   *proc_image = temp;
}
//---------------------------------------------------------------------------
void __fastcall TResizeForm::WidthEditClick(TObject *Sender)
{
   UserRadioButton->Checked = true;        
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::HeightEditClick(TObject *Sender)
{
   UserRadioButton->Checked = true;        
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::UnitComboBoxClick(TObject *Sender)
{
   UserRadioButton->Checked = true;        
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::StandardComboBoxClick(TObject *Sender)
{
   StandardRadioButton->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::WidthEditChange(TObject *Sender)
{
   double ratio = (double)original_image->Rows()/(double)original_image->Cols();
   try{
      switch(UnitComboBox->ItemIndex){
         case(0):  // pixels
            width = WidthEdit->Text.ToInt();
            if(width>MAX_WIDTH){
               WidthEdit->Text = MAX_WIDTH;
               break;
            }
            if(RatioCheckBox->Checked==true)
               if(width>MAX_HEIGHT/ratio){
                  WidthEdit->Text = (int)((double)MAX_HEIGHT/ratio);
                  break;
               }
            WidthUpDown->Position = (short)width;
            if(RatioCheckBox->Checked==true){
                HeightEdit->Text = (int)((double)width*ratio);
                height = HeightEdit->Text.ToInt();
            }
            SetSize(width, height);
            break;
         case(1):  // percent
            double percent = (double)WidthEdit->Text.ToInt()/100;
            width = (double)original_image->Cols()*percent;
            /*
               In case of width bigger than MAX_WIDTH width set to the
               max value
            */
            if(width>MAX_WIDTH){
               WidthEdit->Text = (int)(100*(double)MAX_WIDTH/
                                       (double)original_image->Cols());
               break;                        
            }
            if(RatioCheckBox->Checked==true)
               if(width>MAX_HEIGHT/ratio){
                  WidthEdit->Text = (int)(100*(double)MAX_HEIGHT/ratio/
                                     (double)original_image->Cols());
                  break;
               }
            WidthUpDown->Position = (short)WidthEdit->Text.ToInt();
            if(RatioCheckBox->Checked==true){
                HeightEdit->Text = WidthEdit->Text;
                height = (double)original_image->Rows()*percent;
            }
            SetSize(width, height);
            break;
      }
   }catch(EConvertError& e){
      MessageBox(Handle, e.Message.c_str(), Caption.c_str(),
                                        MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------


void __fastcall TResizeForm::WidthUpDownClick(TObject *Sender,
      TUDBtnType Button)
{
   UserRadioButton->Checked = true;
   width = WidthUpDown->Position;
   WidthEdit->Text = width;
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::HeightUpDownClick(TObject *Sender,
      TUDBtnType Button)
{
   UserRadioButton->Checked = true;
   height = HeightUpDown->Position;
   HeightEdit->Text = height;
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::HeightEditChange(TObject *Sender)
{
   try{
      if(!RatioCheckBox->Checked)
      switch(UnitComboBox->ItemIndex){
         case(0):  // pixels
            height = HeightEdit->Text.ToInt();
            if(height>MAX_HEIGHT){
               HeightEdit->Text = MAX_HEIGHT;
            }
            HeightUpDown->Position = (short)height;
            
            SetSize(width, height);
            break;
         case(1):  // percent
            double percent = (double)HeightEdit->Text.ToInt()/100;
            height = (double)original_image->Rows()*percent;
            if(height>MAX_HEIGHT){
               HeightEdit->Text = (int)(100*(double)MAX_HEIGHT/
                                       (double)original_image->Rows());
            }
            HeightUpDown->Position = (short)HeightEdit->Text.ToInt();
            SetSize(width, height);
            break;
      }
   }catch(EConvertError& e){
      MessageBox(Handle, e.Message.c_str(), Caption.c_str(),
                                        MB_ICONWARNING | MB_OK);
   }
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::UnitComboBoxChange(TObject *Sender)
{
    switch(UnitComboBox->ItemIndex){
         case(0):  // pixels
            WidthEdit->Text = original_image->Cols();
            HeightEdit->Text = original_image->Rows();
            break;
         case(1):  // percent
            WidthEdit->Text = 100;
            HeightEdit->Text = 100;
            break;
      }
}
//---------------------------------------------------------------------------


void __fastcall TResizeForm::StandardComboBoxChange(TObject *Sender)
{
   StandardRadioButton->Checked = true;
   SetStandardSize();
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::StandardRadioButtonClick(TObject *Sender)
{
    SetStandardSize();
}
//---------------------------------------------------------------------------
void __fastcall TResizeForm::SetStandardSize()
{
   switch(StandardComboBox->ItemIndex){
      case(0):  // 128x128
         SetSize(128,128);
         break;
      case(1):  // 256x256
         SetSize(256,256);
         break;
      case(2):  // 512x512
         SetSize(512,512);
         break;
      case(3):  // 1024x1024
         SetSize(1024,1024);
         break;
   }
}

void __fastcall TResizeForm::BitBtn1Click(TObject *Sender)
{
   GreyImage temp;
   Scale scale_orig(*original_image, temp);

   if(StandardRadioButton->Checked == true){
      switch(StandardComboBox->ItemIndex){
         case(0):
            scale_orig.NewSize(128,128,true);
            break;
         case(1):
            scale_orig.NewSize(256,256,true);
            break;
         case(2):
            scale_orig.NewSize(512,512,true);
            break;
         case(3):
            scale_orig.NewSize(1024,1024,true);
            break;
      }
   }
   if(UserRadioButton->Checked == true){
      switch(UnitComboBox->ItemIndex){
         case(0):      // pixels
            scale_orig.NewSize(width,height,true);
            break;
         case(1):    // percent
            scale_orig.NewSize(width,height,true);
            break;
      }
   }
   PictureForm->AssignList(&temp);
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   Release();        
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::StandardComboBoxEnter(TObject *Sender)
{
   StandardRadioButton->Checked = true;        
}
//---------------------------------------------------------------------------

void __fastcall TResizeForm::RatioCheckBoxClick(TObject *Sender)
{
   if(RatioCheckBox->Checked==true){
      HeightEdit->ReadOnly = true;
      HeightUpDown->Enabled = false;
   }else{
      HeightEdit->ReadOnly = false;
      HeightUpDown->Enabled = true;
   }   
}
//---------------------------------------------------------------------------

  