/*
   RotateUnit.cpp

   definin soubor okna pro rotaci

   MIPS 2.0 - Microscopy Image Processing Software
   Gerla Vclav, 2005

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "RotateUnit.h"
#include "PictureUnit.h"
#include "ColoBoxUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
#define MAX_COLS 1024
#define MAX_ROWS 1024
TRotateForm *RotateForm;
//---------------------------------------------------------------------------
__fastcall TRotateForm::TRotateForm(TComponent* Owner)
        : TForm(Owner)
{
   Left = PictureForm->Left+20;
   Top = PictureForm->Top + PictureForm->Height - Height - 40;
   original_image = new GreyImage();
   old_image = new GreyImage();
   rotated_image = new GreyImage();
   rotated_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
   rotated_handler->Set(*original_image);
   rotated_handler->Set(*rotated_image);
   RotateE->Text =  RotateUpDown->Position;
   rotate  = new Rotate(*original_image, *rotated_image);
   color = clBlack;
   Panel2->Color = color;
}
//---------------------------------------------------------------------------
__fastcall TRotateForm::~TRotateForm()
{
   delete (rotated_handler);
   delete (original_image);
   delete (old_image);
   delete (rotated_image);
   delete (rotate);
}
//---------------------------------------------------------------------------
void __fastcall TRotateForm::RotateUpDownClick(TObject *Sender,
      TUDBtnType Button)
{
    RotateE->Text =  RotateUpDown->Position;
}
//---------------------------------------------------------------------------
void __fastcall TRotateForm::LeftBClick(TObject *Sender)
{
   try{
      delete (rotated_handler);
      delete (rotate);
      rotated_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
      rotated_handler->Set(*old_image);
      rotate  = new Rotate(*old_image, *rotated_image);
      rotate->ChangeSizeRotation(RotateUpDown->Position, 1,color);
      PictureForm->AssignImage(rotated_image);
   }catch(VEMsg &E){
         Application->MessageBox(E.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TRotateForm::RightBClick(TObject *Sender)
{
   try{
      delete (rotated_handler);
      delete (rotate);
      rotated_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
      rotated_handler->Set(*old_image);
      rotate  = new Rotate(*old_image, *rotated_image);
      rotate->ChangeSizeRotation(360 - RotateUpDown->Position, 1, color);
      PictureForm->AssignImage(rotated_image);
   }catch(VEMsg &E){
         Application->MessageBox(E.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TRotateForm::FlipHBClick(TObject *Sender)
{
   try{
      delete (rotated_handler);
      rotated_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
      rotated_handler->Set(*old_image);
      Translate* trans = new Translate(*old_image, *rotated_image);
      trans->MirrorImage(0, 0, old_image->Cols(), old_image->Rows(),
                         0, 0, HorizMirror);
      PictureForm->AssignImage(rotated_image);
      delete (trans);
   }catch(VEMsg &E){
         Application->MessageBox(E.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TRotateForm::CancelBitBtnClick(TObject *Sender)
{
   try{
      PictureForm->AssignImage(original_image);
      Close();
   }catch(VEMsg &E){
         Application->MessageBox(E.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TRotateForm::FlipVBitBtnClick(TObject *Sender)
{
   try{
      delete (rotated_handler);
      rotated_handler = new BMPHandler(*(PictureForm->GetBMPHandler()));
      rotated_handler->Set(*old_image);
      Translate* trans = new Translate(*old_image, *rotated_image);
      trans->MirrorImage(0, 0, old_image->Cols(), old_image->Rows(),
                         0, 0, VertMirror);
      PictureForm->AssignImage(rotated_image);
      delete (trans);
   }catch(VEMsg &E){
         Application->MessageBox(E.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------
void __fastcall TRotateForm::ResetBitBtnClick(TObject *Sender)
{
   try{
      PictureForm->AssignImage(original_image);
      RotateUpDown->Position = 90;
      *rotated_image = *original_image;
      RotateE->Text = "90";
   }catch(VEMsg &E){
         Application->MessageBox(E.msg.c_str(),
         "Exception Catched",MB_ICONHAND|MB_OK);
   }
}
//---------------------------------------------------------------------------

void __fastcall TRotateForm::FormClose(TObject *Sender,
      TCloseAction &Action)
{
   PictureForm->MEdit->Visible=true;
   PictureForm->Open1->Visible = true;
   PictureForm->MSave->Visible = true;
   PictureForm->MSaveAs->Visible = true;
   PictureForm->MImage->Visible=true;
   PictureForm->MView->Visible=true;
   PictureForm->MColor->Visible=true;
   Release();
}
//---------------------------------------------------------------------------


void __fastcall TRotateForm::Image1DblClick(TObject *Sender)
{
   Application->CreateForm(__classid(TColorBoxForm),
                          &ColorBoxForm);
   // this functions used from the DrawForm
   ColorBoxForm->KindColor(other);
   ColorBoxForm->ResultPanel->Color = color;
   ColorBoxForm->ShowModal();
   color = ColorBoxForm->ResultPanel->Color;
   Panel2->Color = color;
}
//---------------------------------------------------------------------------

void __fastcall TRotateForm::OKBitBtnClick(TObject *Sender)
{
   PictureForm->AssignList(rotated_image);
   Close();
}
//---------------------------------------------------------------------------


